@@ -106,7 +106,7 @@ class EnterpriseDistro(packager.Distro):
             raise Exception("BUG: unsupported platform?")
 
     def build_os(self, arch):
-        """Return the build os label in the binary package to download ("rhel57", "rhel62" and "rhel70"
+        """Return the build os label in the binary package to download ("rhel57", "rhel62", "rhel67" and "rhel70"
         for redhat, the others are delegated to the super class
         """
         if arch == "ppc64le":
@@ -118,7 +118,7 @@ class EnterpriseDistro(packager.Distro):
                 return []
         if arch == "s390x":
             if self.n == 'redhat':
-                return [ "rhel72" ]
+                return [ "rhel67", "rhel72" ]
             if self.n == 'suse':
                 return [ "suse11", "suse12" ]
             if self.n == 'ubuntu':
@@ -157,10 +157,7 @@ def main(argv):
 
     os.chdir(prefix)
     try:
-      # Download the binaries.
-      urlfmt="http://downloads.mongodb.com/linux/mongodb-linux-%s-enterprise-%s-%s.tgz"
       made_pkg = False
-
       # Build a package for each distro/spec/arch tuple, and
       # accumulate the repository-layout directories.
       for (distro, arch) in packager.crossproduct(distros, args.arches):
@@ -168,12 +165,9 @@ def main(argv):
           for build_os in distro.build_os(arch):
             if build_os in args.distros or not args.distros:
 
-              if args.tarball:
-                filename = tarfile(build_os, arch, spec)
-                packager.ensure_dir(filename)
-                shutil.copyfile(args.tarball,filename)
-              else:
-                packager.httpget(urlfmt % (arch, build_os, spec.version()), packager.ensure_dir(tarfile(build_os, arch, spec)))
+              filename = tarfile(build_os, arch, spec)
+              packager.ensure_dir(filename)
+              shutil.copyfile(args.tarball, filename)
 
               repo = make_package(distro, build_os, arch, spec, srcdir)
               make_repo(repo, distro, build_os, spec)
