@@ -258,7 +258,9 @@ class Distro(object):
             else:
                 raise Exception("unsupported build_os: %s" % build_os)
         elif self.n == 'debian':
-            if build_os == 'debian81':
+            if build_os == 'debian71':
+                return 'wheezy'
+            elif build_os == 'debian81':
                 return 'jessie'
             else:
                 raise Exception("unsupported build_os: %s" % build_os)
@@ -275,7 +277,7 @@ class Distro(object):
 
     def build_os(self, arch):
         """Return the build os label in the binary package to download (e.g. "rhel55" for redhat,
-        "ubuntu1204" for ubuntu, "debian81" for debian, "suse11" for suse, etc.)"""
+        "ubuntu1204" for ubuntu, "debian71" for debian, "suse11" for suse, etc.)"""
         # Community builds only support amd64
         if arch not in ['x86_64', 'ppc64le', 's390x', 'arm64']:
             raise Exception("BUG: unsupported architecture (%s)" % arch)
@@ -289,7 +291,7 @@ class Distro(object):
         elif self.n == 'ubuntu':
             return [ "ubuntu1204", "ubuntu1404", "ubuntu1604", ]
         elif self.n == 'debian':
-            return [ "debian81" ]
+            return [ "debian71", "debian81" ]
         else:
             raise Exception("BUG: unsupported platform?")
 
@@ -354,7 +356,7 @@ def main(argv):
 
               filename = tarfile(build_os, arch, spec)
               ensure_dir(filename)
-              shutil.copyfile(args.tarball,filename)
+              shutil.copyfile(args.tarball, filename)
 
               repo = make_package(distro, build_os, arch, spec, srcdir)
               make_repo(repo, distro, build_os, spec)
@@ -469,8 +471,12 @@ def make_deb(distro, build_os, arch, spec, srcdir):
     sdir=setupdir(distro, build_os, arch, spec)
     if re.search("debian", distro.name()):
         os.unlink(sdir+"debian/mongod.upstart")
-        os.link(sdir+"debian/mongod.service", sdir+"debian/%s%s-server.mongod.service" % (distro.pkgbase(), suffix))
-        os.unlink(sdir+"debian/init.d")
+        if build_os == "debian71":
+            os.link(sdir+"debian/init.d", sdir+"debian/%s%s-server.mongod.init" % (distro.pkgbase(), suffix))
+            os.unlink(sdir+"debian/mongod.service")
+        else:
+            os.link(sdir+"debian/mongod.service", sdir+"debian/%s%s-server.mongod.service" % (distro.pkgbase(), suffix))
+            os.unlink(sdir+"debian/init.d")
     elif re.search("ubuntu", distro.name()):
         os.unlink(sdir+"debian/init.d")
         if build_os in ("ubuntu1204", "ubuntu1404", "ubuntu1410"):
@@ -703,7 +709,7 @@ def make_rpm(distro, build_os, arch, spec, srcdir):
         # This hard-coded hooey came from some box running RPM
         # 4.4.2.3.  It may not work over time, but RPM isn't sanely
         # configurable.
-        flags=["--macros", "/usr/lib/rpm/macros:/usr/lib/rpm/%s-linux/macros:/etc/rpm/macros.*:/etc/rpm/macros:/etc/rpm/%s-linux/macros:~/.rpmmacros:%s" % (distro_arch, distro_arch, macropath)]
+        flags=["--macros", "/usr/lib/rpm/macros:/usr/lib/rpm/%s-linux/macros:/usr/lib/rpm/suse/macros:/etc/rpm/macros.*:/etc/rpm/macros:/etc/rpm/%s-linux/macros:~/.rpmmacros:%s" % (distro_arch, distro_arch, macropath)]
     # Put the specfile and the tar'd up binaries and stuff in
     # place.
     #
