@@ -4,16 +4,27 @@ Interface of the different fixtures for executing JSTests against.
 
 from __future__ import absolute_import
 
-import os.path
 import time
 
 import pymongo
 import pymongo.errors
 
-from ... import config
 from ... import errors
 from ... import logging
-from ... import utils
+from ...utils import registry
+
+
+_FIXTURES = {}
+
+
+def make_fixture(class_name, *args, **kwargs):
+    """
+    Factory function for creating Fixture instances.
+    """
+
+    if class_name not in _FIXTURES:
+        raise ValueError("Unknown fixture class '%s'" % (class_name))
+    return _FIXTURES[class_name](*args, **kwargs)
 
 
 class Fixture(object):
@@ -21,7 +32,13 @@ class Fixture(object):
     Base class for all fixtures.
     """
 
-    def __init__(self, logger, job_num, dbpath_prefix=None):
+    __metaclass__ = registry.make_registry_metaclass(_FIXTURES)
+
+    # We explicitly set the 'REGISTERED_NAME' attribute so that PyLint realizes that the attribute
+    # is defined for all subclasses of Fixture.
+    REGISTERED_NAME = "Fixture"
+
+    def __init__(self, logger, job_num):
         """
         Initializes the fixture with a logger instance.
         """
@@ -36,11 +53,6 @@ class Fixture(object):
 
         self.logger = logger
         self.job_num = job_num
-        self.port = None  # Port that the mongo shell should connect to.
-
-        dbpath_prefix = utils.default_if_none(config.DBPATH_PREFIX, dbpath_prefix)
-        dbpath_prefix = utils.default_if_none(dbpath_prefix, config.DEFAULT_DBPATH_PREFIX)
-        self._dbpath_prefix = os.path.join(dbpath_prefix, "job{}".format(self.job_num))
 
     def setup(self):
         """
@@ -86,17 +98,13 @@ class Fixture(object):
         """
         return True
 
-    def get_dbpath_prefix(self):
-        return self._dbpath_prefix
-
     def get_internal_connection_string(self):
         """
         Returns the connection string for this fixture. This is NOT a
         driver connection string, but a connection string of the format
         expected by the mongo::ConnectionString class.
         """
-        raise NotImplementedError(
-            "get_internal_connection_string must be implemented by Fixture subclasses")
+        raise NotImplementedError("get_connection_string must be implemented by Fixture subclasses")
 
     def get_driver_connection_url(self):
         """
@@ -106,6 +114,24 @@ class Fixture(object):
         raise NotImplementedError(
             "get_driver_connection_url must be implemented by Fixture subclasses")
 
+    def mongo_client(self, read_preference=pymongo.ReadPreference.PRIMARY, timeout_millis=30000):
+        """
+        Returns a pymongo.MongoClient connecting to this fixture with a read
+        preference of 'read_preference'.
+
+        The PyMongo driver will wait up to 'timeout_millis' milliseconds
+        before concluding that the server is unavailable.
+        """
+
+        kwargs = {"connectTimeoutMS": timeout_millis}
+        if pymongo.version_tuple[0] >= 3:
+            kwargs["serverSelectionTimeoutMS"] = timeout_millis
+            kwargs["connect"] = True
+
+        return pymongo.MongoClient(host=self.get_driver_connection_url(),
+                                   read_preference=read_preference,
+                                   **kwargs)
+
     def __str__(self):
         return "%s (Job #%d)" % (self.__class__.__name__, self.job_num)
 
@@ -118,6 +144,8 @@ class ReplFixture(Fixture):
     Base class for all fixtures that support replication.
     """
 
+    REGISTERED_NAME = registry.LEAVE_UNREGISTERED
+
     AWAIT_REPL_TIMEOUT_MINS = 5
 
     def get_primary(self):
@@ -155,8 +183,9 @@ class ReplFixture(Fixture):
             except pymongo.errors.ConnectionFailure:
                 remaining = deadline - time.time()
                 if remaining <= 0.0:
-                    raise errors.ServerFailure("Failed to connect to the primary on port %d" %
-                                               self.port)
+                    raise errors.ServerFailure(
+                        "Failed to connect to ".format(self.get_driver_connection_url()))
+
 
 class NoOpFixture(Fixture):
     """A Fixture implementation that does not start any servers.
@@ -165,6 +194,8 @@ class NoOpFixture(Fixture):
     ReplSetTest, or ShardingTest.
     """
 
+    REGISTERED_NAME = "NoOpFixture"
+
     def get_internal_connection_string(self):
         return None
 
