@@ -9,7 +9,6 @@ import sys
 
 from .. import config
 from .. import errors
-from .. import logging
 from ..utils import queue as _queue
 
 
@@ -18,7 +17,7 @@ class Job(object):
     Runs tests from a queue.
     """
 
-    def __init__(self, logger, fixture, hooks, report, archival):
+    def __init__(self, logger, fixture, hooks, report, suite_options):
         """
         Initializes the job with the specified fixture and custom
         behaviors.
@@ -28,7 +27,7 @@ class Job(object):
         self.fixture = fixture
         self.hooks = hooks
         self.report = report
-        self.archival = archival
+        self.suite_options = suite_options
 
     def __call__(self, queue, interrupt_flag, teardown_flag=None):
         """
@@ -99,37 +98,20 @@ class Job(object):
         self._run_hooks_before_tests(test)
 
         test(self.report)
-        try:
-            if config.FAIL_FAST and not self.report.wasSuccessful():
-                self.logger.info("%s failed, so stopping..." % (test.shortDescription()))
-                raise errors.StopExecution("%s failed" % (test.shortDescription()))
+        if self.suite_options.fail_fast and not self.report.wasSuccessful():
+            self.logger.info("%s failed, so stopping..." % (test.shortDescription()))
+            raise errors.StopExecution("%s failed" % (test.shortDescription()))
 
-            if not self.fixture.is_running():
-                self.logger.error(
-                    "%s marked as a failure because the fixture crashed during the test.",
-                    test.shortDescription())
-                self.report.setFailure(test, return_code=2)
-                # Always fail fast if the fixture fails.
-                raise errors.StopExecution("%s not running after %s" %
-                                           (self.fixture, test.shortDescription()))
-
-        finally:
-            success = self.report.find_test_info(test).status == "pass"
-            if self.archival:
-                self.archival.archive(self.logger, test, success)
+        if not self.fixture.is_running():
+            self.logger.error("%s marked as a failure because the fixture crashed during the test.",
+                              test.shortDescription())
+            self.report.setFailure(test, return_code=2)
+            # Always fail fast if the fixture fails.
+            raise errors.StopExecution("%s not running after %s" %
+                                       (self.fixture, test.shortDescription()))
 
         self._run_hooks_after_tests(test)
 
-    def _run_hook(self, hook, hook_function, test):
-        """Provide helper to run hook and archival."""
-        try:
-            success = False
-            hook_function(test, self.report)
-            success = True
-        finally:
-            if self.archival:
-                self.archival.archive(self.logger, test, success, hook=hook)
-
     def _run_hooks_before_tests(self, test):
         """
         Runs the before_test method on each of the hooks.
@@ -140,7 +122,7 @@ class Job(object):
 
         try:
             for hook in self.hooks:
-                self._run_hook(hook, hook.before_test, test)
+                hook.before_test(test, self.report)
 
         except errors.StopExecution:
             raise
@@ -155,7 +137,7 @@ class Job(object):
             self.logger.exception("%s marked as a failure by a hook's before_test.",
                                   test.shortDescription())
             self._fail_test(test, sys.exc_info(), return_code=1)
-            if config.FAIL_FAST:
+            if self.suite_options.fail_fast:
                 raise errors.StopExecution("A hook's before_test failed")
 
         except:
@@ -174,7 +156,7 @@ class Job(object):
         """
         try:
             for hook in self.hooks:
-                self._run_hook(hook, hook.after_test, test)
+                hook.after_test(test, self.report)
 
         except errors.StopExecution:
             raise
@@ -189,7 +171,7 @@ class Job(object):
             self.logger.exception("%s marked as a failure by a hook's after_test.",
                                   test.shortDescription())
             self.report.setFailure(test, return_code=1)
-            if config.FAIL_FAST:
+            if self.suite_options.fail_fast:
                 raise errors.StopExecution("A hook's after_test failed")
 
         except:
