@@ -90,7 +90,7 @@
     // This write will have to wait on the initial sync to complete before progressing.
     assert.soonNoExcept(() => {
         assert.writeOK(rstest.getPrimary().getDB("db1")["aCollection"].insert(
-            {_id: "afterSecondNodeAdded"}, {writeConcern: {w: 2, wtimeout: 60 * 1000}}));
+            {a: "afterSecondNodeAdded"}, {writeConcern: {w: 2, wtimeout: 60 * 1000}}));
         return true;
     });
 
@@ -134,14 +134,6 @@
         assertListContainsRole(role.inheritedRoles, {role: "dbAdmin", db: "db1"}, node);
     });
 
-    // Verify that dropping the admin database propagates.
-    assert.commandWorked(rstest.getPrimary().getDB("admin").dropDatabase());
-    assert.commandWorked(rstest.getPrimary().getDB("admin").getLastErrorObj(2));
-    rstest.nodes.forEach(function(node) {
-        var roles = node.getDB("db1").getRoles();
-        assert.eq(0, roles.length, node);
-    });
-
     // Verify that applyOps commands propagate.
     // NOTE: This section of the test depends on the oplog and roles schemas.
     assert.commandWorked(rstest.getPrimary().getDB("admin").runCommand({
@@ -170,7 +162,7 @@
                   privileges: [{resource: {db: "db1", collection: "log"}, actions: ["insert"]}]
               }
             },
-            {op: "c", ns: "admin.$cmd", o: {dropDatabase: 1}},
+            {op: "c", ns: "admin.$cmd", o: {drop: "system.roles"}},
             {op: "c", ns: "admin.$cmd", o: {create: "system.roles"}},
             {
               op: "i",
