@@ -9,6 +9,7 @@ var blacklist = [
     'distinct.js',             // SERVER-13116 distinct isn't sharding aware
     'distinct_noindex.js',     // SERVER-13116 distinct isn't sharding aware
     'distinct_projection.js',  // SERVER-13116 distinct isn't sharding aware
+    'create_database.js',      // SERVER-17397 Drops of sharded namespaces may not fully succeed
     'drop_database.js',        // SERVER-17397 Drops of sharded namespaces may not fully succeed
 
     // Disabled due to SERVER-3645, '.count() can be wrong on sharded collections'.
@@ -16,7 +17,6 @@ var blacklist = [
     'agg_match.js',
     'count.js',
     'count_limit_skip.js',
-    'count_noindex.js',
 
     // $lookup and $graphLookup are not supported on sharded collections.
     'agg_graph_lookup.js',
@@ -32,6 +32,7 @@ var blacklist = [
     'map_reduce_reduce_nonatomic.js',
     'map_reduce_replace.js',
     'map_reduce_replace_nonexistent.js',
+    'map_reduce_replace_remove.js',
 
     // Disabled due to MongoDB restrictions and/or workload restrictions
 
@@ -49,7 +50,6 @@ var blacklist = [
     'compact_simultaneous_padding_bytes.js',  // compact can only be run against a mongod
     'convert_to_capped_collection.js',        // convertToCapped can't be run on mongos processes
     'convert_to_capped_collection_index.js',  // convertToCapped can't be run on mongos processes
-    'findAndModify_mixed_queue.js',           // findAndModify requires a shard key
     'findAndModify_mixed_queue_unindexed.js',   // findAndModify requires a shard key
     'findAndModify_remove_queue.js',            // remove cannot be {} for findAndModify
     'findAndModify_remove_queue_unindexed.js',  // findAndModify requires a shard key
@@ -62,11 +62,12 @@ var blacklist = [
     'indexed_insert_eval.js',  // eval doesn't work with sharded collections
     'indexed_insert_eval_nolock.js',  // eval doesn't work with sharded collections
 
-    // This workload sometimes triggers an 'unable to target write op for collection ... caused by
-    // ... database not found' error. Further investigation still needs to be done, but this
-    // workload may be failing due to SERVER-17397 'drops in a sharded cluster may not fully
-    // succeed' because it drops and reuses the same namespaces.
+    // These workloads sometimes triggers an 'unable to target write op for collection ... caused by
+    // ... database not found' error. Further investigation still needs to be done, but these
+    // failures may be due to SERVER-17397 'drops in a sharded cluster may not fully succeed'
+    // because it drops and reuses the same namespaces.
     'kill_multicollection_aggregation.js',
+    'invalidated_cursors.js',
 
     'plan_cache_drop_database.js',  // cannot ensureIndex after dropDatabase without sharding first
     'remove_single_document.js',    // our .remove(query, {justOne: true}) calls lack shard keys
