@@ -44,6 +44,9 @@ var Cluster = function(options) {
             'sharded.enableBalancer',
             'sharded.numMongos',
             'sharded.numShards',
+            'sharded.stepdownOptions',
+            'sharded.stepdownOptions.configStepdown',
+            'sharded.stepdownOptions.shardStepdown',
             'teardownFunctions'
         ];
 
@@ -161,14 +164,13 @@ var Cluster = function(options) {
                "Both 'masterSlave' and 'sharded.enabled' cannot" + "be true");
     }
 
-    function makeReplSetTestConfig(numReplSetNodes, firstNodeOnlyVote = true) {
+    function makeReplSetTestConfig(numReplSetNodes) {
         const REPL_SET_VOTING_LIMIT = 7;
-        var firstNodeNotVoting = firstNodeOnlyVote ? 1 : REPL_SET_VOTING_LIMIT;
         // Workaround for SERVER-26893 to specify when numReplSetNodes > REPL_SET_VOTING_LIMIT.
         var rstConfig = [];
         for (var i = 0; i < numReplSetNodes; i++) {
             rstConfig[i] = {};
-            if (i >= firstNodeNotVoting) {
+            if (i >= REPL_SET_VOTING_LIMIT) {
                 rstConfig[i].rsConfig = {priority: 0, votes: 0};
             }
         }
@@ -220,17 +222,32 @@ var Cluster = function(options) {
                 shardConfig.rsOptions = {};
             }
 
+            if (this.shouldPerformContinuousStepdowns()) {
+                load('jstests/libs/override_methods/continuous_stepdown.js');
+                ContinuousStepdown.configure(options.sharded.stepdownOptions);
+            }
+
             st = new ShardingTest(shardConfig);
 
             conn = st.s;  // mongos
 
-            this.teardown = function teardown(opts) {
+            this.teardown = function teardown() {
                 options.teardownFunctions.mongod.forEach(this.executeOnMongodNodes);
                 options.teardownFunctions.mongos.forEach(this.executeOnMongosNodes);
 
-                st.stop(opts);
+                st.stop();
             };
 
+            if (this.shouldPerformContinuousStepdowns()) {
+                this.startContinuousFailover = function() {
+                    st.startContinuousFailover();
+                };
+
+                this.stopContinuousFailover = function() {
+                    st.stopContinuousFailover({waitForPrimary: true});
+                };
+            }
+
             // Save all mongos and mongod connections
             var i = 0;
             var mongos = st.s0;
@@ -274,9 +291,10 @@ var Cluster = function(options) {
             conn = rst.getPrimary();
             replSets = [rst];
 
-            this.teardown = function teardown(opts) {
+            this.teardown = function teardown() {
                 options.teardownFunctions.mongod.forEach(this.executeOnMongodNodes);
-                rst.stopSet(undefined, undefined, opts);
+
+                rst.stopSet();
             };
 
             this._addReplicaSetConns(rst);
@@ -340,7 +358,7 @@ var Cluster = function(options) {
             throw new Error('mongos function must be a function that takes a db as an argument');
         }
         _conns.mongos.forEach(function(mongosConn) {
-            fn(mongosConn.getDB('admin'));
+            fn(mongosConn.getDB('admin'), true /* isMongos */);
         });
     };
 
@@ -369,7 +387,7 @@ var Cluster = function(options) {
     };
 
     this.isReplication = function isReplication() {
-        return options.replication.enabled;
+        return Cluster.isReplication(options);
     };
 
     this.shardCollection = function shardCollection() {
@@ -474,11 +492,24 @@ var Cluster = function(options) {
     this.validateAllCollections = function validateAllCollections(phase) {
         assert(initialized, 'cluster must be initialized first');
 
-        var _validateCollections = function _validateCollections(db) {
+        const isSteppingDownConfigServers = this.isSteppingDownConfigServers();
+        var _validateCollections = function _validateCollections(db, isMongos = false) {
             // Validate all the collections on each node.
             var res = db.adminCommand({listDatabases: 1});
             assert.commandWorked(res);
             res.databases.forEach(dbInfo => {
+                // Don't perform listCollections on the admin or config database through a mongos
+                // connection when stepping down the config server primary, because both are stored
+                // on the config server, and listCollections may return a not master error if the
+                // mongos is stale.
+                //
+                // TODO SERVER-30949: listCollections through mongos should automatically retry on
+                // NotMaster errors. Once that is true, remove this check.
+                if (isSteppingDownConfigServers && isMongos &&
+                    (dbInfo.name === "admin" || dbInfo.name === "config")) {
+                    return;
+                }
+
                 if (!validateCollections(db.getSiblingDB(dbInfo.name), {full: true})) {
                     throw new Error(phase + ' collection validation failed');
                 }
@@ -586,12 +617,24 @@ var Cluster = function(options) {
         var res = adminDB.runCommand({getCmdLineOpts: 1});
         assert.commandWorked(res, 'failed to get command line options');
 
-        var wiredTigerOptions = res.parsed.storage.wiredTiger || {};
+        var wiredTigerOptions = {};
+        if (res.parsed && res.parsed.storage) {
+            wiredTigerOptions = res.parsed.storage.wiredTiger || {};
+        }
         var wiredTigerCollectionConfig = wiredTigerOptions.collectionConfig || {};
         var wiredTigerConfigString = wiredTigerCollectionConfig.configString || '';
 
         return wiredTigerConfigString === 'type=lsm';
     };
+
+    this.shouldPerformContinuousStepdowns = function shouldPerformContinuousStepdowns() {
+        return this.isSharded() && (typeof options.sharded.stepdownOptions !== 'undefined');
+    };
+
+    this.isSteppingDownConfigServers = function isSteppingDownConfigServers() {
+        return this.shouldPerformContinuousStepdowns() &&
+            options.sharded.stepdownOptions.configStepdown;
+    };
 };
 
 /**
@@ -602,3 +645,10 @@ Cluster.isStandalone = function isStandalone(clusterOptions) {
     return !clusterOptions.sharded.enabled && !clusterOptions.replication.enabled &&
         !clusterOptions.masterSlave;
 };
+
+/**
+ * Returns true if 'clusterOptions' represents a replica set, and returns false otherwise.
+ */
+Cluster.isReplication = function isReplication(clusterOptions) {
+    return clusterOptions.replication.enabled;
+};
