@@ -1,3 +1,5 @@
+// @tags: [requires_non_retryable_writes]
+
 //
 // Test of sample big polygon functionality
 //
@@ -7,8 +9,6 @@ coll.drop();
 
 // coll.ensureIndex({ loc : "2dsphere" });
 
-coll.getMongo().getDB("admin").runCommand({setParameter: 1, verboseQueryLogging: true});
-
 var bigCRS = {type: "name", properties: {name: "urn:x-mongodb:crs:strictwinding:EPSG:4326"}};
 
 var bigPoly20 = {
