@@ -6,20 +6,28 @@
  *   Displays index filters for all query shapes in a collection.
  *
  * - planCacheClearFilters
- *   Clears index filter for a single query shape or,
- *   if the query shape is omitted, all filters for the collection.
+ *   Clears index filter for a single query shape or, if the query shape is omitted, all filters for
+ *   the collection.
  *
  * - planCacheSetFilter
  *   Sets index filter for a query shape. Overrides existing filter.
  *
- * Not a lot of data access in this test suite. Hint commands
- * manage a non-persistent mapping in the server of
- * query shape to list of index specs.
+ * Not a lot of data access in this test suite. Hint commands manage a non-persistent mapping in the
+ * server of query shape to list of index specs.
  *
- * Only time we might need to execute a query is to check the plan
- * cache state. We would do this with the planCacheListPlans command
- * on the same query shape with the index filters.
+ * Only time we might need to execute a query is to check the plan cache state. We would do this
+ * with the planCacheListPlans command on the same query shape with the index filters.
  *
+ * @tags: [
+ *   # Cannot implicitly shard accessed collections because of collection existing when none
+ *   # expected.
+ *   assumes_no_implicit_collection_creation_after_drop,
+ *   # This test attempts to perform queries with plan cache filters set up. The former operation
+ *   # may be routed to a secondary in the replica set, whereas the latter must be routed to the
+ *   # primary.
+ *   assumes_read_preference_unchanged,
+ *   does_not_support_stepdowns,
+ * ]
  */
 
 load("jstests/libs/analyze_plan.js");
@@ -230,3 +238,42 @@ assert.commandWorked(
 
 explain = t.find(queryAA).collation(collationEN).explain();
 assert(isCollscan(explain.queryPlanner.winningPlan), "Expected collscan: " + tojson(explain));
+
+//
+// Test that planCacheSetFilter and planCacheClearFilters allow queries containing $expr.
+//
+
+t.drop();
+assert.writeOK(t.insert({a: "a"}));
+assert.commandWorked(t.createIndex(indexA1, {name: "a_1"}));
+
+assert.commandWorked(t.runCommand(
+    "planCacheSetFilter", {query: {a: "a", $expr: {$eq: ["$a", "a"]}}, indexes: [indexA1]}));
+filters = getFilters();
+assert.eq(1, filters.length, tojson(filters));
+assert.eq({a: "a", $expr: {$eq: ["$a", "a"]}}, filters[0].query, tojson(filters[0]));
+
+assert.commandWorked(
+    t.runCommand("planCacheClearFilters", {query: {a: "a", $expr: {$eq: ["$a", "a"]}}}));
+filters = getFilters();
+assert.eq(0, filters.length, tojson(filters));
+
+//
+// Test that planCacheSetFilter and planCacheClearFilters do not allow queries containing $expr with
+// unbound variables.
+//
+
+t.drop();
+assert.writeOK(t.insert({a: "a"}));
+assert.commandWorked(t.createIndex(indexA1, {name: "a_1"}));
+
+assert.commandFailed(
+    t.runCommand("planCacheSetFilter",
+                 {query: {a: "a", $expr: {$eq: ["$a", "$$unbound"]}}, indexes: [indexA1]}));
+filters = getFilters();
+assert.eq(0, filters.length, tojson(filters));
+
+assert.commandFailed(
+    t.runCommand("planCacheClearFilters", {query: {a: "a", $expr: {$eq: ["$a", "$$unbound"]}}}));
+filters = getFilters();
+assert.eq(0, filters.length, tojson(filters));
