@@ -5,7 +5,7 @@
 //   # failpoint. The former operations may be routed to a secondary in the replica set, whereas the
 //   # latter must be routed to the primary.
 //   assumes_read_preference_unchanged,
-//   requires_collmod_command,
+//   requires_getmore,
 // ]
 
 var t = db.max_time_ms;
@@ -279,15 +279,9 @@ cursor._ensureSpecial();
 assert.eq(0, cursor.next().ok);
 
 // Verify that the $maxTimeMS query option can't be sent with $query-wrapped commands.
-// the shell will throw here as it will validate itself when sending the command.
-// The server uses the same logic.
-// TODO: rewrite to use runCommandWithMetadata when we have a shell helper so that
-// we can test server side validation.
-assert.throws(function() {
-    cursor = t.getDB().$cmd.find({ping: 1}).limit(-1).maxTimeMS(0);
-    cursor._ensureSpecial();
-    cursor.next();
-});
+cursor = t.getDB().$cmd.find({ping: 1}).limit(-1).maxTimeMS(0);
+cursor._ensureSpecial();
+assert.commandFailed(cursor.next());
 
 //
 // Tests for fail points maxTimeAlwaysTimeOut and maxTimeNeverTimeOut.
@@ -387,7 +381,7 @@ assert.eq(1, t.getDB().adminCommand({configureFailPoint: "maxTimeNeverTimeOut",
 //
 
 // "aggregate" command.
-res = t.runCommand("aggregate", {pipeline: [], maxTimeMS: 60 * 1000});
+res = t.runCommand("aggregate", {pipeline: [], cursor: {}, maxTimeMS: 60 * 1000});
 assert(res.ok == 1,
        "expected aggregate with maxtime to succeed, ok=" + res.ok + ", code=" + res.code);
 
@@ -405,7 +399,7 @@ assert.commandWorked(
 //
 res = t.runCommand({parallelCollectionScan: t.getName(), numCursors: 1, maxTimeMS: 60 * 1000});
 assert.commandWorked(res);
-var cursor = new DBCommandCursor(t.getDB().getMongo(), res.cursors[0], 5);
+var cursor = new DBCommandCursor(t.getDB(), res.cursors[0], 5);
 assert.commandWorked(
     t.getDB().adminCommand({configureFailPoint: "maxTimeAlwaysTimeOut", mode: "alwaysOn"}));
 assert.throws(function() {
