@@ -1,3 +1,5 @@
+// @tags: [does_not_support_stepdowns]
+
 // Confirms that profiled aggregation execution contains all expected metrics with proper values.
 
 (function() {
@@ -21,7 +23,10 @@
     }
     assert.commandWorked(coll.createIndex({a: 1}));
 
-    assert.eq(8, coll.aggregate([{$match: {a: {$gte: 2}}}], {collation: {locale: "fr"}}).itcount());
+    assert.eq(8,
+              coll.aggregate([{$match: {a: {$gte: 2}}}],
+                             {collation: {locale: "fr"}, comment: "agg_comment"})
+                  .itcount());
     var profileObj = getLatestProfilerEntry(testDB);
 
     assert.eq(profileObj.ns, coll.getFullName(), tojson(profileObj));
@@ -35,6 +40,7 @@
               tojson(profileObj));
     assert.eq(profileObj.command.aggregate, coll.getName(), tojson(profileObj));
     assert.eq(profileObj.command.collation, {locale: "fr"}, tojson(profileObj));
+    assert.eq(profileObj.command.comment, "agg_comment", tojson(profileObj));
     assert(profileObj.hasOwnProperty("responseLength"), tojson(profileObj));
     assert(profileObj.hasOwnProperty("millis"), tojson(profileObj));
     assert(profileObj.hasOwnProperty("numYield"), tojson(profileObj));
@@ -69,4 +75,32 @@
     profileObj = getLatestProfilerEntry(testDB);
 
     assert.eq(profileObj.fromMultiPlanner, true, tojson(profileObj));
+
+    //
+    // Confirm that the "hint" modifier is in the profiler document.
+    //
+    coll.drop();
+    assert.commandWorked(coll.createIndex({a: 1}));
+    for (i = 0; i < 5; ++i) {
+        assert.writeOK(coll.insert({a: i, b: i}));
+    }
+
+    assert.eq(1, coll.aggregate([{$match: {a: 3, b: 3}}], {hint: {_id: 1}}).itcount());
+    profileObj = getLatestProfilerEntry(testDB);
+    assert.eq(profileObj.command.hint, {_id: 1}, tojson(profileObj));
+
+    //
+    // Confirm that aggregations are truncated in the profiler as { $truncated: <string>, comment:
+    // <string> } when a comment parameter is provided.
+    //
+    let matchPredicate = {};
+
+    for (let i = 0; i < 501; i++) {
+        matchPredicate[i] = "a".repeat(150);
+    }
+
+    assert.eq(coll.aggregate([{$match: matchPredicate}], {comment: "profile_agg"}).itcount(), 0);
+    profileObj = getLatestProfilerEntry(testDB);
+    assert.eq((typeof profileObj.command.$truncated), "string", tojson(profileObj));
+    assert.eq(profileObj.command.comment, "profile_agg", tojson(profileObj));
 })();
