@@ -1,3 +1,7 @@
+// Uses features that require featureCompatibilityVersion 3.6.
+// @tags: [requires_fcv36, does_not_support_stepdowns, requires_getmore,
+// requires_non_retryable_commands, requires_non_retryable_writes]
+
 /*
  * Declaratively-defined tests for views for all database commands. This file contains a map of test
  * definitions as well as code to run them.
@@ -48,8 +52,6 @@
  *
  *  skipStandalone
  *      If true, do not run this command on a standalone mongod.
- *
- * @tags: [requires_collmod_command]
  */
 
 (function() {
@@ -68,10 +70,17 @@
         _configsvrCommitChunkMerge: {skip: isAnInternalCommand},
         _configsvrCommitChunkMigration: {skip: isAnInternalCommand},
         _configsvrCommitChunkSplit: {skip: isAnInternalCommand},
+        _configsvrCreateDatabase: {skip: isAnInternalCommand},
+        _configsvrEnableSharding: {skip: isAnInternalCommand},
         _configsvrMoveChunk: {skip: isAnInternalCommand},
+        _configsvrMovePrimary: {skip: isAnInternalCommand},
+        _configsvrRemoveShard: {skip: isAnInternalCommand},
         _configsvrRemoveShardFromZone: {skip: isAnInternalCommand},
+        _configsvrShardCollection: {skip: isAnInternalCommand},
         _configsvrSetFeatureCompatibilityVersion: {skip: isAnInternalCommand},
         _configsvrUpdateZoneKeyRange: {skip: isAnInternalCommand},
+        _flushRoutingTableCacheUpdates: {skip: isUnrelated},
+        _getNextSessionMods: {skip: isAnInternalCommand},
         _getUserCacheGeneration: {skip: isAnInternalCommand},
         _hashBSONElement: {skip: isAnInternalCommand},
         _isSelf: {skip: isAnInternalCommand},
@@ -84,7 +93,7 @@
         _transferMods: {skip: isAnInternalCommand},
         addShard: {skip: isUnrelated},
         addShardToZone: {skip: isUnrelated},
-        aggregate: {command: {aggregate: "view", pipeline: [{$match: {}}]}},
+        aggregate: {command: {aggregate: "view", pipeline: [{$match: {}}], cursor: {}}},
         appendOplogNote: {skip: isUnrelated},
         applyOps: {
             command: {applyOps: [{op: "i", o: {_id: 1}, ns: "test.view"}]},
@@ -125,6 +134,7 @@
         copydbgetnonce: {skip: isUnrelated},
         copydbsaslstart: {skip: isUnrelated},
         count: {command: {count: "view"}},
+        cpuload: {skip: isAnInternalCommand},
         create: {skip: "tested in views/views_creation.js"},
         createIndexes: {
             command: {createIndexes: "view", indexes: [{key: {x: 1}, name: "x_1"}]},
@@ -149,11 +159,11 @@
             }
         },
         currentOp: {skip: isUnrelated},
-        currentOpCtx: {skip: isUnrelated},
         dataSize: {
             command: {dataSize: "test.view"},
             expectFailure: true,
         },
+        dbCheck: {command: {dbCheck: "view"}, expectFailure: true},
         dbHash: {
             command: function(conn) {
                 let getHash = function() {
@@ -168,14 +178,13 @@
                                      "could not create view 'view2' on 'view'");
                 let hash2 = getHash();
                 assert.neq(hash1, hash2, "expected hash to change after creating new view");
-                assert.commandWorked(conn.runCommand({drop: "view2"}, "problem dropping view2"));
+                assert.commandWorked(conn.runCommand({drop: "view2"}), "problem dropping view2");
                 let hash3 = getHash();
                 assert.eq(hash1, hash3, "hash should be the same again after removing 'view2'");
             }
         },
         dbStats: {skip: "TODO(SERVER-25948)"},
         delete: {command: {delete: "view", deletes: [{q: {x: 1}, limit: 1}]}, expectFailure: true},
-        diagLogging: {skip: isUnrelated},
         distinct: {command: {distinct: "view", key: "_id"}},
         driverOIDTest: {skip: isUnrelated},
         drop: {command: {drop: "view"}},
@@ -199,6 +208,7 @@
             expectFailure: true,
         },
         enableSharding: {skip: "Tested as part of shardCollection"},
+        endSessions: {skip: isUnrelated},
         eval: {skip: isUnrelated},
         explain: {command: {explain: {count: "view"}}},
         features: {skip: isUnrelated},
@@ -296,14 +306,19 @@
             },
             command: function(conn) {
                 // First get and check a partial result for an aggregate command.
-                let aggCmd = {aggregate: "view", pipeline: [], cursor: {batchSize: 2}};
+                let aggCmd = {
+                    aggregate: "view",
+                    pipeline: [{$sort: {_id: 1}}],
+                    cursor: {batchSize: 2}
+                };
                 let res = conn.runCommand(aggCmd);
                 assert.commandWorked(res, aggCmd);
                 let cursor = res.cursor;
                 assert.eq(
                     cursor.ns, "test.view", "expected view namespace in cursor: " + tojson(cursor));
                 let expectedFirstBatch = [{_id: 1}, {_id: 2}];
-                assert.eq(cursor.firstBatch, expectedFirstBatch, "find returned wrong firstBatch");
+                assert.eq(
+                    cursor.firstBatch, expectedFirstBatch, "aggregate returned wrong firstBatch");
 
                 // Then check correct execution of the killCursors command.
                 let killCursorsCmd = {killCursors: "view", cursors: [cursor.id]};
@@ -316,10 +331,15 @@
                     cursorsUnknown: [],
                     ok: 1
                 };
+                delete res.operationTime;
+                delete res.$clusterTime;
                 assert.eq(expectedRes, res, "unexpected result for: " + tojson(killCursorsCmd));
             }
         },
         killOp: {skip: isUnrelated},
+        killSessions: {skip: isUnrelated},
+        killAllSessions: {skip: isUnrelated},
+        killAllSessionsByPattern: {skip: isUnrelated},
         listCollections: {skip: "tested in views/views_creation.js"},
         listCommands: {skip: isUnrelated},
         listDatabases: {skip: isUnrelated},
@@ -351,6 +371,7 @@
             expectedErrorCode: ErrorCodes.NamespaceNotSharded,
         },
         movePrimary: {skip: "Tested in sharding/movePrimary1.js"},
+        multicast: {skip: isUnrelated},
         netstat: {skip: isAnInternalCommand},
         parallelCollectionScan: {command: {parallelCollectionScan: "view"}, expectFailure: true},
         ping: {command: {ping: 1}},
@@ -362,6 +383,10 @@
             {command: {planCacheListQueryShapes: "view"}, expectFailure: true},
         planCacheSetFilter: {command: {planCacheSetFilter: "view"}, expectFailure: true},
         profile: {skip: isUnrelated},
+        refreshLogicalSessionCacheNow: {skip: isAnInternalCommand},
+        reapLogicalSessionCacheNow: {skip: isAnInternalCommand},
+        refreshSessions: {skip: isUnrelated},
+        refreshSessionsInternal: {skip: isAnInternalCommand},
         reIndex: {command: {reIndex: "view"}, expectFailure: true},
         removeShard: {skip: isUnrelated},
         removeShardFromZone: {skip: isUnrelated},
@@ -399,6 +424,7 @@
         replSetSyncFrom: {skip: isUnrelated},
         replSetTest: {skip: isUnrelated},
         replSetUpdatePosition: {skip: isUnrelated},
+        replSetResizeOplog: {skip: isUnrelated},
         resetError: {skip: isUnrelated},
         resync: {skip: isUnrelated},
         revokePrivilegesFromRole: {
@@ -452,11 +478,11 @@
                 max: {x: 0},
                 keyPattern: {x: 1},
                 splitKeys: [{x: -2}, {x: -1}],
-                shardVersion: [1, 2]
+                shardVersion: [ObjectId(), 2]
             },
             skipSharded: true,
             expectFailure: true,
-            expectedErrorCode: null,
+            expectedErrorCode: 193,
             isAdminCommand: true,
         },
         splitVector: {
@@ -468,6 +494,7 @@
             expectFailure: true,
         },
         stageDebug: {skip: isAnInternalCommand},
+        startSession: {skip: isAnInternalCommand},
         top: {skip: "tested in views/views_stats.js"},
         touch: {
             command: {touch: "view", data: true},
