@@ -167,6 +167,8 @@ if (typeof _threadInject != "undefined") {
             "mr_drop.js",
             "mr3.js",
             "indexh.js",
+            "evald.js",
+            "evalf.js",
             "run_program1.js",
             "notablescan.js",
             "dropdb_race.js",
@@ -190,36 +192,36 @@ if (typeof _threadInject != "undefined") {
             // Assumes that other tests are not creating cursors.
             "kill_cursors.js",
 
-            // Use eval command and potentially cause deadlock.
-            "constructors.js",
-            "error2.js",
-            "eval0.js",
-            "eval1.js",
-            "eval3.js",
-            "eval4.js",
-            "eval5.js",
-            "eval6.js",
-            "eval7.js",
-            "eval9.js",
-            "evala.js",
-            "evalb.js",
-            "evald.js",
-            "evale.js",
-            "evalg.js",
-            "evalh.js",
-            "evalj.js",
-            "eval_mr.js",
-            "eval_nolock.js",
-            "fsync.js",
-            "js3.js",
-            "js7.js",
-            "js9.js",
-            "recursion.js",
-            "remove8.js",
-            "rename4.js",
-            "storefunc.js",
+            // Views tests
+            "views/invalid_system_views.js",  // Creates invalid view definitions in system.views.
+            "views/views_all_commands.js",    // Drops test DB.
         ]);
 
+        // The following tests cannot run when shell readMode is legacy.
+        if (db.getMongo().readMode() === "legacy") {
+            var requires_find_command = [
+                "views/views_aggregation.js",
+                "views/views_change.js",
+                "views/views_drop.js",
+                "views/views_find.js"
+            ];
+            Object.assign(skipTests, makeKeys(requires_find_command));
+        }
+
+        // Get files, including files in subdirectories.
+        var getFilesRecursive = function(dir) {
+            var files = listFiles(dir);
+            var fileList = [];
+            files.forEach(file => {
+                if (file.isDirectory) {
+                    getFilesRecursive(file.name).forEach(subDirFile => fileList.push(subDirFile));
+                } else {
+                    fileList.push(file);
+                }
+            });
+            return fileList;
+        };
+
         var parallelFilesDir = "jstests/core";
 
         // some tests can't be run in parallel with each other
@@ -233,8 +235,9 @@ if (typeof _threadInject != "undefined") {
             // be run in parallel with tests that perform fsyncLock. User operations skip writing to
             // the system.profile collection while the server is fsyncLocked.
             //
-            // The profiler tests can be run in parallel with each other as they use test-specific
-            // databases.
+            // Most profiler tests can be run in parallel with each other as they use test-specific
+            // databases, with the exception of tests which modify slowms or the profiler's sampling
+            // rate, since those affect profile settings globally.
             parallelFilesDir + "/apitest_db.js",
             parallelFilesDir + "/evalb.js",
             parallelFilesDir + "/geo_s2cursorlimitskip.js",
@@ -251,16 +254,26 @@ if (typeof _threadInject != "undefined") {
             parallelFilesDir + "/profile_getmore.js",
             parallelFilesDir + "/profile_group.js",
             parallelFilesDir + "/profile_insert.js",
+            parallelFilesDir + "/profile_list_collections.js",
+            parallelFilesDir + "/profile_list_indexes.js",
             parallelFilesDir + "/profile_mapreduce.js",
             parallelFilesDir + "/profile_no_such_db.js",
-            parallelFilesDir + "/profile_update.js"
+            parallelFilesDir + "/profile_parallel_collection_scan.js",
+            parallelFilesDir + "/profile_repair_cursor.js",
+            parallelFilesDir + "/profile_sampling.js",
+            parallelFilesDir + "/profile_update.js",
+
+            // These tests use getLog to examine the slow query logs. Tests which examine the slow
+            // query logs can't be run concurrently with tests that affect the profile sampling
+            // rate, since that also impacts which operations get logged.
+            parallelFilesDir + "/getlog2.js",
         ];
         var serialTests = makeKeys(serialTestsArr);
 
         // prefix the first thread with the serialTests
         // (which we will exclude from the rest of the threads below)
         params[0] = serialTestsArr;
-        var files = listFiles(parallelFilesDir);
+        var files = getFilesRecursive(parallelFilesDir);
         files = Array.shuffle(files);
 
         var i = 0;
@@ -294,10 +307,6 @@ if (typeof _threadInject != "undefined") {
         args.forEach(function(x) {
             print("         S" + suite + " Test : " + x + " ...");
             var time = Date.timeFunc(function() {
-                // Create a new connection to the db for each file. If tests share the same
-                // connection it can create difficult to debug issues.
-                db = new Mongo(db.getMongo().host).getDB(db.getName());
-                gc();
                 load(x);
             }, 1);
             print("         S" + suite + " Test : " + x + " " + time + "ms");
@@ -311,26 +320,32 @@ if (typeof _threadInject != "undefined") {
     // newScopes: if true, each thread starts in a fresh scope
     assert.parallelTests = function(params, msg, newScopes) {
         newScopes = newScopes || false;
-        var wrapper = function(fun, argv) {
-            eval("var z = function() {" + "TestData = " + tojson(TestData) + ";" +
-                 "var __parallelTests__fun = " + fun.toString() + ";" +
-                 "var __parallelTests__argv = " + tojson(argv) + ";" +
-                 "var __parallelTests__passed = false;" + "try {" +
-                 "__parallelTests__fun.apply( 0, __parallelTests__argv );" +
-                 "__parallelTests__passed = true;" + "} catch ( e ) {" + "print('');" +
-                 "print( '********** Parallel Test FAILED: ' + tojson(e) );" + "print('');" + "}" +
-                 "return __parallelTests__passed;" + "}");
-            return z;
-        };
+        function wrapper(fun, argv, globals) {
+            if (globals.hasOwnProperty("TestData")) {
+                TestData = globals.TestData;
+            }
+
+            try {
+                fun.apply(0, argv);
+                return {passed: true};
+            } catch (e) {
+                print("\n********** Parallel Test FAILED: " + tojson(e) + "\n");
+                return {
+                    passed: false,
+                    testName: tojson(e).match(/Error: error loading js file: (.*\.js)/)[1]
+                };
+            }
+        }
+
         var runners = new Array();
         for (var i in params) {
             var param = params[i];
             var test = param.shift();
             var t;
             if (newScopes)
-                t = new ScopedThread(wrapper(test, param));
+                t = new ScopedThread(wrapper, test, param, {TestData: TestData});
             else
-                t = new Thread(wrapper(test, param));
+                t = new Thread(wrapper, test, param, {TestData: TestData});
             runners.push(t);
         }
 
@@ -338,13 +353,16 @@ if (typeof _threadInject != "undefined") {
             x.start();
         });
         var nFailed = 0;
+        var failedTests = [];
         // SpiderMonkey doesn't like it if we exit before all threads are joined
         // (see SERVER-19615 for a similar issue).
         runners.forEach(function(x) {
-            if (!x.returnData()) {
+            if (!x.returnData().passed) {
                 ++nFailed;
+                failedTests.push(x.returnData().testName);
             }
         });
+        msg += ": " + tojsononeline(failedTests);
         assert.eq(0, nFailed, msg);
     };
 }
