@@ -10,6 +10,7 @@ load("./jstests/multiVersion/libs/initial_sync.js");
 var testName = "multiversion_initial_sync_last_stable_from_latest";
 let replSetVersion = "latest";
 let newSecondaryVersion = "last-stable";
-let fcv = "3.2";
+let configSettings = {catchUpTimeoutMillis: 2000};
+let fcv = "3.4";
 
-multversionInitialSyncTest(testName, replSetVersion, newSecondaryVersion, {}, fcv);
+multversionInitialSyncTest(testName, replSetVersion, newSecondaryVersion, configSettings, fcv);
