@@ -8,40 +8,64 @@
  * @param pwd {string} optional, password for authentication. Must be set if user is set.
  */
 ReplSetTest.prototype.upgradeSet = function(options, user, pwd) {
-    options = options || {};
+    let primary = this.getPrimary();
 
-    var primary = this.getPrimary();
+    // Upgrade secondaries first.
+    this.upgradeSecondaries(primary, options, user, pwd);
 
-    // Upgrade secondaries first
-    var nodesToUpgrade = this.getSecondaries();
+    // Then upgrade the primary after stepping down.
+    this.upgradePrimary(primary, options, user, pwd);
 
-    // Then upgrade primaries
-    nodesToUpgrade.push(primary);
+};
 
-    // We can upgrade with no primary downtime if we have enough nodes
-    var noDowntimePossible = this.nodes.length > 2;
+ReplSetTest.prototype.upgradeSecondaries = function(primary, options, user, pwd) {
+    const noDowntimePossible = this.nodes.length > 2;
 
-    for (var i = 0; i < nodesToUpgrade.length; i++) {
-        var node = nodesToUpgrade[i];
-        if (node == primary) {
-            node = this.stepdown(node);
-            primary = this.getPrimary();
-        }
+    // Merge new options into node settings.
+    for (let nodeName in this.nodeOptions) {
+        this.nodeOptions[nodeName] = Object.merge(this.nodeOptions[nodeName], options);
+    }
 
-        var prevPrimaryId = this.getNodeId(primary);
-        // merge new options into node settings...
-        for (var nodeName in this.nodeOptions) {
-            this.nodeOptions[nodeName] = Object.merge(this.nodeOptions[nodeName], options);
-        }
-
-        this.upgradeNode(node, options, user, pwd);
+    for (let secondary of this.getSecondaries()) {
+        this.upgradeNode(secondary, options, user, pwd);
 
         if (noDowntimePossible)
-            assert.eq(this.getNodeId(primary), prevPrimaryId);
+            assert.eq(this.getPrimary(), primary);
     }
 };
 
-ReplSetTest.prototype.upgradeNode = function(node, opts, user, pwd) {
+ReplSetTest.prototype.upgradePrimary = function(primary, options, user, pwd) {
+    const noDowntimePossible = this.nodes.length > 2;
+
+    // Merge new options into node settings.
+    for (let nodeName in this.nodeOptions) {
+        this.nodeOptions[nodeName] = Object.merge(this.nodeOptions[nodeName], options);
+    }
+
+    let oldPrimary = this.stepdown(primary);
+    this.waitForState(oldPrimary, ReplSetTest.State.SECONDARY);
+
+    // stepping down the node can close the connection and lose the authentication state, so
+    // re-authenticate here before calling awaitNodesAgreeOnPrimary().
+    if (user != undefined) {
+        oldPrimary.getDB('admin').auth(user, pwd);
+    }
+    jsTest.authenticate(oldPrimary);
+
+    this.awaitNodesAgreeOnPrimary();
+    primary = this.getPrimary();
+
+    this.upgradeNode(oldPrimary, options, user, pwd);
+
+    let newPrimary = this.getPrimary();
+
+    if (noDowntimePossible)
+        assert.eq(newPrimary, primary);
+
+    return newPrimary;
+};
+
+ReplSetTest.prototype.upgradeNode = function(node, opts = {}, user, pwd) {
     if (user != undefined) {
         assert.eq(1, node.getDB("admin").auth(user, pwd));
     }
@@ -83,10 +107,7 @@ ReplSetTest.prototype.stepdown = function(nodeId) {
         print('Caught exception after stepDown cmd: ' + tojson(ex));
     }
 
-    // Wait for the state to be secondary to avoid the race of closing new connections on stepdown.
-    this.waitForState(node, ReplSetTest.State.SECONDARY);
-
-    return node;
+    return this.reconnect(node);
 };
 
 ReplSetTest.prototype.reconnect = function(node) {
