@@ -6,12 +6,17 @@
     "use strict";
 
     /**
-     * @param {Object} params - Configuration options for the test.
-     * @param {string} params.readMode - The read mode to use for the parallel shell. This allows
+     * @param {string} readMode - The read mode to use for the parallel shell. This allows
      * testing currentOp() output for both OP_QUERY and OP_GET_MORE queries, as well as "find" and
      * "getMore" commands.
+     * @params {function} currentOp - Function which takes a database object and a filter, and
+     * returns an array of matching current operations. This allows us to test output for both the
+     * currentOp command and the $currentOp aggregation stage.
+     * @params {boolean} truncatedOps - if true, we expect operations that exceed the maximum
+     * currentOp size to be truncated in the output 'command' field. If false, we expect the entire
+     * operation to be returned.
      */
-    function runTest(params) {
+    function runTest({readMode, currentOp, truncatedOps}) {
         var conn = MongoRunner.runMongod({smallfiles: "", nojournal: ""});
         assert.neq(null, conn, "mongod was unable to start up");
 
@@ -44,7 +49,7 @@
                 {configureFailPoint: "setYieldAllLocksHang", mode: "alwaysOn"}));
 
             // Set shell read mode for the parallel shell test.
-            TestData.shellReadMode = params.readMode;
+            TestData.shellReadMode = readMode;
             TestData.currentOpTest = testObj.test;
             testObj.test = function() {
                 db.getMongo().forceReadMode(TestData.shellReadMode);
@@ -62,16 +67,19 @@
                     testObj.currentOpFilter.ns = coll.getFullName();
                     testObj.currentOpFilter.planSummary = testObj.planSummary;
                     if (testObj.hasOwnProperty("command")) {
-                        testObj.currentOpFilter["query." + testObj.command] = {$exists: true};
+                        testObj.currentOpFilter["command." + testObj.command] = {$exists: true};
                     } else if (testObj.hasOwnProperty("operation")) {
                         testObj.currentOpFilter.op = testObj.operation;
                     }
 
-                    var result = testDB.currentOp(testObj.currentOpFilter);
+                    var result = currentOp(testDB, testObj.currentOpFilter);
                     assert.commandWorked(result);
 
                     if (result.inprog.length === 1) {
                         assert.eq(result.inprog[0].appName, "MongoDB Shell", tojson(result));
+                        assert.eq(result.inprog[0].clientMetadata.application.name,
+                                  "MongoDB Shell",
+                                  tojson(result));
 
                         return true;
                     }
@@ -80,7 +88,7 @@
                 },
                 function() {
                     return "Failed to find operation from " + tojson(testObj) +
-                        " in currentOp() output: " + tojson(testDB.currentOp());
+                        " in currentOp() output: " + tojson(currentOp(testDB, {}));
                 });
 
             // Allow the query to complete.
@@ -99,16 +107,21 @@
             {
               test: function() {
                   assert.eq(db.currentop_query
-                                .aggregate([{$match: {a: 1, $comment: "currentop_query"}}],
-                                           {collation: {locale: "fr"}})
+                                .aggregate([{$match: {a: 1, $comment: "currentop_query"}}], {
+                                    collation: {locale: "fr"},
+                                    hint: {_id: 1},
+                                    comment: "currentop_query_2"
+                                })
                                 .itcount(),
                             1);
               },
               command: "aggregate",
-              planSummary: "COLLSCAN",
+              planSummary: "IXSCAN { _id: 1 }",
               currentOpFilter: {
-                  "query.pipeline.0.$match.$comment": "currentop_query",
-                  "query.collation": {locale: "fr"}
+                  "command.pipeline.0.$match.$comment": "currentop_query",
+                  "command.comment": "currentop_query_2",
+                  "command.collation": {locale: "fr"},
+                  "command.hint": {_id: 1}
               }
             },
             {
@@ -120,8 +133,10 @@
               },
               command: "count",
               planSummary: "COLLSCAN",
-              currentOpFilter:
-                  {"query.query.$comment": "currentop_query", "query.collation": {locale: "fr"}}
+              currentOpFilter: {
+                  "command.query.$comment": "currentop_query",
+                  "command.collation": {locale: "fr"}
+              }
             },
             {
               test: function() {
@@ -132,8 +147,10 @@
               },
               command: "distinct",
               planSummary: "COLLSCAN",
-              currentOpFilter:
-                  {"query.query.$comment": "currentop_query", "query.collation": {locale: "fr"}}
+              currentOpFilter: {
+                  "command.query.$comment": "currentop_query",
+                  "command.collation": {locale: "fr"}
+              }
             },
             {
               test: function() {
@@ -142,7 +159,7 @@
               },
               command: "find",
               planSummary: "COLLSCAN",
-              currentOpFilter: {"query.comment": "currentop_query"}
+              currentOpFilter: {"command.comment": "currentop_query"}
             },
             {
               test: function() {
@@ -155,8 +172,10 @@
               },
               command: "findandmodify",
               planSummary: "COLLSCAN",
-              currentOpFilter:
-                  {"query.query.$comment": "currentop_query", "query.collation": {locale: "fr"}}
+              currentOpFilter: {
+                  "command.query.$comment": "currentop_query",
+                  "command.collation": {locale: "fr"}
+              }
             },
             {
               test: function() {
@@ -172,8 +191,8 @@
               command: "group",
               planSummary: "COLLSCAN",
               currentOpFilter: {
-                  "query.group.cond.$comment": "currentop_query",
-                  "query.group.collation": {locale: "fr"}
+                  "command.group.cond.$comment": "currentop_query",
+                  "command.group.collation": {locale: "fr"}
               }
             },
             {
@@ -193,8 +212,10 @@
               },
               command: "mapreduce",
               planSummary: "COLLSCAN",
-              currentOpFilter:
-                  {"query.query.$comment": "currentop_query", "query.collation": {locale: "fr"}}
+              currentOpFilter: {
+                  "command.query.$comment": "currentop_query",
+                  "command.collation": {locale: "fr"}
+              }
             },
             {
               test: function() {
@@ -203,7 +224,8 @@
               },
               operation: "remove",
               planSummary: "COLLSCAN",
-              currentOpFilter: {"query.$comment": "currentop_query", "collation": {locale: "fr"}}
+              currentOpFilter:
+                  {"command.q.$comment": "currentop_query", "command.collation": {locale: "fr"}}
             },
             {
               test: function() {
@@ -213,7 +235,8 @@
               },
               operation: "update",
               planSummary: "COLLSCAN",
-              currentOpFilter: {"query.$comment": "currentop_query", "collation": {locale: "fr"}}
+              currentOpFilter:
+                  {"command.q.$comment": "currentop_query", "command.collation": {locale: "fr"}}
             }
         ];
 
@@ -228,7 +251,7 @@
         //
         // Confirm currentOp contains collation for find command.
         //
-        if (params.readMode === "commands") {
+        if (readMode === "commands") {
             confirmCurrentOpContents({
                 test: function() {
                     assert.eq(db.currentop_query.find({a: 1})
@@ -240,7 +263,7 @@
                 command: "find",
                 planSummary: "COLLSCAN",
                 currentOpFilter:
-                    {"query.comment": "currentop_query", "query.collation": {locale: "fr"}}
+                    {"command.comment": "currentop_query", "command.collation": {locale: "fr"}}
             });
         }
 
@@ -265,12 +288,15 @@
             },
             command: "geoNear",
             planSummary: "GEO_NEAR_2DSPHERE { loc: \"2dsphere\" }",
-            currentOpFilter:
-                {"query.query.$comment": "currentop_query", "query.collation": {locale: "fr"}}
+            currentOpFilter: {
+                "command.query.$comment": "currentop_query",
+                "command.collation": {locale: "fr"}
+            }
         });
 
         //
-        // Confirm currentOp content for getMore.
+        // Confirm currentOp content for getMore. This case tests command and legacy getMore with an
+        // originating find command.
         //
         coll.drop();
         for (i = 0; i < 10; ++i) {
@@ -283,19 +309,14 @@
 
         TestData.commandResult = cmdRes;
 
-        var filter;
-        if (params.readMode === "legacy") {
-            filter = {"op": "getmore", "query.filter.$comment": "currentop_query"};
-        } else {
-            filter = {
-                "query.getMore": TestData.commandResult.cursor.id,
-                "originatingCommand.filter.$comment": "currentop_query"
-            };
-        }
+        var filter = {
+            "command.getMore": TestData.commandResult.cursor.id,
+            "originatingCommand.filter.$comment": "currentop_query"
+        };
 
         confirmCurrentOpContents({
             test: function() {
-                var cursor = new DBCommandCursor(db.getMongo(), TestData.commandResult, 5);
+                var cursor = new DBCommandCursor(db, TestData.commandResult, 5);
                 assert.eq(cursor.itcount(), 10);
             },
             planSummary: "COLLSCAN",
@@ -305,7 +326,51 @@
         delete TestData.commandResult;
 
         //
-        // Confirm 512 byte size limit for currentOp query field.
+        // Confirm that currentOp displays upconverted getMore and originatingCommand in the case of
+        // a legacy query.
+        //
+        if (readMode === "legacy") {
+            let filter = {
+                "command.getMore": {$gt: 0},
+                "command.collection": "currentop_query",
+                "command.batchSize": 2,
+                originatingCommand: {
+                    find: "currentop_query",
+                    filter: {},
+                    ntoreturn: 2,
+                    comment: "currentop_query"
+                }
+            };
+
+            confirmCurrentOpContents({
+                test: function() {
+                    // Temporarily disable hanging yields so that we can iterate the first batch.
+                    assert.commandWorked(
+                        db.adminCommand({configureFailPoint: "setYieldAllLocksHang", mode: "off"}));
+
+                    let cursor =
+                        db.currentop_query.find({}).comment("currentop_query").batchSize(2);
+
+                    // Exhaust the current batch so that the next request will force a getMore.
+                    while (cursor.objsLeftInBatch() > 0) {
+                        cursor.next();
+                    }
+
+                    // Set yields to hang so that we can check currentOp output.
+                    assert.commandWorked(db.adminCommand(
+                        {configureFailPoint: "setYieldAllLocksHang", mode: "alwaysOn"}));
+
+                    assert.eq(cursor.itcount(), 8);
+                },
+                operation: "getmore",
+                planSummary: "COLLSCAN",
+                currentOpFilter: filter
+            });
+        }
+
+        //
+        // Confirm ~1000 byte size limit for query field in the case of the currentOp command. For
+        // $currentOp aggregations, this limit does not apply.
         //
         coll.drop();
         assert.writeOK(coll.insert({a: 1}));
@@ -314,31 +379,131 @@
         // values inside it are truncated at 150 characters. To test "total length" truncation we
         // need to pass multiple values, each smaller than 150 bytes.
         TestData.queryFilter = {
-            "1": "1".repeat(100),
-            "2": "2".repeat(100),
-            "3": "3".repeat(100),
-            "4": "4".repeat(100),
-            "5": "5".repeat(100),
-            "6": "6".repeat(100),
+            "1": "1".repeat(149),
+            "2": "2".repeat(149),
+            "3": "3".repeat(149),
+            "4": "4".repeat(149),
+            "5": "5".repeat(149),
+            "6": "6".repeat(149),
+            "7": "7".repeat(149),
         };
-        var truncatedQueryString = "{ find: \"currentop_query\", filter: { " +
-            "1: \"1111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111\", " +
-            "2: \"2222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222222\", " +
-            "3: \"3333333333333333333333333333333333333333333333333333333333333333333333333333333333333333333333333333\", " +
-            "4: \"4444444444444444444444444444444444444444444444444444444444444444444444444444444444444444444444444444\", " +
-            "5: \"5555555555555555555555555555555555555555...";
+
+        var truncatedQueryString = "^\\{ find: \"currentop_query\", filter: \\{ " +
+            "1: \"1{149}\", 2: \"2{149}\", 3: \"3{149}\", 4: \"4{149}\", 5: \"5{149}\", " +
+            "6: \"6{149}\", 7: \"7+\\.\\.\\.";
+
+        let currentOpFilter;
+
+        if (truncatedOps) {
+            currentOpFilter = {
+                "command.$truncated": {$regex: truncatedQueryString},
+                "command.comment": "currentop_query"
+            };
+        } else {
+            currentOpFilter = {
+                "command.filter": TestData.queryFilter,
+                "command.comment": "currentop_query"
+            };
+        }
 
         confirmCurrentOpContents({
             test: function() {
-                assert.eq(db.currentop_query.find(TestData.queryFilter).itcount(), 0);
+                assert.eq(db.currentop_query.find(TestData.queryFilter)
+                              .comment("currentop_query")
+                              .itcount(),
+                          0);
             },
             planSummary: "COLLSCAN",
-            currentOpFilter: {"query": truncatedQueryString}
+            currentOpFilter: currentOpFilter
+        });
+
+        // Verify that an originatingCommand truncated by currentOp appears as { $truncated:
+        // <string>, comment: <string> }.
+        cmdRes = testDB.runCommand({
+            find: "currentop_query",
+            filter: TestData.queryFilter,
+            comment: "currentop_query",
+            batchSize: 0
+        });
+        assert.commandWorked(cmdRes);
+
+        TestData.commandResult = cmdRes;
+
+        if (truncatedOps) {
+            currentOpFilter = {
+                "command.getMore": TestData.commandResult.cursor.id,
+                "originatingCommand.$truncated": {$regex: truncatedQueryString},
+                "originatingCommand.comment": "currentop_query"
+            };
+        } else {
+            currentOpFilter = {
+                "command.getMore": TestData.commandResult.cursor.id,
+                "originatingCommand.filter": TestData.queryFilter,
+                "originatingCommand.comment": "currentop_query"
+            };
+        }
+
+        confirmCurrentOpContents({
+            test: function() {
+                var cursor = new DBCommandCursor(db, TestData.commandResult, 5);
+                assert.eq(cursor.itcount(), 0);
+            },
+            planSummary: "COLLSCAN",
+            currentOpFilter: currentOpFilter
+        });
+
+        delete TestData.commandResult;
+
+        // Verify that an aggregation truncated by currentOp appears as { $truncated: <string>,
+        // comment: <string> } when a comment parameter is present.
+        truncatedQueryString =
+            "^\\{ aggregate: \"currentop_query\", pipeline: \\[ \\{ \\$match: \\{ " +
+            "1: \"1{149}\", 2: \"2{149}\", 3: \"3{149}\", 4: \"4{149}\", 5: \"5{149}\", " +
+            "6: \"6{149}\", 7: \"7+\\.\\.\\.";
+
+        if (truncatedOps) {
+            currentOpFilter = {
+                "command.$truncated": {$regex: truncatedQueryString},
+                "command.comment": "currentop_query"
+            };
+        } else {
+            currentOpFilter = {
+                "command.pipeline.0.$match": TestData.queryFilter,
+                "command.comment": "currentop_query"
+            };
+        }
+
+        confirmCurrentOpContents({
+            test: function() {
+                assert.eq(
+                    db.currentop_query
+                        .aggregate([{$match: TestData.queryFilter}], {comment: "currentop_query"})
+                        .itcount(),
+                    0);
+            },
+            planSummary: "COLLSCAN",
+            currentOpFilter: currentOpFilter
         });
 
         delete TestData.queryFilter;
     }
 
-    runTest({readMode: "commands"});
-    runTest({readMode: "legacy"});
+    function currentOpCommand(inputDB, filter) {
+        return inputDB.currentOp(filter);
+    }
+
+    function currentOpAgg(inputDB, filter) {
+        return {
+            inprog: inputDB.getSiblingDB("admin")
+                        .aggregate([{$currentOp: {}}, {$match: filter}])
+                        .toArray(),
+            ok: 1
+        };
+    }
+
+    runTest({readMode: "commands", currentOp: currentOpCommand, truncatedOps: true});
+    runTest({readMode: "legacy", currentOp: currentOpCommand, truncatedOps: true});
+
+    runTest({readMode: "commands", currentOp: currentOpAgg, truncatedOps: false});
+    runTest({readMode: "legacy", currentOp: currentOpAgg, truncatedOps: false});
 })();
