@@ -8,10 +8,8 @@
     const worksPerYield = 50;
 
     // Start a mongod that will yield every 50 work cycles.
-    // clang-format off
-    var conn = MongoRunner.runMongod(
-        {setParameter: `internalQueryExecYieldIterations=${worksPerYield}`});
-    // clang-format on
+    var conn =
+        MongoRunner.runMongod({setParameter: `internalQueryExecYieldIterations=${worksPerYield}`});
     assert.neq(null, conn, 'mongod was unable to start up');
 
     var coll = conn.getDB('test').yield_group;
