@@ -10,6 +10,7 @@ var testOnlyCommands = [
     'journalLatencyTest',
     'godinsert',
     'sleep',
+    'cpuload',
     'captrunc',
     'emptycapped'
 ];
@@ -36,7 +37,7 @@ var conn = MongoRunner.runMongod({});
 for (i in testOnlyCommands) {
     assertCmdNotFound(conn.getDB('test'), testOnlyCommands[i]);
 }
-MongoRunner.stopMongod(conn.port);
+MongoRunner.stopMongod(conn);
 
 // Now enable the commands
 jsTest.setOption('enableTestCommands', true);
@@ -45,4 +46,4 @@ var conn = MongoRunner.runMongod({});
 for (i in testOnlyCommands) {
     assertCmdFound(conn.getDB('test'), testOnlyCommands[i]);
 }
-MongoRunner.stopMongod(conn.port);
+MongoRunner.stopMongod(conn);
