@@ -22,7 +22,10 @@ var coll = mongod.getCollection("test.batch_write_command_wc");
 //
 // Basic insert, default WC
 coll.remove({});
-printjson(request = {insert: coll.getName(), documents: [{a: 1}]});
+printjson(request = {
+    insert: coll.getName(),
+    documents: [{a: 1}]
+});
 printjson(result = coll.runCommand(request));
 assert(result.ok);
 assert.eq(1, result.n);
@@ -31,7 +34,11 @@ assert.eq(1, coll.find().itcount());
 //
 // Basic insert, majority WC
 coll.remove({});
-printjson(request = {insert: coll.getName(), documents: [{a: 1}], writeConcern: {w: 'majority'}});
+printjson(request = {
+    insert: coll.getName(),
+    documents: [{a: 1}],
+    writeConcern: {w: 'majority'}
+});
 printjson(result = coll.runCommand(request));
 assert(result.ok);
 assert.eq(1, result.n);
@@ -40,7 +47,11 @@ assert.eq(1, coll.find().itcount());
 //
 // Basic insert,  w:2 WC
 coll.remove({});
-printjson(request = {insert: coll.getName(), documents: [{a: 1}], writeConcern: {w: 2}});
+printjson(request = {
+    insert: coll.getName(),
+    documents: [{a: 1}],
+    writeConcern: {w: 2}
+});
 printjson(result = coll.runCommand(request));
 assert(result.ok);
 assert.eq(1, result.n);
@@ -49,7 +60,11 @@ assert.eq(1, coll.find().itcount());
 //
 // Basic insert, immediate nojournal error
 coll.remove({});
-printjson(request = {insert: coll.getName(), documents: [{a: 1}], writeConcern: {j: true}});
+printjson(request = {
+    insert: coll.getName(),
+    documents: [{a: 1}],
+    writeConcern: {j: true}
+});
 printjson(result = coll.runCommand(request));
 assert(!result.ok);
 assert.eq(0, coll.find().itcount());
@@ -57,8 +72,11 @@ assert.eq(0, coll.find().itcount());
 //
 // Basic insert, timeout wc error
 coll.remove({});
-printjson(
-    request = {insert: coll.getName(), documents: [{a: 1}], writeConcern: {w: 3, wtimeout: 1}});
+printjson(request = {
+    insert: coll.getName(),
+    documents: [{a: 1}],
+    writeConcern: {w: 3, wtimeout: 1}
+});
 printjson(result = coll.runCommand(request));
 assert(result.ok);
 assert.eq(1, result.n);
@@ -69,7 +87,11 @@ assert.eq(1, coll.find().itcount());
 //
 // Basic insert, wmode wc error
 coll.remove({});
-printjson(request = {insert: coll.getName(), documents: [{a: 1}], writeConcern: {w: 'invalid'}});
+printjson(request = {
+    insert: coll.getName(),
+    documents: [{a: 1}],
+    writeConcern: {w: 'invalid'}
+});
 printjson(result = coll.runCommand(request));
 assert(result.ok);
 assert.eq(1, result.n);
@@ -132,11 +154,10 @@ coll.remove({});
 request = {
     insert: coll.getName(),
     documents: [{_id: 1}, {_id: 1}],
-    writeConcern: {wTimeout: 1},
+    writeConcern: {wtimeout: 1},
     ordered: false
 };
-result = coll.runCommand(request);
-assert(result.ok);
+result = assert.commandWorked(coll.runCommand(request));
 assert.eq(1, result.n);
 assert.eq(result.writeErrors.length, 1);
 assert.eq(result.writeErrors[0].index, 1);
