@@ -12,21 +12,21 @@ var secondary = replTest.liveNodes.slaves[0].getDB(name);
 
 // populate the collection
 for (i = 0; i < 5; i++) {
-    primary.in .insert({x: i});
+    primary.coll.insert({x: i});
 }
 replTest.awaitReplication();
 
 // make sure $out cannot be run on a secondary
 assert.throws(function() {
-    secondary.in .aggregate({$out: "out"}).itcount;
+    secondary.coll.aggregate({$out: "out"}).itcount();
 });
 // even if slaveOk
 secondary.setSlaveOk();
 assert.throws(function() {
-    secondary.in .aggregate({$out: "out"}).itcount;
+    secondary.coll.aggregate({$out: "out"}).itcount();
 });
 
 // run one and check for proper replication
-primary.in .aggregate({$out: "out"}).itcount;
+primary.coll.aggregate({$out: "out"}).itcount();
 replTest.awaitReplication();
 assert.eq(primary.out.find().sort({x: 1}).toArray(), secondary.out.find().sort({x: 1}).toArray());
