@@ -68,6 +68,10 @@ load("jstests/replsets/rslib.js");       // For startSetIfSupportsReadMajority.
         dropDB: {
             prepare: function(db) {
                 assert.writeOK(db.coll.insert({_id: 1}));
+                // Drop collection explicitly during the preparation phase while we are still able
+                // to write to a majority. Otherwise, dropDatabase() will drop the collection
+                // and wait for the collection drop to be replicated to a majority of the nodes.
+                assert(db.coll.drop());
             },
             performOp: function(db) {
                 assert.commandWorked(db.dropDatabase());
@@ -90,6 +94,10 @@ load("jstests/replsets/rslib.js");       // For startSetIfSupportsReadMajority.
         dropAndRecreateDB: {
             prepare: function(db) {
                 assert.writeOK(db.coll.insert({_id: 1}));
+                // Drop collection explicitly during the preparation phase while we are still able
+                // to write to a majority. Otherwise, dropDatabase() will drop the collection
+                // and wait for the collection drop to be replicated to a majority of the nodes.
+                assert(db.coll.drop());
             },
             performOp: function(db) {
                 assert.commandWorked(db.dropDatabase());
@@ -219,7 +227,7 @@ load("jstests/replsets/rslib.js");       // For startSetIfSupportsReadMajority.
             coll.runCommand('find', {"readConcern": {"level": "majority"}, "maxTimeMS": timeoutMs});
         assert.commandWorked(res, 'reading from ' + coll.getFullName());
         // Exhaust the cursor to avoid leaking cursors on the server.
-        new DBCommandCursor(coll.getMongo(), res).itcount();
+        new DBCommandCursor(coll.getDB(), res).itcount();
     }
 
     // Set up a set and grab things for later.
@@ -254,8 +262,8 @@ load("jstests/replsets/rslib.js");       // For startSetIfSupportsReadMajority.
     // This DB won't be used by any tests so it should always be unblocked.
     var otherDB = primary.getDB('otherDB');
     var otherDBCollection = otherDB.collection;
-    assert.writeOK(
-        otherDBCollection.insert({}, {writeConcern: {w: "majority", wtimeout: 60 * 1000}}));
+    assert.writeOK(otherDBCollection.insert(
+        {}, {writeConcern: {w: "majority", wtimeout: ReplSetTest.kDefaultTimeoutMS}}));
     assertReadsSucceed(otherDBCollection);
 
     for (var testName in testCases) {
