@@ -1,11 +1,10 @@
-
-doTest = function (signal) {
-
+var doTest = function(signal) {
+    "use strict";
     // Test replica set step down
 
     // Replica set testing API
     // Create a new replica set test. Specify set name and the number of nodes you want.
-    var replTest = new ReplSetTest({ name: 'testSet', nodes: 3 });
+    var replTest = new ReplSetTest({name: 'testSet', nodes: 3});
 
     // call startSet() to start each mongod in the replica set
     // this returns a list of nodes
@@ -16,19 +15,19 @@ doTest = function (signal) {
     replTest.initiate();
 
     // Get master node
-    var master = replTest.getMaster();
+    var master = replTest.getPrimary();
 
     // Write some data to master
     // NOTE: this test fails unless we write some data.
-    master.getDB("foo").foo.insert({ a: 1 }, { writeConcern: { w: 3, wtimeout: 20000 }});
+    master.getDB("foo").foo.insert({a: 1}, {writeConcern: {w: 3, wtimeout: 20000}});
 
     var phase = 1;
 
     print(phase++);
 
-    // Step down master.  Note: this may close our connection!
+    // Step down master.  Note: this will close our connection!
     try {
-        master.getDB("admin").runCommand({ replSetStepDown: true, force: 1 });
+        master.getDB("admin").runCommand({replSetStepDown: 0, force: 1});
     } catch (err) {
         print("caught: " + err + " on stepdown");
     }
@@ -36,10 +35,9 @@ doTest = function (signal) {
     print(phase++);
 
     try {
-        var new_master = replTest.getMaster();
-    }
-    catch (err) {
-        throw ("Could not elect new master before timeout.");
+        var new_master = replTest.getPrimary();
+    } catch (err) {
+        throw("Could not elect new master before timeout.");
     }
 
     print(phase++);
@@ -49,31 +47,33 @@ doTest = function (signal) {
     print(phase++);
 
     // Make sure that slaves are still up
-    var result = new_master.getDB("admin").runCommand({ replSetGetStatus: 1 });
+    var result = new_master.getDB("admin").runCommand({replSetGetStatus: 1});
     assert(result['ok'] == 1, "Could not verify that slaves were still up:" + result);
 
     print(phase++);
 
-    slaves = replTest.liveNodes.slaves;
-    assert.soon(function () {
+    var slaves = replTest.liveNodes.slaves;
+    assert.soon(function() {
         try {
-            res = slaves[0].getDB("admin").runCommand({ replSetGetStatus: 1 })
-        } catch (err) { }
+            var res = slaves[0].getDB("admin").runCommand({replSetGetStatus: 1});
+        } catch (err) {
+        }
         return res.myState == 2;
     }, "Slave 0 state not ready.");
 
     print(phase++);
 
-    assert.soon(function () {
+    assert.soon(function() {
         try {
-            res = slaves[1].getDB("admin").runCommand({ replSetGetStatus: 1 })
-        } catch (err) { }
+            var res = slaves[1].getDB("admin").runCommand({replSetGetStatus: 1});
+        } catch (err) {
+        }
         return res.myState == 2;
     }, "Slave 1 state not ready.");
 
     print("replset3.js SUCCESS");
 
     replTest.stopSet(15);
-}
+};
 
-doTest( 15 );
+doTest(15);
