@@ -9,6 +9,8 @@
 
 (function() {
     "use strict";
+    load("jstests/replsets/rslib.js");  // For getLatestOp()
+
     // set up a set and grab things for later
     var name = "rollback_too_new";
     var replTest = new ReplSetTest({name: name, nodes: 3});
@@ -28,25 +30,35 @@
 
     // get master and do an initial write
     var master = replTest.getPrimary();
-    var options = {writeConcern: {w: 2, wtimeout: 60000}};
+    var options = {writeConcern: {w: 2, wtimeout: ReplSetTest.kDefaultTimeoutMS}};
     assert.writeOK(master.getDB(name).foo.insert({x: 1}, options));
 
     // add an oplog entry from the distant future as the most recent entry on node C
     var future_oplog_entry = conns[2].getDB("local").oplog.rs.find().sort({$natural: -1})[0];
     future_oplog_entry["ts"] = new Timestamp(future_oplog_entry["ts"].getTime() + 200000, 1);
-    options = {writeConcern: {w: 1, wtimeout: 60000}};
+    options = {writeConcern: {w: 1, wtimeout: ReplSetTest.kDefaultTimeoutMS}};
     assert.writeOK(conns[2].getDB("local").oplog.rs.insert(future_oplog_entry, options));
 
     replTest.stop(CID);
 
-    // do one write to master
-    // in order to trigger a rollback on C
-    assert.writeOK(master.getDB(name).foo.insert({x: 2}, options));
+    // We bump the term to make sure node 0's oplog is ahead of node 2's.
+    var term = getLatestOp(conns[0]).t;
+    assert.adminCommandWorkedAllowingNetworkError(conns[0], {replSetStepDown: 1, force: true});
+
+    // After stepping down due to the higher term, it will eventually get reelected.
+    replTest.waitForState(conns[0], ReplSetTest.State.PRIMARY);
+    // Wait for the node to increase its term.
+    assert.soon(function() {
+        return getLatestOp(conns[0]).t > term;
+    });
 
     // Node C should connect to new master as a sync source because chaining is disallowed.
     // C is ahead of master but it will still connect to it.
     clearRawMongoProgramOutput();
-    replTest.restart(CID);
+    // Don't wait for a connection to the node after startup, since it might roll back and crash
+    // immediately.
+    replTest.start(CID, {waitForConnect: false}, true /*restart*/);
+
     assert.soon(function() {
         try {
             return rawMongoProgramOutput().match(
@@ -56,6 +68,6 @@
         }
     }, "node C failed to fassert", 60 * 1000);
 
-    replTest.stopSet(undefined, undefined, {allowedExitCodes: [MongoRunner.EXIT_ABRUPT]});
-
+    replTest.stop(CID, undefined, {allowedExitCode: MongoRunner.EXIT_ABRUPT});
+    replTest.stopSet();
 }());
