@@ -1,5 +1,10 @@
 // Ensures that if the config servers are blackholed from the point of view of MongoS, metadata
 // operations do not get stuck forever.
+
+// Checking UUID consistency involves talking to config servers through mongos, but mongos is
+// blackholed from the config servers in this test.
+TestData.skipCheckingUUIDsConsistentAcrossCluster = true;
+
 (function() {
     'use strict';
 
