@@ -3,6 +3,7 @@
  */
 (function() {
     'use strict';
+
     load("jstests/replsets/rslib.js");
 
     var st = new ShardingTest({
@@ -47,7 +48,7 @@
     jsTestLog('Creating initial data');
 
     st.adminCommand({enablesharding: "test"});
-    st.ensurePrimaryShard('test', 'test-rs0');
+    st.ensurePrimaryShard('test', st.shard0.shardName);
     st.adminCommand({shardcollection: "test.foo", key: {i: 1, j: 1}});
 
     // Balancer is stopped by default, so no moveChunks will interfere with the splits we're testing
@@ -95,20 +96,16 @@
     };
 
     var checkCommandSucceeded = function(db, cmdObj) {
-        print("Running command that should succeed: ");
-        printjson(cmdObj);
-        var resultObj = db.runCommand(cmdObj);
+        print("Running command that should succeed: " + tojson(cmdObj));
+        var resultObj = assert.commandWorked(db.runCommand(cmdObj));
         printjson(resultObj);
-        assert(resultObj.ok);
         return resultObj;
     };
 
     var checkCommandFailed = function(db, cmdObj) {
-        print("Running command that should fail: ");
-        printjson(cmdObj);
-        var resultObj = db.runCommand(cmdObj);
+        print("Running command that should fail: " + tojson(cmdObj));
+        var resultObj = assert.commandFailed(db.runCommand(cmdObj));
         printjson(resultObj);
-        assert(!resultObj.ok);
         return resultObj;
     };
 
@@ -117,9 +114,9 @@
             print("Checking read operations, should work");
             assert.eq(expectedDocs, testDB.foo.find().itcount());
             assert.eq(expectedDocs, testDB.foo.count());
+
             // NOTE: This is an explicit check that GLE can be run with read prefs, not the result
-            // of
-            // above.
+            // of above.
             assert.eq(null, testDB.runCommand({getlasterror: 1}).err);
             checkCommandSucceeded(testDB, {dbstats: 1});
             checkCommandSucceeded(testDB, {collstats: 'foo'});
@@ -132,9 +129,10 @@
 
             res = checkCommandSucceeded(testDB, {
                 aggregate: 'foo',
-                pipeline: [{$project: {j: 1}}, {$group: {_id: 'j', sum: {$sum: '$j'}}}]
+                pipeline: [{$project: {j: 1}}, {$group: {_id: 'j', sum: {$sum: '$j'}}}],
+                cursor: {}
             });
-            assert.eq(4500, res.result[0].sum);
+            assert.eq(4500, res.cursor.firstBatch[0].sum);
         } else {
             print("Checking read operations, should fail");
             assert.throws(function() {
@@ -146,7 +144,8 @@
                                {mapreduce: 'foo', map: map, reduce: reduce, out: {inline: 1}});
             checkCommandFailed(testDB, {
                 aggregate: 'foo',
-                pipeline: [{$project: {j: 1}}, {$group: {_id: 'j', sum: {$sum: '$j'}}}]
+                pipeline: [{$project: {j: 1}}, {$group: {_id: 'j', sum: {$sum: '$j'}}}],
+                cursor: {}
             });
         }
     };
@@ -163,7 +162,6 @@
             testDB.foo.remove({a: 1});
             assert.eq(null, testDB.runCommand({getlasterror: 1}).err);
             checkCommandSucceeded(testDB, {reIndex: 'foo'});
-            checkCommandSucceeded(testDB, {repairDatabase: 1});
             checkCommandSucceeded(testDB,
                                   {mapreduce: 'foo', map: map, reduce: reduce, out: 'mrOutput'});
             assert.eq(100, testDB.mrOutput.count());
@@ -183,7 +181,6 @@
             checkCommandFailed(
                 testDB, {findAndModify: "foo", query: {a: 1, i: 1, j: 1}, update: {$set: {b: 1}}});
             checkCommandFailed(testDB, {reIndex: 'foo'});
-            checkCommandFailed(testDB, {repairDatabase: 1});
             checkCommandFailed(testDB,
                                {mapreduce: 'foo', map: map, reduce: reduce, out: 'mrOutput'});
             checkCommandFailed(testDB, {drop: 'foo'});
@@ -214,7 +211,7 @@
             checkCommandSucceeded(adminDB, {isdbgrid: 1});
             checkCommandSucceeded(adminDB, {ismaster: 1});
             checkCommandSucceeded(adminDB, {split: 'test.foo', find: {i: 1, j: 1}});
-            var chunk = configDB.chunks.findOne({shard: st.rs0.name});
+            var chunk = configDB.chunks.findOne({ns: 'test.foo', shard: st.rs0.name});
             checkCommandSucceeded(
                 adminDB,
                 {moveChunk: 'test.foo', find: chunk.min, to: st.rs1.name, _waitForDelete: true});
