@@ -6,28 +6,22 @@
 
     var mongos = st.s;
     var staleMongos = st.s1;
-    var config = mongos.getDB("config");
     var admin = mongos.getDB("admin");
-    var shards = config.shards.find().toArray();
-
-    for (var i = 0; i < shards.length; i++) {
-        shards[i].conn = new Mongo(shards[i].host);
-    }
 
     var collSh = mongos.getCollection(jsTestName() + ".collSharded");
     var collUn = mongos.getCollection(jsTestName() + ".collUnsharded");
-    var collDi = shards[0].conn.getCollection(jsTestName() + ".collDirect");
+    var collDi = st.shard0.getCollection(jsTestName() + ".collDirect");
 
     jsTest.log('Checking write to config collections...');
     assert.writeOK(admin.TestColl.insert({SingleDoc: 1}));
-    assert.writeError(admin.TestColl.insert([{Doc1: 1}, {Doc2: 1}]));
 
     jsTest.log("Setting up collections...");
 
     assert.commandWorked(admin.runCommand({enableSharding: collSh.getDB() + ""}));
-    st.ensurePrimaryShard(collSh.getDB() + "", shards[0]._id);
+    st.ensurePrimaryShard(collSh.getDB() + "", st.shard0.shardName);
 
-    assert.commandWorked(admin.runCommand({movePrimary: collUn.getDB() + "", to: shards[1]._id}));
+    assert.commandWorked(
+        admin.runCommand({movePrimary: collUn.getDB() + "", to: st.shard1.shardName}));
 
     printjson(collSh.ensureIndex({ukey: 1}, {unique: true}));
     printjson(collUn.ensureIndex({ukey: 1}, {unique: true}));
@@ -36,7 +30,7 @@
     assert.commandWorked(admin.runCommand({shardCollection: collSh + "", key: {ukey: 1}}));
     assert.commandWorked(admin.runCommand({split: collSh + "", middle: {ukey: 0}}));
     assert.commandWorked(admin.runCommand(
-        {moveChunk: collSh + "", find: {ukey: 0}, to: shards[0]._id, _waitForDelete: true}));
+        {moveChunk: collSh + "", find: {ukey: 0}, to: st.shard0.shardName, _waitForDelete: true}));
 
     var resetColls = function() {
         assert.writeOK(collSh.remove({}));
@@ -248,38 +242,42 @@
     assert.eq(null, staleCollSh.findOne(), 'Collections should be empty');
 
     assert.commandWorked(admin.runCommand(
-        {moveChunk: collSh + "", find: {ukey: 0}, to: shards[1]._id, _waitForDelete: true}));
+        {moveChunk: collSh + "", find: {ukey: 0}, to: st.shard1.shardName, _waitForDelete: true}));
     assert.commandWorked(admin.runCommand(
-        {moveChunk: collSh + "", find: {ukey: 0}, to: shards[0]._id, _waitForDelete: true}));
+        {moveChunk: collSh + "", find: {ukey: 0}, to: st.shard0.shardName, _waitForDelete: true}));
 
     assert.writeOK(staleCollSh.insert(inserts));
 
     //
-    // Test when the objects to be bulk inserted are 10MB, and so can't be inserted
-    // together with WBL.
+    // Test when the legacy batch exceeds the BSON object size limit
     //
 
-    jsTest.log("Testing bulk insert (no COE) with WBL and large objects...");
+    jsTest.log("Testing bulk insert (no COE) with large objects...");
     resetColls();
 
-    var data10MB = 'x'.repeat(10 * 1024 * 1024);
-    var inserts = [
-        {ukey: 1, data: data10MB},
-        {ukey: 2, data: data10MB},
-        {ukey: -1, data: data10MB},
-        {ukey: -2, data: data10MB}
-    ];
+    var inserts = (function() {
+        var data = 'x'.repeat(10 * 1024 * 1024);
+        return [
+            {ukey: 1, data: data},
+            {ukey: 2, data: data},
+            {ukey: -1, data: data},
+            {ukey: -2, data: data}
+        ];
+    })();
+
+    var staleMongosWithLegacyWrites = new Mongo(staleMongos.name);
+    staleMongosWithLegacyWrites.forceWriteMode('legacy');
 
     staleCollSh = staleMongos.getCollection(collSh + "");
     assert.eq(null, staleCollSh.findOne(), 'Collections should be empty');
 
     assert.commandWorked(admin.runCommand(
-        {moveChunk: collSh + "", find: {ukey: 0}, to: shards[1]._id, _waitForDelete: true}));
+        {moveChunk: collSh + "", find: {ukey: 0}, to: st.shard1.shardName, _waitForDelete: true}));
     assert.commandWorked(admin.runCommand(
-        {moveChunk: collSh + "", find: {ukey: 0}, to: shards[0]._id, _waitForDelete: true}));
+        {moveChunk: collSh + "", find: {ukey: 0}, to: st.shard0.shardName, _waitForDelete: true}));
 
-    assert.writeOK(staleCollSh.insert(inserts));
+    staleCollSh.insert(inserts);
+    staleCollSh.getDB().getLastError();
 
     st.stop();
-
 })();
