@@ -1,6 +1,3 @@
-load('jstests/libs/write_concern_util.js');
-load('jstests/multiVersion/libs/auth_helpers.js');
-
 /**
  * This file tests that commands that do writes accept a write concern in a sharded cluster. This
  * test defines various database commands and what they expect to be true before and after the fact.
@@ -9,7 +6,14 @@ load('jstests/multiVersion/libs/auth_helpers.js');
  * replication between nodes to make sure the write concern is actually being waited for. This only
  * tests commands that get sent to config servers and must have w: majority specified. If these
  * commands fail, they should return an actual error, not just a writeConcernError.
+ *
+ * This test is labeled resource intensive because its total io_write is 70MB compared to a median
+ * of 5MB across all sharding tests in wiredTiger. Its total io_write is 1900MB compared to a median
+ * of 135MB in mmapv1.
+ * @tags: [resource_intensive]
  */
+load('jstests/libs/write_concern_util.js');
+load('jstests/multiVersion/libs/auth_helpers.js');
 
 (function() {
     "use strict";
@@ -93,10 +97,15 @@ load('jstests/multiVersion/libs/auth_helpers.js');
         setupFunc: function() {
             coll.insert({type: 'oak'});
             db.pine_needles.insert({type: 'pine'});
+            // As of SERVER-29277, dropping a database with any replicated collections requires a
+            // majority of nodes to be able to complete the drop. Since this test case may run with
+            // less than a majority of nodes available, we empty out the database in the "setup"
+            // phase to allow the dropDatabase command to always run to completion.
+            db.pine_needles.drop();
+            coll.drop();
         },
         confirmFunc: function() {
-            assert.eq(coll.find().itcount(), 0);
-            assert.eq(db.pine_needles.find().itcount(), 0);
+            assert.isnull(db.getMongo().getDBNames().find(dbName => dbName == db.getName()));
         },
         requiresMajority: false,
         runsOnShards: true,
@@ -111,10 +120,15 @@ load('jstests/multiVersion/libs/auth_helpers.js');
             shardCollectionWithChunks(st, coll);
             coll.insert({type: 'oak', x: 11});
             db.pine_needles.insert({type: 'pine'});
+            // As of SERVER-29277, dropping a database with any replicated collections requires a
+            // majority of nodes to be able to complete the drop. Since this test case may run with
+            // less than a majority of nodes available, we empty out the database in the "setup"
+            // phase to allow the dropDatabase command to always run to completion.
+            db.pine_needles.drop();
+            coll.drop();
         },
         confirmFunc: function() {
-            assert.eq(coll.find().itcount(), 0);
-            assert.eq(db.pine_needles.find().itcount(), 0);
+            assert.isnull(db.getMongo().getDBNames().find(dbName => dbName == db.getName()));
         },
         requiresMajority: false,
         runsOnShards: true,
