@@ -2,8 +2,8 @@
 // checks that cursors survive a chunk's move
 (function() {
 
-    // Turn off auto-splitting, because this test handles chunk splitting manually.
     var s = new ShardingTest({name: "sharding_cursor1", shards: 2});
+
     s.config.settings.find().forEach(printjson);
 
     // create a sharded 'test.foo', for the moment with just one chunk
@@ -12,7 +12,7 @@
     s.adminCommand({shardcollection: "test.foo", key: {_id: 1}});
 
     db = s.getDB("test");
-    primary = s.getServer("test").getDB("test");
+    primary = s.getPrimaryShard("test").getDB("test");
     secondary = s.getOther(primary).getDB("test");
 
     var numObjs = 30;
@@ -21,7 +21,9 @@
         bulk.insert({_id: i});
     }
     assert.writeOK(bulk.execute());
-    assert.eq(1, s.config.chunks.count(), "test requires collection to have one chunk initially");
+    assert.eq(1,
+              s.config.chunks.count({"ns": "test.foo"}),
+              "test requires collection to have one chunk initially");
 
     // we'll split the collection in two and move the second chunk while three cursors are open
     // cursor1 still has more data in the first chunk, the one that didn't move
@@ -36,7 +38,7 @@
 
     s.adminCommand({split: "test.foo", middle: {_id: 5}});
     s.adminCommand({movechunk: "test.foo", find: {_id: 5}, to: secondary.getMongo().name});
-    assert.eq(2, s.config.chunks.count());
+    assert.eq(2, s.config.chunks.count({"ns": "test.foo"}));
 
     // the cursors should not have been affected
     assert.eq(numObjs, cursor1.itcount(), "c1");
@@ -44,8 +46,6 @@
     assert.eq(numObjs, cursor3.itcount(), "c3");
 
     // Test that a cursor with a 1 second timeout eventually times out.
-    gc();
-    gc();
     var cur = db.foo.find().batchSize(2);
     assert(cur.next(), "T1");
     assert(cur.next(), "T2");
@@ -65,9 +65,6 @@
         }
     }, "cursor failed to time out", /*timeout*/ 30000, /*interval*/ 5000);
 
-    gc();
-    gc();
-
     s.stop();
 
 })();
