@@ -11,13 +11,13 @@
     db.setSlaveOk(true);
 
     assert.writeOK(db.foo.insert({a: 1}, {writeConcern: {w: 3}}));
-    assert.commandWorked(
-        db.runCommand({aggregate: 'foo', pipeline: [{$project: {total: {'$add': ['$a', 1]}}}]}));
+    assert.commandWorked(db.runCommand(
+        {aggregate: 'foo', pipeline: [{$project: {total: {'$add': ['$a', 1]}}}], cursor: {}}));
 
     assert.writeOK(db.foo.insert({a: [1, 2]}, {writeConcern: {w: 3}}));
 
-    var res =
-        db.runCommand({aggregate: 'foo', pipeline: [{$project: {total: {'$add': ['$a', 1]}}}]});
+    var res = db.runCommand(
+        {aggregate: 'foo', pipeline: [{$project: {total: {'$add': ['$a', 1]}}}], cursor: {}});
     assert.commandFailed(res);
-    assert.eq("$add only supports numeric or date types, not Array", res.errmsg, printjson(res));
+    assert.eq("$add only supports numeric or date types, not array", res.errmsg, printjson(res));
 }());
