@@ -1,6 +1,10 @@
 // replica set as solo shard
 // TODO: Add assertion code that catches hang
 
+// The UUID check must be able to contact the shard primaries, but this test manually stops 2/3
+// nodes of a replica set.
+TestData.skipCheckingUUIDsConsistentAcrossCluster = true;
+
 (function() {
     "use strict";
 
@@ -42,7 +46,7 @@
     var testDB = mongosConn.getDB(testDBName);
 
     // Add replSet1 as only shard
-    mongosConn.adminCommand({addshard: replSet1.getURL()});
+    assert.commandWorked(mongosConn.adminCommand({addshard: replSet1.getURL()}));
 
     // Enable sharding on test db and its collection foo
     assert.commandWorked(mongosConn.getDB('admin').runCommand({enablesharding: testDBName}));
@@ -51,7 +55,7 @@
         {shardcollection: testDBName + '.' + testCollName, key: {x: 1}}));
 
     // Test case where GLE should return an error
-    testDB.foo.insert({_id: 'a', x: 1});
+    assert.writeOK(testDB.foo.insert({_id: 'a', x: 1}));
     assert.writeError(testDB.foo.insert({_id: 'a', x: 1}, {writeConcern: {w: 2, wtimeout: 30000}}));
 
     // Add more data
@@ -62,8 +66,14 @@
     assert.writeOK(bulk.execute({w: replNodes, wtimeout: 30000}));
 
     // Take down two nodes and make sure slaveOk reads still work
-    replSet1.stop(1);
-    replSet1.stop(2);
+    var nodes = replSet1.liveNodes;
+    var primary = nodes.master;
+    var secondary1 = nodes.slaves[0];
+    var secondary2 = nodes.slaves[1];
+    replSet1.stop(secondary1);
+    replSet1.stop(secondary2);
+    replSet1.waitForState(primary, ReplSetTest.State.SECONDARY);
+
     testDB.getMongo().adminCommand({setParameter: 1, logLevel: 1});
     testDB.getMongo().setSlaveOk();
     print("trying some queries");
