@@ -6,6 +6,7 @@ var KEYFILE = "jstests/libs/key1";
 var SERVER_CERT = "jstests/libs/server.pem";
 var CA_CERT = "jstests/libs/ca.pem";
 var CLIENT_CERT = "jstests/libs/client.pem";
+var DH_PARAM = "jstests/libs/8k-prime.dhparam";
 
 // Note: "sslAllowInvalidCertificates" is enabled to avoid
 // hostname conflicts with our testing certificates
@@ -29,6 +30,14 @@ var requireSSL = {
     sslCAFile: CA_CERT
 };
 
+var dhparamSSL = {
+    sslMode: "requireSSL",
+    sslAllowInvalidCertificates: "",
+    sslPEMKeyFile: SERVER_CERT,
+    sslCAFile: CA_CERT,
+    setParameter: {"opensslDiffieHellmanParameters": DH_PARAM}
+};
+
 // Test if ssl replset  configs work
 
 var replSetTestFile = "jstests/replsets/replset1.js";
@@ -50,7 +59,7 @@ var replShouldFail = function(name, opt1, opt2) {
     assert.throws(load, [replSetTestFile], "This setup should have failed");
     // clean up to continue running...
     if (replTest) {
-        replTest.stopSet(15);
+        replTest.stopSet();
     }
 };
 
