@@ -27,18 +27,18 @@
 
 #pragma once
 
-#include <boost/config.hpp>
 #include <iosfwd>
 #include <string>
 
 #include "mongo/base/error_codes.h"
 #include "mongo/platform/atomic_word.h"
+#include "mongo/platform/compiler.h"
 
 namespace mongoutils {
 namespace str {
 class stream;
-}
-}
+}  // namespace str
+}  // namespace mongoutils
 
 namespace mongo {
 
@@ -47,8 +47,7 @@ namespace mongo {
  *
  * A Status uses the standardized error codes -- from file 'error_codes.h' -- to
  * determine an error's cause. It further clarifies the error with a textual
- * description. Optionally, a Status may also have an error location number, which
- * should be a unique, grep-able point in the code base (including assert numbers).
+ * description.
  *
  * Example usage:
  *
@@ -60,57 +59,67 @@ namespace mongo {
  *       *c = a+b;
  *       return Status::OK();
  *   }
- *
- * TODO: expand base/error_codes.h to capture common errors in current code
- * TODO: generate base/error_codes.h out of a description file
- * TODO: check 'location' duplicates against assert numbers
  */
-class Status {
+class MONGO_WARN_UNUSED_RESULT_CLASS Status {
 public:
     // Short-hand for returning an OK status.
     static inline Status OK();
 
     /**
-     * Builds an error status given the error code, a textual description of what
-     * caused the error, and a unique position in the where the error occurred
-     * (similar to an assert number).
+     * Builds an error status given the error code and a textual description of what
+     * caused the error.
      *
      * For OK Statuses prefer using Status::OK(). If code is OK, the remaining arguments are
      * ignored.
+     *
+     * For adding context to the reason string, use withContext/addContext rather than making a new
+     * Status manually.
      */
+    MONGO_COMPILER_COLD_FUNCTION Status(ErrorCodes::Error code, std::string reason);
+    MONGO_COMPILER_COLD_FUNCTION Status(ErrorCodes::Error code, const char* reason);
+    MONGO_COMPILER_COLD_FUNCTION Status(ErrorCodes::Error code, StringData reason);
     MONGO_COMPILER_COLD_FUNCTION Status(ErrorCodes::Error code,
-                                        std::string reason,
-                                        int location = 0);
-    MONGO_COMPILER_COLD_FUNCTION Status(ErrorCodes::Error code,
-                                        const char* reason,
-                                        int location = 0);
-    MONGO_COMPILER_COLD_FUNCTION Status(ErrorCodes::Error code,
-                                        const mongoutils::str::stream& reason,
-                                        int location = 0);
+                                        const mongoutils::str::stream& reason);
 
     inline Status(const Status& other);
     inline Status& operator=(const Status& other);
 
-    inline Status(Status&& other) BOOST_NOEXCEPT;
-    inline Status& operator=(Status&& other) BOOST_NOEXCEPT;
+    inline Status(Status&& other) noexcept;
+    inline Status& operator=(Status&& other) noexcept;
 
     inline ~Status();
 
     /**
-     * Returns true if 'other's error code and location are equal/different to this
-     * instance's. Otherwise returns false.
+     * Returns a new Status with the same data as this, but with the reason string prefixed with
+     * reasonPrefix and our standard " :: caused by :: " separator. The new reason is not visible to
+     * any other Statuses that share the same ErrorInfo object.
+     *
+     * No-op when called on an OK status.
      */
-    bool compare(const Status& other) const;
-    bool operator==(const Status& other) const;
-    bool operator!=(const Status& other) const;
+    Status withContext(StringData reasonPrefix) const;
+    void addContext(StringData reasonPrefix) {
+        *this = this->withContext(reasonPrefix);
+    }
 
     /**
-     * Returns true if 'other's error code is equal/different to this instance's.
-     * Otherwise returns false.
+     * Only compares codes. Ignores reason strings.
      */
-    bool compareCode(const ErrorCodes::Error other) const;
-    bool operator==(const ErrorCodes::Error other) const;
-    bool operator!=(const ErrorCodes::Error other) const;
+    bool operator==(const Status& other) const {
+        return code() == other.code();
+    }
+    bool operator!=(const Status& other) const {
+        return !(*this == other);
+    }
+
+    /**
+     * Compares this Status's code with an error code.
+     */
+    bool operator==(const ErrorCodes::Error other) const {
+        return code() == other;
+    }
+    bool operator!=(const ErrorCodes::Error other) const {
+        return !(*this == other);
+    }
 
     //
     // accessors
@@ -122,12 +131,49 @@ public:
 
     inline std::string codeString() const;
 
-    inline std::string reason() const;
 
-    inline int location() const;
+    /**
+     * Returns the reason string or the empty string if isOK().
+     */
+    const std::string& reason() const {
+        if (_error)
+            return _error->reason;
+
+        static const std::string empty;
+        return empty;
+    }
 
     std::string toString() const;
 
+    /**
+     * Returns true if this Status's code is a member of the given category.
+     */
+    template <ErrorCategory category>
+    bool isA() const {
+        return ErrorCodes::isA<category>(code());
+    }
+
+    /**
+     * Call this method to indicate that it is your intention to ignore a returned status. Ignoring
+     * is only possible if the value being ignored is an xvalue -- it is not appropriate to create a
+     * status variable and then ignore it.
+     */
+    inline void ignore() && noexcept {}
+    inline void ignore() const& noexcept = delete;
+
+    /**
+     * This method is a transitional tool, to facilitate transition to compile-time enforced status
+     * checking.
+     *
+     * NOTE: DO NOT ADD NEW CALLS TO THIS METHOD. This method serves the same purpose as
+     * `.ignore()`; however, it indicates a situation where the code that presently ignores a status
+     * code has not been audited for correctness. This method will be removed at some point. If you
+     * encounter a compiler error from ignoring the result of a status-returning function be sure to
+     * check the return value, or deliberately ignore the return value.
+     */
+    inline void transitional_ignore() && noexcept {};
+    inline void transitional_ignore() const& noexcept = delete;
+
     //
     // Below interface used for testing code only.
     //
@@ -141,11 +187,10 @@ private:
         AtomicUInt32 refs;             // reference counter
         const ErrorCodes::Error code;  // error code
         const std::string reason;      // description of error cause
-        const int location;            // unique location of the triggering line in the code
 
-        static ErrorInfo* create(ErrorCodes::Error code, std::string reason, int location);
+        static ErrorInfo* create(ErrorCodes::Error code, std::string reason);
 
-        ErrorInfo(ErrorCodes::Error code, std::string reason, int location);
+        ErrorInfo(ErrorCodes::Error code, std::string reason);
     };
 
     ErrorInfo* _error;
@@ -163,12 +208,7 @@ inline bool operator==(const ErrorCodes::Error lhs, const Status& rhs);
 
 inline bool operator!=(const ErrorCodes::Error lhs, const Status& rhs);
 
-//
-// Convenience method for unittest code. Please use accessors otherwise.
-//
-
 std::ostream& operator<<(std::ostream& os, const Status& status);
-std::ostream& operator<<(std::ostream& os, ErrorCodes::Error);
 
 }  // namespace mongo
 
