@@ -1333,8 +1333,7 @@ TEST(ReplicaSetMonitor, MaxStalenessMSMatch) {
     Refresher refresher(state);
     repl::OpTime opTime{Timestamp{10, 10}, 10};
 
-    const ReadPreferenceSetting secondary(
-        ReadPreference::SecondaryOnly, TagSet(), Milliseconds(100000));
+    const ReadPreferenceSetting secondary(ReadPreference::SecondaryOnly, TagSet(), Seconds(100));
     BSONArray hosts = BSON_ARRAY("a"
                                  << "b"
                                  << "c");
@@ -1361,7 +1360,7 @@ TEST(ReplicaSetMonitor, MaxStalenessMSMatch) {
                                         << BSON("lastWriteDate" << (nonStale ? lastWriteDateNonStale
                                                                              : lastWriteDateStale)
                                                                 << "opTime"
-                                                                << opTime.toBSON())
+                                                                << opTime)
                                         << "ok"
                                         << true));
         ns = refresher.getNextStep();
@@ -1411,7 +1410,7 @@ TEST(ReplicaSetMonitor, MaxStalenessMSNoMatch) {
                                         << BSON("lastWriteDate" << (primary ? lastWriteDateNonStale
                                                                             : lastWriteDateStale)
                                                                 << "opTime"
-                                                                << opTime.toBSON())
+                                                                << opTime)
                                         << "ok"
                                         << true));
 
@@ -1463,7 +1462,7 @@ TEST(ReplicaSetMonitor, MaxStalenessMSNoPrimaryMatch) {
                                                 << (isNonStale ? lastWriteDateNonStale
                                                                : lastWriteDateStale)
                                                 << "opTime"
-                                                << opTime.toBSON())
+                                                << opTime)
                                         << "ok"
                                         << true));
 
@@ -1517,7 +1516,7 @@ TEST(ReplicaSetMonitor, MaxStalenessMSAllFailed) {
                                                 << (isNonStale ? lastWriteDateNonStale
                                                                : lastWriteDateStale)
                                                 << "opTime"
-                                                << opTime.toBSON())
+                                                << opTime)
                                         << "ok"
                                         << true));
 
@@ -1569,7 +1568,7 @@ TEST(ReplicaSetMonitor, MaxStalenessMSAllButPrimaryFailed) {
                                         << BSON("lastWriteDate" << (primary ? lastWriteDateNonStale
                                                                             : lastWriteDateStale)
                                                                 << "opTime"
-                                                                << opTime.toBSON())
+                                                                << opTime)
                                         << "ok"
                                         << true));
         ns = refresher.getNextStep();
@@ -1621,7 +1620,7 @@ TEST(ReplicaSetMonitor, MaxStalenessMSOneSecondaryFailed) {
                                         << BSON("lastWriteDate" << (primary ? lastWriteDateNonStale
                                                                             : lastWriteDateStale)
                                                                 << "opTime"
-                                                                << opTime.toBSON())
+                                                                << opTime)
                                         << "ok"
                                         << true));
         ns = refresher.getNextStep();
@@ -1674,7 +1673,7 @@ TEST(ReplicaSetMonitor, MaxStalenessMSNonStaleSecondaryMatched) {
                                                 << (isNonStale ? lastWriteDateNonStale
                                                                : lastWriteDateStale)
                                                 << "opTime"
-                                                << opTime.toBSON())
+                                                << opTime)
                                         << "ok"
                                         << true));
         ns = refresher.getNextStep();
@@ -1739,7 +1738,7 @@ TEST(ReplicaSetMonitor, MaxStalenessMSZeroNoLastWrite) {
     SetStatePtr state = std::make_shared<SetState>("name", basicSeedsSet);
     Refresher refresher(state);
 
-    const ReadPreferenceSetting secondary(ReadPreference::SecondaryOnly, TagSet(), Milliseconds(0));
+    const ReadPreferenceSetting secondary(ReadPreference::SecondaryOnly, TagSet(), Seconds(0));
     BSONArray hosts = BSON_ARRAY("a"
                                  << "b"
                                  << "c");
