@@ -31,12 +31,15 @@
 #include "mongo/db/auth/authz_manager_external_state_local.h"
 
 #include "mongo/base/status.h"
+#include "mongo/bson/mutable/algorithm.h"
 #include "mongo/bson/mutable/document.h"
 #include "mongo/bson/mutable/element.h"
 #include "mongo/bson/util/bson_extract.h"
 #include "mongo/db/auth/authorization_manager.h"
+#include "mongo/db/auth/privilege_parser.h"
 #include "mongo/db/auth/user_document_parser.h"
 #include "mongo/db/operation_context.h"
+#include "mongo/db/server_options.h"
 #include "mongo/util/log.h"
 #include "mongo/util/mongoutils/str.h"
 
@@ -44,8 +47,8 @@ namespace mongo {
 
 using std::vector;
 
-Status AuthzManagerExternalStateLocal::initialize(OperationContext* txn) {
-    Status status = _initializeRoleGraph(txn);
+Status AuthzManagerExternalStateLocal::initialize(OperationContext* opCtx) {
+    Status status = _initializeRoleGraph(opCtx);
     if (!status.isOK()) {
         if (status == ErrorCodes::GraphContainsCycle) {
             error() << "Cycle detected in admin.system.roles; role inheritance disabled. "
@@ -61,10 +64,10 @@ Status AuthzManagerExternalStateLocal::initialize(OperationContext* txn) {
     return Status::OK();
 }
 
-Status AuthzManagerExternalStateLocal::getStoredAuthorizationVersion(OperationContext* txn,
+Status AuthzManagerExternalStateLocal::getStoredAuthorizationVersion(OperationContext* opCtx,
                                                                      int* outVersion) {
     BSONObj versionDoc;
-    Status status = findOne(txn,
+    Status status = findOne(opCtx,
                             AuthorizationManager::versionCollectionNamespace,
                             AuthorizationManager::versionDocumentQuery,
                             &versionDoc);
@@ -132,12 +135,20 @@ void addPrivilegeObjectsOrWarningsToArrayElement(mutablebson::Element privileges
         }
     }
 }
+
+void addAuthenticationRestrictionObjectsToArrayElement(
+    mutablebson::Element restrictionsElement,
+    const std::vector<SharedRestrictionDocument>& restrictions) {
+    for (const auto& r : restrictions) {
+        fassert(40560, restrictionsElement.appendArray("", r->toBSON()));
+    }
+}
 }  // namespace
 
-bool AuthzManagerExternalStateLocal::hasAnyPrivilegeDocuments(OperationContext* txn) {
+bool AuthzManagerExternalStateLocal::hasAnyPrivilegeDocuments(OperationContext* opCtx) {
     BSONObj userBSONObj;
     Status statusFindUsers =
-        findOne(txn, AuthorizationManager::usersCollectionNamespace, BSONObj(), &userBSONObj);
+        findOne(opCtx, AuthorizationManager::usersCollectionNamespace, BSONObj(), &userBSONObj);
 
     // If we were unable to complete the query,
     // it's best to assume that there _are_ privilege documents.
@@ -145,23 +156,24 @@ bool AuthzManagerExternalStateLocal::hasAnyPrivilegeDocuments(OperationContext*
         return true;
     }
     Status statusFindRoles =
-        findOne(txn, AuthorizationManager::rolesCollectionNamespace, BSONObj(), &userBSONObj);
+        findOne(opCtx, AuthorizationManager::rolesCollectionNamespace, BSONObj(), &userBSONObj);
     return statusFindRoles != ErrorCodes::NoMatchingDocument;
 }
 
-Status AuthzManagerExternalStateLocal::getUserDescription(OperationContext* txn,
+Status AuthzManagerExternalStateLocal::getUserDescription(OperationContext* opCtx,
                                                           const UserName& userName,
                                                           BSONObj* result) {
     Status status = Status::OK();
 
-    if (!shouldUseRolesFromConnection(txn, userName)) {
-        status = _getUserDocument(txn, userName, result);
+    if (!shouldUseRolesFromConnection(opCtx, userName)) {
+        status = _getUserDocument(opCtx, userName, result);
         if (!status.isOK())
             return status;
     } else {
         // We are able to artifically construct the external user from the request
         BSONArrayBuilder userRoles;
-        for (const RoleName& role : txn->getClient()->session()->getX509PeerInfo().roles) {
+        auto& sslPeerInfo = SSLPeerInfo::forSession(opCtx->getClient()->session());
+        for (const RoleName& role : sslPeerInfo.roles) {
             userRoles << BSON("role" << role.getRole() << "db" << role.getDB());
         }
         *result = BSON("_id" << userName.getUser() << "user" << userName.getUser() << "db"
@@ -193,60 +205,89 @@ void AuthzManagerExternalStateLocal::resolveUserRoles(mutablebson::Document* use
                                                       const std::vector<RoleName>& directRoles) {
     unordered_set<RoleName> indirectRoles;
     PrivilegeVector allPrivileges;
-    bool isRoleGraphInconsistent;
+    std::vector<SharedRestrictionDocument> allAuthenticationRestrictions;
+    bool isRoleGraphConsistent = false;
+
     {
         stdx::lock_guard<stdx::mutex> lk(_roleGraphMutex);
-        isRoleGraphInconsistent = _roleGraphState == roleGraphStateConsistent;
-        for (size_t i = 0; i < directRoles.size(); ++i) {
-            const RoleName& role(directRoles[i]);
+        isRoleGraphConsistent = _roleGraphState == roleGraphStateConsistent;
+        for (const auto& role : directRoles) {
             indirectRoles.insert(role);
-            if (isRoleGraphInconsistent) {
+            if (isRoleGraphConsistent) {
                 for (RoleNameIterator subordinates = _roleGraph.getIndirectSubordinates(role);
                      subordinates.more();
                      subordinates.next()) {
                     indirectRoles.insert(subordinates.get());
                 }
             }
-            const PrivilegeVector& rolePrivileges(isRoleGraphInconsistent
-                                                      ? _roleGraph.getAllPrivileges(role)
-                                                      : _roleGraph.getDirectPrivileges(role));
-            for (PrivilegeVector::const_iterator priv = rolePrivileges.begin(),
-                                                 end = rolePrivileges.end();
-                 priv != end;
-                 ++priv) {
-                Privilege::addPrivilegeToPrivilegeVector(&allPrivileges, *priv);
+
+            const auto& currentPrivileges = isRoleGraphConsistent
+                ? _roleGraph.getAllPrivileges(role)
+                : _roleGraph.getDirectPrivileges(role);
+            for (const auto& priv : currentPrivileges) {
+                Privilege::addPrivilegeToPrivilegeVector(&allPrivileges, priv);
+            }
+
+            if (isRoleGraphConsistent) {
+                const auto& currentAuthenticationRestrictions =
+                    _roleGraph.getAllAuthenticationRestrictions(role);
+                allAuthenticationRestrictions.insert(allAuthenticationRestrictions.end(),
+                                                     currentAuthenticationRestrictions.begin(),
+                                                     currentAuthenticationRestrictions.end());
+            } else {
+                const auto& dar = _roleGraph.getDirectAuthenticationRestrictions(role);
+                if (dar.get()) {
+                    allAuthenticationRestrictions.push_back(dar);
+                }
             }
         }
     }
 
-    mutablebson::Element inheritedRolesElement = userDoc->makeElementArray("inheritedRoles");
-    mutablebson::Element privilegesElement = userDoc->makeElementArray("inheritedPrivileges");
-    mutablebson::Element warningsElement = userDoc->makeElementArray("warnings");
+    auto warningsElement = userDoc->makeElementArray("warnings");
+
+    auto inheritedRolesElement = userDoc->makeElementArray("inheritedRoles");
     fassert(17159, userDoc->root().pushBack(inheritedRolesElement));
+    addRoleNameObjectsToArrayElement(inheritedRolesElement,
+                                     makeRoleNameIteratorForContainer(indirectRoles));
+
+    auto privilegesElement = userDoc->makeElementArray("inheritedPrivileges");
     fassert(17158, userDoc->root().pushBack(privilegesElement));
-    if (!isRoleGraphInconsistent) {
+    addPrivilegeObjectsOrWarningsToArrayElement(privilegesElement, warningsElement, allPrivileges);
+
+    auto inheritedAuthenticationRestrictionsElement =
+        userDoc->makeElementArray("inheritedAuthenticationRestrictions");
+    fassert(40558, userDoc->root().pushBack(inheritedAuthenticationRestrictionsElement));
+    addAuthenticationRestrictionObjectsToArrayElement(inheritedAuthenticationRestrictionsElement,
+                                                      allAuthenticationRestrictions);
+
+    if (!mutablebson::findFirstChildNamed(userDoc->root(), "authenticationRestrictions").ok()) {
+        auto authenticationRestrictionsElement =
+            userDoc->makeElementArray("authenticationRestrictions");
+        fassert(40572, userDoc->root().pushBack(authenticationRestrictionsElement));
+    }
+
+    if (!isRoleGraphConsistent) {
         fassert(17160,
                 warningsElement.appendString(
                     "", "Role graph inconsistent, only direct privileges available."));
     }
-    addRoleNameObjectsToArrayElement(inheritedRolesElement,
-                                     makeRoleNameIteratorForContainer(indirectRoles));
-    addPrivilegeObjectsOrWarningsToArrayElement(privilegesElement, warningsElement, allPrivileges);
+
     if (warningsElement.hasChildren()) {
         fassert(17161, userDoc->root().pushBack(warningsElement));
     }
 }
 
-Status AuthzManagerExternalStateLocal::_getUserDocument(OperationContext* txn,
+Status AuthzManagerExternalStateLocal::_getUserDocument(OperationContext* opCtx,
                                                         const UserName& userName,
                                                         BSONObj* userDoc) {
-    Status status = findOne(txn,
+    Status status = findOne(opCtx,
                             AuthorizationManager::usersCollectionNamespace,
                             BSON(AuthorizationManager::USER_NAME_FIELD_NAME
                                  << userName.getUser()
                                  << AuthorizationManager::USER_DB_FIELD_NAME
                                  << userName.getDB()),
                             userDoc);
+
     if (status == ErrorCodes::NoMatchingDocument) {
         status =
             Status(ErrorCodes::UserNotFound,
@@ -255,10 +296,12 @@ Status AuthzManagerExternalStateLocal::_getUserDocument(OperationContext* txn,
     return status;
 }
 
-Status AuthzManagerExternalStateLocal::getRoleDescription(OperationContext* txn,
-                                                          const RoleName& roleName,
-                                                          PrivilegeFormat showPrivileges,
-                                                          BSONObj* result) {
+Status AuthzManagerExternalStateLocal::getRoleDescription(
+    OperationContext* opCtx,
+    const RoleName& roleName,
+    PrivilegeFormat showPrivileges,
+    AuthenticationRestrictionsFormat showRestrictions,
+    BSONObj* result) {
     if (showPrivileges == PrivilegeFormat::kShowAsUserFragment) {
         mutablebson::Document resultDoc;
         mutablebson::Element rolesElement = resultDoc.makeElementArray("roles");
@@ -270,13 +313,15 @@ Status AuthzManagerExternalStateLocal::getRoleDescription(OperationContext* txn,
         return Status::OK();
     }
     stdx::lock_guard<stdx::mutex> lk(_roleGraphMutex);
-    return _getRoleDescription_inlock(roleName, showPrivileges, result);
+    return _getRoleDescription_inlock(roleName, showPrivileges, showRestrictions, result);
 }
 
-Status AuthzManagerExternalStateLocal::getRolesDescription(OperationContext* txn,
-                                                           const std::vector<RoleName>& roles,
-                                                           PrivilegeFormat showPrivileges,
-                                                           BSONObj* result) {
+Status AuthzManagerExternalStateLocal::getRolesDescription(
+    OperationContext* opCtx,
+    const std::vector<RoleName>& roles,
+    PrivilegeFormat showPrivileges,
+    AuthenticationRestrictionsFormat showRestrictions,
+    BSONObj* result) {
     if (showPrivileges == PrivilegeFormat::kShowAsUserFragment) {
         mutablebson::Document resultDoc;
         mutablebson::Element rolesElement = resultDoc.makeElementArray("roles");
@@ -291,7 +336,8 @@ Status AuthzManagerExternalStateLocal::getRolesDescription(OperationContext* txn
     BSONArrayBuilder resultBuilder;
     for (const RoleName& role : roles) {
         BSONObj roleDoc;
-        Status status = _getRoleDescription_inlock(role, showPrivileges, &roleDoc);
+        Status status =
+            _getRoleDescription_inlock(role, showPrivileges, showRestrictions, &roleDoc);
         if (!status.isOK()) {
             if (status.code() == ErrorCodes::RoleNotFound) {
                 continue;
@@ -304,9 +350,11 @@ Status AuthzManagerExternalStateLocal::getRolesDescription(OperationContext* txn
     return Status::OK();
 }
 
-Status AuthzManagerExternalStateLocal::_getRoleDescription_inlock(const RoleName& roleName,
-                                                                  PrivilegeFormat showPrivileges,
-                                                                  BSONObj* result) {
+Status AuthzManagerExternalStateLocal::_getRoleDescription_inlock(
+    const RoleName& roleName,
+    PrivilegeFormat showPrivileges,
+    AuthenticationRestrictionsFormat showRestrictions,
+    BSONObj* result) {
     if (!_roleGraph.roleExists(roleName))
         return Status(ErrorCodes::RoleNotFound, "No role named " + roleName.toString());
 
@@ -318,23 +366,39 @@ Status AuthzManagerExternalStateLocal::_getRoleDescription_inlock(const RoleName
         17163,
         resultDoc.root().appendString(AuthorizationManager::ROLE_DB_FIELD_NAME, roleName.getDB()));
     fassert(17267, resultDoc.root().appendBool("isBuiltin", _roleGraph.isBuiltinRole(roleName)));
-    mutablebson::Element rolesElement = resultDoc.makeElementArray("roles");
+
+    auto warningsElement = resultDoc.makeElementArray("warnings");
+
+    auto rolesElement = resultDoc.makeElementArray("roles");
     fassert(17164, resultDoc.root().pushBack(rolesElement));
-    mutablebson::Element inheritedRolesElement = resultDoc.makeElementArray("inheritedRoles");
+    addRoleNameObjectsToArrayElement(rolesElement, _roleGraph.getDirectSubordinates(roleName));
+
+    auto inheritedRolesElement = resultDoc.makeElementArray("inheritedRoles");
     fassert(17165, resultDoc.root().pushBack(inheritedRolesElement));
-    mutablebson::Element privilegesElement = resultDoc.makeElementArray("privileges");
-    mutablebson::Element inheritedPrivilegesElement =
-        resultDoc.makeElementArray("inheritedPrivileges");
+
+    auto privilegesElement = resultDoc.makeElementArray("privileges");
     if (showPrivileges == PrivilegeFormat::kShowSeparate) {
         fassert(17166, resultDoc.root().pushBack(privilegesElement));
     }
-    mutablebson::Element warningsElement = resultDoc.makeElementArray("warnings");
 
-    addRoleNameObjectsToArrayElement(rolesElement, _roleGraph.getDirectSubordinates(roleName));
+    if (showRestrictions == AuthenticationRestrictionsFormat::kShow) {
+        auto authenticationRestrictionsElement =
+            resultDoc.makeElementArray("authenticationRestrictions");
+        fassert(40559, resultDoc.root().pushBack(authenticationRestrictionsElement));
+
+        const auto& restrictions = _roleGraph.getDirectAuthenticationRestrictions(roleName);
+        if (restrictions.get()) {
+            fassert(40561,
+                    authenticationRestrictionsElement.appendArray("", restrictions->toBSON()));
+        }
+    }
+
     if (_roleGraphState == roleGraphStateConsistent) {
         addRoleNameObjectsToArrayElement(inheritedRolesElement,
                                          _roleGraph.getIndirectSubordinates(roleName));
+
         if (showPrivileges == PrivilegeFormat::kShowSeparate) {
+            auto inheritedPrivilegesElement = resultDoc.makeElementArray("inheritedPrivileges");
             addPrivilegeObjectsOrWarningsToArrayElement(
                 privilegesElement, warningsElement, _roleGraph.getDirectPrivileges(roleName));
 
@@ -343,12 +407,27 @@ Status AuthzManagerExternalStateLocal::_getRoleDescription_inlock(const RoleName
 
             fassert(17323, resultDoc.root().pushBack(inheritedPrivilegesElement));
         }
+
+        if (showRestrictions == AuthenticationRestrictionsFormat::kShow) {
+            auto inheritedAuthenticationRestrictionsElement =
+                resultDoc.makeElementArray("inheritedAuthenticationRestrictions");
+            fassert(40563, resultDoc.root().pushBack(inheritedAuthenticationRestrictionsElement));
+
+            for (const auto& restrictions : _roleGraph.getAllAuthenticationRestrictions(roleName)) {
+                fassert(40562,
+                        inheritedAuthenticationRestrictionsElement.appendArray(
+                            "", restrictions->toBSON()));
+            }
+        }
     } else if (showPrivileges == PrivilegeFormat::kShowSeparate) {
-        warningsElement.appendString(
-            "", "Role graph state inconsistent; only direct privileges available.");
         addPrivilegeObjectsOrWarningsToArrayElement(
             privilegesElement, warningsElement, _roleGraph.getDirectPrivileges(roleName));
+        fassert(40557,
+                warningsElement.appendString("",
+                                             "Role graph state inconsistent; only direct "
+                                             "privileges and restrictions available."));
     }
+
     if (warningsElement.hasChildren()) {
         fassert(17167, resultDoc.root().pushBack(warningsElement));
     }
@@ -356,11 +435,13 @@ Status AuthzManagerExternalStateLocal::_getRoleDescription_inlock(const RoleName
     return Status::OK();
 }
 
-Status AuthzManagerExternalStateLocal::getRoleDescriptionsForDB(OperationContext* txn,
-                                                                const std::string dbname,
-                                                                PrivilegeFormat showPrivileges,
-                                                                bool showBuiltinRoles,
-                                                                vector<BSONObj>* result) {
+Status AuthzManagerExternalStateLocal::getRoleDescriptionsForDB(
+    OperationContext* opCtx,
+    const std::string& dbname,
+    PrivilegeFormat showPrivileges,
+    AuthenticationRestrictionsFormat showRestrictions,
+    bool showBuiltinRoles,
+    vector<BSONObj>* result) {
     if (showPrivileges == PrivilegeFormat::kShowAsUserFragment) {
         return Status(ErrorCodes::IllegalOperation,
                       "Cannot get user fragment for all roles in a database");
@@ -372,7 +453,8 @@ Status AuthzManagerExternalStateLocal::getRoleDescriptionsForDB(OperationContext
             continue;
         }
         BSONObj roleDoc;
-        Status status = _getRoleDescription_inlock(it.get(), showPrivileges, &roleDoc);
+        Status status =
+            _getRoleDescription_inlock(it.get(), showPrivileges, showRestrictions, &roleDoc);
         if (!status.isOK()) {
             return status;
         }
@@ -399,7 +481,7 @@ void addRoleFromDocumentOrWarn(RoleGraph* roleGraph, const BSONObj& doc) {
 
 }  // namespace
 
-Status AuthzManagerExternalStateLocal::_initializeRoleGraph(OperationContext* txn) {
+Status AuthzManagerExternalStateLocal::_initializeRoleGraph(OperationContext* opCtx) {
     stdx::lock_guard<stdx::mutex> lkInitialzeRoleGraph(_roleGraphMutex);
 
     _roleGraphState = roleGraphStateInitial;
@@ -407,7 +489,7 @@ Status AuthzManagerExternalStateLocal::_initializeRoleGraph(OperationContext* tx
 
     RoleGraph newRoleGraph;
     Status status =
-        query(txn,
+        query(opCtx,
               AuthorizationManager::rolesCollectionNamespace,
               BSONObj(),
               BSONObj(),
@@ -431,26 +513,26 @@ Status AuthzManagerExternalStateLocal::_initializeRoleGraph(OperationContext* tx
     }
 
     if (status.isOK()) {
-        _roleGraph.swap(newRoleGraph);
-        _roleGraphState = newState;
+        _roleGraph = std::move(newRoleGraph);
+        _roleGraphState = std::move(newState);
     }
     return status;
 }
 
 class AuthzManagerExternalStateLocal::AuthzManagerLogOpHandler : public RecoveryUnit::Change {
 public:
-    // None of the parameters below (except txn and externalState) need to live longer than the
+    // None of the parameters below (except opCtx and externalState) need to live longer than the
     // instantiations of this class
-    AuthzManagerLogOpHandler(OperationContext* txn,
+    AuthzManagerLogOpHandler(OperationContext* opCtx,
                              AuthzManagerExternalStateLocal* externalState,
                              const char* op,
-                             const char* ns,
+                             const NamespaceString& nss,
                              const BSONObj& o,
                              const BSONObj* o2)
-        : _txn(txn),
+        : _opCtx(opCtx),
           _externalState(externalState),
           _op(op),
-          _ns(ns),
+          _nss(nss),
           _o(o.getOwned()),
 
           _isO2Set(o2 ? true : false),
@@ -459,13 +541,13 @@ public:
     virtual void commit() {
         stdx::lock_guard<stdx::mutex> lk(_externalState->_roleGraphMutex);
         Status status = _externalState->_roleGraph.handleLogOp(
-            _txn, _op.c_str(), NamespaceString(_ns.c_str()), _o, _isO2Set ? &_o2 : NULL);
+            _opCtx, _op.c_str(), _nss, _o, _isO2Set ? &_o2 : NULL);
 
         if (status == ErrorCodes::OplogOperationUnsupported) {
             _externalState->_roleGraph = RoleGraph();
             _externalState->_roleGraphState = _externalState->roleGraphStateInitial;
             BSONObjBuilder oplogEntryBuilder;
-            oplogEntryBuilder << "op" << _op << "ns" << _ns << "o" << _o;
+            oplogEntryBuilder << "op" << _op << "ns" << _nss.ns() << "o" << _o;
             if (_isO2Set)
                 oplogEntryBuilder << "o2" << _o2;
             error() << "Unsupported modification to roles collection in oplog; "
@@ -490,21 +572,25 @@ public:
     virtual void rollback() {}
 
 private:
-    OperationContext* _txn;
+    OperationContext* _opCtx;
     AuthzManagerExternalStateLocal* _externalState;
     const std::string _op;
-    const std::string _ns;
+    const NamespaceString _nss;
     const BSONObj _o;
 
     const bool _isO2Set;
     const BSONObj _o2;
 };
 
-void AuthzManagerExternalStateLocal::logOp(
-    OperationContext* txn, const char* op, const char* ns, const BSONObj& o, const BSONObj* o2) {
-    if (ns == AuthorizationManager::rolesCollectionNamespace.ns() ||
-        ns == AuthorizationManager::adminCommandNamespace.ns()) {
-        txn->recoveryUnit()->registerChange(new AuthzManagerLogOpHandler(txn, this, op, ns, o, o2));
+void AuthzManagerExternalStateLocal::logOp(OperationContext* opCtx,
+                                           const char* op,
+                                           const NamespaceString& nss,
+                                           const BSONObj& o,
+                                           const BSONObj* o2) {
+    if (nss == AuthorizationManager::rolesCollectionNamespace ||
+        nss == AuthorizationManager::adminCommandNamespace) {
+        opCtx->recoveryUnit()->registerChange(
+            new AuthzManagerLogOpHandler(opCtx, this, op, nss, o, o2));
     }
 }
 
