@@ -43,16 +43,6 @@ namespace {
 PrivilegeVector emptyPrivilegeVector;
 }  // namespace
 
-RoleGraph::RoleGraph(){};
-RoleGraph::RoleGraph(const RoleGraph& other)
-    : _roleToSubordinates(other._roleToSubordinates),
-      _roleToIndirectSubordinates(other._roleToIndirectSubordinates),
-      _roleToMembers(other._roleToMembers),
-      _directPrivilegesForRole(other._directPrivilegesForRole),
-      _allPrivilegesForRole(other._allPrivilegesForRole),
-      _allRoles(other._allRoles) {}
-RoleGraph::~RoleGraph(){};
-
 void RoleGraph::swap(RoleGraph& other) {
     using std::swap;
     swap(this->_roleToSubordinates, other._roleToSubordinates);
@@ -60,13 +50,11 @@ void RoleGraph::swap(RoleGraph& other) {
     swap(this->_roleToMembers, other._roleToMembers);
     swap(this->_directPrivilegesForRole, other._directPrivilegesForRole);
     swap(this->_allPrivilegesForRole, other._allPrivilegesForRole);
+    swap(this->_directRestrictionsForRole, other._directRestrictionsForRole);
+    swap(this->_allRestrictionsForRole, other._allRestrictionsForRole);
     swap(this->_allRoles, other._allRoles);
 }
 
-void swap(RoleGraph& lhs, RoleGraph& rhs) {
-    lhs.swap(rhs);
-}
-
 bool RoleGraph::roleExists(const RoleName& role) {
     _createBuiltinRoleIfNeeded(role);
     return _roleExistsDontCreateBuiltin(role);
@@ -91,8 +79,7 @@ Status RoleGraph::createRole(const RoleName& role) {
     if (roleExists(role)) {
         return Status(ErrorCodes::DuplicateKey,
                       mongoutils::str::stream() << "Role: " << role.getFullName()
-                                                << " already exists",
-                      0);
+                                                << " already exists");
     }
 
     _createRoleDontCheckIfRoleExists(role);
@@ -114,14 +101,12 @@ Status RoleGraph::deleteRole(const RoleName& role) {
     if (!roleExists(role)) {
         return Status(ErrorCodes::RoleNotFound,
                       mongoutils::str::stream() << "Role: " << role.getFullName()
-                                                << " does not exist",
-                      0);
+                                                << " does not exist");
     }
     if (isBuiltinRole(role)) {
         return Status(ErrorCodes::InvalidRoleModification,
-                      mongoutils::str::stream()
-                          << "Cannot delete built-in role: " << role.getFullName(),
-                      0);
+                      mongoutils::str::stream() << "Cannot delete built-in role: "
+                                                << role.getFullName());
     }
 
     for (std::vector<RoleName>::iterator it = _roleToSubordinates[role].begin();
@@ -183,8 +168,8 @@ Status RoleGraph::addRoleToRole(const RoleName& recipient, const RoleName& role)
     }
     if (isBuiltinRole(recipient)) {
         return Status(ErrorCodes::InvalidRoleModification,
-                      mongoutils::str::stream()
-                          << "Cannot grant roles to built-in role: " << role.getFullName());
+                      mongoutils::str::stream() << "Cannot grant roles to built-in role: "
+                                                << role.getFullName());
     }
     if (!roleExists(role)) {
         return Status(ErrorCodes::RoleNotFound,
@@ -207,20 +192,17 @@ Status RoleGraph::removeRoleFromRole(const RoleName& recipient, const RoleName&
     if (!roleExists(recipient)) {
         return Status(ErrorCodes::RoleNotFound,
                       mongoutils::str::stream() << "Role: " << recipient.getFullName()
-                                                << " does not exist",
-                      0);
+                                                << " does not exist");
     }
     if (isBuiltinRole(recipient)) {
         return Status(ErrorCodes::InvalidRoleModification,
-                      mongoutils::str::stream()
-                          << "Cannot remove roles from built-in role: " << role.getFullName(),
-                      0);
+                      mongoutils::str::stream() << "Cannot remove roles from built-in role: "
+                                                << role.getFullName());
     }
     if (!roleExists(role)) {
         return Status(ErrorCodes::RoleNotFound,
                       mongoutils::str::stream() << "Role: " << role.getFullName()
-                                                << " does not exist",
-                      0);
+                                                << " does not exist");
     }
 
     std::vector<RoleName>::iterator itToRm =
@@ -231,8 +213,7 @@ Status RoleGraph::removeRoleFromRole(const RoleName& recipient, const RoleName&
         return Status(ErrorCodes::RolesNotRelated,
                       mongoutils::str::stream() << recipient.getFullName() << " is not a member"
                                                                               " of "
-                                                << role.getFullName(),
-                      0);
+                                                << role.getFullName());
     }
 
     itToRm = std::find(
@@ -247,14 +228,12 @@ Status RoleGraph::removeAllRolesFromRole(const RoleName& victim) {
     if (!roleExists(victim)) {
         return Status(ErrorCodes::RoleNotFound,
                       mongoutils::str::stream() << "Role: " << victim.getFullName()
-                                                << " does not exist",
-                      0);
+                                                << " does not exist");
     }
     if (isBuiltinRole(victim)) {
         return Status(ErrorCodes::InvalidRoleModification,
-                      mongoutils::str::stream()
-                          << "Cannot remove roles from built-in role: " << victim.getFullName(),
-                      0);
+                      mongoutils::str::stream() << "Cannot remove roles from built-in role: "
+                                                << victim.getFullName());
     }
 
     RoleNameVector& subordinatesOfVictim = _roleToSubordinates[victim];
@@ -276,14 +255,12 @@ Status RoleGraph::addPrivilegeToRole(const RoleName& role, const Privilege& priv
     if (!roleExists(role)) {
         return Status(ErrorCodes::RoleNotFound,
                       mongoutils::str::stream() << "Role: " << role.getFullName()
-                                                << " does not exist",
-                      0);
+                                                << " does not exist");
     }
     if (isBuiltinRole(role)) {
         return Status(ErrorCodes::InvalidRoleModification,
-                      mongoutils::str::stream()
-                          << "Cannot grant privileges to built-in role: " << role.getFullName(),
-                      0);
+                      mongoutils::str::stream() << "Cannot grant privileges to built-in role: "
+                                                << role.getFullName());
     }
 
     _addPrivilegeToRoleNoChecks(role, privilegeToAdd);
@@ -303,14 +280,12 @@ Status RoleGraph::addPrivilegesToRole(const RoleName& role,
     if (!roleExists(role)) {
         return Status(ErrorCodes::RoleNotFound,
                       mongoutils::str::stream() << "Role: " << role.getFullName()
-                                                << " does not exist",
-                      0);
+                                                << " does not exist");
     }
     if (isBuiltinRole(role)) {
         return Status(ErrorCodes::InvalidRoleModification,
-                      mongoutils::str::stream()
-                          << "Cannot grant privileges to built-in role: " << role.getFullName(),
-                      0);
+                      mongoutils::str::stream() << "Cannot grant privileges to built-in role: "
+                                                << role.getFullName());
     }
 
     for (PrivilegeVector::const_iterator it = privilegesToAdd.begin(); it != privilegesToAdd.end();
@@ -325,13 +300,12 @@ Status RoleGraph::removePrivilegeFromRole(const RoleName& role,
     if (!roleExists(role)) {
         return Status(ErrorCodes::RoleNotFound,
                       mongoutils::str::stream() << "Role: " << role.getFullName()
-                                                << " does not exist",
-                      0);
+                                                << " does not exist");
     }
     if (isBuiltinRole(role)) {
         return Status(ErrorCodes::InvalidRoleModification,
-                      mongoutils::str::stream()
-                          << "Cannot remove privileges from built-in role: " << role.getFullName());
+                      mongoutils::str::stream() << "Cannot remove privileges from built-in role: "
+                                                << role.getFullName());
     }
 
     PrivilegeVector& currentPrivileges = _directPrivilegesForRole[role];
@@ -343,13 +317,13 @@ Status RoleGraph::removePrivilegeFromRole(const RoleName& role,
 
             if (!curActions.isSupersetOf(privilegeToRemove.getActions())) {
                 // Didn't possess all the actions being removed.
-                return Status(ErrorCodes::PrivilegeNotFound,
-                              mongoutils::str::stream()
-                                  << "Role: " << role.getFullName()
-                                  << " does not contain a privilege on "
-                                  << privilegeToRemove.getResourcePattern().toString()
-                                  << " with actions: " << privilegeToRemove.getActions().toString(),
-                              0);
+                return Status(
+                    ErrorCodes::PrivilegeNotFound,
+                    mongoutils::str::stream() << "Role: " << role.getFullName()
+                                              << " does not contain a privilege on "
+                                              << privilegeToRemove.getResourcePattern().toString()
+                                              << " with actions: "
+                                              << privilegeToRemove.getActions().toString());
             }
 
             curPrivilege.removeActions(privilegeToRemove.getActions());
@@ -363,8 +337,7 @@ Status RoleGraph::removePrivilegeFromRole(const RoleName& role,
                   mongoutils::str::stream() << "Role: " << role.getFullName()
                                             << " does not "
                                                "contain any privileges on "
-                                            << privilegeToRemove.getResourcePattern().toString(),
-                  0);
+                                            << privilegeToRemove.getResourcePattern().toString());
 }
 
 Status RoleGraph::removePrivilegesFromRole(const RoleName& role,
@@ -384,21 +357,37 @@ Status RoleGraph::removeAllPrivilegesFromRole(const RoleName& role) {
     if (!roleExists(role)) {
         return Status(ErrorCodes::RoleNotFound,
                       mongoutils::str::stream() << "Role: " << role.getFullName()
-                                                << " does not exist",
-                      0);
+                                                << " does not exist");
     }
     if (isBuiltinRole(role)) {
         return Status(ErrorCodes::InvalidRoleModification,
-                      mongoutils::str::stream()
-                          << "Cannot remove privileges from built-in role: " << role.getFullName());
+                      mongoutils::str::stream() << "Cannot remove privileges from built-in role: "
+                                                << role.getFullName());
     }
     _directPrivilegesForRole[role].clear();
     return Status::OK();
 }
 
+Status RoleGraph::replaceRestrictionsForRole(const RoleName& role,
+                                             SharedRestrictionDocument restrictions) {
+    if (!roleExists(role)) {
+        return Status(ErrorCodes::RoleNotFound,
+                      mongoutils::str::stream() << "Role: " << role.getFullName()
+                                                << " does not exist");
+    }
+    if (isBuiltinRole(role)) {
+        return Status(ErrorCodes::InvalidRoleModification,
+                      mongoutils::str::stream() << "Cannot remove restrictions from built-in role: "
+                                                << role.getFullName());
+    }
+    _directRestrictionsForRole[role] = std::move(restrictions);
+    return Status::OK();
+}
+
 Status RoleGraph::replaceRole(const RoleName& roleName,
                               const std::vector<RoleName>& roles,
-                              const PrivilegeVector& privileges) {
+                              const PrivilegeVector& privileges,
+                              SharedRestrictionDocument restrictions) {
     Status status = removeAllPrivilegesFromRole(roleName);
     if (status == ErrorCodes::RoleNotFound) {
         fassert(17168, createRole(roleName));
@@ -406,6 +395,7 @@ Status RoleGraph::replaceRole(const RoleName& roleName,
         return status;
     }
     fassert(17169, removeAllRolesFromRole(roleName));
+    fassert(40556, replaceRestrictionsForRole(roleName, restrictions));
     for (size_t i = 0; i < roles.size(); ++i) {
         const RoleName& grantedRole = roles[i];
         status = createRole(grantedRole);
@@ -457,8 +447,7 @@ Status RoleGraph::_recomputePrivilegeDataHelper(const RoleName& startingRole,
         if (!roleExists(currentRole)) {
             return Status(ErrorCodes::RoleNotFound,
                           mongoutils::str::stream() << "Role: " << currentRole.getFullName()
-                                                    << " does not exist",
-                          0);
+                                                    << " does not exist");
         }
 
         // Check for cycles
@@ -508,36 +497,40 @@ Status RoleGraph::_recomputePrivilegeDataHelper(const RoleName& startingRole,
         unordered_set<RoleName>& currentRoleIndirectRoles =
             _roleToIndirectSubordinates[currentRole];
         currentRoleIndirectRoles.clear();
-        for (std::vector<RoleName>::const_iterator it = currentRoleDirectRoles.begin();
-             it != currentRoleDirectRoles.end();
-             ++it) {
-            currentRoleIndirectRoles.insert(*it);
+        for (const auto& role : currentRoleDirectRoles) {
+            currentRoleIndirectRoles.insert(role);
+        }
+
+        // Also clear the "all restrictions" to rebuild in loop
+        auto& currentRoleAllRestrictions = _allRestrictionsForRole[currentRole];
+        currentRoleAllRestrictions.clear();
+        auto& currentRoleDirectRestrictions = _directRestrictionsForRole[currentRole];
+        if (currentRoleDirectRestrictions) {
+            currentRoleAllRestrictions.push_back(currentRoleDirectRestrictions);
         }
 
         // Recursively add children's privileges to current role's "all privileges" vector, and
         // children's roles to current roles's "indirect roles" vector.
-        for (std::vector<RoleName>::const_iterator roleIt = currentRoleDirectRoles.begin();
-             roleIt != currentRoleDirectRoles.end();
-             ++roleIt) {
+        for (const auto& childRole : currentRoleDirectRoles) {
             // At this point, we already know that the "all privilege" set for the child is
             // correct, so add those privileges to our "all privilege" set.
-            const RoleName& childRole = *roleIt;
 
-            const PrivilegeVector& childsPrivileges = _allPrivilegesForRole[childRole];
-            for (PrivilegeVector::const_iterator privIt = childsPrivileges.begin();
-                 privIt != childsPrivileges.end();
-                 ++privIt) {
-                Privilege::addPrivilegeToPrivilegeVector(&currentRoleAllPrivileges, *privIt);
+            for (const auto& priv : _allPrivilegesForRole[childRole]) {
+                Privilege::addPrivilegeToPrivilegeVector(&currentRoleAllPrivileges, priv);
             }
 
             // We also know that the "indirect roles" for the child is also correct, so we can
             // add those roles to our "indirect roles" set.
-            const unordered_set<RoleName>& childsRoles = _roleToIndirectSubordinates[childRole];
-            for (unordered_set<RoleName>::const_iterator childsRoleIt = childsRoles.begin();
-                 childsRoleIt != childsRoles.end();
-                 ++childsRoleIt) {
-                currentRoleIndirectRoles.insert(*childsRoleIt);
-            }
+            const auto& childAllRolesToIndirectSubordinates =
+                _roleToIndirectSubordinates[childRole];
+            currentRoleIndirectRoles.insert(childAllRolesToIndirectSubordinates.begin(),
+                                            childAllRolesToIndirectSubordinates.end());
+
+            // Similarly, "indirect restrictions" are ready to append
+            const auto& childAllRestrictionsForRole = _allRestrictionsForRole[childRole];
+            currentRoleAllRestrictions.insert(currentRoleAllRestrictions.end(),
+                                              childAllRestrictionsForRole.begin(),
+                                              childAllRestrictionsForRole.end());
         }
 
         visitedRoles.insert(currentRole);
