@@ -37,7 +37,9 @@
 #include "mongo/db/auth/authorization_manager.h"
 #include "mongo/db/auth/user_document_parser.h"
 #include "mongo/db/jsobj.h"
+#include "mongo/db/server_options.h"
 #include "mongo/unittest/unittest.h"
+#include "mongo/util/net/sock.h"
 
 #define ASSERT_NULL(EXPR) ASSERT_FALSE(EXPR)
 #define ASSERT_NON_NULL(EXPR) ASSERT_TRUE(EXPR)
@@ -229,6 +231,8 @@ public:
     V2UserDocumentParser v2parser;
 
     void setUp() {
+        serverGlobalParams.featureCompatibility.setVersion(
+            ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo36);
         user.reset(new User(UserName("spencer", "test")));
         adminUser.reset(new User(UserName("admin", "admin")));
     }
@@ -281,6 +285,27 @@ TEST_F(V2UserDocumentParsing, V2DocumentValidation) {
                                                        << BSON("MONGODB-CR"
                                                                << "a"))));
 
+    // authenticationRestricitons must be an array if it exists
+    ASSERT_NOT_OK(v2parser.checkValidUserDocument(BSON("user"
+                                                       << "spencer"
+                                                       << "db"
+                                                       << "test"
+                                                       << "authenticationRestrictions"
+                                                       << "bogus")));
+
+    // Empty authenticationRestrictions is OK
+    ASSERT_OK(v2parser.checkValidUserDocument(BSON("user"
+                                                   << "spencer"
+                                                   << "db"
+                                                   << "test"
+                                                   << "credentials"
+                                                   << BSON("MONGODB-CR"
+                                                           << "a")
+                                                   << "roles"
+                                                   << emptyArray
+                                                   << "authenticationRestrictions"
+                                                   << emptyArray)));
+
     // Empty roles arrays are OK
     ASSERT_OK(v2parser.checkValidUserDocument(BSON("user"
                                                    << "spencer"
@@ -370,6 +395,25 @@ TEST_F(V2UserDocumentParsing, V2DocumentValidation) {
                                                                          << "db"
                                                                          << "dbB")))));
 
+    // Optional authenticationRestrictions field OK
+    ASSERT_OK(v2parser.checkValidUserDocument(BSON("user"
+                                                   << "spencer"
+                                                   << "db"
+                                                   << "test"
+                                                   << "credentials"
+                                                   << BSON("MONGODB-CR"
+                                                           << "a")
+                                                   << "authenticationRestrictions"
+                                                   << BSON_ARRAY(BSON("clientSource"
+                                                                      << BSON_ARRAY("127.0.0.1/8")
+                                                                      << "serverAddress"
+                                                                      << BSON_ARRAY("127.0.0.1/8")))
+                                                   << "roles"
+                                                   << BSON_ARRAY(BSON("role"
+                                                                      << "roleA"
+                                                                      << "db"
+                                                                      << "dbA")))));
+
     // Optional extraData field OK
     ASSERT_OK(v2parser.checkValidUserDocument(BSON("user"
                                                    << "spencer"
@@ -525,5 +569,114 @@ TEST_F(V2UserDocumentParsing, V2RoleExtraction) {
     ASSERT_FALSE(roles.more());
 }
 
+TEST_F(V2UserDocumentParsing, V2AuthenticationRestrictionsExtraction) {
+    const auto emptyArray = BSONArrayBuilder().arr();
+    const auto emptyObj = BSONObjBuilder().obj();
+
+    // "authenticationRestrictions" field is optional
+    ASSERT_OK(v2parser.initializeAuthenticationRestrictionsFromUserDocument(BSON("user"
+                                                                                 << "spencer"),
+                                                                            user.get()));
+    ASSERT_OK(v2parser.initializeAuthenticationRestrictionsFromUserDocument(
+        BSON("user"
+             << "spencer"
+             << "authenticationRestrictions"
+             << emptyArray),
+        user.get()));
+
+    // authenticationRestrictions must have at least one of "clientSource"/"serverAdddress" fields
+    ASSERT_NOT_OK(v2parser.initializeAuthenticationRestrictionsFromUserDocument(
+        BSON("user"
+             << "spencer"
+             << "authenticationRestrictions"
+             << BSON_ARRAY(emptyObj)),
+        user.get()));
+
+    // authenticationRestrictions must not have unexpected elements
+    ASSERT_NOT_OK(v2parser.initializeAuthenticationRestrictionsFromUserDocument(
+        BSON("user"
+             << "spencer"
+             << "authenticationRestrictions"
+             << BSON_ARRAY(BSON("foo"
+                                << "bar"))),
+        user.get()));
+
+    // authenticationRestrictions may have only one of "clientSource"/"serverAddress" fields
+    ASSERT_OK(v2parser.initializeAuthenticationRestrictionsFromUserDocument(
+        BSON("user"
+             << "spencer"
+             << "authenticationRestrictions"
+             << BSON_ARRAY(BSON("clientSource" << BSON_ARRAY("::1")))),
+        user.get()));
+    ASSERT_OK(v2parser.initializeAuthenticationRestrictionsFromUserDocument(
+        BSON("user"
+             << "spencer"
+             << "authenticationRestrictions"
+             << BSON_ARRAY(BSON("serverAddress" << BSON_ARRAY("::1")))),
+        user.get()));
+
+    // authenticationRestrictions may have both "clientSource"/"serverAddress" fields
+    ASSERT_OK(v2parser.initializeAuthenticationRestrictionsFromUserDocument(
+        BSON("user"
+             << "spencer"
+             << "authenticationRestrictions"
+             << BSON_ARRAY(BSON("clientSource" << BSON_ARRAY("::1") << "serverAddress"
+                                               << BSON_ARRAY("::1")))),
+        user.get()));
+
+    // authenticationRestrictions addresses must be valid CIDR strings
+    ASSERT_NOT_OK(v2parser.initializeAuthenticationRestrictionsFromUserDocument(
+        BSON("user"
+             << "spencer"
+             << "authenticationRestrictions"
+             << BSON_ARRAY(BSON("clientSource" << BSON_ARRAY("1.2.3.4.5")))),
+        user.get()));
+    ASSERT_NOT_OK(v2parser.initializeAuthenticationRestrictionsFromUserDocument(
+        BSON("user"
+             << "spencer"
+             << "authenticationRestrictions"
+             << BSON_ARRAY(BSON("serverAddress" << BSON_ARRAY(":::1")))),
+        user.get()));
+    ASSERT_NOT_OK(v2parser.initializeAuthenticationRestrictionsFromUserDocument(
+        BSON("user"
+             << "spencer"
+             << "authenticationRestrictions"
+             << BSON_ARRAY(BSON("clientSource" << BSON_ARRAY("::1") << "serverAddress"
+                                               << BSON_ARRAY(":::1")))),
+        user.get()));
+}
+
+TEST_F(V2UserDocumentParsing, V2AuthenticationRestrictionsExtractionAndRetreival) {
+    serverGlobalParams.featureCompatibility.setVersion(
+        ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo36);
+    enableIPv6(true);
+    ASSERT_OK(v2parser.initializeAuthenticationRestrictionsFromUserDocument(
+        BSON("user"
+             << "spencer"
+             << "authenticationRestrictions"
+             << BSON_ARRAY(BSON("clientSource" << BSON_ARRAY("169.254.12.0/22") << "serverAddress"
+                                               << BSON_ARRAY("fe80::/10")))),
+        user.get()));
+
+    const auto& doc = user->getRestrictions();
+    const struct {
+        std::string client;
+        std::string server;
+        bool valid;
+    } tests[] = {
+        {"169.254.12.1", "fe80::1", true},
+        {"169.254.15.255", "fe80:0000:0000:0000:ffff:ffff:ffff:ffff", true},
+        {"169.254.12.1", "fec0::1", false},
+        {"169.254.16.0", "fe80::1", false},
+        {"169.254.16.0", "fec0::1", false},
+        {"127.0.0.1", "::1", false},
+    };
+    for (const auto& p : tests) {
+        const RestrictionEnvironment re(SockAddr(p.client, 1024, AF_UNSPEC),
+                                        SockAddr(p.server, 1025, AF_UNSPEC));
+        ASSERT_EQ(doc.validate(re).isOK(), p.valid);
+    }
+}
+
 }  // namespace
 }  // namespace mongo
