@@ -40,6 +40,7 @@
 #include "mongo/db/catalog/index_key_validate.h"
 #include "mongo/db/client.h"
 #include "mongo/db/commands.h"
+#include "mongo/db/concurrency/d_concurrency.h"
 #include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/index/index_descriptor.h"
@@ -65,7 +66,6 @@ namespace {
 
 const StringData kIndexesFieldName = "indexes"_sd;
 const StringData kCommandName = "createIndexes"_sd;
-const StringData kWriteConcern = "writeConcern"_sd;
 
 /**
  * Parses the index specifications from 'cmdObj', validates them, and returns equivalent index
@@ -73,6 +73,7 @@ const StringData kWriteConcern = "writeConcern"_sd;
  * malformed, then an error status is returned.
  */
 StatusWith<std::vector<BSONObj>> parseAndValidateIndexSpecs(
+    OperationContext* opCtx,
     const NamespaceString& ns,
     const BSONObj& cmdObj,
     const ServerGlobalParams::FeatureCompatibility& featureCompatibility) {
@@ -99,7 +100,7 @@ StatusWith<std::vector<BSONObj>> parseAndValidateIndexSpecs(
                 }
 
                 auto indexSpecStatus = index_key_validate::validateIndexSpec(
-                    indexesElem.Obj(), ns, featureCompatibility);
+                    opCtx, indexesElem.Obj(), ns, featureCompatibility);
                 if (!indexSpecStatus.isOK()) {
                     return indexSpecStatus.getStatus();
                 }
@@ -116,14 +117,19 @@ StatusWith<std::vector<BSONObj>> parseAndValidateIndexSpecs(
                             str::stream() << "The index name '_id_' is reserved for the _id index, "
                                              "which must have key pattern {_id: 1}, found "
                                           << indexSpec[IndexDescriptor::kKeyPatternFieldName]};
+                } else if (indexSpec[IndexDescriptor::kIndexNameFieldName].String() == "*"_sd) {
+                    // An index named '*' cannot be dropped on its own, because a dropIndex oplog
+                    // entry with a '*' as an index name means "drop all indexes in this
+                    // collection".  We disallow creation of such indexes to avoid this conflict.
+                    return {ErrorCodes::BadValue, "The index name '*' is not valid."};
                 }
 
                 indexSpecs.push_back(std::move(indexSpec));
             }
 
             hasIndexesField = true;
-        } else if (kCommandName == cmdElemFieldName || kWriteConcern == cmdElemFieldName) {
-            // Both the command name and writeConcern are valid top-level fields.
+        } else if (kCommandName == cmdElemFieldName ||
+                   Command::isGenericArgument(cmdElemFieldName)) {
             continue;
         } else {
             return {ErrorCodes::BadValue,
@@ -156,12 +162,12 @@ StatusWith<std::vector<BSONObj>> parseAndValidateIndexSpecs(
  * form stored in the IndexCatalog should any of these indexes already exist.
  */
 StatusWith<std::vector<BSONObj>> resolveCollectionDefaultProperties(
-    OperationContext* txn, const Collection* collection, std::vector<BSONObj> indexSpecs) {
+    OperationContext* opCtx, const Collection* collection, std::vector<BSONObj> indexSpecs) {
     std::vector<BSONObj> indexSpecsWithDefaults = std::move(indexSpecs);
 
     for (size_t i = 0, numIndexSpecs = indexSpecsWithDefaults.size(); i < numIndexSpecs; ++i) {
         auto indexSpecStatus = index_key_validate::validateIndexSpecCollation(
-            txn, indexSpecsWithDefaults[i], collection->getDefaultCollator());
+            opCtx, indexSpecsWithDefaults[i], collection->getDefaultCollator());
         if (!indexSpecStatus.isOK()) {
             return indexSpecStatus.getStatus();
         }
@@ -171,7 +177,7 @@ StatusWith<std::vector<BSONObj>> resolveCollectionDefaultProperties(
                 indexSpec[IndexDescriptor::kKeyPatternFieldName].Obj())) {
             std::unique_ptr<CollatorInterface> indexCollator;
             if (auto collationElem = indexSpec[IndexDescriptor::kCollationFieldName]) {
-                auto collatorStatus = CollatorFactoryInterface::get(txn->getServiceContext())
+                auto collatorStatus = CollatorFactoryInterface::get(opCtx->getServiceContext())
                                           ->makeFromBSON(collationElem.Obj());
                 // validateIndexSpecCollation() should have checked that the index collation spec is
                 // valid.
@@ -203,9 +209,9 @@ StatusWith<std::vector<BSONObj>> resolveCollectionDefaultProperties(
 /**
  * { createIndexes : "bar", indexes : [ { ns : "test.bar", key : { x : 1 }, name: "x_1" } ] }
  */
-class CmdCreateIndex : public Command {
+class CmdCreateIndex : public ErrmsgCommandDeprecated {
 public:
-    CmdCreateIndex() : Command(kCommandName) {}
+    CmdCreateIndex() : ErrmsgCommandDeprecated(kCommandName) {}
 
     virtual bool supportsWriteConcern(const BSONObj& cmd) const override {
         return true;
@@ -225,20 +231,25 @@ public:
         return Status(ErrorCodes::Unauthorized, "Unauthorized");
     }
 
-    virtual bool run(OperationContext* txn,
-                     const string& dbname,
-                     BSONObj& cmdObj,
-                     int options,
-                     string& errmsg,
-                     BSONObjBuilder& result) {
-        const NamespaceString ns(parseNs(dbname, cmdObj));
+    virtual bool errmsgRun(OperationContext* opCtx,
+                           const string& dbname,
+                           const BSONObj& cmdObj,
+                           string& errmsg,
+                           BSONObjBuilder& result) {
+        const NamespaceString ns(parseNsCollectionRequired(dbname, cmdObj));
 
         Status status = userAllowedWriteNS(ns);
         if (!status.isOK())
             return appendCommandStatus(result, status);
 
+        // Disallow users from creating new indexes on config.transactions since the sessions
+        // code was optimized to not update indexes.
+        uassert(ErrorCodes::IllegalOperation,
+                str::stream() << "not allowed to create index on " << ns.ns(),
+                ns != NamespaceString::kSessionTransactionsTableNamespace);
+
         auto specsWithStatus =
-            parseAndValidateIndexSpecs(ns, cmdObj, serverGlobalParams.featureCompatibility);
+            parseAndValidateIndexSpecs(opCtx, ns, cmdObj, serverGlobalParams.featureCompatibility);
         if (!specsWithStatus.isOK()) {
             return appendCommandStatus(result, specsWithStatus.getStatus());
         }
@@ -246,56 +257,48 @@ public:
 
         // now we know we have to create index(es)
         // Note: createIndexes command does not currently respect shard versioning.
-        ScopedTransaction transaction(txn, MODE_IX);
-        Lock::DBLock dbLock(txn->lockState(), ns.db(), MODE_X);
-        if (!repl::getGlobalReplicationCoordinator()->canAcceptWritesFor(ns)) {
+        Lock::DBLock dbLock(opCtx, ns.db(), MODE_X);
+        if (!repl::getGlobalReplicationCoordinator()->canAcceptWritesFor(opCtx, ns)) {
             return appendCommandStatus(
                 result,
                 Status(ErrorCodes::NotMaster,
                        str::stream() << "Not primary while creating indexes in " << ns.ns()));
         }
 
-        Database* db = dbHolder().get(txn, ns.db());
+        Database* db = dbHolder().get(opCtx, ns.db());
         if (!db) {
-            db = dbHolder().openDb(txn, ns.db());
+            db = dbHolder().openDb(opCtx, ns.db());
         }
 
-        Collection* collection = db->getCollection(ns.ns());
+        Collection* collection = db->getCollection(opCtx, ns);
         if (collection) {
             result.appendBool("createdCollectionAutomatically", false);
         } else {
-            if (db->getViewCatalog()->lookup(txn, ns.ns())) {
+            if (db->getViewCatalog()->lookup(opCtx, ns.ns())) {
                 errmsg = "Cannot create indexes on a view";
                 return appendCommandStatus(result, {ErrorCodes::CommandNotSupportedOnView, errmsg});
             }
 
-            MONGO_WRITE_CONFLICT_RETRY_LOOP_BEGIN {
-                WriteUnitOfWork wunit(txn);
-                collection = db->createCollection(txn, ns.ns(), CollectionOptions());
+            writeConflictRetry(opCtx, kCommandName, ns.ns(), [&] {
+                WriteUnitOfWork wunit(opCtx);
+                collection = db->createCollection(opCtx, ns.ns(), CollectionOptions());
                 invariant(collection);
                 wunit.commit();
-            }
-            MONGO_WRITE_CONFLICT_RETRY_LOOP_END(txn, kCommandName, ns.ns());
+            });
             result.appendBool("createdCollectionAutomatically", true);
         }
 
         auto indexSpecsWithDefaults =
-            resolveCollectionDefaultProperties(txn, collection, std::move(specs));
+            resolveCollectionDefaultProperties(opCtx, collection, std::move(specs));
         if (!indexSpecsWithDefaults.isOK()) {
             return appendCommandStatus(result, indexSpecsWithDefaults.getStatus());
         }
         specs = std::move(indexSpecsWithDefaults.getValue());
 
-        const int numIndexesBefore = collection->getIndexCatalog()->numIndexesTotal(txn);
+        const int numIndexesBefore = collection->getIndexCatalog()->numIndexesTotal(opCtx);
         result.append("numIndexesBefore", numIndexesBefore);
 
-        auto client = txn->getClient();
-        ScopeGuard lastOpSetterGuard =
-            MakeObjGuard(repl::ReplClientInfo::forClient(client),
-                         &repl::ReplClientInfo::setLastOpToSystemLastOpTime,
-                         txn);
-
-        MultiIndexBlock indexer(txn, collection);
+        MultiIndexBlock indexer(opCtx, collection);
         indexer.allowBackgroundBuilding();
         indexer.allowInterruption();
 
@@ -315,7 +318,7 @@ public:
         for (size_t i = 0; i < specs.size(); i++) {
             const BSONObj& spec = specs[i];
             if (spec["unique"].trueValue()) {
-                status = checkUniqueIndexConstraints(txn, ns.ns(), spec["key"].Obj());
+                status = checkUniqueIndexConstraints(opCtx, ns.ns(), spec["key"].Obj());
 
                 if (!status.isOK()) {
                     return appendCommandStatus(result, status);
@@ -323,18 +326,17 @@ public:
             }
         }
 
-        std::vector<BSONObj> indexInfoObjs;
-        MONGO_WRITE_CONFLICT_RETRY_LOOP_BEGIN {
-            indexInfoObjs = uassertStatusOK(indexer.init(specs));
-        }
-        MONGO_WRITE_CONFLICT_RETRY_LOOP_END(txn, kCommandName, ns.ns());
+        std::vector<BSONObj> indexInfoObjs =
+            writeConflictRetry(opCtx, kCommandName, ns.ns(), [&indexer, &specs] {
+                return uassertStatusOK(indexer.init(specs));
+            });
 
         // If we're a background index, replace exclusive db lock with an intent lock, so that
         // other readers and writers can proceed during this phase.
         if (indexer.getBuildInBackground()) {
-            txn->recoveryUnit()->abandonSnapshot();
+            opCtx->recoveryUnit()->abandonSnapshot();
             dbLock.relockWithMode(MODE_IX);
-            if (!repl::getGlobalReplicationCoordinator()->canAcceptWritesFor(ns)) {
+            if (!repl::getGlobalReplicationCoordinator()->canAcceptWritesFor(opCtx, ns)) {
                 return appendCommandStatus(
                     result,
                     Status(ErrorCodes::NotMaster,
@@ -344,19 +346,19 @@ public:
         }
 
         try {
-            Lock::CollectionLock colLock(txn->lockState(), ns.ns(), MODE_IX);
+            Lock::CollectionLock colLock(opCtx->lockState(), ns.ns(), MODE_IX);
             uassertStatusOK(indexer.insertAllDocumentsInCollection());
         } catch (const DBException& e) {
-            invariant(e.getCode() != ErrorCodes::WriteConflict);
+            invariant(e.code() != ErrorCodes::WriteConflict);
             // Must have exclusive DB lock before we clean up the index build via the
             // destructor of 'indexer'.
             if (indexer.getBuildInBackground()) {
                 try {
                     // This function cannot throw today, but we will preemptively prepare for
                     // that day, to avoid data corruption due to lack of index cleanup.
-                    txn->recoveryUnit()->abandonSnapshot();
+                    opCtx->recoveryUnit()->abandonSnapshot();
                     dbLock.relockWithMode(MODE_X);
-                    if (!repl::getGlobalReplicationCoordinator()->canAcceptWritesFor(ns)) {
+                    if (!repl::getGlobalReplicationCoordinator()->canAcceptWritesFor(opCtx, ns)) {
                         return appendCommandStatus(
                             result,
                             Status(ErrorCodes::NotMaster,
@@ -374,48 +376,42 @@ public:
         }
         // Need to return db lock back to exclusive, to complete the index build.
         if (indexer.getBuildInBackground()) {
-            txn->recoveryUnit()->abandonSnapshot();
+            opCtx->recoveryUnit()->abandonSnapshot();
             dbLock.relockWithMode(MODE_X);
             uassert(ErrorCodes::NotMaster,
                     str::stream() << "Not primary while completing index build in " << dbname,
-                    repl::getGlobalReplicationCoordinator()->canAcceptWritesFor(ns));
+                    repl::getGlobalReplicationCoordinator()->canAcceptWritesFor(opCtx, ns));
 
-            Database* db = dbHolder().get(txn, ns.db());
+            Database* db = dbHolder().get(opCtx, ns.db());
             uassert(28551, "database dropped during index build", db);
-            uassert(28552, "collection dropped during index build", db->getCollection(ns.ns()));
+            uassert(28552, "collection dropped during index build", db->getCollection(opCtx, ns));
         }
 
-        MONGO_WRITE_CONFLICT_RETRY_LOOP_BEGIN {
-            WriteUnitOfWork wunit(txn);
+        writeConflictRetry(opCtx, kCommandName, ns.ns(), [&] {
+            WriteUnitOfWork wunit(opCtx);
 
             indexer.commit();
 
             for (auto&& infoObj : indexInfoObjs) {
-                std::string systemIndexes = ns.getSystemIndexesCollection();
-                auto opObserver = getGlobalServiceContext()->getOpObserver();
-                if (opObserver) {
-                    opObserver->onCreateIndex(txn, systemIndexes, infoObj);
-                }
+                getGlobalServiceContext()->getOpObserver()->onCreateIndex(
+                    opCtx, ns, collection->uuid(), infoObj, false);
             }
 
             wunit.commit();
-        }
-        MONGO_WRITE_CONFLICT_RETRY_LOOP_END(txn, kCommandName, ns.ns());
+        });
 
-        result.append("numIndexesAfter", collection->getIndexCatalog()->numIndexesTotal(txn));
-
-        lastOpSetterGuard.Dismiss();
+        result.append("numIndexesAfter", collection->getIndexCatalog()->numIndexesTotal(opCtx));
 
         return true;
     }
 
 private:
-    static Status checkUniqueIndexConstraints(OperationContext* txn,
+    static Status checkUniqueIndexConstraints(OperationContext* opCtx,
                                               StringData ns,
                                               const BSONObj& newIdxKey) {
-        invariant(txn->lockState()->isCollectionLockedForMode(ns, MODE_X));
+        invariant(opCtx->lockState()->isCollectionLockedForMode(ns, MODE_X));
 
-        auto metadata(CollectionShardingState::get(txn, ns.toString())->getMetadata());
+        auto metadata(CollectionShardingState::get(opCtx, ns.toString())->getMetadata());
         if (metadata) {
             ShardKeyPattern shardKeyPattern(metadata->getKeyPattern());
             if (!shardKeyPattern.isUniqueIndexCompatible(newIdxKey)) {
