@@ -64,9 +64,9 @@ using std::stringstream;
  *    data: <Object> // optional arbitrary object to store.
  * }
  */
-class FaultInjectCmd : public Command {
+class FaultInjectCmd : public ErrmsgCommandDeprecated {
 public:
-    FaultInjectCmd() : Command("configureFailPoint") {}
+    FaultInjectCmd() : ErrmsgCommandDeprecated("configureFailPoint") {}
 
     virtual bool slaveOk() const {
         return true;
@@ -90,17 +90,16 @@ public:
         h << "modifies the settings of a fail point";
     }
 
-    bool run(OperationContext* txn,
-             const string& dbname,
-             BSONObj& cmdObj,
-             int,
-             string& errmsg,
-             BSONObjBuilder& result) {
+    bool errmsgRun(OperationContext* opCtx,
+                   const string& dbname,
+                   const BSONObj& cmdObj,
+                   string& errmsg,
+                   BSONObjBuilder& result) {
         const string failPointName(cmdObj.firstElement().str());
         FailPointRegistry* registry = getGlobalFailPointRegistry();
         FailPoint* failPoint = registry->getFailPoint(failPointName);
 
-        if (failPoint == NULL) {
+        if (failPoint == nullptr) {
             errmsg = failPointName + " not found";
             return false;
         }
