@@ -30,10 +30,7 @@
 
 #include "mongo/platform/basic.h"
 
-#include "mongo/db/commands/find_and_modify.h"
-
 #include <boost/optional.hpp>
-#include <memory>
 
 #include "mongo/base/status_with.h"
 #include "mongo/bson/bsonobj.h"
@@ -42,6 +39,7 @@
 #include "mongo/db/catalog/document_validation.h"
 #include "mongo/db/client.h"
 #include "mongo/db/commands.h"
+#include "mongo/db/commands/find_and_modify_common.h"
 #include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/exec/update.h"
@@ -50,11 +48,13 @@
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/ops/delete_request.h"
+#include "mongo/db/ops/find_and_modify_result.h"
 #include "mongo/db/ops/insert.h"
 #include "mongo/db/ops/parsed_delete.h"
 #include "mongo/db/ops/parsed_update.h"
 #include "mongo/db/ops/update_lifecycle_impl.h"
 #include "mongo/db/ops/update_request.h"
+#include "mongo/db/ops/write_ops_retryability.h"
 #include "mongo/db/query/explain.h"
 #include "mongo/db/query/find_and_modify_request.h"
 #include "mongo/db/query/get_executor.h"
@@ -62,14 +62,15 @@
 #include "mongo/db/query/plan_summary_stats.h"
 #include "mongo/db/repl/repl_client_info.h"
 #include "mongo/db/repl/replication_coordinator_global.h"
+#include "mongo/db/retryable_writes_stats.h"
 #include "mongo/db/s/collection_sharding_state.h"
+#include "mongo/db/session_catalog.h"
 #include "mongo/db/stats/top.h"
 #include "mongo/db/write_concern.h"
 #include "mongo/util/log.h"
 #include "mongo/util/scopeguard.h"
 
 namespace mongo {
-
 namespace {
 
 const UpdateStats* getUpdateStats(const PlanExecutor* exec) {
@@ -105,7 +106,7 @@ const DeleteStats* getDeleteStats(const PlanExecutor* exec) {
  *
  * If the operation failed, then an error Status is returned.
  */
-StatusWith<boost::optional<BSONObj>> advanceExecutor(OperationContext* txn,
+StatusWith<boost::optional<BSONObj>> advanceExecutor(OperationContext* opCtx,
                                                      PlanExecutor* exec,
                                                      bool isRemove) {
     BSONObj value;
@@ -144,6 +145,7 @@ void makeUpdateRequest(const FindAndModifyRequest& args,
     requestOut->setUpdates(args.getUpdateObj());
     requestOut->setSort(args.getSort());
     requestOut->setCollation(args.getCollation());
+    requestOut->setArrayFilters(args.getArrayFilters());
     requestOut->setUpsert(args.isUpsert());
     requestOut->setReturnDocs(args.shouldReturnNew() ? UpdateRequest::RETURN_NEW
                                                      : UpdateRequest::RETURN_OLD);
@@ -164,35 +166,27 @@ void makeDeleteRequest(const FindAndModifyRequest& args, bool explain, DeleteReq
     requestOut->setExplain(explain);
 }
 
-void appendCommandResponse(PlanExecutor* exec,
+void appendCommandResponse(const PlanExecutor* exec,
                            bool isRemove,
                            const boost::optional<BSONObj>& value,
-                           BSONObjBuilder& result) {
-    BSONObjBuilder lastErrorObjBuilder(result.subobjStart("lastErrorObject"));
-
+                           BSONObjBuilder* result) {
     if (isRemove) {
-        lastErrorObjBuilder.appendNumber("n", getDeleteStats(exec)->docsDeleted);
+        find_and_modify::serializeRemove(getDeleteStats(exec)->docsDeleted, value, result);
     } else {
-        const UpdateStats* updateStats = getUpdateStats(exec);
-        lastErrorObjBuilder.appendBool("updatedExisting", updateStats->nMatched > 0);
-        lastErrorObjBuilder.appendNumber("n", updateStats->inserted ? 1 : updateStats->nMatched);
-        // Note we have to use the objInserted from the stats here, rather than 'value'
-        // because the _id field could have been excluded by a projection.
-        if (!updateStats->objInserted.isEmpty()) {
-            lastErrorObjBuilder.appendAs(updateStats->objInserted["_id"], kUpsertedFieldName);
-        }
-    }
-    lastErrorObjBuilder.done();
+        const auto updateStats = getUpdateStats(exec);
 
-    if (value) {
-        result.append("value", *value);
-    } else {
-        result.appendNull("value");
+        // Note we have to use the objInserted from the stats here, rather than 'value' because the
+        // _id field could have been excluded by a projection.
+        find_and_modify::serializeUpsert(updateStats->inserted ? 1 : updateStats->nMatched,
+                                         value,
+                                         updateStats->nMatched > 0,
+                                         updateStats->objInserted,
+                                         result);
     }
 }
 
-Status checkCanAcceptWritesForDatabase(const NamespaceString& nsString) {
-    if (!repl::getGlobalReplicationCoordinator()->canAcceptWritesFor(nsString)) {
+Status checkCanAcceptWritesForDatabase(OperationContext* opCtx, const NamespaceString& nsString) {
+    if (!repl::getGlobalReplicationCoordinator()->canAcceptWritesFor(opCtx, nsString)) {
         return Status(ErrorCodes::NotMaster,
                       str::stream()
                           << "Not primary while running findAndModify command on collection "
@@ -201,25 +195,22 @@ Status checkCanAcceptWritesForDatabase(const NamespaceString& nsString) {
     return Status::OK();
 }
 
-void recordStatsForTopCommand(OperationContext* txn) {
-    auto curOp = CurOp::get(txn);
-    const int writeLocked = 1;
-
-    Top::get(txn->getClient()->getServiceContext())
-        .record(txn,
+void recordStatsForTopCommand(OperationContext* opCtx) {
+    auto curOp = CurOp::get(opCtx);
+    Top::get(opCtx->getClient()->getServiceContext())
+        .record(opCtx,
                 curOp->getNS(),
                 curOp->getLogicalOp(),
-                writeLocked,
-                curOp->elapsedMicros(),
+                Top::LockType::WriteLocked,
+                durationCount<Microseconds>(curOp->elapsedTimeExcludingPauses()),
                 curOp->isCommand(),
                 curOp->getReadWriteType());
 }
 
-}  // namespace
-
-/* Find and Modify an object returning either the old (default) or new value*/
-class CmdFindAndModify : public Command {
+class CmdFindAndModify : public BasicCommand {
 public:
+    CmdFindAndModify() : BasicCommand("findAndModify", "findandmodify") {}
+
     void help(std::stringstream& help) const override {
         help << "{ findAndModify: \"collection\", query: {processed:false}, update: {$set: "
                 "{processed:true}}, new: true}\n"
@@ -229,13 +220,11 @@ public:
                 "Output is in the \"value\" field\n";
     }
 
-    CmdFindAndModify() : Command("findAndModify", false, "findandmodify") {}
-
     bool slaveOk() const override {
         return false;
     }
 
-    virtual bool supportsWriteConcern(const BSONObj& cmd) const override {
+    bool supportsWriteConcern(const BSONObj& cmd) const override {
         return true;
     }
 
@@ -245,15 +234,14 @@ public:
         find_and_modify::addPrivilegesRequiredForFindAndModify(this, dbname, cmdObj, out);
     }
 
-    ReadWriteType getReadWriteType() const {
+    ReadWriteType getReadWriteType() const override {
         return ReadWriteType::kWrite;
     }
 
-    Status explain(OperationContext* txn,
+    Status explain(OperationContext* opCtx,
                    const std::string& dbName,
                    const BSONObj& cmdObj,
-                   ExplainCommon::Verbosity verbosity,
-                   const rpc::ServerSelectionMetadata&,
+                   ExplainOptions::Verbosity verbosity,
                    BSONObjBuilder* out) const override {
         const NamespaceString fullNs = parseNsCollectionRequired(dbName, cmdObj);
         Status allowedWriteStatus = userAllowedWriteNS(fullNs.ns());
@@ -269,14 +257,14 @@ public:
 
         const FindAndModifyRequest& args = parseStatus.getValue();
         const NamespaceString& nsString = args.getNamespaceString();
-        OpDebug* opDebug = &CurOp::get(txn)->debug();
+        OpDebug* opDebug = &CurOp::get(opCtx)->debug();
 
         if (args.isRemove()) {
             DeleteRequest request(nsString);
             const bool isExplain = true;
             makeDeleteRequest(args, isExplain, &request);
 
-            ParsedDelete parsedDelete(txn, &request);
+            ParsedDelete parsedDelete(opCtx, &request);
             Status parsedDeleteStatus = parsedDelete.parseRequest();
             if (!parsedDeleteStatus.isOK()) {
                 return parsedDeleteStatus;
@@ -284,22 +272,22 @@ public:
 
             // Explain calls of the findAndModify command are read-only, but we take write
             // locks so that the timing information is more accurate.
-            AutoGetCollection autoColl(txn, nsString, MODE_IX);
+            AutoGetCollection autoColl(opCtx, nsString, MODE_IX);
             if (!autoColl.getDb()) {
                 return {ErrorCodes::NamespaceNotFound,
                         str::stream() << "database " << dbName << " does not exist."};
             }
 
-            auto css = CollectionShardingState::get(txn, nsString);
-            css->checkShardVersionOrThrow(txn);
+            auto css = CollectionShardingState::get(opCtx, nsString);
+            css->checkShardVersionOrThrow(opCtx);
 
             Collection* const collection = autoColl.getCollection();
             auto statusWithPlanExecutor =
-                getExecutorDelete(txn, opDebug, collection, &parsedDelete);
+                getExecutorDelete(opCtx, opDebug, collection, &parsedDelete);
             if (!statusWithPlanExecutor.isOK()) {
                 return statusWithPlanExecutor.getStatus();
             }
-            const std::unique_ptr<PlanExecutor> exec = std::move(statusWithPlanExecutor.getValue());
+            const auto exec = std::move(statusWithPlanExecutor.getValue());
             Explain::explainStages(exec.get(), collection, verbosity, out);
         } else {
             UpdateRequest request(nsString);
@@ -307,7 +295,7 @@ public:
             const bool isExplain = true;
             makeUpdateRequest(args, isExplain, &updateLifecycle, &request);
 
-            ParsedUpdate parsedUpdate(txn, &request);
+            ParsedUpdate parsedUpdate(opCtx, &request);
             Status parsedUpdateStatus = parsedUpdate.parseRequest();
             if (!parsedUpdateStatus.isOK()) {
                 return parsedUpdateStatus;
@@ -315,36 +303,34 @@ public:
 
             // Explain calls of the findAndModify command are read-only, but we take write
             // locks so that the timing information is more accurate.
-            AutoGetCollection autoColl(txn, nsString, MODE_IX);
+            AutoGetCollection autoColl(opCtx, nsString, MODE_IX);
             if (!autoColl.getDb()) {
                 return {ErrorCodes::NamespaceNotFound,
                         str::stream() << "database " << dbName << " does not exist."};
             }
 
-            auto css = CollectionShardingState::get(txn, nsString);
-            css->checkShardVersionOrThrow(txn);
+            auto css = CollectionShardingState::get(opCtx, nsString);
+            css->checkShardVersionOrThrow(opCtx);
 
-            Collection* collection = autoColl.getCollection();
+            Collection* const collection = autoColl.getCollection();
             auto statusWithPlanExecutor =
-                getExecutorUpdate(txn, opDebug, collection, &parsedUpdate);
+                getExecutorUpdate(opCtx, opDebug, collection, &parsedUpdate);
             if (!statusWithPlanExecutor.isOK()) {
                 return statusWithPlanExecutor.getStatus();
             }
-            const std::unique_ptr<PlanExecutor> exec = std::move(statusWithPlanExecutor.getValue());
+            const auto exec = std::move(statusWithPlanExecutor.getValue());
             Explain::explainStages(exec.get(), collection, verbosity, out);
         }
 
         return Status::OK();
     }
 
-    bool run(OperationContext* txn,
+    bool run(OperationContext* opCtx,
              const std::string& dbName,
-             BSONObj& cmdObj,
-             int options,
-             std::string& errmsg,
+             const BSONObj& cmdObj,
              BSONObjBuilder& result) override {
         // findAndModify command is not replicated directly.
-        invariant(txn->writesAreReplicated());
+        invariant(opCtx->writesAreReplicated());
         const NamespaceString fullNs = parseNsCollectionRequired(dbName, cmdObj);
         Status allowedWriteStatus = userAllowedWriteNS(fullNs.ns());
         if (!allowedWriteStatus.isOK()) {
@@ -362,79 +348,87 @@ public:
 
         boost::optional<DisableDocumentValidation> maybeDisableValidation;
         if (shouldBypassDocumentValidationForCommand(cmdObj))
-            maybeDisableValidation.emplace(txn);
+            maybeDisableValidation.emplace(opCtx);
 
-        auto client = txn->getClient();
-        auto lastOpAtOperationStart = repl::ReplClientInfo::forClient(client).getLastOp();
-        ScopeGuard lastOpSetterGuard =
-            MakeObjGuard(repl::ReplClientInfo::forClient(client),
-                         &repl::ReplClientInfo::setLastOpToSystemLastOpTime,
-                         txn);
-
-        // If this is the local database, don't set last op.
-        if (dbName == "local") {
-            lastOpSetterGuard.Dismiss();
+        const auto stmtId = 0;
+        if (opCtx->getTxnNumber()) {
+            auto session = OperationContextSession::get(opCtx);
+            if (auto entry =
+                    session->checkStatementExecuted(opCtx, *opCtx->getTxnNumber(), stmtId)) {
+                RetryableWritesStats::get(opCtx)->incrementRetriedCommandsCount();
+                RetryableWritesStats::get(opCtx)->incrementRetriedStatementsCount();
+                parseOplogEntryForFindAndModify(opCtx, args, *entry, &result);
+                return true;
+            }
         }
 
-        auto curOp = CurOp::get(txn);
+        auto curOp = CurOp::get(opCtx);
         OpDebug* opDebug = &curOp->debug();
 
         // Although usually the PlanExecutor handles WCE internally, it will throw WCEs when it is
         // executing a findAndModify. This is done to ensure that we can always match, modify, and
         // return the document under concurrency, if a matching document exists.
-        MONGO_WRITE_CONFLICT_RETRY_LOOP_BEGIN {
+        return writeConflictRetry(opCtx, "findAndModify", nsString.ns(), [&] {
             if (args.isRemove()) {
                 DeleteRequest request(nsString);
                 const bool isExplain = false;
                 makeDeleteRequest(args, isExplain, &request);
 
-                ParsedDelete parsedDelete(txn, &request);
-                Status parsedDeleteStatus = parsedDelete.parseRequest();
-                if (!parsedDeleteStatus.isOK()) {
-                    return appendCommandStatus(result, parsedDeleteStatus);
+                if (opCtx->getTxnNumber()) {
+                    request.setStmtId(stmtId);
                 }
 
-                AutoGetOrCreateDb autoDb(txn, dbName, MODE_IX);
-                Lock::CollectionLock collLock(txn->lockState(), nsString.ns(), MODE_IX);
+                ParsedDelete parsedDelete(opCtx, &request);
+                Status parsedDeleteStatus = parsedDelete.parseRequest();
+                if (!parsedDeleteStatus.isOK()) {
+                    appendCommandStatus(result, parsedDeleteStatus);
+                    return false;
+                }
+
+                AutoGetOrCreateDb autoDb(opCtx, dbName, MODE_IX);
+                Lock::CollectionLock collLock(opCtx->lockState(), nsString.ns(), MODE_IX);
 
                 // Attach the namespace and database profiling level to the current op.
                 {
-                    stdx::lock_guard<Client> lk(*txn->getClient());
-                    CurOp::get(txn)->enter_inlock(nsString.ns().c_str(),
-                                                  autoDb.getDb()->getProfilingLevel());
+                    stdx::lock_guard<Client> lk(*opCtx->getClient());
+                    CurOp::get(opCtx)->enter_inlock(nsString.ns().c_str(),
+                                                    autoDb.getDb()->getProfilingLevel());
                 }
 
-                auto css = CollectionShardingState::get(txn, nsString);
-                css->checkShardVersionOrThrow(txn);
+                auto css = CollectionShardingState::get(opCtx, nsString);
+                css->checkShardVersionOrThrow(opCtx);
 
-                Status isPrimary = checkCanAcceptWritesForDatabase(nsString);
+                Status isPrimary = checkCanAcceptWritesForDatabase(opCtx, nsString);
                 if (!isPrimary.isOK()) {
-                    return appendCommandStatus(result, isPrimary);
+                    appendCommandStatus(result, isPrimary);
+                    return false;
                 }
 
-                Collection* const collection = autoDb.getDb()->getCollection(nsString.ns());
-                if (!collection && autoDb.getDb()->getViewCatalog()->lookup(txn, nsString.ns())) {
-                    return appendCommandStatus(result,
-                                               {ErrorCodes::CommandNotSupportedOnView,
-                                                "findAndModify not supported on a view"});
+                Collection* const collection = autoDb.getDb()->getCollection(opCtx, nsString);
+                if (!collection && autoDb.getDb()->getViewCatalog()->lookup(opCtx, nsString.ns())) {
+                    appendCommandStatus(result,
+                                        {ErrorCodes::CommandNotSupportedOnView,
+                                         "findAndModify not supported on a view"});
+                    return false;
                 }
                 auto statusWithPlanExecutor =
-                    getExecutorDelete(txn, opDebug, collection, &parsedDelete);
+                    getExecutorDelete(opCtx, opDebug, collection, &parsedDelete);
                 if (!statusWithPlanExecutor.isOK()) {
-                    return appendCommandStatus(result, statusWithPlanExecutor.getStatus());
+                    appendCommandStatus(result, statusWithPlanExecutor.getStatus());
+                    return false;
                 }
-                const std::unique_ptr<PlanExecutor> exec =
-                    std::move(statusWithPlanExecutor.getValue());
+                const auto exec = std::move(statusWithPlanExecutor.getValue());
 
                 {
-                    stdx::lock_guard<Client>(*txn->getClient());
-                    CurOp::get(txn)->setPlanSummary_inlock(Explain::getPlanSummary(exec.get()));
+                    stdx::lock_guard<Client> lk(*opCtx->getClient());
+                    CurOp::get(opCtx)->setPlanSummary_inlock(Explain::getPlanSummary(exec.get()));
                 }
 
                 StatusWith<boost::optional<BSONObj>> advanceStatus =
-                    advanceExecutor(txn, exec.get(), args.isRemove());
+                    advanceExecutor(opCtx, exec.get(), args.isRemove());
                 if (!advanceStatus.isOK()) {
-                    return appendCommandStatus(result, advanceStatus.getStatus());
+                    appendCommandStatus(result, advanceStatus.getStatus());
+                    return false;
                 }
                 // Nothing after advancing the plan executor should throw a WriteConflictException,
                 // so the following bookkeeping with execution stats won't end up being done
@@ -443,7 +437,7 @@ public:
                 PlanSummaryStats summaryStats;
                 Explain::getSummaryStats(*exec, &summaryStats);
                 if (collection) {
-                    collection->infoCache()->notifyOfQuery(txn, summaryStats.indexesUsed);
+                    collection->infoCache()->notifyOfQuery(opCtx, summaryStats.indexesUsed);
                 }
                 opDebug->setPlanSummaryMetrics(summaryStats);
 
@@ -455,45 +449,52 @@ public:
                     Explain::getWinningPlanStats(exec.get(), &execStatsBob);
                     curOp->debug().execStats = execStatsBob.obj();
                 }
-                recordStatsForTopCommand(txn);
+                recordStatsForTopCommand(opCtx);
 
-                boost::optional<BSONObj> value = advanceStatus.getValue();
-                appendCommandResponse(exec.get(), args.isRemove(), value, result);
+                appendCommandResponse(
+                    exec.get(), args.isRemove(), advanceStatus.getValue(), &result);
             } else {
                 UpdateRequest request(nsString);
                 UpdateLifecycleImpl updateLifecycle(nsString);
                 const bool isExplain = false;
                 makeUpdateRequest(args, isExplain, &updateLifecycle, &request);
 
-                ParsedUpdate parsedUpdate(txn, &request);
-                Status parsedUpdateStatus = parsedUpdate.parseRequest();
-                if (!parsedUpdateStatus.isOK()) {
-                    return appendCommandStatus(result, parsedUpdateStatus);
+                if (opCtx->getTxnNumber()) {
+                    request.setStmtId(stmtId);
                 }
 
-                AutoGetOrCreateDb autoDb(txn, dbName, MODE_IX);
-                Lock::CollectionLock collLock(txn->lockState(), nsString.ns(), MODE_IX);
+                ParsedUpdate parsedUpdate(opCtx, &request);
+                Status parsedUpdateStatus = parsedUpdate.parseRequest();
+                if (!parsedUpdateStatus.isOK()) {
+                    appendCommandStatus(result, parsedUpdateStatus);
+                    return false;
+                }
+
+                AutoGetOrCreateDb autoDb(opCtx, dbName, MODE_IX);
+                Lock::CollectionLock collLock(opCtx->lockState(), nsString.ns(), MODE_IX);
 
                 // Attach the namespace and database profiling level to the current op.
                 {
-                    stdx::lock_guard<Client> lk(*txn->getClient());
-                    CurOp::get(txn)->enter_inlock(nsString.ns().c_str(),
-                                                  autoDb.getDb()->getProfilingLevel());
+                    stdx::lock_guard<Client> lk(*opCtx->getClient());
+                    CurOp::get(opCtx)->enter_inlock(nsString.ns().c_str(),
+                                                    autoDb.getDb()->getProfilingLevel());
                 }
 
-                auto css = CollectionShardingState::get(txn, nsString);
-                css->checkShardVersionOrThrow(txn);
+                auto css = CollectionShardingState::get(opCtx, nsString);
+                css->checkShardVersionOrThrow(opCtx);
 
-                Status isPrimary = checkCanAcceptWritesForDatabase(nsString);
+                Status isPrimary = checkCanAcceptWritesForDatabase(opCtx, nsString);
                 if (!isPrimary.isOK()) {
-                    return appendCommandStatus(result, isPrimary);
+                    appendCommandStatus(result, isPrimary);
+                    return false;
                 }
 
-                Collection* collection = autoDb.getDb()->getCollection(nsString.ns());
-                if (!collection && autoDb.getDb()->getViewCatalog()->lookup(txn, nsString.ns())) {
-                    return appendCommandStatus(result,
-                                               {ErrorCodes::CommandNotSupportedOnView,
-                                                "findAndModify not supported on a view"});
+                Collection* collection = autoDb.getDb()->getCollection(opCtx, nsString.ns());
+                if (!collection && autoDb.getDb()->getViewCatalog()->lookup(opCtx, nsString.ns())) {
+                    appendCommandStatus(result,
+                                        {ErrorCodes::CommandNotSupportedOnView,
+                                         "findAndModify not supported on a view"});
+                    return false;
                 }
 
                 // Create the collection if it does not exist when performing an upsert
@@ -502,45 +503,48 @@ public:
                     // Release the collection lock and reacquire a lock on the database
                     // in exclusive mode in order to create the collection.
                     collLock.relockAsDatabaseExclusive(autoDb.lock());
-                    collection = autoDb.getDb()->getCollection(nsString.ns());
-                    Status isPrimaryAfterRelock = checkCanAcceptWritesForDatabase(nsString);
+                    collection = autoDb.getDb()->getCollection(opCtx, nsString);
+                    Status isPrimaryAfterRelock = checkCanAcceptWritesForDatabase(opCtx, nsString);
                     if (!isPrimaryAfterRelock.isOK()) {
-                        return appendCommandStatus(result, isPrimaryAfterRelock);
+                        appendCommandStatus(result, isPrimaryAfterRelock);
+                        return false;
                     }
 
                     if (collection) {
                         // Someone else beat us to creating the collection, do nothing.
                     } else {
-                        WriteUnitOfWork wuow(txn);
+                        WriteUnitOfWork wuow(opCtx);
                         Status createCollStatus =
-                            userCreateNS(txn, autoDb.getDb(), nsString.ns(), BSONObj());
+                            userCreateNS(opCtx, autoDb.getDb(), nsString.ns(), BSONObj());
                         if (!createCollStatus.isOK()) {
-                            return appendCommandStatus(result, createCollStatus);
+                            appendCommandStatus(result, createCollStatus);
+                            return false;
                         }
                         wuow.commit();
 
-                        collection = autoDb.getDb()->getCollection(nsString.ns());
+                        collection = autoDb.getDb()->getCollection(opCtx, nsString);
                         invariant(collection);
                     }
                 }
 
                 auto statusWithPlanExecutor =
-                    getExecutorUpdate(txn, opDebug, collection, &parsedUpdate);
+                    getExecutorUpdate(opCtx, opDebug, collection, &parsedUpdate);
                 if (!statusWithPlanExecutor.isOK()) {
-                    return appendCommandStatus(result, statusWithPlanExecutor.getStatus());
+                    appendCommandStatus(result, statusWithPlanExecutor.getStatus());
+                    return false;
                 }
-                const std::unique_ptr<PlanExecutor> exec =
-                    std::move(statusWithPlanExecutor.getValue());
+                const auto exec = std::move(statusWithPlanExecutor.getValue());
 
                 {
-                    stdx::lock_guard<Client>(*txn->getClient());
-                    CurOp::get(txn)->setPlanSummary_inlock(Explain::getPlanSummary(exec.get()));
+                    stdx::lock_guard<Client> lk(*opCtx->getClient());
+                    CurOp::get(opCtx)->setPlanSummary_inlock(Explain::getPlanSummary(exec.get()));
                 }
 
                 StatusWith<boost::optional<BSONObj>> advanceStatus =
-                    advanceExecutor(txn, exec.get(), args.isRemove());
+                    advanceExecutor(opCtx, exec.get(), args.isRemove());
                 if (!advanceStatus.isOK()) {
-                    return appendCommandStatus(result, advanceStatus.getStatus());
+                    appendCommandStatus(result, advanceStatus.getStatus());
+                    return false;
                 }
                 // Nothing after advancing the plan executor should throw a WriteConflictException,
                 // so the following bookkeeping with execution stats won't end up being done
@@ -549,7 +553,7 @@ public:
                 PlanSummaryStats summaryStats;
                 Explain::getSummaryStats(*exec, &summaryStats);
                 if (collection) {
-                    collection->infoCache()->notifyOfQuery(txn, summaryStats.indexesUsed);
+                    collection->infoCache()->notifyOfQuery(opCtx, summaryStats.indexesUsed);
                 }
                 UpdateStage::recordUpdateStatsInOpDebug(getUpdateStats(exec.get()), opDebug);
                 opDebug->setPlanSummaryMetrics(summaryStats);
@@ -559,24 +563,17 @@ public:
                     Explain::getWinningPlanStats(exec.get(), &execStatsBob);
                     curOp->debug().execStats = execStatsBob.obj();
                 }
-                recordStatsForTopCommand(txn);
+                recordStatsForTopCommand(opCtx);
 
-                boost::optional<BSONObj> value = advanceStatus.getValue();
-                appendCommandResponse(exec.get(), args.isRemove(), value, result);
+                appendCommandResponse(
+                    exec.get(), args.isRemove(), advanceStatus.getValue(), &result);
             }
-        }
-        MONGO_WRITE_CONFLICT_RETRY_LOOP_END(txn, "findAndModify", nsString.ns());
 
-        if (repl::ReplClientInfo::forClient(client).getLastOp() != lastOpAtOperationStart) {
-            // If this operation has already generated a new lastOp, don't bother setting it here.
-            // No-op updates will not generate a new lastOp, so we still need the guard to fire in
-            // that case.
-            lastOpSetterGuard.Dismiss();
-        }
-
-        return true;
+            return true;
+        });
     }
 
 } cmdFindAndModify;
 
+}  // namespace
 }  // namespace mongo
