@@ -33,6 +33,7 @@
 
 #include "mongo/base/disallow_copying.h"
 #include "mongo/db/commands.h"
+#include "mongo/db/cursor_id.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/server_options.h"
 #include "mongo/platform/atomic_word.h"
@@ -81,7 +82,6 @@ public:
     // Similarly, the return value will be dbGetMore for both OP_GET_MORE and getMore command.
     LogicalOp logicalOp{LogicalOp::opInvalid};  // only set this through setNetworkOp_inlock()
     bool iscommand{false};
-    BSONObj updateobj{};
 
     // detailed options
     long long cursorid{-1};
@@ -121,7 +121,7 @@ public:
     BSONObj execStats;  // Owned here.
 
     // error handling
-    ExceptionInfo exceptionInfo;
+    Status exceptionInfo = Status::OK();
 
     // response info
     long long executionTimeMicros{0};
@@ -163,24 +163,16 @@ public:
     explicit CurOp(OperationContext* opCtx);
     ~CurOp();
 
-    bool haveQuery() const {
-        return !_query.isEmpty();
+    bool haveOpDescription() const {
+        return !_opDescription.isEmpty();
     }
 
     /**
      * The BSONObj returned may not be owned by CurOp. Callers should call getOwned() if they plan
      * to reference beyond the lifetime of this CurOp instance.
      */
-    BSONObj query() const {
-        return _query;
-    }
-
-    /**
-     * The BSONObj returned may not be owned by CurOp. Callers should call getOwned() if they plan
-     * to reference beyond the lifetime of this CurOp instance.
-     */
-    BSONObj collation() const {
-        return _collation;
+    BSONObj opDescription() const {
+        return _opDescription;
     }
 
     /**
@@ -191,7 +183,7 @@ public:
         return _originatingCommand;
     }
 
-    void enter_inlock(const char* ns, int dbProfileLevel);
+    void enter_inlock(const char* ns, boost::optional<int> dbProfileLevel);
 
     /**
      * Sets the type of the current network operation.
@@ -232,18 +224,20 @@ public:
     }
 
     /**
-     * Returns true iff the elapsed time of this operation is such that it should be profiled.
-     * Uses total time if the operation is done, current elapsed time otherwise.
+     * Returns true if the elapsed time of this operation is such that it should be profiled or
+     * profile level is set to 2. Uses total time if the operation is done, current elapsed time
+     * otherwise. The argument shouldSample prevents slow diagnostic logging at profile 1
+     * when set to false.
      */
-    bool shouldDBProfile() {
-        if (_dbprofile <= 0)
-            return false;
-
+    bool shouldDBProfile(bool shouldSample = true) {
+        // Profile level 2 should override any sample rate or slowms settings.
         if (_dbprofile >= 2)
             return true;
 
-        long long opMicros = isDone() ? totalTimeMicros() : elapsedMicros();
-        return opMicros >= serverGlobalParams.slowMS * 1000LL;
+        if (!shouldSample || _dbprofile <= 0)
+            return false;
+
+        return elapsedTimeExcludingPauses() >= Milliseconds{serverGlobalParams.slowMS};
     }
 
     /**
@@ -297,37 +291,79 @@ public:
         return _end > 0;
     }
 
-    long long totalTimeMicros() {
-        massert(12601, "CurOp not marked done yet", _end);
-        return _end - startTime();
-    }
-
-    long long elapsedMicros() {
-        return curTimeMicros64() - startTime();
-    }
-
-    int elapsedSeconds() {
-        return static_cast<int>(elapsedMicros() / (1000 * 1000));
-    }
-
     /**
-     * 'query' must be either an owned BSONObj or guaranteed to outlive the OperationContext it is
-     * associated with.
+     * Stops the operation latency timer from "ticking". Time spent paused is not included in the
+     * latencies returned by elapsedTimeExcludingPauses().
+     *
+     * Illegal to call if either the CurOp has not been started, or the CurOp is already in a paused
+     * state.
      */
-    void setQuery_inlock(const BSONObj& query) {
-        _query = query;
+    void pauseTimer() {
+        invariant(isStarted());
+        invariant(_lastPauseTime == 0);
+        _lastPauseTime = curTimeMicros64();
     }
 
     /**
-     * 'collation' must be either an owned BSONObj or guaranteed to outlive the OperationContext it
-     * is associated with.
+     * Starts the operation latency timer "ticking" again. Illegal to call if the CurOp has not been
+     * started and then subsequently paused.
      */
-    void setCollation_inlock(const BSONObj& collation) {
-        _collation = collation;
+    void resumeTimer() {
+        invariant(isStarted());
+        invariant(_lastPauseTime > 0);
+        _totalPausedDuration +=
+            Microseconds{static_cast<long long>(curTimeMicros64()) - _lastPauseTime};
+        _lastPauseTime = 0;
     }
 
     /**
-     * Sets the original command object. Used only by the getMore command.
+     * If this op has been marked as done(), returns the wall clock duration between being marked as
+     * started with ensureStarted() and the call to done().
+     *
+     * Otherwise, returns the wall clock duration between the start time and now.
+     *
+     * If this op has not yet been started, returns 0.
+     */
+    Microseconds elapsedTimeTotal() {
+        if (!isStarted()) {
+            return Microseconds{0};
+        }
+
+        if (!_end) {
+            return Microseconds{static_cast<long long>(curTimeMicros64() - startTime())};
+        } else {
+            return Microseconds{static_cast<long long>(_end - startTime())};
+        }
+    }
+
+    /**
+     * Returns the total elapsed duration minus any time spent in a paused state. See
+     * elapsedTimeTotal() for the definition of the total duration and pause/resumeTimer() for
+     * details on pausing.
+     *
+     * If this op has not yet been started, returns 0.
+     *
+     * Illegal to call while the timer is paused.
+     */
+    Microseconds elapsedTimeExcludingPauses() {
+        invariant(!_lastPauseTime);
+        if (!isStarted()) {
+            return Microseconds{0};
+        }
+
+        return elapsedTimeTotal() - _totalPausedDuration;
+    }
+
+    /**
+     * 'opDescription' must be either an owned BSONObj or guaranteed to outlive the OperationContext
+     * it is associated with.
+     */
+    void setOpDescription_inlock(const BSONObj& opDescription) {
+        _opDescription = opDescription;
+    }
+
+    /**
+     * Sets the original command object.
      */
     void setOriginatingCommand_inlock(const BSONObj& commandObj) {
         _originatingCommand = commandObj.getOwned();
@@ -346,12 +382,14 @@ public:
     Command::ReadWriteType getReadWriteType() const;
 
     /**
-     * Appends information about this CurOp to "builder".
+     * Appends information about this CurOp to "builder". If "truncateOps" is true, appends a string
+     * summary of any objects which exceed the threshold size. If truncateOps is false, append the
+     * entire object.
      *
      * If called from a thread other than the one executing the operation associated with this
      * CurOp, it is necessary to lock the associated Client object before executing this method.
      */
-    void reportState(BSONObjBuilder* builder);
+    void reportState(BSONObjBuilder* builder, bool truncateOps = false);
 
     /**
      * Sets the message and the progress meter for this CurOp.
@@ -389,17 +427,6 @@ public:
         return _numYields;
     }
 
-    /**
-     * Access to _expectedLatencyMs is not synchronized, so it is illegal for threads other than the
-     * one executing the operation to call getExpectedLatencyMs() and setExpectedLatencyMs().
-     */
-    long long getExpectedLatencyMs() const {
-        return _expectedLatencyMs;
-    }
-    void setExpectedLatencyMs(long long latency) {
-        _expectedLatencyMs = latency;
-    }
-
     /**
      * this should be used very sparingly
      * generally the Context should set this up
@@ -429,9 +456,20 @@ private:
     CurOpStack* _stack;
     CurOp* _parent{nullptr};
     Command* _command{nullptr};
+
+    // The time at which this CurOp instance was marked as started.
     long long _start{0};
+
+    // The time at which this CurOp instance was marked as done.
     long long _end{0};
 
+    // The time at which this CurOp instance had its timer paused, or 0 if the timer is not
+    // currently paused.
+    long long _lastPauseTime{0};
+
+    // The cumulative duration for which the timer has been paused.
+    Microseconds _totalPausedDuration{0};
+
     // _networkOp represents the network-level op code: OP_QUERY, OP_GET_MORE, OP_COMMAND, etc.
     NetworkOp _networkOp{opInvalid};  // only set this through setNetworkOp_inlock() to keep synced
     // _logicalOp is the logical operation type, ie 'dbQuery' regardless of whether this is an
@@ -442,19 +480,27 @@ private:
     bool _isCommand{false};
     int _dbprofile{0};  // 0=off, 1=slow, 2=all
     std::string _ns;
-    BSONObj _query;
-    BSONObj _collation;
+    BSONObj _opDescription;
     BSONObj _originatingCommand;  // Used by getMore to display original command.
     OpDebug _debug;
     std::string _message;
     ProgressMeter _progressMeter;
     int _numYields{0};
 
-    // this is how much "extra" time a query might take
-    // a writebacklisten for example will block for 30s
-    // so this should be 30000 in that case
-    long long _expectedLatencyMs{0};
-
     std::string _planSummary;
 };
+
+/**
+ * Upconverts a legacy query object such that it matches the format of the find command.
+ */
+BSONObj upconvertQueryEntry(const BSONObj& query,
+                            const NamespaceString& nss,
+                            int ntoreturn,
+                            int ntoskip);
+
+/**
+ * Generates a getMore command object from the specified namespace, cursor ID and batchsize.
+ */
+BSONObj upconvertGetMoreEntry(const NamespaceString& nss, CursorId cursorId, int ntoreturn);
+
 }  // namespace mongo
