@@ -37,6 +37,9 @@
 
 #include "mongo/base/error_codes.h"
 #include "mongo/base/status.h"
+#include "mongo/db/catalog/collection_impl.h"
+#include "mongo/db/catalog/index_catalog.h"
+#include "mongo/db/catalog/index_consistency.h"
 #include "mongo/db/client.h"
 #include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/curop.h"
@@ -112,16 +115,16 @@ IndexAccessMethod::IndexAccessMethod(IndexCatalogEntry* btreeState, SortedDataIn
     verify(IndexDescriptor::isIndexVersionSupported(_descriptor->version()));
 }
 
-bool IndexAccessMethod::ignoreKeyTooLong(OperationContext* txn) {
+bool IndexAccessMethod::ignoreKeyTooLong(OperationContext* opCtx) {
     // Ignore this error if we cannot write to the collection or if the user requested it
     const auto shouldRelaxConstraints =
-        repl::ReplicationCoordinator::get(txn)->shouldRelaxIndexConstraints(
-            NamespaceString(_btreeState->ns()));
-    return shouldRelaxConstraints || !failIndexKeyTooLong;
+        repl::ReplicationCoordinator::get(opCtx)->shouldRelaxIndexConstraints(
+            opCtx, NamespaceString(_btreeState->ns()));
+    return shouldRelaxConstraints || !failIndexKeyTooLong.load();
 }
 
 // Find the keys for obj, put them in the tree pointing to loc
-Status IndexAccessMethod::insert(OperationContext* txn,
+Status IndexAccessMethod::insert(OperationContext* opCtx,
                                  const BSONObj& obj,
                                  const RecordId& loc,
                                  const InsertDeleteOptions& options,
@@ -135,24 +138,26 @@ Status IndexAccessMethod::insert(OperationContext* txn,
 
     Status ret = Status::OK();
     for (BSONObjSet::const_iterator i = keys.begin(); i != keys.end(); ++i) {
-        Status status = _newInterface->insert(txn, *i, loc, options.dupsAllowed);
+        Status status = _newInterface->insert(opCtx, *i, loc, options.dupsAllowed);
 
         // Everything's OK, carry on.
         if (status.isOK()) {
             ++*numInserted;
+            IndexKeyEntry indexEntry = IndexKeyEntry(*i, loc);
             continue;
         }
 
         // Error cases.
 
-        if (status.code() == ErrorCodes::KeyTooLong && ignoreKeyTooLong(txn)) {
+        if (status.code() == ErrorCodes::KeyTooLong && ignoreKeyTooLong(opCtx)) {
+            IndexKeyEntry indexEntry = IndexKeyEntry(*i, loc);
             continue;
         }
 
         if (status.code() == ErrorCodes::DuplicateKeyValue) {
             // A document might be indexed multiple times during a background index build
             // if it moves ahead of the collection scan cursor (e.g. via an update).
-            if (!_btreeState->isReady(txn)) {
+            if (!_btreeState->isReady(opCtx)) {
                 LOG(3) << "key " << *i << " already in index during background indexing (ok)";
                 continue;
             }
@@ -160,7 +165,7 @@ Status IndexAccessMethod::insert(OperationContext* txn,
 
         // Clean up after ourselves.
         for (BSONObjSet::const_iterator j = keys.begin(); j != i; ++j) {
-            removeOneKey(txn, *j, loc, options.dupsAllowed);
+            removeOneKey(opCtx, *j, loc, options.dupsAllowed);
             *numInserted = 0;
         }
 
@@ -168,18 +173,20 @@ Status IndexAccessMethod::insert(OperationContext* txn,
     }
 
     if (*numInserted > 1 || isMultikeyFromPaths(multikeyPaths)) {
-        _btreeState->setMultikey(txn, multikeyPaths);
+        _btreeState->setMultikey(opCtx, multikeyPaths);
     }
 
     return ret;
 }
 
-void IndexAccessMethod::removeOneKey(OperationContext* txn,
+void IndexAccessMethod::removeOneKey(OperationContext* opCtx,
                                      const BSONObj& key,
                                      const RecordId& loc,
                                      bool dupsAllowed) {
+
     try {
-        _newInterface->unindex(txn, key, loc, dupsAllowed);
+        _newInterface->unindex(opCtx, key, loc, dupsAllowed);
+        IndexKeyEntry indexEntry = IndexKeyEntry(key, loc);
     } catch (AssertionException& e) {
         log() << "Assertion failure: _unindex failed " << _descriptor->indexNamespace();
         log() << "Assertion failure: _unindex failed: " << redact(e) << "  key:" << key.toString()
@@ -188,18 +195,18 @@ void IndexAccessMethod::removeOneKey(OperationContext* txn,
     }
 }
 
-std::unique_ptr<SortedDataInterface::Cursor> IndexAccessMethod::newCursor(OperationContext* txn,
+std::unique_ptr<SortedDataInterface::Cursor> IndexAccessMethod::newCursor(OperationContext* opCtx,
                                                                           bool isForward) const {
-    return _newInterface->newCursor(txn, isForward);
+    return _newInterface->newCursor(opCtx, isForward);
 }
 
 std::unique_ptr<SortedDataInterface::Cursor> IndexAccessMethod::newRandomCursor(
-    OperationContext* txn) const {
-    return _newInterface->newRandomCursor(txn);
+    OperationContext* opCtx) const {
+    return _newInterface->newRandomCursor(opCtx);
 }
 
 // Remove the provided doc from the index.
-Status IndexAccessMethod::remove(OperationContext* txn,
+Status IndexAccessMethod::remove(OperationContext* opCtx,
                                  const BSONObj& obj,
                                  const RecordId& loc,
                                  const InsertDeleteOptions& options,
@@ -214,25 +221,25 @@ Status IndexAccessMethod::remove(OperationContext* txn,
     getKeys(obj, options.getKeysMode, &keys, multikeyPaths);
 
     for (BSONObjSet::const_iterator i = keys.begin(); i != keys.end(); ++i) {
-        removeOneKey(txn, *i, loc, options.dupsAllowed);
+        removeOneKey(opCtx, *i, loc, options.dupsAllowed);
         ++*numDeleted;
     }
 
     return Status::OK();
 }
 
-Status IndexAccessMethod::initializeAsEmpty(OperationContext* txn) {
-    return _newInterface->initAsEmpty(txn);
+Status IndexAccessMethod::initializeAsEmpty(OperationContext* opCtx) {
+    return _newInterface->initAsEmpty(opCtx);
 }
 
-Status IndexAccessMethod::touch(OperationContext* txn, const BSONObj& obj) {
+Status IndexAccessMethod::touch(OperationContext* opCtx, const BSONObj& obj) {
     BSONObjSet keys = SimpleBSONObjComparator::kInstance.makeBSONObjSet();
     // There's no need to compute the prefixes of the indexed fields that cause the index to be
     // multikey when paging a document's index entries into memory.
     MultikeyPaths* multikeyPaths = nullptr;
     getKeys(obj, GetKeysMode::kEnforceConstraints, &keys, multikeyPaths);
 
-    std::unique_ptr<SortedDataInterface::Cursor> cursor(_newInterface->newCursor(txn));
+    std::unique_ptr<SortedDataInterface::Cursor> cursor(_newInterface->newCursor(opCtx));
     for (BSONObjSet::const_iterator i = keys.begin(); i != keys.end(); ++i) {
         cursor->seekExact(*i);
     }
@@ -241,11 +248,11 @@ Status IndexAccessMethod::touch(OperationContext* txn, const BSONObj& obj) {
 }
 
 
-Status IndexAccessMethod::touch(OperationContext* txn) const {
-    return _newInterface->touch(txn);
+Status IndexAccessMethod::touch(OperationContext* opCtx) const {
+    return _newInterface->touch(opCtx);
 }
 
-RecordId IndexAccessMethod::findSingle(OperationContext* txn, const BSONObj& requestedKey) const {
+RecordId IndexAccessMethod::findSingle(OperationContext* opCtx, const BSONObj& requestedKey) const {
     // Generate the key for this index.
     BSONObj actualKey;
     if (_btreeState->getCollator()) {
@@ -259,7 +266,7 @@ RecordId IndexAccessMethod::findSingle(OperationContext* txn, const BSONObj& req
         actualKey = requestedKey;
     }
 
-    std::unique_ptr<SortedDataInterface::Cursor> cursor(_newInterface->newCursor(txn));
+    std::unique_ptr<SortedDataInterface::Cursor> cursor(_newInterface->newCursor(opCtx));
     const auto requestedInfo = kDebugBuild ? SortedDataInterface::Cursor::kKeyAndLoc
                                            : SortedDataInterface::Cursor::kWantLoc;
     if (auto kv = cursor->seekExact(actualKey, requestedInfo)) {
@@ -274,23 +281,22 @@ RecordId IndexAccessMethod::findSingle(OperationContext* txn, const BSONObj& req
     return RecordId();
 }
 
-Status IndexAccessMethod::validate(OperationContext* txn,
-                                   int64_t* numKeys,
-                                   ValidateResults* fullResults) {
+void IndexAccessMethod::validate(OperationContext* opCtx,
+                                 int64_t* numKeys,
+                                 ValidateResults* fullResults) {
     long long keys = 0;
-    _newInterface->fullValidate(txn, &keys, fullResults);
+    _newInterface->fullValidate(opCtx, &keys, fullResults);
     *numKeys = keys;
-    return Status::OK();
 }
 
-bool IndexAccessMethod::appendCustomStats(OperationContext* txn,
+bool IndexAccessMethod::appendCustomStats(OperationContext* opCtx,
                                           BSONObjBuilder* output,
                                           double scale) const {
-    return _newInterface->appendCustomStats(txn, output, scale);
+    return _newInterface->appendCustomStats(opCtx, output, scale);
 }
 
-long long IndexAccessMethod::getSpaceUsedBytes(OperationContext* txn) const {
-    return _newInterface->getSpaceUsedBytes(txn);
+long long IndexAccessMethod::getSpaceUsedBytes(OperationContext* opCtx) const {
+    return _newInterface->getSpaceUsedBytes(opCtx);
 }
 
 pair<vector<BSONObj>, vector<BSONObj>> IndexAccessMethod::setDifference(const BSONObjSet& left,
@@ -329,7 +335,7 @@ pair<vector<BSONObj>, vector<BSONObj>> IndexAccessMethod::setDifference(const BS
     return {std::move(onlyLeft), std::move(onlyRight)};
 }
 
-Status IndexAccessMethod::validateUpdate(OperationContext* txn,
+Status IndexAccessMethod::validateUpdate(OperationContext* opCtx,
                                          const BSONObj& from,
                                          const BSONObj& to,
                                          const RecordId& record,
@@ -358,7 +364,7 @@ Status IndexAccessMethod::validateUpdate(OperationContext* txn,
     return Status::OK();
 }
 
-Status IndexAccessMethod::update(OperationContext* txn,
+Status IndexAccessMethod::update(OperationContext* opCtx,
                                  const UpdateTicket& ticket,
                                  int64_t* numInserted,
                                  int64_t* numDeleted) {
@@ -374,23 +380,28 @@ Status IndexAccessMethod::update(OperationContext* txn,
 
     if (ticket.oldKeys.size() + ticket.added.size() - ticket.removed.size() > 1 ||
         isMultikeyFromPaths(ticket.newMultikeyPaths)) {
-        _btreeState->setMultikey(txn, ticket.newMultikeyPaths);
+        _btreeState->setMultikey(opCtx, ticket.newMultikeyPaths);
     }
 
     for (size_t i = 0; i < ticket.removed.size(); ++i) {
-        _newInterface->unindex(txn, ticket.removed[i], ticket.loc, ticket.dupsAllowed);
+        _newInterface->unindex(opCtx, ticket.removed[i], ticket.loc, ticket.dupsAllowed);
+        IndexKeyEntry indexEntry = IndexKeyEntry(ticket.removed[i], ticket.loc);
     }
 
     for (size_t i = 0; i < ticket.added.size(); ++i) {
-        Status status = _newInterface->insert(txn, ticket.added[i], ticket.loc, ticket.dupsAllowed);
+        Status status =
+            _newInterface->insert(opCtx, ticket.added[i], ticket.loc, ticket.dupsAllowed);
         if (!status.isOK()) {
-            if (status.code() == ErrorCodes::KeyTooLong && ignoreKeyTooLong(txn)) {
+            if (status.code() == ErrorCodes::KeyTooLong && ignoreKeyTooLong(opCtx)) {
                 // Ignore.
+                IndexKeyEntry indexEntry = IndexKeyEntry(ticket.added[i], ticket.loc);
                 continue;
             }
 
             return status;
         }
+
+        IndexKeyEntry indexEntry = IndexKeyEntry(ticket.added[i], ticket.loc);
     }
 
     *numInserted = ticket.added.size();
@@ -399,8 +410,8 @@ Status IndexAccessMethod::update(OperationContext* txn,
     return Status::OK();
 }
 
-Status IndexAccessMethod::compact(OperationContext* txn) {
-    return this->_newInterface->compact(txn);
+Status IndexAccessMethod::compact(OperationContext* opCtx) {
+    return this->_newInterface->compact(opCtx);
 }
 
 std::unique_ptr<IndexAccessMethod::BulkBuilder> IndexAccessMethod::initiateBulk(
@@ -419,7 +430,7 @@ IndexAccessMethod::BulkBuilder::BulkBuilder(const IndexAccessMethod* index,
           BtreeExternalSortComparison(descriptor->keyPattern(), descriptor->version()))),
       _real(index) {}
 
-Status IndexAccessMethod::BulkBuilder::insert(OperationContext* txn,
+Status IndexAccessMethod::BulkBuilder::insert(OperationContext* opCtx,
                                               const BSONObj& obj,
                                               const RecordId& loc,
                                               const InsertDeleteOptions& options,
@@ -455,7 +466,7 @@ Status IndexAccessMethod::BulkBuilder::insert(OperationContext* txn,
 }
 
 
-Status IndexAccessMethod::commitBulk(OperationContext* txn,
+Status IndexAccessMethod::commitBulk(OperationContext* opCtx,
                                      std::unique_ptr<BulkBuilder> bulk,
                                      bool mayInterrupt,
                                      bool dupsAllowed,
@@ -464,38 +475,38 @@ Status IndexAccessMethod::commitBulk(OperationContext* txn,
 
     std::unique_ptr<BulkBuilder::Sorter::Iterator> i(bulk->_sorter->done());
 
-    stdx::unique_lock<Client> lk(*txn->getClient());
-    ProgressMeterHolder pm(*txn->setMessage_inlock("Index Bulk Build: (2/3) btree bottom up",
-                                                   "Index: (2/3) BTree Bottom Up Progress",
-                                                   bulk->_keysInserted,
-                                                   10));
+    stdx::unique_lock<Client> lk(*opCtx->getClient());
+    ProgressMeterHolder pm(
+        CurOp::get(opCtx)->setMessage_inlock("Index Bulk Build: (2/3) btree bottom up",
+                                             "Index: (2/3) BTree Bottom Up Progress",
+                                             bulk->_keysInserted,
+                                             10));
     lk.unlock();
 
     std::unique_ptr<SortedDataBuilderInterface> builder;
 
-    MONGO_WRITE_CONFLICT_RETRY_LOOP_BEGIN {
-        WriteUnitOfWork wunit(txn);
+    writeConflictRetry(opCtx, "setting index multikey flag", "", [&] {
+        WriteUnitOfWork wunit(opCtx);
 
         if (bulk->_everGeneratedMultipleKeys || isMultikeyFromPaths(bulk->_indexMultikeyPaths)) {
-            _btreeState->setMultikey(txn, bulk->_indexMultikeyPaths);
+            _btreeState->setMultikey(opCtx, bulk->_indexMultikeyPaths);
         }
 
-        builder.reset(_newInterface->getBulkBuilder(txn, dupsAllowed));
+        builder.reset(_newInterface->getBulkBuilder(opCtx, dupsAllowed));
         wunit.commit();
-    }
-    MONGO_WRITE_CONFLICT_RETRY_LOOP_END(txn, "setting index multikey flag", "");
+    });
 
     while (i->more()) {
         if (mayInterrupt) {
-            txn->checkForInterrupt();
+            opCtx->checkForInterrupt();
         }
 
-        WriteUnitOfWork wunit(txn);
+        WriteUnitOfWork wunit(opCtx);
         // Improve performance in the btree-building phase by disabling rollback tracking.
         // This avoids copying all the written bytes to a buffer that is only used to roll back.
         // Note that this is safe to do, as this entire index-build-in-progress will be cleaned
         // up by the index system.
-        txn->recoveryUnit()->setRollbackWritesDisabled();
+        opCtx->recoveryUnit()->setRollbackWritesDisabled();
 
         // Get the next datum and add it to the builder.
         BulkBuilder::Sorter::Data d = i->next();
@@ -503,7 +514,7 @@ Status IndexAccessMethod::commitBulk(OperationContext* txn,
 
         if (!status.isOK()) {
             // Overlong key that's OK to skip?
-            if (status.code() == ErrorCodes::KeyTooLong && ignoreKeyTooLong(txn)) {
+            if (status.code() == ErrorCodes::KeyTooLong && ignoreKeyTooLong(opCtx)) {
                 continue;
             }
 
@@ -529,9 +540,9 @@ Status IndexAccessMethod::commitBulk(OperationContext* txn,
     pm.finished();
 
     {
-        stdx::lock_guard<Client> lk(*txn->getClient());
-        CurOp::get(txn)->setMessage_inlock("Index Bulk Build: (3/3) btree-middle",
-                                           "Index: (3/3) BTree Middle Progress");
+        stdx::lock_guard<Client> lk(*opCtx->getClient());
+        CurOp::get(opCtx)->setMessage_inlock("Index Bulk Build: (3/3) btree-middle",
+                                             "Index: (3/3) BTree Middle Progress");
     }
 
     LOG(timer.seconds() > 10 ? 0 : 1) << "\t done building bottom layer, going to commit";
@@ -552,6 +563,8 @@ void IndexAccessMethod::getKeys(const BSONObj& obj,
                                               16732,
                                               16733,
                                               16675,
+                                              17261,
+                                              17262,
                                               // Hash
                                               16766,
                                               // Haystack
@@ -568,7 +581,7 @@ void IndexAccessMethod::getKeys(const BSONObj& obj,
                                               13027};
     try {
         doGetKeys(obj, keys, multikeyPaths);
-    } catch (const UserException& ex) {
+    } catch (const AssertionException& ex) {
         if (mode == GetKeysMode::kEnforceConstraints) {
             throw;
         }
@@ -579,7 +592,7 @@ void IndexAccessMethod::getKeys(const BSONObj& obj,
             multikeyPaths->clear();
         }
         // Only suppress the errors in the whitelist.
-        if (whiteList.find(ex.getCode()) == whiteList.end()) {
+        if (whiteList.find(ex.code()) == whiteList.end()) {
             throw;
         }
         LOG(1) << "Ignoring indexing error for idempotency reasons: " << redact(ex)
