@@ -36,5 +36,5 @@ class OperationContext;
  * Restarts building indexes that were in progress during shutdown.
  * Only call this at startup before taking requests.
  */
-void restartInProgressIndexesFromLastShutdown(OperationContext* txn);
+void restartInProgressIndexesFromLastShutdown(OperationContext* opCtx);
 }
