@@ -38,6 +38,7 @@
 #include <signal.h>
 
 #ifndef _WIN32
+#include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <syslog.h>
@@ -51,6 +52,7 @@
 #include "mongo/db/auth/internal_user_auth.h"
 #include "mongo/db/auth/security_key.h"
 #include "mongo/db/server_options.h"
+#include "mongo/db/server_parameters.h"
 #include "mongo/logger/console_appender.h"
 #include "mongo/logger/logger.h"
 #include "mongo/logger/message_event.h"
@@ -69,6 +71,10 @@
 #include "mongo/util/quick_exit.h"
 #include "mongo/util/signal_handlers_synchronous.h"
 
+#if defined(__APPLE__)
+#include <TargetConditionals.h>
+#endif
+
 namespace fs = boost::filesystem;
 
 namespace mongo {
@@ -100,7 +106,7 @@ void signalForkSuccess() {
 
 
 static bool forkServer() {
-#ifndef _WIN32
+#if !defined(_WIN32) && !(defined(__APPLE__) && TARGET_OS_TV)
     if (serverGlobalParams.doFork) {
         fassert(16447, !serverGlobalParams.logpath.empty() || serverGlobalParams.logWithSyslog);
 
@@ -131,7 +137,9 @@ static bool forkServer() {
             if (WIFEXITED(pstat)) {
                 if (WEXITSTATUS(pstat)) {
                     cout << "ERROR: child process failed, exited with error number "
-                         << WEXITSTATUS(pstat) << endl;
+                         << WEXITSTATUS(pstat) << endl
+                         << "To see additional information in this output, start without "
+                         << "the \"--fork\" option." << endl;
                 } else {
                     cout << "child process started successfully, parent exiting" << endl;
                 }
@@ -330,8 +338,27 @@ MONGO_INITIALIZER(RegisterShortCircuitExitHandler)(InitializerContext*) {
     return Status::OK();
 }
 
+// On non-windows platforms, drop rwx for group and other unless the
+// user has opted into using the system umask. To do so, we first read
+// out the current umask (by temporarily setting it to
+// no-permissions), and then or the returned umask with the
+// restrictions we want to apply and set it back. The overall effect
+// is to set the bits for 'other' and 'group', but leave umask bits
+// bits for 'user' unaltered.
+#ifndef _WIN32
+namespace {
+MONGO_EXPORT_STARTUP_SERVER_PARAMETER(honorSystemUmask, bool, false);
+MONGO_INITIALIZER(MungeUmask)(InitializerContext*) {
+    if (!honorSystemUmask) {
+        umask(umask(S_IRWXU | S_IRWXG | S_IRWXO) | S_IRWXG | S_IRWXO);
+    }
+    return Status::OK();
+}
+}  // namespace
+#endif
+
 bool initializeServerGlobalState() {
-    Listener::globalTicketHolder.resize(serverGlobalParams.maxConns);
+    Listener::globalTicketHolder.resize(serverGlobalParams.maxConns).transitional_ignore();
 
 #ifndef _WIN32
     if (!fs::is_directory(serverGlobalParams.socket)) {
@@ -373,7 +400,7 @@ bool initializeServerGlobalState() {
                  << saslCommandUserDBFieldName
                  << "$external"
                  << saslCommandUserFieldName
-                 << getSSLManager()->getSSLConfiguration().clientSubjectName.toString()));
+                 << getSSLManager()->getSSLConfiguration().clientSubjectName));
     }
 #endif
     return true;
