@@ -65,34 +65,55 @@ public:
         return _expressions[i];
     }
 
+    /*
+     * Replaces the ith child with nullptr, and releases ownership of the child.
+     */
     virtual std::unique_ptr<MatchExpression> releaseChild(size_t i) {
         auto child = std::unique_ptr<MatchExpression>(_expressions[i]);
         _expressions[i] = nullptr;
         return child;
     }
 
+    /*
+     * Removes the ith child, and releases ownership of the child.
+     */
+    virtual std::unique_ptr<MatchExpression> removeChild(size_t i) {
+        auto child = std::unique_ptr<MatchExpression>(_expressions[i]);
+        _expressions.erase(_expressions.begin() + i);
+        return child;
+    }
+
     virtual std::vector<MatchExpression*>* getChildVector() {
         return &_expressions;
     }
 
     bool equivalent(const MatchExpression* other) const;
 
+    MatchCategory getCategory() const final {
+        return MatchCategory::kLogical;
+    }
+
 protected:
     void _debugList(StringBuilder& debug, int level) const;
 
     void _listToBSON(BSONArrayBuilder* out) const;
 
 private:
+    ExpressionOptimizerFunc getOptimizer() const final;
+
     std::vector<MatchExpression*> _expressions;
 };
 
 class AndMatchExpression : public ListOfMatchExpression {
 public:
+    static constexpr StringData kName = "$and"_sd;
+
     AndMatchExpression() : ListOfMatchExpression(AND) {}
     virtual ~AndMatchExpression() {}
 
     virtual bool matches(const MatchableDocument* doc, MatchDetails* details = 0) const;
-    virtual bool matchesSingleElement(const BSONElement& e) const;
+
+    bool matchesSingleElement(const BSONElement&, MatchDetails* details = nullptr) const final;
 
     virtual std::unique_ptr<MatchExpression> shallowClone() const {
         std::unique_ptr<AndMatchExpression> self = stdx::make_unique<AndMatchExpression>();
@@ -112,11 +133,14 @@ public:
 
 class OrMatchExpression : public ListOfMatchExpression {
 public:
+    static constexpr StringData kName = "$or"_sd;
+
     OrMatchExpression() : ListOfMatchExpression(OR) {}
     virtual ~OrMatchExpression() {}
 
     virtual bool matches(const MatchableDocument* doc, MatchDetails* details = 0) const;
-    virtual bool matchesSingleElement(const BSONElement& e) const;
+
+    bool matchesSingleElement(const BSONElement&, MatchDetails* details = nullptr) const final;
 
     virtual std::unique_ptr<MatchExpression> shallowClone() const {
         std::unique_ptr<OrMatchExpression> self = stdx::make_unique<OrMatchExpression>();
@@ -136,11 +160,14 @@ public:
 
 class NorMatchExpression : public ListOfMatchExpression {
 public:
+    static constexpr StringData kName = "$nor"_sd;
+
     NorMatchExpression() : ListOfMatchExpression(NOR) {}
     virtual ~NorMatchExpression() {}
 
     virtual bool matches(const MatchableDocument* doc, MatchDetails* details = 0) const;
-    virtual bool matchesSingleElement(const BSONElement& e) const;
+
+    bool matchesSingleElement(const BSONElement&, MatchDetails* details = nullptr) const final;
 
     virtual std::unique_ptr<MatchExpression> shallowClone() const {
         std::unique_ptr<NorMatchExpression> self = stdx::make_unique<NorMatchExpression>();
@@ -158,7 +185,7 @@ public:
     virtual void serialize(BSONObjBuilder* out) const;
 };
 
-class NotMatchExpression : public MatchExpression {
+class NotMatchExpression final : public MatchExpression {
 public:
     NotMatchExpression() : MatchExpression(NOT) {}
     NotMatchExpression(MatchExpression* e) : MatchExpression(NOT), _exp(e) {}
@@ -172,7 +199,7 @@ public:
 
     virtual std::unique_ptr<MatchExpression> shallowClone() const {
         std::unique_ptr<NotMatchExpression> self = stdx::make_unique<NotMatchExpression>();
-        self->init(_exp->shallowClone().release());
+        self->init(_exp->shallowClone().release()).transitional_ignore();
         if (getTag()) {
             self->setTag(getTag()->clone());
         }
@@ -183,8 +210,8 @@ public:
         return !_exp->matches(doc, NULL);
     }
 
-    virtual bool matchesSingleElement(const BSONElement& e) const {
-        return !_exp->matchesSingleElement(e);
+    bool matchesSingleElement(const BSONElement& elt, MatchDetails* details = nullptr) const final {
+        return !_exp->matchesSingleElement(elt, details);
     }
 
     virtual void debugString(StringBuilder& debug, int level = 0) const;
@@ -193,14 +220,18 @@ public:
 
     bool equivalent(const MatchExpression* other) const;
 
-    virtual size_t numChildren() const {
+    size_t numChildren() const final {
         return 1;
     }
 
-    virtual MatchExpression* getChild(size_t i) const {
+    MatchExpression* getChild(size_t i) const final {
         return _exp.get();
     }
 
+    std::vector<MatchExpression*>* getChildVector() final {
+        return nullptr;
+    }
+
     MatchExpression* releaseChild(void) {
         return _exp.release();
     }
@@ -209,7 +240,13 @@ public:
         _exp.reset(newChild);
     }
 
+    MatchCategory getCategory() const final {
+        return MatchCategory::kLogical;
+    }
+
 private:
+    ExpressionOptimizerFunc getOptimizer() const final;
+
     std::unique_ptr<MatchExpression> _exp;
 };
 }
