@@ -1,54 +1,48 @@
-// @file namespacestring.h
-
 /**
-*    Copyright (C) 2008 10gen Inc.
-*
-*    This program is free software: you can redistribute it and/or  modify
-*    it under the terms of the GNU Affero General Public License, version 3,
-*    as published by the Free Software Foundation.
-*
-*    This program is distributed in the hope that it will be useful,
-*    but WITHOUT ANY WARRANTY; without even the implied warranty of
-*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-*    GNU Affero General Public License for more details.
-*
-*    You should have received a copy of the GNU Affero General Public License
-*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*
-*    As a special exception, the copyright holders give permission to link the
-*    code of portions of this program with the OpenSSL library under certain
-*    conditions as described in each individual source file and distribute
-*    linked combinations including the program with the OpenSSL library. You
-*    must comply with the GNU Affero General Public License in all respects for
-*    all of the code used other than as permitted herein. If you modify file(s)
-*    with this exception, you may extend this exception to your version of the
-*    file(s), but you are not obligated to do so. If you do not wish to do so,
-*    delete this exception statement from your version. If you delete this
-*    exception statement from all source files in the program, then also delete
-*    it in the license file.
-*/
+ *    Copyright (C) 2017 MongoDB, Inc.
+ *
+ *    This program is free software: you can redistribute it and/or  modify
+ *    it under the terms of the GNU Affero General Public License, version 3,
+ *    as published by the Free Software Foundation.
+ *
+ *    This program is distributed in the hope that it will be useful,
+ *    but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *    GNU Affero General Public License for more details.
+ *
+ *    You should have received a copy of the GNU Affero General Public License
+ *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ *    As a special exception, the copyright holders give permission to link the
+ *    code of portions of this program with the OpenSSL library under certain
+ *    conditions as described in each individual source file and distribute
+ *    linked combinations including the program with the OpenSSL library. You
+ *    must comply with the GNU Affero General Public License in all respects for
+ *    all of the code used other than as permitted herein. If you modify file(s)
+ *    with this exception, you may extend this exception to your version of the
+ *    file(s), but you are not obligated to do so. If you do not wish to do so,
+ *    delete this exception statement from your version. If you delete this
+ *    exception statement from all source files in the program, then also delete
+ *    it in the license file.
+ */
 
 #pragma once
 
 #include <algorithm>
+#include <boost/optional.hpp>
 #include <iosfwd>
 #include <string>
 
+#include "mongo/base/status_with.h"
 #include "mongo/base/string_data.h"
+#include "mongo/db/repl/optime.h"
 #include "mongo/platform/hash_namespace.h"
 #include "mongo/util/assert_util.h"
 
 namespace mongo {
 
-/* in the mongo source code, "client" means "database". */
-
 const size_t MaxDatabaseNameLen = 128;  // max str len for the db name, including null char
 
-/** @return true if a client can modify this namespace even though it is under ".system."
-    For example <dbname>.system.users is ok for regular clients to update.
-*/
-bool legalClientSystemNS(StringData ns);
-
 /* e.g.
    NamespaceString ns("acme.orders");
    cout << ns.coll; // "orders"
@@ -63,15 +57,32 @@ public:
     // Namespace for the local database
     static constexpr StringData kLocalDb = "local"_sd;
 
+    // Namespace for the sharding config database
+    static constexpr StringData kConfigDb = "config"_sd;
+
     // Name for the system views collection
     static constexpr StringData kSystemDotViewsCollectionName = "system.views"_sd;
 
+    // Name for a shard's collections metadata collection, each document of which indicates the
+    // state of a specific collection.
+    static constexpr StringData kShardConfigCollectionsCollectionName =
+        "config.cache.collections"_sd;
+
+    // Name for causal consistency's key collection.
+    static constexpr StringData kSystemKeysCollectionName = "admin.system.keys"_sd;
+
     // Namespace for storing configuration data, which needs to be replicated if the server is
     // running as a replica set. Documents in this collection should represent some configuration
     // state of the server, which needs to be recovered/consulted at startup. Each document in this
     // namespace should have its _id set to some string, which meaningfully describes what it
     // represents.
-    static const NamespaceString kConfigCollectionNamespace;
+    static const NamespaceString kServerConfigurationNamespace;
+
+    // Namespace for storing the transaction information for each session
+    static const NamespaceString kSessionTransactionsTableNamespace;
+
+    // Namespace of the the oplog collection.
+    static const NamespaceString kRsOplogNamespace;
 
     /**
      * Constructs an empty NamespaceString.
@@ -90,13 +101,19 @@ public:
     NamespaceString(StringData dbName, StringData collectionName);
 
     /**
-     * Contructs a NamespaceString representing a listCollections namespace. The format for this
+     * Constructs the namespace '<dbName>.$cmd.aggregate', which we use as the namespace for
+     * aggregation commands with the format {aggregate: 1}.
+     */
+    static NamespaceString makeCollectionlessAggregateNSS(StringData dbName);
+
+    /**
+     * Constructs a NamespaceString representing a listCollections namespace. The format for this
      * namespace is "<dbName>.$cmd.listCollections".
      */
     static NamespaceString makeListCollectionsNSS(StringData dbName);
 
     /**
-     * Contructs a NamespaceString representing a listIndexes namespace. The format for this
+     * Constructs a NamespaceString representing a listIndexes namespace. The format for this
      * namespace is "<dbName>.$cmd.listIndexes.<collectionName>".
      */
     static NamespaceString makeListIndexesNSS(StringData dbName, StringData collectionName);
@@ -143,6 +160,10 @@ public:
         return _ns.size();
     }
 
+    bool isEmpty() const {
+        return _ns.empty();
+    }
+
     struct Hasher {
         size_t operator()(const NamespaceString& nss) const {
             return std::hash<std::string>()(nss._ns);
@@ -153,6 +174,9 @@ public:
     // The following methods assume isValid() is true for this NamespaceString.
     //
 
+    bool isHealthlog() const {
+        return isLocal() && coll() == "system.healthlog";
+    }
     bool isSystem() const {
         return coll().startsWith("system.");
     }
@@ -168,6 +192,9 @@ public:
     bool isSystemDotViews() const {
         return coll() == kSystemDotViewsCollectionName;
     }
+    bool isAdminDotSystemDotVersion() const {
+        return ((db() == "admin") && (coll() == "system.version"));
+    }
     bool isConfigDB() const {
         return db() == "config";
     }
@@ -177,9 +204,6 @@ public:
     bool isOplog() const {
         return oplog(_ns);
     }
-    bool isSpecialCommand() const {
-        return coll().startsWith("$cmd.sys");
-    }
     bool isSpecial() const {
         return special(_ns);
     }
@@ -195,9 +219,59 @@ public:
     bool isVirtualized() const {
         return virtualized(_ns);
     }
+
+    /**
+     * Returns true if cursors for this namespace are registered with the global cursor manager.
+     */
+    bool isGloballyManagedNamespace() const {
+        return coll().startsWith("$cmd."_sd);
+    }
+
+    bool isCollectionlessAggregateNS() const;
     bool isListCollectionsCursorNS() const;
     bool isListIndexesCursorNS() const;
 
+    /**
+     * Returns true if a client can modify this namespace even though it is under ".system."
+     * For example <dbname>.system.users is ok for regular clients to update.
+     */
+    bool isLegalClientSystemNS() const;
+
+    /**
+     * Given a NamespaceString for which isGloballyManagedNamespace() returns true, returns the
+     * namespace the command targets, or boost::none for commands like 'listCollections' which
+     * do not target a collection.
+     */
+    boost::optional<NamespaceString> getTargetNSForGloballyManagedNamespace() const;
+
+    /**
+     * Returns true if this namespace refers to a drop-pending collection.
+     */
+    bool isDropPendingNamespace() const;
+
+    /**
+     * Returns the drop-pending namespace name for this namespace, provided the given optime.
+     *
+     * Example:
+     *     test.foo -> test.system.drop.<timestamp seconds>i<timestamp increment>t<term>.foo
+     *
+     * Original collection name may be truncated so that the generated namespace length does not
+     * exceed MaxNsCollectionLen.
+     */
+    NamespaceString makeDropPendingNamespace(const repl::OpTime& opTime) const;
+
+    /**
+     * Returns the optime used to generate the drop-pending namespace.
+     * Returns an error if this namespace is not drop-pending.
+     */
+    StatusWith<repl::OpTime> getDropPendingNamespaceOpTime() const;
+
+    /**
+     * Checks if this namespace is valid as a target namespace for a rename operation, given
+     * the length of the longest index name in the source collection.
+     */
+    Status checkLengthForRename(const std::string::size_type longestIndexNameLength) const;
+
     /**
      * Given a NamespaceString for which isListIndexesCursorNS() returns true, returns the
      * NamespaceString for the collection that the "listIndexes" targets.
@@ -240,8 +314,8 @@ public:
     // @return db() + ".system.indexes"
     std::string getSystemIndexesCollection() const;
 
-    // @return db() + ".$cmd"
-    std::string getCommandNS() const;
+    // @return {db(), "$cmd"}
+    NamespaceString getCommandNS() const;
 
     /**
      * Function to escape most non-alpha characters from file names
