@@ -28,12 +28,15 @@
 
 #pragma once
 
+#include <boost/optional.hpp>
 #include <memory>
 
 #include "mongo/base/disallow_copying.h"
 #include "mongo/base/status.h"
 #include "mongo/db/client.h"
-#include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/concurrency/locker.h"
+#include "mongo/db/logical_session_id.h"
+#include "mongo/db/storage/recovery_unit.h"
 #include "mongo/db/storage/recovery_unit.h"
 #include "mongo/db/storage/storage_options.h"
 #include "mongo/db/write_concern_options.h"
@@ -48,12 +51,15 @@ namespace mongo {
 
 class Client;
 class CurOp;
-class Locker;
 class ProgressMeter;
 class ServiceContext;
 class StringData;
 class WriteUnitOfWork;
 
+namespace repl {
+class UnreplicatedWritesBlock;
+}  // namespace repl
+
 /**
  * This class encompasses the state required by an operation and lives from the time a network
  * operation is dispatched until its execution is finished. Note that each "getmore" on a cursor
@@ -78,6 +84,8 @@ public:
         kFailedUnitOfWork   // in a unit of work that has failed and must be aborted
     };
 
+    OperationContext(Client* client, unsigned int opId);
+
     virtual ~OperationContext() = default;
 
     /**
@@ -129,7 +137,7 @@ public:
     std::unique_ptr<Locker> releaseLockState();
 
     /**
-     * Raises a UserException if this operation is in a killed state.
+     * Raises a AssertionException if this operation is in a killed state.
      */
     void checkForInterrupt();
 
@@ -138,10 +146,20 @@ public:
      */
     Status checkForInterruptNoAssert();
 
+    /**
+     * Sleeps until "deadline"; throws an exception if the operation is interrupted before then.
+     */
+    void sleepUntil(Date_t deadline);
+
+    /**
+     * Sleeps for "duration" ms; throws an exception if the operation is interrupted before then.
+     */
+    void sleepFor(Milliseconds duration);
+
     /**
      * Waits for either the condition "cv" to be signaled, this operation to be interrupted, or the
      * deadline on this operation to expire.  In the event of interruption or operation deadline
-     * expiration, raises a UserException with an error code indicating the interruption type.
+     * expiration, raises a AssertionException with an error code indicating the interruption type.
      */
     void waitForConditionOrInterrupt(stdx::condition_variable& cv,
                                      stdx::unique_lock<stdx::mutex>& m);
@@ -189,31 +207,10 @@ public:
      * cv_status::no_timeout indicating that "pred" finally returned true.
      */
     template <typename Pred>
-    stdx::cv_status waitForConditionOrInterruptUntil(stdx::condition_variable& cv,
-                                                     stdx::unique_lock<stdx::mutex>& m,
-                                                     Date_t deadline,
-                                                     Pred pred) {
-        while (!pred()) {
-            if (stdx::cv_status::timeout == waitForConditionOrInterruptUntil(cv, m, deadline)) {
-                return stdx::cv_status::timeout;
-            }
-        }
-        return stdx::cv_status::no_timeout;
-    }
-
-    /**
-     * Waits on condition "cv" for "pred" until "pred" returns true, or the given "deadline"
-     * expires, or this operation is interrupted, or this operation's own deadline expires.
-     *
-     *
-     * If the operation deadline expires or the operation is interrupted, throws a DBException.  If
-     * the given "deadline" expires, returns pred. Otherwise, returns true.
-     */
-    template <typename Pred>
-    bool waitForConditionOrInterruptUntilPred(stdx::condition_variable& cv,
-                                              stdx::unique_lock<stdx::mutex>& m,
-                                              Date_t deadline,
-                                              Pred pred) {
+    bool waitForConditionOrInterruptUntil(stdx::condition_variable& cv,
+                                          stdx::unique_lock<stdx::mutex>& m,
+                                          Date_t deadline,
+                                          Pred pred) {
         while (!pred()) {
             if (stdx::cv_status::timeout == waitForConditionOrInterruptUntil(cv, m, deadline)) {
                 return pred();
@@ -223,7 +220,7 @@ public:
     }
 
     /**
-     * Same as the predicate form of waitForConditionOrInterruptUntilPred, but takes a relative
+     * Same as the predicate form of waitForConditionOrInterruptUntil, but takes a relative
      * amount of time to wait instead of an absolute time point.
      */
     template <typename Pred>
@@ -231,8 +228,8 @@ public:
                                         stdx::unique_lock<stdx::mutex>& m,
                                         Milliseconds duration,
                                         Pred pred) {
-        return waitForConditionOrInterruptUntilPred(
-            cv, m, getServiceContext()->getPreciseClockSource()->now() + duration, pred);
+        return waitForConditionOrInterruptUntil(
+            cv, m, getExpirationDateForWaitForValue(duration), pred);
     }
 
     /**
@@ -243,20 +240,14 @@ public:
         stdx::condition_variable& cv, stdx::unique_lock<stdx::mutex>& m, Date_t deadline) noexcept;
 
     /**
-     * Delegates to CurOp, but is included here to break dependencies.
-     * Caller does not own the pointer.
-     *
-     * Caller must have locked the "Client" associated with this context.
-     */
-    virtual ProgressMeter* setMessage_inlock(const char* msg,
-                                             const std::string& name = "Progress",
-                                             unsigned long long progressMeterTotal = 0,
-                                             int secondsBetween = 3) = 0;
-
-    /**
-     * Returns the service context under which this operation context runs.
+     * Returns the service context under which this operation context runs, or nullptr if there is
+     * no such service context.
      */
     ServiceContext* getServiceContext() const {
+        if (!_client) {
+            return nullptr;
+        }
+
         return _client->getServiceContext();
     }
 
@@ -274,6 +265,33 @@ public:
         return _opId;
     }
 
+    /**
+     * Returns the session ID associated with this operation, if there is one.
+     */
+    boost::optional<LogicalSessionId> getLogicalSessionId() const {
+        return _lsid;
+    }
+
+    /**
+     * Associates a logical session id with this operation context. May only be called once for the
+     * lifetime of the operation.
+     */
+    void setLogicalSessionId(LogicalSessionId lsid);
+
+    /**
+     * Returns the transaction number associated with thes operation. The combination of logical
+     * session id + transaction number is what constitutes the operation transaction id.
+     */
+    boost::optional<TxnNumber> getTxnNumber() const {
+        return _txnNumber;
+    }
+
+    /**
+     * Associates a transaction number with this operation context. May only be called once for the
+     * lifetime of the operation and the operation must have a logical session id assigned.
+     */
+    void setTxnNumber(TxnNumber txnNumber);
+
     /**
      * Returns WriteConcernOptions of the current operation
      */
@@ -285,14 +303,6 @@ public:
         _writeConcern = writeConcern;
     }
 
-    /**
-     * Set whether or not operations should generate oplog entries.
-     * TODO SERVER-26965: Make this private.
-     */
-    void setReplicatedWrites(bool writesAreReplicated = true) {
-        _writesAreReplicated = writesAreReplicated;
-    }
-
     /**
      * Returns true if operations should generate oplog entries.
      */
@@ -378,6 +388,14 @@ public:
         return _deadline;
     }
 
+    /**
+     * Reset the deadline for this operation.
+     */
+    void clearDeadline() {
+        _deadline = Date_t::max();
+        _maxTime = computeMaxTimeFromDeadline(_deadline);
+    }
+
     /**
      * Returns the number of milliseconds remaining for this operation's time limit or
      * Milliseconds::max() if the operation has no time limit.
@@ -393,9 +411,6 @@ public:
      */
     Microseconds getRemainingMaxTimeMicros() const;
 
-protected:
-    OperationContext(Client* client, unsigned int opId);
-
 private:
     /**
      * Returns true if this operation has a deadline and it has passed according to the fast clock
@@ -409,10 +424,32 @@ private:
      */
     void setDeadlineAndMaxTime(Date_t when, Microseconds maxTime);
 
+    /**
+     * Compute maxTime based on the given deadline.
+     */
+    Microseconds computeMaxTimeFromDeadline(Date_t when);
+
+    /**
+     * Returns the timepoint that is "waitFor" ms after now according to the
+     * ServiceContext's precise clock.
+     */
+    Date_t getExpirationDateForWaitForValue(Milliseconds waitFor);
+
+    /**
+     * Set whether or not operations should generate oplog entries.
+     */
+    void setReplicatedWrites(bool writesAreReplicated = true) {
+        _writesAreReplicated = writesAreReplicated;
+    }
+
     friend class WriteUnitOfWork;
+    friend class repl::UnreplicatedWritesBlock;
     Client* const _client;
     const unsigned int _opId;
 
+    boost::optional<LogicalSessionId> _lsid;
+    boost::optional<TxnNumber> _txnNumber;
+
     std::unique_ptr<Locker> _locker;
 
     std::unique_ptr<RecoveryUnit> _recoveryUnit;
@@ -459,83 +496,52 @@ class WriteUnitOfWork {
     MONGO_DISALLOW_COPYING(WriteUnitOfWork);
 
 public:
-    WriteUnitOfWork(OperationContext* txn)
-        : _txn(txn),
+    WriteUnitOfWork(OperationContext* opCtx)
+        : _opCtx(opCtx),
           _committed(false),
-          _toplevel(txn->_ruState == OperationContext::kNotInUnitOfWork) {
+          _toplevel(opCtx->_ruState == OperationContext::kNotInUnitOfWork) {
         uassert(ErrorCodes::IllegalOperation,
                 "Cannot execute a write operation in read-only mode",
                 !storageGlobalParams.readOnly);
-        _txn->lockState()->beginWriteUnitOfWork();
+        _opCtx->lockState()->beginWriteUnitOfWork();
         if (_toplevel) {
-            _txn->recoveryUnit()->beginUnitOfWork(_txn);
-            _txn->_ruState = OperationContext::kActiveUnitOfWork;
+            _opCtx->recoveryUnit()->beginUnitOfWork(_opCtx);
+            _opCtx->_ruState = OperationContext::kActiveUnitOfWork;
         }
     }
 
     ~WriteUnitOfWork() {
         dassert(!storageGlobalParams.readOnly);
         if (!_committed) {
-            invariant(_txn->_ruState != OperationContext::kNotInUnitOfWork);
+            invariant(_opCtx->_ruState != OperationContext::kNotInUnitOfWork);
             if (_toplevel) {
-                _txn->recoveryUnit()->abortUnitOfWork();
-                _txn->_ruState = OperationContext::kNotInUnitOfWork;
+                _opCtx->recoveryUnit()->abortUnitOfWork();
+                _opCtx->_ruState = OperationContext::kNotInUnitOfWork;
             } else {
-                _txn->_ruState = OperationContext::kFailedUnitOfWork;
+                _opCtx->_ruState = OperationContext::kFailedUnitOfWork;
             }
-            _txn->lockState()->endWriteUnitOfWork();
+            _opCtx->lockState()->endWriteUnitOfWork();
         }
     }
 
     void commit() {
         invariant(!_committed);
-        invariant(_txn->_ruState == OperationContext::kActiveUnitOfWork);
+        invariant(_opCtx->_ruState == OperationContext::kActiveUnitOfWork);
         if (_toplevel) {
-            _txn->recoveryUnit()->commitUnitOfWork();
-            _txn->_ruState = OperationContext::kNotInUnitOfWork;
+            _opCtx->recoveryUnit()->commitUnitOfWork();
+            _opCtx->_ruState = OperationContext::kNotInUnitOfWork;
         }
-        _txn->lockState()->endWriteUnitOfWork();
+        _opCtx->lockState()->endWriteUnitOfWork();
         _committed = true;
     }
 
 private:
-    OperationContext* const _txn;
+    OperationContext* const _opCtx;
 
     bool _committed;
     bool _toplevel;
 };
 
-
-/**
- * RAII-style class to mark the scope of a transaction. ScopedTransactions may be nested.
- * An outermost ScopedTransaction calls abandonSnapshot() on destruction, so that the storage
- * engine can release resources, such as snapshots or locks, that it may have acquired during
- * the transaction. Note that any writes are committed in nested WriteUnitOfWork scopes,
- * so write conflicts cannot happen on completing a ScopedTransaction.
- *
- * TODO: The ScopedTransaction should hold the global lock
- */
-class ScopedTransaction {
-    MONGO_DISALLOW_COPYING(ScopedTransaction);
-
-public:
-    /**
-     * The mode for the transaction indicates whether the transaction will write (MODE_IX) or
-     * only read (MODE_IS), or needs to run without other writers (MODE_S) or any other
-     * operations (MODE_X) on the server.
-     */
-    ScopedTransaction(OperationContext* txn, LockMode mode) : _txn(txn) {}
-
-    ~ScopedTransaction() {
-        if (!_txn->lockState()->isLocked()) {
-            _txn->recoveryUnit()->abandonSnapshot();
-        }
-    }
-
-private:
-    OperationContext* _txn;
-};
-
 namespace repl {
 /**
  * RAII-style class to turn off replicated writes. Writes do not create oplog entries while the
@@ -545,19 +551,18 @@ class UnreplicatedWritesBlock {
     MONGO_DISALLOW_COPYING(UnreplicatedWritesBlock);
 
 public:
-    UnreplicatedWritesBlock(OperationContext* txn)
-        : _txn(txn), _shouldReplicateWrites(txn->writesAreReplicated()) {
-        txn->setReplicatedWrites(false);
+    UnreplicatedWritesBlock(OperationContext* opCtx)
+        : _opCtx(opCtx), _shouldReplicateWrites(opCtx->writesAreReplicated()) {
+        opCtx->setReplicatedWrites(false);
     }
 
     ~UnreplicatedWritesBlock() {
-        _txn->setReplicatedWrites(_shouldReplicateWrites);
+        _opCtx->setReplicatedWrites(_shouldReplicateWrites);
     }
 
 private:
-    OperationContext* _txn;
+    OperationContext* _opCtx;
     const bool _shouldReplicateWrites;
 };
 }  // namespace repl
-
 }  // namespace mongo
