@@ -28,8 +28,13 @@
 
 #include "mongo/platform/basic.h"
 
+#include <boost/optional.hpp>
+
 #include "mongo/db/client.h"
+#include "mongo/db/json.h"
+#include "mongo/db/logical_session_id.h"
 #include "mongo/db/operation_context.h"
+#include "mongo/db/operation_context_group.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/service_context_noop.h"
 #include "mongo/stdx/future.h"
@@ -42,9 +47,10 @@
 #include "mongo/util/time_support.h"
 
 namespace mongo {
-
 namespace {
 
+using unittest::assertGet;
+
 std::ostream& operator<<(std::ostream& os, stdx::cv_status cvStatus) {
     switch (cvStatus) {
         case stdx::cv_status::timeout:
@@ -69,6 +75,103 @@ std::ostream& operator<<(std::ostream& os, stdx::future_status futureStatus) {
     }
 }
 
+TEST(OperationContextTest, NoSessionIdNoTransactionNumber) {
+    auto serviceCtx = stdx::make_unique<ServiceContextNoop>();
+    auto client = serviceCtx->makeClient("OperationContextTest");
+    auto opCtx = client->makeOperationContext();
+
+    ASSERT(!opCtx->getLogicalSessionId());
+    ASSERT(!opCtx->getTxnNumber());
+}
+
+TEST(OperationContextTest, SessionIdNoTransactionNumber) {
+    auto serviceCtx = stdx::make_unique<ServiceContextNoop>();
+    auto client = serviceCtx->makeClient("OperationContextTest");
+    auto opCtx = client->makeOperationContext();
+
+    const auto lsid = makeLogicalSessionIdForTest();
+    opCtx->setLogicalSessionId(lsid);
+
+    ASSERT(opCtx->getLogicalSessionId());
+    ASSERT_EQUALS(lsid, *opCtx->getLogicalSessionId());
+
+    ASSERT(!opCtx->getTxnNumber());
+}
+
+TEST(OperationContextTest, SessionIdAndTransactionNumber) {
+    auto serviceCtx = stdx::make_unique<ServiceContextNoop>();
+    auto client = serviceCtx->makeClient("OperationContextTest");
+    auto opCtx = client->makeOperationContext();
+
+    const auto lsid = makeLogicalSessionIdForTest();
+    opCtx->setLogicalSessionId(lsid);
+    opCtx->setTxnNumber(5);
+
+    ASSERT(opCtx->getTxnNumber());
+    ASSERT_EQUALS(5, *opCtx->getTxnNumber());
+}
+
+TEST(OperationContextTest, OpCtxGroup) {
+    OperationContextGroup group1;
+    ASSERT_TRUE(group1.isEmpty());
+    {
+        auto serviceCtx1 = stdx::make_unique<ServiceContextNoop>();
+        auto client1 = serviceCtx1->makeClient("OperationContextTest1");
+        auto opCtx1 = group1.makeOperationContext(*client1);
+        ASSERT_FALSE(group1.isEmpty());
+
+        auto serviceCtx2 = stdx::make_unique<ServiceContextNoop>();
+        auto client2 = serviceCtx2->makeClient("OperationContextTest2");
+        {
+            auto opCtx2 = group1.makeOperationContext(*client2);
+            opCtx1.discard();
+            ASSERT_FALSE(group1.isEmpty());
+        }
+        ASSERT_TRUE(group1.isEmpty());
+
+        auto opCtx3 = group1.makeOperationContext(*client1);
+        auto opCtx4 = group1.makeOperationContext(*client2);
+        ASSERT_TRUE(opCtx3->checkForInterruptNoAssert().isOK());    // use member op->
+        ASSERT_TRUE((*opCtx4).checkForInterruptNoAssert().isOK());  // use conversion to OC*
+        group1.interrupt(ErrorCodes::InternalError);
+        ASSERT_FALSE(opCtx3->checkForInterruptNoAssert().isOK());
+        ASSERT_FALSE((*opCtx4).checkForInterruptNoAssert().isOK());
+    }
+    ASSERT_TRUE(group1.isEmpty());
+
+    OperationContextGroup group2;
+    {
+        auto serviceCtx = stdx::make_unique<ServiceContextNoop>();
+        auto client = serviceCtx->makeClient("OperationContextTest1");
+        auto opCtx2 = group2.adopt(client->makeOperationContext());
+        ASSERT_FALSE(group2.isEmpty());
+        ASSERT_TRUE(opCtx2->checkForInterruptNoAssert().isOK());
+        group2.interrupt(ErrorCodes::InternalError);
+        ASSERT_FALSE(opCtx2->checkForInterruptNoAssert().isOK());
+        opCtx2.discard();
+        ASSERT(opCtx2.opCtx() == nullptr);
+        ASSERT_TRUE(group2.isEmpty());
+    }
+
+    OperationContextGroup group3;
+    OperationContextGroup group4;
+    {
+        auto serviceCtx = stdx::make_unique<ServiceContextNoop>();
+        auto client3 = serviceCtx->makeClient("OperationContextTest3");
+        auto opCtx3 = group3.makeOperationContext(*client3);
+        auto p3 = opCtx3.opCtx();
+        auto opCtx4 = group4.take(std::move(opCtx3));
+        ASSERT_EQ(p3, opCtx4.opCtx());
+        ASSERT(opCtx3.opCtx() == nullptr);
+        ASSERT_TRUE(group3.isEmpty());
+        ASSERT_FALSE(group4.isEmpty());
+        group3.interrupt(ErrorCodes::InternalError);
+        ASSERT_TRUE(opCtx4->checkForInterruptNoAssert().isOK());
+        group4.interrupt(ErrorCodes::InternalError);
+        ASSERT_FALSE(opCtx4->checkForInterruptNoAssert().isOK());
+    }
+}
+
 class OperationDeadlineTests : public unittest::Test {
 public:
     void setUp() {
@@ -85,37 +188,37 @@ public:
 };
 
 TEST_F(OperationDeadlineTests, OperationDeadlineExpiration) {
-    auto txn = client->makeOperationContext();
-    txn->setDeadlineAfterNowBy(Seconds{1});
+    auto opCtx = client->makeOperationContext();
+    opCtx->setDeadlineAfterNowBy(Seconds{1});
     mockClock->advance(Milliseconds{500});
-    ASSERT_OK(txn->checkForInterruptNoAssert());
+    ASSERT_OK(opCtx->checkForInterruptNoAssert());
 
     // 1ms before relative deadline reports no interrupt
     mockClock->advance(Milliseconds{499});
-    ASSERT_OK(txn->checkForInterruptNoAssert());
+    ASSERT_OK(opCtx->checkForInterruptNoAssert());
 
     // Exactly at deadline reports no interrupt, because setDeadlineAfterNowBy adds one clock
     // precision unit to the deadline, to ensure that the deadline does not expire in less than the
     // requested amount of time.
     mockClock->advance(Milliseconds{1});
-    ASSERT_OK(txn->checkForInterruptNoAssert());
+    ASSERT_OK(opCtx->checkForInterruptNoAssert());
 
     // Since the mock clock's precision is 1ms, at test start + 1001 ms, we expect
     // checkForInterruptNoAssert to return ExceededTimeLimit.
     mockClock->advance(Milliseconds{1});
-    ASSERT_EQ(ErrorCodes::ExceededTimeLimit, txn->checkForInterruptNoAssert());
+    ASSERT_EQ(ErrorCodes::ExceededTimeLimit, opCtx->checkForInterruptNoAssert());
 
     // Also at times greater than start + 1001ms, we expect checkForInterruptNoAssert to keep
     // returning ExceededTimeLimit.
     mockClock->advance(Milliseconds{1});
-    ASSERT_EQ(ErrorCodes::ExceededTimeLimit, txn->checkForInterruptNoAssert());
+    ASSERT_EQ(ErrorCodes::ExceededTimeLimit, opCtx->checkForInterruptNoAssert());
 }
 
 template <typename D>
 void assertLargeRelativeDeadlineLikeInfinity(Client& client, D maxTime) {
-    auto txn = client.makeOperationContext();
-    txn->setDeadlineAfterNowBy(maxTime);
-    ASSERT_FALSE(txn->hasDeadline()) << "Tried to set maxTime to " << maxTime;
+    auto opCtx = client.makeOperationContext();
+    opCtx->setDeadlineAfterNowBy(maxTime);
+    ASSERT_FALSE(opCtx->hasDeadline()) << "Tried to set maxTime to " << maxTime;
 }
 
 TEST_F(OperationDeadlineTests, VeryLargeRelativeDeadlinesHours) {
@@ -142,76 +245,90 @@ TEST_F(OperationDeadlineTests, VeryLargeRelativeDeadlinesMicroseconds) {
 TEST_F(OperationDeadlineTests, VeryLargeRelativeDeadlinesNanoseconds) {
     // Nanoseconds::max() is less than Microseconds::max(), so it is possible to set
     // a deadline of that duration.
-    auto txn = client->makeOperationContext();
-    txn->setDeadlineAfterNowBy(Nanoseconds::max());
-    ASSERT_TRUE(txn->hasDeadline());
+    auto opCtx = client->makeOperationContext();
+    opCtx->setDeadlineAfterNowBy(Nanoseconds::max());
+    ASSERT_TRUE(opCtx->hasDeadline());
     ASSERT_EQ(mockClock->now() + mockClock->getPrecision() +
                   duration_cast<Milliseconds>(Nanoseconds::max()),
-              txn->getDeadline());
+              opCtx->getDeadline());
 }
 
 TEST_F(OperationDeadlineTests, WaitForMaxTimeExpiredCV) {
-    auto txn = client->makeOperationContext();
-    txn->setDeadlineByDate(mockClock->now());
+    auto opCtx = client->makeOperationContext();
+    opCtx->setDeadlineByDate(mockClock->now());
     stdx::mutex m;
     stdx::condition_variable cv;
     stdx::unique_lock<stdx::mutex> lk(m);
-    ASSERT_EQ(ErrorCodes::ExceededTimeLimit, txn->waitForConditionOrInterruptNoAssert(cv, lk));
+    ASSERT_EQ(ErrorCodes::ExceededTimeLimit, opCtx->waitForConditionOrInterruptNoAssert(cv, lk));
 }
 
 TEST_F(OperationDeadlineTests, WaitForMaxTimeExpiredCVWithWaitUntilSet) {
-    auto txn = client->makeOperationContext();
-    txn->setDeadlineByDate(mockClock->now());
+    auto opCtx = client->makeOperationContext();
+    opCtx->setDeadlineByDate(mockClock->now());
     stdx::mutex m;
     stdx::condition_variable cv;
     stdx::unique_lock<stdx::mutex> lk(m);
-    ASSERT_EQ(ErrorCodes::ExceededTimeLimit,
-              txn->waitForConditionOrInterruptNoAssertUntil(cv, lk, mockClock->now() + Seconds{10})
-                  .getStatus());
+    ASSERT_EQ(
+        ErrorCodes::ExceededTimeLimit,
+        opCtx->waitForConditionOrInterruptNoAssertUntil(cv, lk, mockClock->now() + Seconds{10})
+            .getStatus());
 }
 
 TEST_F(OperationDeadlineTests, WaitForKilledOpCV) {
-    auto txn = client->makeOperationContext();
-    txn->markKilled();
+    auto opCtx = client->makeOperationContext();
+    opCtx->markKilled();
     stdx::mutex m;
     stdx::condition_variable cv;
     stdx::unique_lock<stdx::mutex> lk(m);
-    ASSERT_EQ(ErrorCodes::Interrupted, txn->waitForConditionOrInterruptNoAssert(cv, lk));
+    ASSERT_EQ(ErrorCodes::Interrupted, opCtx->waitForConditionOrInterruptNoAssert(cv, lk));
 }
 
 TEST_F(OperationDeadlineTests, WaitForUntilExpiredCV) {
-    auto txn = client->makeOperationContext();
+    auto opCtx = client->makeOperationContext();
     stdx::mutex m;
     stdx::condition_variable cv;
     stdx::unique_lock<stdx::mutex> lk(m);
     ASSERT(stdx::cv_status::timeout ==
            unittest::assertGet(
-               txn->waitForConditionOrInterruptNoAssertUntil(cv, lk, mockClock->now())));
+               opCtx->waitForConditionOrInterruptNoAssertUntil(cv, lk, mockClock->now())));
 }
 
 TEST_F(OperationDeadlineTests, WaitForUntilExpiredCVWithMaxTimeSet) {
-    auto txn = client->makeOperationContext();
-    txn->setDeadlineByDate(mockClock->now() + Seconds{10});
+    auto opCtx = client->makeOperationContext();
+    opCtx->setDeadlineByDate(mockClock->now() + Seconds{10});
     stdx::mutex m;
     stdx::condition_variable cv;
     stdx::unique_lock<stdx::mutex> lk(m);
     ASSERT(stdx::cv_status::timeout ==
            unittest::assertGet(
-               txn->waitForConditionOrInterruptNoAssertUntil(cv, lk, mockClock->now())));
+               opCtx->waitForConditionOrInterruptNoAssertUntil(cv, lk, mockClock->now())));
+}
+
+TEST_F(OperationDeadlineTests, WaitForDurationExpired) {
+    auto opCtx = client->makeOperationContext();
+    stdx::mutex m;
+    stdx::condition_variable cv;
+    stdx::unique_lock<stdx::mutex> lk(m);
+    ASSERT_FALSE(opCtx->waitForConditionOrInterruptFor(
+        cv, lk, Milliseconds(-1000), []() -> bool { return false; }));
 }
 
 TEST_F(OperationDeadlineTests, DuringWaitMaxTimeExpirationDominatesUntilExpiration) {
-    auto txn = client->makeOperationContext();
-    txn->setDeadlineByDate(mockClock->now());
+    auto opCtx = client->makeOperationContext();
+    opCtx->setDeadlineByDate(mockClock->now());
     stdx::mutex m;
     stdx::condition_variable cv;
     stdx::unique_lock<stdx::mutex> lk(m);
     ASSERT(ErrorCodes::ExceededTimeLimit ==
-           txn->waitForConditionOrInterruptNoAssertUntil(cv, lk, mockClock->now()));
+           opCtx->waitForConditionOrInterruptNoAssertUntil(cv, lk, mockClock->now()));
 }
 
 class ThreadedOperationDeadlineTests : public OperationDeadlineTests {
 public:
+    using CvPred = stdx::function<bool()>;
+    using WaitFn = stdx::function<bool(
+        OperationContext*, stdx::condition_variable&, stdx::unique_lock<stdx::mutex>&, CvPred)>;
+
     struct WaitTestState {
         void signal() {
             stdx::lock_guard<stdx::mutex> lk(mutex);
@@ -225,25 +342,20 @@ public:
         bool isSignaled = false;
     };
 
-    stdx::future<stdx::cv_status> startWaiterWithUntilAndMaxTime(OperationContext* txn,
-                                                                 WaitTestState* state,
-                                                                 Date_t until,
-                                                                 Date_t maxTime) {
+    stdx::future<bool> startWaiterWithMaxTime(OperationContext* opCtx,
+                                              WaitTestState* state,
+                                              WaitFn waitFn,
+                                              Date_t maxTime) {
 
         auto barrier = std::make_shared<unittest::Barrier>(2);
-        auto task = stdx::packaged_task<stdx::cv_status()>([=]() -> stdx::cv_status {
+        auto task = stdx::packaged_task<bool()>([=] {
             if (maxTime < Date_t::max()) {
-                txn->setDeadlineByDate(maxTime);
+                opCtx->setDeadlineByDate(maxTime);
             }
             auto predicate = [state] { return state->isSignaled; };
             stdx::unique_lock<stdx::mutex> lk(state->mutex);
             barrier->countDownAndWait();
-            if (until < Date_t::max()) {
-                return txn->waitForConditionOrInterruptUntil(state->cv, lk, until, predicate);
-            } else {
-                txn->waitForConditionOrInterrupt(state->cv, lk, predicate);
-                return stdx::cv_status::no_timeout;
-            }
+            return waitFn(opCtx, state->cv, lk, predicate);
         });
         auto result = task.get_future();
         stdx::thread(std::move(task)).detach();
@@ -253,36 +365,101 @@ public:
         // barrier until it does.
         stdx::lock_guard<stdx::mutex> lk(state->mutex);
 
-        // Assuming that txn has not already been interrupted and that maxTime and until are
+        // Assuming that opCtx has not already been interrupted and that maxTime and until are
         // unexpired, we know that the waiter must be blocked in the condition variable, because it
         // held the mutex before we tried to acquire it, and only releases it on condition variable
         // wait.
         return result;
     }
 
-    stdx::future<stdx::cv_status> startWaiter(OperationContext* txn, WaitTestState* state) {
-        return startWaiterWithUntilAndMaxTime(txn, state, Date_t::max(), Date_t::max());
+    stdx::future<bool> startWaiterWithUntilAndMaxTime(OperationContext* opCtx,
+                                                      WaitTestState* state,
+                                                      Date_t until,
+                                                      Date_t maxTime) {
+        const auto waitFn = [until](OperationContext* opCtx,
+                                    stdx::condition_variable& cv,
+                                    stdx::unique_lock<stdx::mutex>& lk,
+                                    CvPred predicate) {
+            if (until < Date_t::max()) {
+                return opCtx->waitForConditionOrInterruptUntil(cv, lk, until, predicate);
+            } else {
+                opCtx->waitForConditionOrInterrupt(cv, lk, predicate);
+                return true;
+            }
+        };
+        return startWaiterWithMaxTime(opCtx, state, waitFn, maxTime);
+    }
+
+    template <typename Period>
+    stdx::future<bool> startWaiterWithDurationAndMaxTime(OperationContext* opCtx,
+                                                         WaitTestState* state,
+                                                         Duration<Period> duration,
+                                                         Date_t maxTime) {
+        const auto waitFn = [duration](OperationContext* opCtx,
+                                       stdx::condition_variable& cv,
+                                       stdx::unique_lock<stdx::mutex>& lk,
+                                       CvPred predicate) {
+            return opCtx->waitForConditionOrInterruptFor(cv, lk, duration, predicate);
+        };
+        return startWaiterWithMaxTime(opCtx, state, waitFn, maxTime);
+    }
+
+    stdx::future<bool> startWaiter(OperationContext* opCtx, WaitTestState* state) {
+        return startWaiterWithUntilAndMaxTime(opCtx, state, Date_t::max(), Date_t::max());
+    }
+
+    stdx::future<bool> startWaiterWithSleepUntilAndMaxTime(OperationContext* opCtx,
+                                                           WaitTestState* state,
+                                                           Date_t sleepUntil,
+                                                           Date_t maxTime) {
+        auto waitFn = [sleepUntil](OperationContext* opCtx,
+                                   stdx::condition_variable& cv,
+                                   stdx::unique_lock<stdx::mutex>& lk,
+                                   CvPred predicate) {
+            lk.unlock();
+            opCtx->sleepUntil(sleepUntil);
+            lk.lock();
+            return false;
+        };
+        return startWaiterWithMaxTime(opCtx, state, waitFn, maxTime);
+    }
+
+    template <typename Period>
+    stdx::future<bool> startWaiterWithSleepForAndMaxTime(OperationContext* opCtx,
+                                                         WaitTestState* state,
+                                                         Duration<Period> sleepFor,
+                                                         Date_t maxTime) {
+        auto waitFn = [sleepFor](OperationContext* opCtx,
+                                 stdx::condition_variable& cv,
+                                 stdx::unique_lock<stdx::mutex>& lk,
+                                 CvPred predicate) {
+            lk.unlock();
+            opCtx->sleepFor(sleepFor);
+            lk.lock();
+            return false;
+        };
+        return startWaiterWithMaxTime(opCtx, state, waitFn, maxTime);
     }
 };
 
 TEST_F(ThreadedOperationDeadlineTests, KillArrivesWhileWaiting) {
-    auto txn = client->makeOperationContext();
+    auto opCtx = client->makeOperationContext();
     WaitTestState state;
-    auto waiterResult = startWaiter(txn.get(), &state);
+    auto waiterResult = startWaiter(opCtx.get(), &state);
     ASSERT(stdx::future_status::ready !=
            waiterResult.wait_for(Milliseconds::zero().toSystemDuration()));
     {
-        stdx::lock_guard<Client> clientLock(*txn->getClient());
-        txn->markKilled();
+        stdx::lock_guard<Client> clientLock(*opCtx->getClient());
+        opCtx->markKilled();
     }
     ASSERT_THROWS_CODE(waiterResult.get(), DBException, ErrorCodes::Interrupted);
 }
 
 TEST_F(ThreadedOperationDeadlineTests, MaxTimeExpiresWhileWaiting) {
-    auto txn = client->makeOperationContext();
+    auto opCtx = client->makeOperationContext();
     WaitTestState state;
     const auto startDate = mockClock->now();
-    auto waiterResult = startWaiterWithUntilAndMaxTime(txn.get(),
+    auto waiterResult = startWaiterWithUntilAndMaxTime(opCtx.get(),
                                                        &state,
                                                        startDate + Seconds{60},   // until
                                                        startDate + Seconds{10});  // maxTime
@@ -297,10 +474,10 @@ TEST_F(ThreadedOperationDeadlineTests, MaxTimeExpiresWhileWaiting) {
 }
 
 TEST_F(ThreadedOperationDeadlineTests, UntilExpiresWhileWaiting) {
-    auto txn = client->makeOperationContext();
+    auto opCtx = client->makeOperationContext();
     WaitTestState state;
     const auto startDate = mockClock->now();
-    auto waiterResult = startWaiterWithUntilAndMaxTime(txn.get(),
+    auto waiterResult = startWaiterWithUntilAndMaxTime(opCtx.get(),
                                                        &state,
                                                        startDate + Seconds{10},   // until
                                                        startDate + Seconds{60});  // maxTime
@@ -311,19 +488,35 @@ TEST_F(ThreadedOperationDeadlineTests, UntilExpiresWhileWaiting) {
     ASSERT(stdx::future_status::ready !=
            waiterResult.wait_for(Milliseconds::zero().toSystemDuration()));
     mockClock->advance(Seconds{2});
-    ASSERT(stdx::cv_status::timeout == waiterResult.get());
+    ASSERT_FALSE(waiterResult.get());
+}
+
+TEST_F(ThreadedOperationDeadlineTests, ForExpiresWhileWaiting) {
+    auto opCtx = client->makeOperationContext();
+    WaitTestState state;
+    const auto startDate = mockClock->now();
+    auto waiterResult = startWaiterWithDurationAndMaxTime(
+        opCtx.get(), &state, Seconds{10}, startDate + Seconds{60});  // maxTime
+    ASSERT(stdx::future_status::ready !=
+           waiterResult.wait_for(Milliseconds::zero().toSystemDuration()))
+        << waiterResult.get();
+    mockClock->advance(Seconds{9});
+    ASSERT(stdx::future_status::ready !=
+           waiterResult.wait_for(Milliseconds::zero().toSystemDuration()));
+    mockClock->advance(Seconds{2});
+    ASSERT_FALSE(waiterResult.get());
 }
 
 TEST_F(ThreadedOperationDeadlineTests, SignalOne) {
-    auto txn = client->makeOperationContext();
+    auto opCtx = client->makeOperationContext();
     WaitTestState state;
-    auto waiterResult = startWaiter(txn.get(), &state);
+    auto waiterResult = startWaiter(opCtx.get(), &state);
 
     ASSERT(stdx::future_status::ready !=
            waiterResult.wait_for(Milliseconds::zero().toSystemDuration()))
         << waiterResult.get();
     state.signal();
-    ASSERT(stdx::cv_status::no_timeout == waiterResult.get());
+    ASSERT_TRUE(waiterResult.get());
 }
 
 TEST_F(ThreadedOperationDeadlineTests, KillOneSignalAnother) {
@@ -347,14 +540,14 @@ TEST_F(ThreadedOperationDeadlineTests, KillOneSignalAnother) {
     ASSERT(stdx::future_status::ready !=
            waiterResult2.wait_for(Milliseconds::zero().toSystemDuration()));
     state2.signal();
-    ASSERT(stdx::cv_status::no_timeout == waiterResult2.get());
+    ASSERT_TRUE(waiterResult2.get());
 }
 
 TEST_F(ThreadedOperationDeadlineTests, SignalBeforeUntilExpires) {
-    auto txn = client->makeOperationContext();
+    auto opCtx = client->makeOperationContext();
     WaitTestState state;
     const auto startDate = mockClock->now();
-    auto waiterResult = startWaiterWithUntilAndMaxTime(txn.get(),
+    auto waiterResult = startWaiterWithUntilAndMaxTime(opCtx.get(),
                                                        &state,
                                                        startDate + Seconds{10},   // until
                                                        startDate + Seconds{60});  // maxTime
@@ -365,14 +558,14 @@ TEST_F(ThreadedOperationDeadlineTests, SignalBeforeUntilExpires) {
     ASSERT(stdx::future_status::ready !=
            waiterResult.wait_for(Milliseconds::zero().toSystemDuration()));
     state.signal();
-    ASSERT(stdx::cv_status::no_timeout == waiterResult.get());
+    ASSERT_TRUE(waiterResult.get());
 }
 
 TEST_F(ThreadedOperationDeadlineTests, SignalBeforeMaxTimeExpires) {
-    auto txn = client->makeOperationContext();
+    auto opCtx = client->makeOperationContext();
     WaitTestState state;
     const auto startDate = mockClock->now();
-    auto waiterResult = startWaiterWithUntilAndMaxTime(txn.get(),
+    auto waiterResult = startWaiterWithUntilAndMaxTime(opCtx.get(),
                                                        &state,
                                                        startDate + Seconds{60},   // until
                                                        startDate + Seconds{10});  // maxTime
@@ -383,7 +576,44 @@ TEST_F(ThreadedOperationDeadlineTests, SignalBeforeMaxTimeExpires) {
     ASSERT(stdx::future_status::ready !=
            waiterResult.wait_for(Milliseconds::zero().toSystemDuration()));
     state.signal();
-    ASSERT(stdx::cv_status::no_timeout == waiterResult.get());
+    ASSERT_TRUE(waiterResult.get());
+}
+
+TEST_F(ThreadedOperationDeadlineTests, SleepUntilWithExpiredUntilDoesNotBlock) {
+    auto opCtx = client->makeOperationContext();
+    WaitTestState state;
+    const auto startDate = mockClock->now();
+    auto waiterResult = startWaiterWithSleepUntilAndMaxTime(opCtx.get(),
+                                                            &state,
+                                                            startDate - Seconds{10},   // until
+                                                            startDate + Seconds{60});  // maxTime
+    ASSERT_FALSE(waiterResult.get());
+}
+
+TEST_F(ThreadedOperationDeadlineTests, SleepUntilExpires) {
+    auto opCtx = client->makeOperationContext();
+    WaitTestState state;
+    const auto startDate = mockClock->now();
+    auto waiterResult = startWaiterWithSleepUntilAndMaxTime(opCtx.get(),
+                                                            &state,
+                                                            startDate + Seconds{10},   // until
+                                                            startDate + Seconds{60});  // maxTime
+    ASSERT(stdx::future_status::ready !=
+           waiterResult.wait_for(Milliseconds::zero().toSystemDuration()));
+    mockClock->advance(Seconds{9});
+    ASSERT(stdx::future_status::ready !=
+           waiterResult.wait_for(Milliseconds::zero().toSystemDuration()));
+    mockClock->advance(Seconds{2});
+    ASSERT_FALSE(waiterResult.get());
+}
+
+TEST_F(ThreadedOperationDeadlineTests, SleepForWithExpiredForDoesNotBlock) {
+    auto opCtx = client->makeOperationContext();
+    WaitTestState state;
+    const auto startDate = mockClock->now();
+    auto waiterResult = startWaiterWithSleepForAndMaxTime(
+        opCtx.get(), &state, Seconds{-10}, startDate + Seconds{60});  // maxTime
+    ASSERT_FALSE(waiterResult.get());
 }
 
 }  // namespace
