@@ -38,6 +38,7 @@
 
 namespace mongo {
 
+class CollatorInterface;
 class LogBuilder;
 
 class ModifierCompare : public ModifierInterface {
@@ -82,6 +83,11 @@ public:
      */
     virtual Status log(LogBuilder* logBuilder) const;
 
+    virtual void setCollator(const CollatorInterface* collator) {
+        invariant(!_collator);
+        _collator = collator;
+    }
+
 private:
     // Compare mode: min/max
     const ModifierCompareMode _mode;
@@ -100,6 +106,8 @@ private:
     // being prepared against must be live throughout all the calls.
     struct PreparedState;
     std::unique_ptr<PreparedState> _preparedState;
+
+    const CollatorInterface* _collator = nullptr;
 };
 
 }  // namespace mongo
