@@ -33,6 +33,7 @@
 #include "mongo/bson/mutable/element.h"
 #include "mongo/db/field_ref.h"
 #include "mongo/db/jsobj.h"
+#include "mongo/db/pipeline/expression_context.h"
 
 namespace mongo {
 
@@ -168,20 +169,19 @@ public:
  * Options used to control Modifier behavior
  */
 struct ModifierInterface::Options {
-    Options() = default;
-    Options(bool repl, bool ofs, const CollatorInterface* collator)
-        : fromReplication(repl), enforceOkForStorage(ofs), collator(collator) {}
+    Options(bool fromOpLog, bool ofs, boost::intrusive_ptr<ExpressionContext> expCtx)
+        : fromOplogApplication(fromOpLog), enforceOkForStorage(ofs), expCtx(std::move(expCtx)) {}
 
-    static Options normal(const CollatorInterface* collator = nullptr) {
-        return Options(false, true, collator);
+    static Options normal(boost::intrusive_ptr<ExpressionContext> expCtx) {
+        return Options(false, true, std::move(expCtx));
     }
-    static Options fromRepl(const CollatorInterface* collator = nullptr) {
-        return Options(true, false, collator);
+    static Options fromRepl(boost::intrusive_ptr<ExpressionContext> expCtx) {
+        return Options(true, false, std::move(expCtx));
     }
 
-    bool fromReplication = false;
+    bool fromOplogApplication = false;
     bool enforceOkForStorage = true;
-    const CollatorInterface* collator = nullptr;
+    boost::intrusive_ptr<ExpressionContext> expCtx;
 };
 
 struct ModifierInterface::ExecInfo {
@@ -205,11 +205,17 @@ struct ModifierInterface::ExecInfo {
     ExecInfo() : noOp(false), context(ANY_CONTEXT) {
         for (int i = 0; i < MAX_NUM_FIELDS; i++) {
             fieldRef[i] = NULL;
+            indexOfArrayWithNewElement[i] = boost::none;
         }
     }
 
     // The fields of concern to the driver: no other op may modify the fields listed here.
     FieldRef* fieldRef[MAX_NUM_FIELDS];  // not owned here
+
+    // For each modified field ref, the index of the path component representing an existing array
+    // that gained a new element.
+    boost::optional<size_t> indexOfArrayWithNewElement[MAX_NUM_FIELDS];
+
     bool noOp;
     UpdateContext context;
 };
