@@ -42,8 +42,9 @@
 #include "mongo/db/bson/dotted_path_support.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/json.h"
-#include "mongo/db/ops/log_builder.h"
+#include "mongo/db/pipeline/expression_context_for_test.h"
 #include "mongo/db/query/collation/collator_interface_mock.h"
+#include "mongo/db/update/log_builder.h"
 #include "mongo/unittest/unittest.h"
 #include "mongo/util/mongoutils/str.h"
 
@@ -53,6 +54,7 @@ using mongo::BSONObj;
 using mongo::BSONObjBuilder;
 using mongo::BSONArrayBuilder;
 using mongo::CollatorInterfaceMock;
+using mongo::ExpressionContextForTest;
 using mongo::fromjson;
 using mongo::LogBuilder;
 using mongo::ModifierInterface;
@@ -165,8 +167,9 @@ void combineAndSortVec(const vector<BSONObj>& origVec,
 TEST(Init, SimplePush) {
     BSONObj modObj = fromjson("{$push: {x: 0}}");
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                       ModifierInterface::Options::normal()));
+                       ModifierInterface::Options::normal(expCtx)));
 }
 
 //
@@ -176,46 +179,52 @@ TEST(Init, SimplePush) {
 TEST(Init, PushEachNormal) {
     BSONObj modObj = fromjson("{$push: {x: {$each: [1, 2]}}}");
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                       ModifierInterface::Options::normal()));
+                       ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachMixed) {
     BSONObj modObj = fromjson("{$push: {x: {$each: [1, {a: 2}]}}}");
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                       ModifierInterface::Options::normal()));
+                       ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachObject) {
     // $each must be an array
     BSONObj modObj = fromjson("{$push: {x: {$each: {'0': 1}}}}");
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_NOT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                           ModifierInterface::Options::normal()));
+                           ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachSimpleType) {
     // $each must be an array.
     BSONObj modObj = fromjson("{$push: {x: {$each: 1}}}");
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_NOT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                           ModifierInterface::Options::normal()));
+                           ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachEmpty) {
     BSONObj modObj = fromjson("{$push: {x: {$each: []}}}");
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                       ModifierInterface::Options::normal()));
+                       ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachInvalidType) {
     // $each must be an array.
     BSONObj modObj = fromjson("{$push: {x: {$each: {b: 1}}}}");
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_NOT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                           ModifierInterface::Options::normal()));
+                           ModifierInterface::Options::normal(expCtx)));
 }
 
 //
@@ -225,50 +234,57 @@ TEST(Init, PushEachInvalidType) {
 TEST(Init, PushEachWithSliceBottom) {
     BSONObj modObj = fromjson("{$push: {x: {$each: [1, 2], $slice: -3}}}");
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                       ModifierInterface::Options::normal()));
+                       ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachWithSliceTop) {
     BSONObj modObj = fromjson("{$push: {x: {$each: [1, 2], $slice: 3}}}");
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                       ModifierInterface::Options::normal()));
+                       ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachWithInvalidSliceObject) {
     BSONObj modObj = fromjson("{$push: {x: {$each: [1, 2], $slice: {a: 1}}}}");
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_NOT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                           ModifierInterface::Options::normal()));
+                           ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachWithInvalidSliceDouble) {
     BSONObj modObj = fromjson("{$push: {x: {$each: [1, 2], $slice: -2.1}}}");
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_NOT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                           ModifierInterface::Options::normal()));
+                           ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachWithValidSliceDouble) {
     BSONObj modObj = fromjson("{$push: {x: {$each: [1, 2], $slice: -2.0}}}");
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                       ModifierInterface::Options::normal()));
+                       ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachWithUnsupportedFullSlice) {
     BSONObj modObj = fromjson("{$push: {x: {$each: [1, 2], $slice: [1,2]}}}");
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_NOT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                           ModifierInterface::Options::normal()));
+                           ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachWithWrongTypeSlice) {
     BSONObj modObj = fromjson("{$push: {x: {$each: [1, 2], $slice: '-1'}}}");
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_NOT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                           ModifierInterface::Options::normal()));
+                           ModifierInterface::Options::normal(expCtx)));
 }
 
 //
@@ -279,134 +295,99 @@ TEST(Init, PushEachWithObjectSort) {
     const char* c = "{$push: {x: {$each: [{a:1},{a:2}], $slice: -2.0, $sort: {a:1}}}}";
     BSONObj modObj = fromjson(c);
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                       ModifierInterface::Options::normal()));
+                       ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachWithNumbericSort) {
     const char* c = "{$push: {x: {$each: [{a:1},{a:2}], $slice: -2.0, $sort:1 }}}";
     BSONObj modObj = fromjson(c);
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                       ModifierInterface::Options::normal()));
+                       ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachWithInvalidSortType) {
     const char* c = "{$push: {x: {$each: [{a:1},{a:2}], $slice: -2.0, $sort: [{a:1}]}}}";
     BSONObj modObj = fromjson(c);
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_NOT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                           ModifierInterface::Options::normal()));
+                           ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachDuplicateSortPattern) {
     const char* c = "{$push: {x: {$each: [{a:1},{a:2}], $slice: -2.0, $sort: [{a:1,a:1}]}}}";
     BSONObj modObj = fromjson(c);
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_NOT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                           ModifierInterface::Options::normal()));
+                           ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachWithInvalidSortValue) {
     const char* c = "{$push: {x: {$each: [{a:1},{a:2}], $slice: -2.0, $sort: {a:100}}}}";
     BSONObj modObj = fromjson(c);
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_NOT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                           ModifierInterface::Options::normal()));
+                           ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachWithEmptySortField) {
     const char* c = "{$push: {x: {$each: [{a:1},{a:2}], $slice: -2.0, $sort: {'':1}}}}";
     BSONObj modObj = fromjson(c);
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_NOT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                           ModifierInterface::Options::normal()));
+                           ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachWithEmptyDottedSortField) {
     const char* c = "{$push: {x: {$each: [{a:1},{a:2}], $slice: -2.0, $sort: {'.':1}}}}";
     BSONObj modObj = fromjson(c);
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_NOT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                           ModifierInterface::Options::normal()));
+                           ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachWithMissingSortFieldSuffix) {
     const char* c = "{$push: {x: {$each: [{a:1},{a:2}], $slice: -2.0, $sort: {'a.':1}}}}";
     BSONObj modObj = fromjson(c);
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_NOT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                           ModifierInterface::Options::normal()));
+                           ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachWithMissingSortFieldPreffix) {
     const char* c = "{$push: {x: {$each: [{a:1},{a:2}], $slice: -2.0, $sort: {'.b':1}}}}";
     BSONObj modObj = fromjson(c);
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_NOT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                           ModifierInterface::Options::normal()));
+                           ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachWithMissingSortFieldMiddle) {
     const char* c = "{$push: {x: {$each: [{a:1},{a:2}], $slice: -2.0, $sort: {'a..b':1}}}}";
     BSONObj modObj = fromjson(c);
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_NOT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                           ModifierInterface::Options::normal()));
+                           ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachWithEmptySort) {
     const char* c = "{$push: {x: {$each: [{a:1},{a:2}], $slice: -2.0, $sort:{} }}}";
     BSONObj modObj = fromjson(c);
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_NOT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                           ModifierInterface::Options::normal()));
-}
-
-//
-// If in $pushAll semantics, do we check the array and that nothing else is there?
-//
-
-TEST(Init, PushAllSimple) {
-    BSONObj modObj = fromjson("{$pushAll: {x: [0]}}");
-    ModifierPush mod(ModifierPush::PUSH_ALL);
-    ASSERT_OK(mod.init(modObj["$pushAll"].embeddedObject().firstElement(),
-                       ModifierInterface::Options::normal()));
-}
-
-TEST(Init, PushAllMultiple) {
-    BSONObj modObj = fromjson("{$pushAll: {x: [1,2,3]}}");
-    ModifierPush mod(ModifierPush::PUSH_ALL);
-    ASSERT_OK(mod.init(modObj["$pushAll"].embeddedObject().firstElement(),
-                       ModifierInterface::Options::normal()));
-}
-
-TEST(Init, PushAllObject) {
-    BSONObj modObj = fromjson("{$pushAll: {x: [{a:1},{a:2}]}}");
-    ModifierPush mod(ModifierPush::PUSH_ALL);
-    ASSERT_OK(mod.init(modObj["$pushAll"].embeddedObject().firstElement(),
-                       ModifierInterface::Options::normal()));
-}
-
-TEST(Init, PushAllMixed) {
-    BSONObj modObj = fromjson("{$pushAll: {x: [1,{a:2}]}}");
-    ModifierPush mod(ModifierPush::PUSH_ALL);
-    ASSERT_OK(mod.init(modObj["$pushAll"].embeddedObject().firstElement(),
-                       ModifierInterface::Options::normal()));
-}
-
-TEST(Init, PushAllWrongType) {
-    BSONObj modObj = fromjson("{$pushAll: {x: 1}}");
-    ModifierPush mod(ModifierPush::PUSH_ALL);
-    ASSERT_NOT_OK(mod.init(modObj["$pushAll"].embeddedObject().firstElement(),
-                           ModifierInterface::Options::normal()));
-}
-
-TEST(Init, PushAllNotArray) {
-    BSONObj modObj = fromjson("{$pushAll: {x: {a:1}}}");
-    ModifierPush mod(ModifierPush::PUSH_ALL);
-    ASSERT_NOT_OK(mod.init(modObj["$pushAll"].embeddedObject().firstElement(),
-                           ModifierInterface::Options::normal()));
+                           ModifierInterface::Options::normal(expCtx)));
 }
 
 //
@@ -417,80 +398,87 @@ TEST(Init, PushEachWithSortMissingSlice) {
     const char* c = "{$push: {x: {$each: [{a:1},{a:2}], $sort:{a:1}}}}";
     BSONObj modObj = fromjson(c);
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                       ModifierInterface::Options::normal()));
+                       ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachInvalidClause) {
     const char* c = "{$push: {x: {$each: [{a:1},{a:2}], $xxx: -1, $sort:{a:1}}}}";
     BSONObj modObj = fromjson(c);
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_NOT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                           ModifierInterface::Options::normal()));
+                           ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachExtraField) {
     const char* c = "{$push: {x: {$each: [{a:1},{a:2}], $slice: -2.0, $sort: {a:1}, b: 1}}}";
     BSONObj modObj = fromjson(c);
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_NOT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                           ModifierInterface::Options::normal()));
+                           ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachDuplicateSortClause) {
     const char* c = "{$push: {x:{$each:[{a:1},{a:2}], $slice:-2.0, $sort:{a:1}, $sort:{a:1}}}}";
     BSONObj modObj = fromjson(c);
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_NOT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                           ModifierInterface::Options::normal()));
+                           ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachDuplicateSliceClause) {
     const char* c = "{$push: {x: {$each:[{a:1},{a:2}], $slice:-2.0, $slice:-2, $sort:{a:1}}}}";
     BSONObj modObj = fromjson(c);
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_NOT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                           ModifierInterface::Options::normal()));
+                           ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachDuplicateEachClause) {
     const char* c = "{$push: {x: {$each:[{a:1}], $each:[{a:2}], $slice:-3, $sort:{a:1}}}}";
     BSONObj modObj = fromjson(c);
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_NOT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                           ModifierInterface::Options::normal()));
+                           ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachWithSliceFirst) {
     const char* c = "{$push: {x: {$slice: -2.0, $each: [{a:1},{a:2}], $sort: {a:1}}}}";
     BSONObj modObj = fromjson(c);
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                       ModifierInterface::Options::normal()));
+                       ModifierInterface::Options::normal(expCtx)));
 }
 
 TEST(Init, PushEachWithSortFirst) {
     const char* c = "{$push: {x: {$sort: {a:1}, $slice: -2.0, $each: [{a:1},{a:2}]}}}";
     BSONObj modObj = fromjson(c);
     ModifierPush mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_OK(mod.init(modObj["$push"].embeddedObject().firstElement(),
-                       ModifierInterface::Options::normal()));
+                       ModifierInterface::Options::normal(expCtx)));
 }
 
 //
 // Simple mod
 //
 
-/** Helper to build and manipulate a $push or a $pushAll mod. */
+/** Helper to build and manipulate a $push mod. */
 class Mod {
 public:
     Mod() : _mod() {}
 
     explicit Mod(BSONObj modObj,
-                 ModifierInterface::Options options = ModifierInterface::Options::normal())
-        : _mod(mongoutils::str::equals(modObj.firstElement().fieldName(), "$pushAll")
-                   ? ModifierPush::PUSH_ALL
-                   : ModifierPush::PUSH_NORMAL) {
+                 ModifierInterface::Options options =
+                     ModifierInterface::Options::normal(new ExpressionContextForTest()))
+        : _mod() {
         _modObj = modObj;
         StringData modName = modObj.firstElement().fieldName();
         ASSERT_OK(_mod.init(_modObj[modName].embeddedObject().firstElement(), options));
@@ -697,73 +685,6 @@ TEST(SimpleObjMod, PrepareApplyDotted) {
 }
 
 
-//
-// $pushAll Variation
-//
-
-TEST(PushAll, PrepareApplyEmpty) {
-    Document doc(fromjson("{a: []}"));
-    Mod pushMod(fromjson("{$pushAll: {a: [1]}}"));
-
-    ModifierInterface::ExecInfo execInfo;
-    ASSERT_OK(pushMod.prepare(doc.root(), "", &execInfo));
-
-    ASSERT_EQUALS(execInfo.fieldRef[0]->dottedField(), "a");
-    ASSERT_FALSE(execInfo.noOp);
-
-    ASSERT_OK(pushMod.apply());
-    ASSERT_FALSE(doc.isInPlaceModeEnabled());
-    ASSERT_EQUALS(doc, fromjson("{a: [1]}"));
-
-    Document logDoc;
-    LogBuilder logBuilder(logDoc.root());
-    ASSERT_OK(pushMod.log(&logBuilder));
-    ASSERT_EQUALS(countChildren(logDoc.root()), 1u);
-    ASSERT_EQUALS(logDoc, fromjson("{$set: {a: [1]}}"));
-}
-
-TEST(PushAll, PrepareApplyInexistent) {
-    Document doc(fromjson("{}"));
-    Mod pushMod(fromjson("{$pushAll: {a: [1]}}"));
-
-    ModifierInterface::ExecInfo execInfo;
-    ASSERT_OK(pushMod.prepare(doc.root(), "", &execInfo));
-
-    ASSERT_EQUALS(execInfo.fieldRef[0]->dottedField(), "a");
-    ASSERT_FALSE(execInfo.noOp);
-
-    ASSERT_OK(pushMod.apply());
-    ASSERT_FALSE(doc.isInPlaceModeEnabled());
-    ASSERT_EQUALS(doc, fromjson("{a: [1]}"));
-
-    Document logDoc;
-    LogBuilder logBuilder(logDoc.root());
-    ASSERT_OK(pushMod.log(&logBuilder));
-    ASSERT_EQUALS(countChildren(logDoc.root()), 1u);
-    ASSERT_EQUALS(logDoc, fromjson("{$set: {a: [1]}}"));
-}
-
-TEST(PushAll, PrepareApplyNormal) {
-    Document doc(fromjson("{a: [0]}"));
-    Mod pushMod(fromjson("{$pushAll: {a: [1,2]}}"));
-
-    ModifierInterface::ExecInfo execInfo;
-    ASSERT_OK(pushMod.prepare(doc.root(), "", &execInfo));
-
-    ASSERT_EQUALS(execInfo.fieldRef[0]->dottedField(), "a");
-    ASSERT_FALSE(execInfo.noOp);
-
-    ASSERT_OK(pushMod.apply());
-    ASSERT_FALSE(doc.isInPlaceModeEnabled());
-    ASSERT_EQUALS(doc, fromjson("{a: [0,1,2]}"));
-
-    Document logDoc;
-    LogBuilder logBuilder(logDoc.root());
-    ASSERT_OK(pushMod.log(&logBuilder));
-    ASSERT_EQUALS(countChildren(logDoc.root()), 1u);
-    ASSERT_EQUALS(logDoc, fromjson("{$set: {'a.1': 1, 'a.2':2}}"));
-}
-
 //
 // Simple $each mod
 //
@@ -1019,8 +940,10 @@ TEST(SortPushEach, MixedSortEmbeddedField) {
 TEST(SortPushEach, SortRespectsCollationFromOptions) {
     Document doc(fromjson("{a: ['dd', 'fc', 'gb'] }"));
     CollatorInterfaceMock collator(CollatorInterfaceMock::MockType::kReverseString);
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    expCtx->setCollator(&collator);
     Mod pushMod(fromjson("{$push: {a: {$each: ['ha'], $sort: 1}}}"),
-                ModifierInterface::Options::normal(&collator));
+                ModifierInterface::Options::normal(expCtx));
     const BSONObj expectedObj = fromjson("{a: ['ha', 'gb', 'fc', 'dd']}");
 
     ModifierInterface::ExecInfo execInfo;
@@ -1082,8 +1005,9 @@ public:
         _modObj =
             BSON("$push" << BSON("a" << BSON("$each" << arrBuilder.arr() << "$slice" << slice)));
 
+        boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
         ASSERT_OK(_mod.init(_modObj["$push"].embeddedObject().firstElement(),
-                            ModifierInterface::Options::normal()));
+                            ModifierInterface::Options::normal(expCtx)));
     }
 
     /** Sets up the mod to be {$push: {a: {$each:[<Obj>,...], $slice:<slice>, $sort:<Obj>}}} */
@@ -1097,8 +1021,9 @@ public:
             "$push" << BSON(
                 "a" << BSON("$each" << arrBuilder.arr() << "$slice" << slice << "$sort" << sort)));
 
+        boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
         ASSERT_OK(_mod.init(_modObj["$push"].embeddedObject().firstElement(),
-                            ModifierInterface::Options::normal()));
+                            ModifierInterface::Options::normal(expCtx)));
     }
 
     /** Returns an object {a: [<'vec's content>]} */
@@ -1305,8 +1230,9 @@ TEST_F(SlicedMod, ObjectArrayFromExisting) {
 // Push to position tests
 
 TEST(ToPosition, BadInputs) {
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+
     const char* const bad[] = {
-        "{$push: {a: { $each: [1], $position:-1}}}",
         "{$push: {a: { $each: [1], $position:'s'}}}",
         "{$push: {a: { $each: [1], $position:{}}}}",
         "{$push: {a: { $each: [1], $position:[0]}}}",
@@ -1322,10 +1248,10 @@ TEST(ToPosition, BadInputs) {
 
     int i = 0;
     while (bad[i] != NULL) {
-        ModifierPush pushMod(ModifierPush::PUSH_NORMAL);
+        ModifierPush pushMod;
         BSONObj modObj = fromjson(bad[i]);
         ASSERT_NOT_OK(pushMod.init(modObj.firstElement().embeddedObject().firstElement(),
-                                   ModifierInterface::Options::normal()));
+                                   ModifierInterface::Options::normal(expCtx)));
         i++;
     }
 }
@@ -1473,6 +1399,111 @@ TEST(ToPosition, Back) {
     ASSERT_EQUALS(fromjson("{$set: {'a.1':1}}"), logDoc);
 }
 
+TEST(ToPosition, NegativePositionEmptyArray) {
+    Document doc(fromjson("{a: []}"));
+    Mod pushMod(fromjson("{$push: {a: {$each: [1], $position: -1}}}"));
+
+    ModifierInterface::ExecInfo execInfo;
+    ASSERT_OK(pushMod.prepare(doc.root(), "", &execInfo));
+
+    ASSERT_EQUALS(execInfo.fieldRef[0]->dottedField(), "a");
+    ASSERT_FALSE(execInfo.noOp);
+
+    ASSERT_OK(pushMod.apply());
+    ASSERT_FALSE(doc.isInPlaceModeEnabled());
+    ASSERT_EQUALS(fromjson("{a: [1]}"), doc);
+
+    Document logDoc;
+    LogBuilder logBuilder(logDoc.root());
+    ASSERT_OK(pushMod.log(&logBuilder));
+    ASSERT_EQUALS(countChildren(logDoc.root()), 1u);
+    ASSERT_EQUALS(fromjson("{$set: {'a': [1]}}"), logDoc);
+}
+
+TEST(ToPosition, NegativePositionOneElementArray) {
+    Document doc(fromjson("{a: [0]}"));
+    Mod pushMod(fromjson("{$push: {a: {$each: [1], $position: -1}}}"));
+
+    ModifierInterface::ExecInfo execInfo;
+    ASSERT_OK(pushMod.prepare(doc.root(), "", &execInfo));
+
+    ASSERT_EQUALS(execInfo.fieldRef[0]->dottedField(), "a");
+    ASSERT_FALSE(execInfo.noOp);
+
+    ASSERT_OK(pushMod.apply());
+    ASSERT_FALSE(doc.isInPlaceModeEnabled());
+    ASSERT_EQUALS(fromjson("{a: [1, 0]}"), doc);
+
+    Document logDoc;
+    LogBuilder logBuilder(logDoc.root());
+    ASSERT_OK(pushMod.log(&logBuilder));
+    ASSERT_EQUALS(countChildren(logDoc.root()), 1u);
+    ASSERT_EQUALS(fromjson("{$set: {'a': [1, 0]}}"), logDoc);
+}
+
+TEST(ToPosition, NegativePositionMultiElementArray) {
+    Document doc(fromjson("{a: [0, 1, 2, 3, 4]}"));
+    Mod pushMod(fromjson("{$push: {a: {$each: [5], $position: -2}}}"));
+
+    ModifierInterface::ExecInfo execInfo;
+    ASSERT_OK(pushMod.prepare(doc.root(), "", &execInfo));
+
+    ASSERT_EQUALS(execInfo.fieldRef[0]->dottedField(), "a");
+    ASSERT_FALSE(execInfo.noOp);
+
+    ASSERT_OK(pushMod.apply());
+    ASSERT_FALSE(doc.isInPlaceModeEnabled());
+    ASSERT_EQUALS(fromjson("{a: [0, 1, 2, 5, 3, 4]}"), doc);
+
+    Document logDoc;
+    LogBuilder logBuilder(logDoc.root());
+    ASSERT_OK(pushMod.log(&logBuilder));
+    ASSERT_EQUALS(countChildren(logDoc.root()), 1u);
+    ASSERT_EQUALS(fromjson("{$set: {'a': [0, 1, 2, 5, 3, 4]}}"), logDoc);
+}
+
+TEST(ToPosition, NegativePositionOutOfBoundsDefaultsToBeginning) {
+    Document doc(fromjson("{a: [0]}"));
+    Mod pushMod(fromjson("{$push: {a: {$each: [1], $position: -2}}}"));
+
+    ModifierInterface::ExecInfo execInfo;
+    ASSERT_OK(pushMod.prepare(doc.root(), "", &execInfo));
+
+    ASSERT_EQUALS(execInfo.fieldRef[0]->dottedField(), "a");
+    ASSERT_FALSE(execInfo.noOp);
+
+    ASSERT_OK(pushMod.apply());
+    ASSERT_FALSE(doc.isInPlaceModeEnabled());
+    ASSERT_EQUALS(fromjson("{a: [1, 0]}"), doc);
+
+    Document logDoc;
+    LogBuilder logBuilder(logDoc.root());
+    ASSERT_OK(pushMod.log(&logBuilder));
+    ASSERT_EQUALS(countChildren(logDoc.root()), 1u);
+    ASSERT_EQUALS(fromjson("{$set: {'a': [1, 0]}}"), logDoc);
+}
+
+TEST(ToPosition, NegativePositionPushMultipleElements) {
+    Document doc(fromjson("{a: [0, 1, 2, 3, 4]}"));
+    Mod pushMod(fromjson("{$push: {a: {$each: [5, 6, 7], $position: -2}}}"));
+
+    ModifierInterface::ExecInfo execInfo;
+    ASSERT_OK(pushMod.prepare(doc.root(), "", &execInfo));
+
+    ASSERT_EQUALS(execInfo.fieldRef[0]->dottedField(), "a");
+    ASSERT_FALSE(execInfo.noOp);
+
+    ASSERT_OK(pushMod.apply());
+    ASSERT_FALSE(doc.isInPlaceModeEnabled());
+    ASSERT_EQUALS(fromjson("{a: [0, 1, 2, 5, 6, 7, 3, 4]}"), doc);
+
+    Document logDoc;
+    LogBuilder logBuilder(logDoc.root());
+    ASSERT_OK(pushMod.log(&logBuilder));
+    ASSERT_EQUALS(countChildren(logDoc.root()), 1u);
+    ASSERT_EQUALS(fromjson("{$set: {'a': [0, 1, 2, 5, 6, 7, 3, 4]}}"), logDoc);
+}
+
 TEST(IndexedMod, PrepareReportCreatedArrayElement) {
     Document doc(fromjson("{a: [{b: 0}]}"));
     Mod mod(fromjson("{$push: {'a.1.c': 2}}"));
@@ -1486,22 +1517,6 @@ TEST(IndexedMod, PrepareReportCreatedArrayElement) {
     ASSERT_FALSE(execInfo.noOp);
 }
 
-TEST(IndexedMod, PrepareReportCreatedArrayElementPushAll) {
-    Document doc(fromjson("{a: [{b: 0}]}"));
-    auto modObj = fromjson("{$pushAll: {'a.1.c': [2]}}");
-    ModifierPush mod(ModifierPush::PUSH_ALL);
-    ASSERT_OK(mod.init(modObj["$pushAll"].embeddedObject().firstElement(),
-                       ModifierInterface::Options::normal()));
-
-    ModifierInterface::ExecInfo execInfo;
-    ASSERT_OK(mod.prepare(doc.root(), "", &execInfo));
-
-    ASSERT_EQUALS(execInfo.fieldRef[0]->dottedField(), "a.1.c");
-    ASSERT_TRUE(execInfo.indexOfArrayWithNewElement[0]);
-    ASSERT_EQUALS(*execInfo.indexOfArrayWithNewElement[0], 0u);
-    ASSERT_FALSE(execInfo.noOp);
-}
-
 TEST(IndexedMod, PrepareDoNotReportModifiedArrayElement) {
     Document doc(fromjson("{a: [{b: 0}]}"));
     Mod mod(fromjson("{$push: {'a.0.c': 2}}"));
@@ -1514,21 +1529,6 @@ TEST(IndexedMod, PrepareDoNotReportModifiedArrayElement) {
     ASSERT_FALSE(execInfo.noOp);
 }
 
-TEST(IndexedMod, PrepareDoNotReportModifiedArrayElementPushAll) {
-    Document doc(fromjson("{a: [{b: 0}]}"));
-    auto modObj = fromjson("{$pushAll: {'a.0.c': [2]}}");
-    ModifierPush mod(ModifierPush::PUSH_ALL);
-    ASSERT_OK(mod.init(modObj["$pushAll"].embeddedObject().firstElement(),
-                       ModifierInterface::Options::normal()));
-
-    ModifierInterface::ExecInfo execInfo;
-    ASSERT_OK(mod.prepare(doc.root(), "", &execInfo));
-
-    ASSERT_EQUALS(execInfo.fieldRef[0]->dottedField(), "a.0.c");
-    ASSERT_FALSE(execInfo.indexOfArrayWithNewElement[0]);
-    ASSERT_FALSE(execInfo.noOp);
-}
-
 TEST(IndexedMod, PrepareDoNotReportCreatedNumericObjectField) {
     Document doc(fromjson("{a: {'0': {b: 0}}}"));
     Mod mod(fromjson("{$push: {'a.1.c': 2}}"));
@@ -1541,19 +1541,4 @@ TEST(IndexedMod, PrepareDoNotReportCreatedNumericObjectField) {
     ASSERT_FALSE(execInfo.noOp);
 }
 
-TEST(IndexedMod, PrepareDoNotReportCreatedNumericObjectFieldPushAll) {
-    Document doc(fromjson("{a: {'0': {b: 0}}}"));
-    auto modObj = fromjson("{$pushAll: {'a.1.c': [2]}}");
-    ModifierPush mod(ModifierPush::PUSH_ALL);
-    ASSERT_OK(mod.init(modObj["$pushAll"].embeddedObject().firstElement(),
-                       ModifierInterface::Options::normal()));
-
-    ModifierInterface::ExecInfo execInfo;
-    ASSERT_OK(mod.prepare(doc.root(), "", &execInfo));
-
-    ASSERT_EQUALS(execInfo.fieldRef[0]->dottedField(), "a.1.c");
-    ASSERT_FALSE(execInfo.indexOfArrayWithNewElement[0]);
-    ASSERT_FALSE(execInfo.noOp);
-}
-
 }  // unnamed namespace
