@@ -31,9 +31,9 @@
 #include "mongo/base/error_codes.h"
 #include "mongo/bson/mutable/algorithm.h"
 #include "mongo/bson/mutable/document.h"
-#include "mongo/db/ops/field_checker.h"
-#include "mongo/db/ops/log_builder.h"
-#include "mongo/db/ops/path_support.h"
+#include "mongo/db/update/field_checker.h"
+#include "mongo/db/update/log_builder.h"
+#include "mongo/db/update/path_support.h"
 #include "mongo/util/mongoutils/str.h"
 
 namespace mongo {
@@ -255,7 +255,8 @@ Status ModifierRename::apply() const {
     return pathsupport::createPathAt(_toFieldRef,
                                      tempElem == doc.end() ? 0 : tempIdx + 1,
                                      tempElem == doc.end() ? doc.root() : tempElem,
-                                     elemToSet);
+                                     elemToSet)
+        .getStatus();
 }
 
 Status ModifierRename::log(LogBuilder* logBuilder) const {
