@@ -38,12 +38,14 @@
 #include "mongo/bson/mutable/mutable_bson_test_utils.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/json.h"
-#include "mongo/db/ops/log_builder.h"
+#include "mongo/db/pipeline/expression_context_for_test.h"
+#include "mongo/db/update/log_builder.h"
 #include "mongo/unittest/unittest.h"
 
 namespace {
 
 using mongo::BSONObj;
+using mongo::ExpressionContextForTest;
 using mongo::fromjson;
 using mongo::LogBuilder;
 using mongo::ModifierInterface;
@@ -67,9 +69,10 @@ public:
                    : ModifierSet::SET_NORMAL) {
         _modObj = modObj;
         StringData modName = modObj.firstElement().fieldName();
-        ASSERT_OK(_mod.init(_modObj[modName].embeddedObject().firstElement(),
-                            !fromRepl ? ModifierInterface::Options::normal()
-                                      : ModifierInterface::Options::fromRepl()));
+        ASSERT_OK(_mod.init(
+            _modObj[modName].embeddedObject().firstElement(),
+            !fromRepl ? ModifierInterface::Options::normal(new ExpressionContextForTest())
+                      : ModifierInterface::Options::fromRepl(new ExpressionContextForTest())));
     }
 
     Status prepare(Element root, StringData matchedField, ModifierInterface::ExecInfo* execInfo) {
@@ -100,8 +103,9 @@ private:
 TEST(Init, EmptyOperation) {
     BSONObj modObj = fromjson("{$set: {}}");
     ModifierSet mod;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     ASSERT_NOT_OK(mod.init(modObj["$set"].embeddedObject().firstElement(),
-                           ModifierInterface::Options::normal()));
+                           ModifierInterface::Options::normal(expCtx)));
 }
 
 //
@@ -134,7 +138,7 @@ TEST(SimpleMod, PrepareIdentityOpOnDeserializedIsNotANoOp) {
     Document doc(fromjson("{a: { b: NumberInt(0)}}"));
 
     // Apply a mutation to the document that will make it non-serialized.
-    doc.root()["a"]["b"].setValueInt(2);
+    doc.root()["a"]["b"].setValueInt(2).transitional_ignore();
 
     // Apply an op that would be a no-op.
     Mod setMod(fromjson("{$set: {a: {b : NumberInt(2)}}}"));
@@ -823,9 +827,9 @@ TEST(Ephemeral, ApplySetModToEphemeralDocument) {
     // $set.
     Document doc;
     Element x = doc.makeElementObject("x");
-    doc.root().pushBack(x);
+    doc.root().pushBack(x).transitional_ignore();
     Element a = doc.makeElementInt("a", 100);
-    x.pushBack(a);
+    x.pushBack(a).transitional_ignore();
 
     Mod setMod(fromjson("{ $set: { x: { a: 100, b: 2 }}}"), true);
 
