@@ -53,11 +53,12 @@ void AccumulatorFirst::processInternal(const Value& input, bool merging) {
     }
 }
 
-Value AccumulatorFirst::getValue(bool toBeMerged) const {
+Value AccumulatorFirst::getValue(bool toBeMerged) {
     return _first;
 }
 
-AccumulatorFirst::AccumulatorFirst() : _haveFirst(false) {
+AccumulatorFirst::AccumulatorFirst(const boost::intrusive_ptr<ExpressionContext>& expCtx)
+    : Accumulator(expCtx), _haveFirst(false) {
     _memUsageBytes = sizeof(*this);
 }
 
@@ -68,7 +69,8 @@ void AccumulatorFirst::reset() {
 }
 
 
-intrusive_ptr<Accumulator> AccumulatorFirst::create() {
-    return new AccumulatorFirst();
+intrusive_ptr<Accumulator> AccumulatorFirst::create(
+    const boost::intrusive_ptr<ExpressionContext>& expCtx) {
+    return new AccumulatorFirst(expCtx);
 }
 }
