@@ -41,14 +41,17 @@
 #include "mongo/bson/simple_bsonobj_comparator.h"
 #include "mongo/client/dbclientinterface.h"
 #include "mongo/db/collection_index_usage_tracker.h"
+#include "mongo/db/commands.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/pipeline/dependencies.h"
 #include "mongo/db/pipeline/document.h"
 #include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/pipeline/field_path.h"
 #include "mongo/db/pipeline/lite_parsed_document_source.h"
 #include "mongo/db/pipeline/pipeline.h"
 #include "mongo/db/pipeline/value.h"
+#include "mongo/db/query/explain_options.h"
 #include "mongo/stdx/functional.h"
 #include "mongo/util/intrusive_counter.h"
 
@@ -80,11 +83,14 @@ class Document;
  * If your stage is actually an alias which needs to return more than one stage (such as
  * $sortByCount), you should use the REGISTER_MULTI_STAGE_ALIAS macro instead.
  */
-#define REGISTER_DOCUMENT_SOURCE(key, liteParser, fullParser)                                \
+#define REGISTER_DOCUMENT_SOURCE_CONDITIONALLY(key, liteParser, fullParser, ...)             \
     MONGO_INITIALIZER(addToDocSourceParserMap_##key)(InitializerContext*) {                  \
+        if (!__VA_ARGS__) {                                                                  \
+            return Status::OK();                                                             \
+        }                                                                                    \
         auto fullParserWrapper = [](BSONElement stageSpec,                                   \
                                     const boost::intrusive_ptr<ExpressionContext>& expCtx) { \
-            return std::vector<boost::intrusive_ptr<DocumentSource>>{                        \
+            return std::list<boost::intrusive_ptr<DocumentSource>>{                          \
                 (fullParser)(stageSpec, expCtx)};                                            \
         };                                                                                   \
         LiteParsedDocumentSource::registerParser("$" #key, liteParser);                      \
@@ -92,6 +98,13 @@ class Document;
         return Status::OK();                                                                 \
     }
 
+#define REGISTER_DOCUMENT_SOURCE(key, liteParser, fullParser) \
+    REGISTER_DOCUMENT_SOURCE_CONDITIONALLY(key, liteParser, fullParser, true)
+
+#define REGISTER_TEST_DOCUMENT_SOURCE(key, liteParser, fullParser) \
+    REGISTER_DOCUMENT_SOURCE_CONDITIONALLY(                        \
+        key, liteParser, fullParser, Command::testCommandsEnabled)
+
 /**
  * Registers a multi-stage alias (such as $sortByCount) to have the single name 'key'. When a stage
  * with name '$key' is found, 'liteParser' will be used to produce a LiteParsedDocumentSource,
@@ -114,9 +127,184 @@ class Document;
 
 class DocumentSource : public IntrusiveCounterUnsigned {
 public:
-    using Parser = stdx::function<std::vector<boost::intrusive_ptr<DocumentSource>>(
+    using Parser = stdx::function<std::list<boost::intrusive_ptr<DocumentSource>>(
         BSONElement, const boost::intrusive_ptr<ExpressionContext>&)>;
 
+    /**
+     * A struct describing various constraints about where this stage can run, where it must be in
+     * the pipeline, what resources it may require, etc.
+     */
+    struct StageConstraints {
+        /**
+         * A StreamType defines whether this stage is streaming (can produce output based solely on
+         * the current input document) or blocking (must examine subsequent documents before
+         * producing an output document).
+         */
+        enum class StreamType { kStreaming, kBlocking };
+
+        /**
+         * A PositionRequirement stipulates what specific position the stage must occupy within the
+         * pipeline, if any.
+         */
+        enum class PositionRequirement { kNone, kFirst, kLast };
+
+        /**
+         * A HostTypeRequirement defines where this stage is permitted to be executed when the
+         * pipeline is run on a sharded cluster.
+         */
+        enum class HostTypeRequirement {
+            // Indicates that the stage can run either on mongoD or mongoS.
+            kNone,
+            // Indicates that the stage must run on the host to which it was originally sent and
+            // cannot be forwarded from mongoS to the shards.
+            kLocalOnly,
+            // Indicates that the stage must run on the primary shard.
+            kPrimaryShard,
+            // Indicates that the stage must run on any participating shard.
+            kAnyShard,
+            // Indicates that the stage can only run on mongoS.
+            kMongoS,
+        };
+
+        /**
+         * A DiskUseRequirement indicates whether this stage writes permanent data to disk, or
+         * whether it may spill temporary data to disk if its memory usage exceeds a given
+         * threshold. Note that this only indicates that the stage has the ability to spill; if
+         * 'allowDiskUse' is set to false, it will be prevented from doing so.
+         */
+        enum class DiskUseRequirement { kNoDiskUse, kWritesTmpData, kWritesPersistentData };
+
+        /**
+         * A ChangeStreamRequirement determines whether a particular stage is itself a ChangeStream
+         * stage, whether it is allowed to exist in a $changeStream pipeline, or whether it is
+         * blacklisted from $changeStream.
+         */
+        enum class ChangeStreamRequirement { kChangeStreamStage, kWhitelist, kBlacklist };
+
+        /**
+         * A FacetRequirement indicates whether this stage may be used within a $facet pipeline.
+         */
+        enum class FacetRequirement { kAllowed, kNotAllowed };
+
+        StageConstraints(
+            StreamType streamType,
+            PositionRequirement requiredPosition,
+            HostTypeRequirement hostRequirement,
+            DiskUseRequirement diskRequirement,
+            FacetRequirement facetRequirement,
+            ChangeStreamRequirement changeStreamRequirement = ChangeStreamRequirement::kBlacklist)
+            : requiredPosition(requiredPosition),
+              hostRequirement(hostRequirement),
+              diskRequirement(diskRequirement),
+              changeStreamRequirement(changeStreamRequirement),
+              facetRequirement(facetRequirement),
+              streamType(streamType) {
+            // Stages which are allowed to run in $facet must not have any position requirements.
+            invariant(
+                !(isAllowedInsideFacetStage() && requiredPosition != PositionRequirement::kNone));
+
+            // No change stream stages are permitted to run in a $facet pipeline.
+            invariant(!(isChangeStreamStage() && isAllowedInsideFacetStage()));
+
+            // Only streaming stages are permitted in $changeStream pipelines.
+            invariant(!(isAllowedInChangeStream() && streamType == StreamType::kBlocking));
+
+            // A stage which is whitelisted for $changeStream cannot have a requirement to run on a
+            // shard, since it needs to be able to run on mongoS in a cluster.
+            invariant(!(changeStreamRequirement == ChangeStreamRequirement::kWhitelist &&
+                        (hostRequirement == HostTypeRequirement::kAnyShard ||
+                         hostRequirement == HostTypeRequirement::kPrimaryShard)));
+
+            // A stage which is whitelisted for $changeStream cannot have a position requirement.
+            invariant(!(changeStreamRequirement == ChangeStreamRequirement::kWhitelist &&
+                        requiredPosition != PositionRequirement::kNone));
+        }
+
+        /**
+         * Returns the literal HostTypeRequirement used to initialize the StageConstraints, or the
+         * effective HostTypeRequirement (kAnyShard or kMongoS) if kLocalOnly was specified.
+         */
+        HostTypeRequirement resolvedHostTypeRequirement(
+            const boost::intrusive_ptr<ExpressionContext>& expCtx) const {
+            return (hostRequirement != HostTypeRequirement::kLocalOnly
+                        ? hostRequirement
+                        : (expCtx->inMongos ? HostTypeRequirement::kMongoS
+                                            : HostTypeRequirement::kAnyShard));
+        }
+
+        /**
+         * True if this stage must run on the same host to which it was originally sent.
+         */
+        bool mustRunLocally() const {
+            return hostRequirement == HostTypeRequirement::kLocalOnly;
+        }
+
+        /**
+         * True if this stage is permitted to run in a $facet pipeline.
+         */
+        bool isAllowedInsideFacetStage() const {
+            return facetRequirement == FacetRequirement::kAllowed;
+        }
+
+        /**
+         * True if this stage is permitted to run in a pipeline which starts with $changeStream.
+         */
+        bool isAllowedInChangeStream() const {
+            return changeStreamRequirement != ChangeStreamRequirement::kBlacklist;
+        }
+
+        /**
+         * True if this stage is itself a $changeStream stage, and is therefore implicitly allowed
+         * to run in a pipeline which begins with $changeStream.
+         */
+        bool isChangeStreamStage() const {
+            return changeStreamRequirement == ChangeStreamRequirement::kChangeStreamStage;
+        }
+
+        // Indicates whether this stage needs to be at a particular position in the pipeline.
+        const PositionRequirement requiredPosition;
+
+        // Indicates whether this stage can only be executed on specific components of a sharded
+        // cluster.
+        const HostTypeRequirement hostRequirement;
+
+        // Indicates whether this stage may write persistent data to disk, or may spill to temporary
+        // files if its memory usage becomes excessive.
+        const DiskUseRequirement diskRequirement;
+
+        // Indicates whether this stage is itself a $changeStream stage, or if not whether it may
+        // exist in a pipeline which begins with $changeStream.
+        const ChangeStreamRequirement changeStreamRequirement;
+
+        // Indicates whether this stage may run inside a $facet stage.
+        const FacetRequirement facetRequirement;
+
+        // Indicates whether this is a streaming or blocking stage.
+        const StreamType streamType;
+
+        // True if this stage does not generate results itself, and instead pulls inputs from an
+        // input DocumentSource (via 'pSource').
+        bool requiresInputDocSource = true;
+
+        // True if this stage operates on a global or database level, like $currentOp.
+        bool isIndependentOfAnyCollection = false;
+
+        // True if this stage can ever be safely swapped with a subsequent $match stage, provided
+        // that the match does not depend on the paths returned by getModifiedPaths().
+        //
+        // Stages that want to participate in match swapping should set this to true. Such a stage
+        // must also override getModifiedPaths() to provide information about which particular
+        // $match predicates be swapped before itself.
+        bool canSwapWithMatch = false;
+    };
+
+    using ChangeStreamRequirement = StageConstraints::ChangeStreamRequirement;
+    using HostTypeRequirement = StageConstraints::HostTypeRequirement;
+    using PositionRequirement = StageConstraints::PositionRequirement;
+    using DiskUseRequirement = StageConstraints::DiskUseRequirement;
+    using FacetRequirement = StageConstraints::FacetRequirement;
+    using StreamType = StageConstraints::StreamType;
+
     /**
      * This is what is returned from the main DocumentSource API: getNext(). It is essentially a
      * (ReturnStatus, Document) pair, with the first entry being used to communicate information
@@ -199,65 +387,61 @@ public:
      * this DocumentSource.
      *
      * All implementers must call pExpCtx->checkForInterrupt().
+     *
+     * For performance reasons, a streaming stage must not keep references to documents across calls
+     * to getNext(). Such stages must retrieve a result from their child and then release it (or
+     * return it) before asking for another result. Failing to do so can result in extra work, since
+     * the Document/Value library must copy data on write when that data has a refcount above one.
      */
     virtual GetNextResult getNext() = 0;
 
     /**
-     * Inform the source that it is no longer needed and may release its resources.  After
-     * dispose() is called the source must still be able to handle iteration requests, but may
-     * become eof().
-     * NOTE: For proper mutex yielding, dispose() must be called on any DocumentSource that will
-     * not be advanced until eof(), see SERVER-6123.
+     * Returns a struct containing information about any special constraints imposed on using this
+     * stage. Input parameter Pipeline::SplitState is used by stages whose requirements change
+     * depending on whether they are in a split or unsplit pipeline.
      */
-    virtual void dispose();
+    virtual StageConstraints constraints(
+        Pipeline::SplitState = Pipeline::SplitState::kUnsplit) const = 0;
 
     /**
-       Get the source's name.
+     * Informs the stage that it is no longer needed and can release its resources. After dispose()
+     * is called the stage must still be able to handle calls to getNext(), but can return kEOF.
+     *
+     * This is a non-virtual public interface to ensure dispose() is threaded through the entire
+     * pipeline. Subclasses should override doDispose() to implement their disposal.
+     */
+    void dispose() {
+        doDispose();
+        if (pSource) {
+            pSource->dispose();
+        }
+    }
 
-       @returns the std::string name of the source as a constant string;
-         this is static, and there's no need to worry about adopting it
+    /**
+     * Get the stage's name.
      */
     virtual const char* getSourceName() const;
 
     /**
-      Set the underlying source this source should use to get Documents
-      from.
-
-      It is an error to set the source more than once.  This is to
-      prevent changing sources once the original source has been started;
-      this could break the state maintained by the DocumentSource.
-
-      This pointer is not reference counted because that has led to
-      some circular references.  As a result, this doesn't keep
-      sources alive, and is only intended to be used temporarily for
-      the lifetime of a Pipeline::run().
-
-      @param pSource the underlying source to use
+     * Set the underlying source this source should use to get Documents from. Must not throw
+     * exceptions.
      */
-    virtual void setSource(DocumentSource* pSource);
+    virtual void setSource(DocumentSource* source) {
+        pSource = source;
+    }
 
     /**
      * In the default case, serializes the DocumentSource and adds it to the std::vector<Value>.
      *
-     * A subclass may choose to overwrite this, rather than serialize,
-     * if it should output multiple stages (eg, $sort sometimes also outputs a $limit).
+     * A subclass may choose to overwrite this, rather than serialize, if it should output multiple
+     * stages (eg, $sort sometimes also outputs a $limit).
+     *
+     * The 'explain' parameter indicates the explain verbosity mode, or is equal boost::none if no
+     * explain is requested.
      */
-
-    virtual void serializeToArray(std::vector<Value>& array, bool explain = false) const;
-
-    /**
-     * Returns true if doesn't require an input source (most DocumentSources do).
-     */
-    virtual bool isValidInitialSource() const {
-        return false;
-    }
-
-    /**
-     * Returns true if the DocumentSource needs to be run on the primary shard.
-     */
-    virtual bool needsPrimaryShard() const {
-        return false;
-    }
+    virtual void serializeToArray(
+        std::vector<Value>& array,
+        boost::optional<ExplainOptions::Verbosity> explain = boost::none) const;
 
     /**
      * If DocumentSource uses additional collections, it adds the namespaces to the input vector.
@@ -268,23 +452,11 @@ public:
 
     virtual void reattachToOperationContext(OperationContext* opCtx) {}
 
-    /**
-     * Injects a new ExpressionContext into this DocumentSource and propagates the ExpressionContext
-     * to all child expressions, accumulators, etc.
-     *
-     * Stages which require work to propagate the ExpressionContext to their private execution
-     * machinery should override doInjectExpressionContext().
-     */
-    void injectExpressionContext(const boost::intrusive_ptr<ExpressionContext>& expCtx) {
-        pExpCtx = expCtx;
-        doInjectExpressionContext();
-    }
-
     /**
      * Create a DocumentSource pipeline stage from 'stageObj'.
      */
-    static std::vector<boost::intrusive_ptr<DocumentSource>> parse(
-        const boost::intrusive_ptr<ExpressionContext> expCtx, BSONObj stageObj);
+    static std::list<boost::intrusive_ptr<DocumentSource>> parse(
+        const boost::intrusive_ptr<ExpressionContext>& expCtx, BSONObj stageObj);
 
     /**
      * Registers a DocumentSource with a parsing function, so that when a stage with the given name
@@ -375,11 +547,25 @@ public:
             kAllExcept,
         };
 
-        GetModPathsReturn(Type type, std::set<std::string>&& paths)
-            : type(type), paths(std::move(paths)) {}
+        GetModPathsReturn(Type type,
+                          std::set<std::string>&& paths,
+                          StringMap<std::string>&& renames)
+            : type(type), paths(std::move(paths)), renames(std::move(renames)) {}
 
         Type type;
         std::set<std::string> paths;
+
+        // Stages may fill out 'renames' to contain information about path renames. Each entry in
+        // 'renames' maps from the new name of the path (valid in documents flowing *out* of this
+        // stage) to the old name of the path (valid in documents flowing *into* this stage).
+        //
+        // For example, consider the stage
+        //
+        //   {$project: {_id: 0, a: 1, b: "$c"}}
+        //
+        // This stage should return kAllExcept, since it modifies all paths other than "a". It can
+        // also fill out 'renames' with the mapping "b" => "c".
+        StringMap<std::string> renames;
     };
 
     /**
@@ -390,19 +576,7 @@ public:
      * See GetModPathsReturn above for the possible return values and what they mean.
      */
     virtual GetModPathsReturn getModifiedPaths() const {
-        return {GetModPathsReturn::Type::kNotSupported, std::set<std::string>{}};
-    }
-
-    /**
-     * Returns whether this stage can swap with a subsequent $match stage, provided that the match
-     * does not depend on the paths returned by getModifiedPaths().
-     *
-     * Subclasses which want to participate in match swapping should override this to return true.
-     * Such a subclass must also override getModifiedPaths() to provide information about which
-     * $match predicates be swapped before itself.
-     */
-    virtual bool canSwapWithMatch() const {
-        return false;
+        return {GetModPathsReturn::Type::kNotSupported, std::set<std::string>{}, {}};
     }
 
     enum GetDepsReturn {
@@ -419,7 +593,7 @@ public:
         EXHAUSTIVE_FIELDS = 0x2,
 
         // Later stages won't need more metadata from input. For example, a $group stage will group
-        // documents together, discarding their text score.
+        // documents together, discarding their text score and sort keys.
         EXHAUSTIVE_META = 0x4,
 
         // Later stages won't need either fields or metadata.
@@ -438,20 +612,8 @@ public:
     }
 
 protected:
-    /**
-       Base constructor.
-     */
     explicit DocumentSource(const boost::intrusive_ptr<ExpressionContext>& pExpCtx);
 
-    /**
-     * DocumentSources which need to update their internal state when attaching to a new
-     * ExpressionContext should override this method.
-     *
-     * Any stage subclassing from DocumentSource should override this method if it contains
-     * expressions or accumulators which need to attach to the newly injected ExpressionContext.
-     */
-    virtual void doInjectExpressionContext() {}
-
     /**
      * Attempt to perform an optimization with the following source in the pipeline. 'container'
      * refers to the entire pipeline, and 'itr' points to this stage within the pipeline. The caller
@@ -469,6 +631,11 @@ protected:
         return std::next(itr);
     };
 
+    /**
+     * Release any resources held by this stage. After doDispose() is called the stage must still be
+     * able to handle calls to getNext(), but can return kEOF.
+     */
+    virtual void doDispose() {}
 
     /*
       Most DocumentSources have an underlying source they get their data
@@ -489,24 +656,38 @@ private:
      * This is used by the default implementation of serializeToArray() to add this object
      * to a pipeline being serialized. Returning a missing() Value results in no entry
      * being added to the array for this stage (DocumentSource).
+     *
+     * The 'explain' parameter indicates the explain verbosity mode, or is equal boost::none if no
+     * explain is requested.
      */
-    virtual Value serialize(bool explain = false) const = 0;
+    virtual Value serialize(
+        boost::optional<ExplainOptions::Verbosity> explain = boost::none) const = 0;
 };
 
-/** This class marks DocumentSources that should be split between the merger and the shards.
- *  See Pipeline::Optimizations::Sharded::findSplitPoint() for details.
+/**
+ * This class marks DocumentSources that should be split between the merger and the shards. See
+ * Pipeline::Optimizations::Sharded::findSplitPoint() for details.
  */
 class SplittableDocumentSource {
 public:
-    /** returns a source to be run on the shards.
-     *  if NULL, don't run on shards
+    /**
+     * Returns a source to be run on the shards, or NULL if no work should be done on the shards for
+     * this stage. Must not mutate the existing source object; if different behaviour is required in
+     * the split-pipeline case, a new source should be created and configured appropriately. It is
+     * an error for getShardSource() to return a pointer to the same object as getMergeSource(),
+     * since this can result in the source being stitched into both the shard and merge pipelines
+     * when the latter is executed on mongoS.
      */
     virtual boost::intrusive_ptr<DocumentSource> getShardSource() = 0;
 
-    /** returns a source that combines results from shards.
-     *  if NULL, don't run on merger
+    /**
+     * Returns a list of stages that combine results from the shards, or an empty list if no work
+     * should be done in the merge pipeline for this stage. Must not mutate the existing source
+     * object; if different behaviour is required, a new source should be created and configured
+     * appropriately. It is an error for getMergeSources() to return a pointer to the same object as
+     * getShardSource().
      */
-    virtual boost::intrusive_ptr<DocumentSource> getMergeSource() = 0;
+    virtual std::list<boost::intrusive_ptr<DocumentSource>> getMergeSources() = 0;
 
 protected:
     // It is invalid to delete through a SplittableDocumentSource-typed pointer.
@@ -514,16 +695,40 @@ protected:
 };
 
 
-/** This class marks DocumentSources which need mongod-specific functionality.
- *  It causes a MongodInterface to be injected when in a mongod and prevents mongos from
- *  merging pipelines containing this stage.
+/**
+ * This class marks DocumentSources which need functionality specific to a mongos or a mongod. It
+ * causes a MongodProcessInterface to be injected when in a mongod and a MongosProcessInterface when
+ * in a mongos.
  */
-class DocumentSourceNeedsMongod : public DocumentSource {
+class DocumentSourceNeedsMongoProcessInterface : public DocumentSource {
 public:
-    // Wraps mongod-specific functions to allow linking into mongos.
-    class MongodInterface {
+    /**
+     * Any functionality needed by an aggregation stage that is either context specific to a mongod
+     * or mongos process, or is only compiled in to one of those two binaries must be accessed via
+     * this interface. This allows all DocumentSources to be parsed on either mongos or mongod, but
+     * only executable where it makes sense.
+     */
+    class MongoProcessInterface {
     public:
-        virtual ~MongodInterface(){};
+        enum class CurrentOpConnectionsMode { kIncludeIdle, kExcludeIdle };
+        enum class CurrentOpUserMode { kIncludeAll, kExcludeOthers };
+        enum class CurrentOpTruncateMode { kNoTruncation, kTruncateOps };
+
+        struct MakePipelineOptions {
+            MakePipelineOptions(){};
+
+            bool optimize = true;
+            bool attachCursorSource = true;
+
+            // Ordinarily, a MongoProcessInterface is injected into the pipeline at the point
+            // when the cursor source is added. If true, 'forceInjectMongoProcessInterface' will
+            // inject MongoProcessInterfaces into the pipeline even if 'attachCursorSource' is
+            // false. If 'attachCursorSource' is true, then the value of
+            // 'forceInjectMongoProcessInterface' is irrelevant.
+            bool forceInjectMongoProcessInterface = false;
+        };
+
+        virtual ~MongoProcessInterface(){};
 
         /**
          * Sets the OperationContext of the DBDirectClient returned by directClient(). This method
@@ -565,6 +770,12 @@ public:
                                           const BSONObj& param,
                                           BSONObjBuilder* builder) const = 0;
 
+        /**
+         * Appends the record count for collection "nss" to "builder".
+         */
+        virtual Status appendRecordCount(const NamespaceString& nss,
+                                         BSONObjBuilder* builder) const = 0;
+
         /**
          * Gets the collection options for the collection given by 'nss'.
          */
@@ -582,34 +793,85 @@ public:
             const std::list<BSONObj>& originalIndexes) = 0;
 
         /**
-         * Parses a Pipeline from a vector of BSONObjs representing DocumentSources and readies it
-         * for execution. The returned pipeline is optimized and has a cursor source prepared.
+         * Parses a Pipeline from a vector of BSONObjs representing DocumentSources. The state of
+         * the returned pipeline will depend upon the supplied MakePipelineOptions:
+         * - The boolean opts.optimize determines whether the pipeline will be optimized.
+         * - If opts.attachCursorSource is false, the pipeline will be returned without attempting
+         *   to add an initial cursor source.
+         * - If opts.forceInjectMongoProcessInterface is true, then a MongoProcessInterface will be
+         *   provided to each stage which requires one, regardless of whether a cursor source is
+         *   attached to the pipeline.
          *
          * This function returns a non-OK status if parsing the pipeline failed.
          */
-        virtual StatusWith<boost::intrusive_ptr<Pipeline>> makePipeline(
+        virtual StatusWith<std::unique_ptr<Pipeline, Pipeline::Deleter>> makePipeline(
             const std::vector<BSONObj>& rawPipeline,
-            const boost::intrusive_ptr<ExpressionContext>& expCtx) = 0;
+            const boost::intrusive_ptr<ExpressionContext>& expCtx,
+            const MakePipelineOptions opts = MakePipelineOptions{}) = 0;
+
+        /**
+         * Attaches a cursor source to the start of a pipeline. Performs no further optimization.
+         * This function asserts if the collection to be aggregated is sharded. NamespaceNotFound
+         * will be returned if ExpressionContext has a UUID and that UUID doesn't exist anymore.
+         * That should be the only case where NamespaceNotFound is returned.
+         */
+        virtual Status attachCursorSourceToPipeline(
+            const boost::intrusive_ptr<ExpressionContext>& expCtx, Pipeline* pipeline) = 0;
+
+        /**
+         * Returns a vector of owned BSONObjs, each of which contains details of an in-progress
+         * operation or, optionally, an idle connection. If userMode is kIncludeAllUsers, report
+         * operations for all authenticated users; otherwise, report only the current user's
+         * operations.
+         */
+        virtual std::vector<BSONObj> getCurrentOps(CurrentOpConnectionsMode connMode,
+                                                   CurrentOpUserMode userMode,
+                                                   CurrentOpTruncateMode) const = 0;
+
+        /**
+         * Returns the name of the local shard if sharding is enabled, or an empty string.
+         */
+        virtual std::string getShardName(OperationContext* opCtx) const = 0;
+
+        /**
+         * Returns the fields of the document key (in order) for the current collection, including
+         * the shard key and _id.  If _id is not in the shard key, it is added last.
+         */
+        virtual std::vector<FieldPath> collectDocumentKeyFields(UUID) const = 0;
+
+        /**
+         * Returns zero or one documents with the document key 'documentKey'. 'documentKey' is
+         * treated as a unique identifier of a document, and may include an _id or all fields from
+         * the shard key and an _id. Throws if more than one match was found. Returns boost::none if
+         * no matching documents were found, including cases where the given namespace does not
+         * exist.
+         */
+        virtual boost::optional<Document> lookupSingleDocument(
+            const NamespaceString& nss,
+            UUID collectionUUID,
+            const Document& documentKey,
+            boost::optional<BSONObj> readConcern) = 0;
 
         // Add new methods as needed.
     };
 
-    DocumentSourceNeedsMongod(const boost::intrusive_ptr<ExpressionContext>& expCtx)
+    DocumentSourceNeedsMongoProcessInterface(const boost::intrusive_ptr<ExpressionContext>& expCtx)
         : DocumentSource(expCtx) {}
 
-    void injectMongodInterface(std::shared_ptr<MongodInterface> mongod) {
-        _mongod = mongod;
-        doInjectMongodInterface(mongod);
+    void injectMongoProcessInterface(std::shared_ptr<MongoProcessInterface> mongoProcessInterface) {
+        _mongoProcessInterface = mongoProcessInterface;
+        doInjectMongoProcessInterface(mongoProcessInterface);
     }
 
     /**
      * Derived classes may override this method to register custom inject functionality.
      */
-    virtual void doInjectMongodInterface(std::shared_ptr<MongodInterface> mongod) {}
+    virtual void doInjectMongoProcessInterface(
+        std::shared_ptr<MongoProcessInterface> mongoProcessInterface) {}
 
     void detachFromOperationContext() override {
-        invariant(_mongod);
-        _mongod->setOperationContext(nullptr);
+        invariant(_mongoProcessInterface);
+        _mongoProcessInterface->setOperationContext(nullptr);
         doDetachFromOperationContext();
     }
 
@@ -619,8 +881,8 @@ public:
     virtual void doDetachFromOperationContext() {}
 
     void reattachToOperationContext(OperationContext* opCtx) final {
-        invariant(_mongod);
-        _mongod->setOperationContext(opCtx);
+        invariant(_mongoProcessInterface);
+        _mongoProcessInterface->setOperationContext(opCtx);
         doReattachToOperationContext(opCtx);
     }
 
@@ -630,11 +892,11 @@ public:
     virtual void doReattachToOperationContext(OperationContext* opCtx) {}
 
 protected:
-    // It is invalid to delete through a DocumentSourceNeedsMongod-typed pointer.
-    virtual ~DocumentSourceNeedsMongod() {}
+    // It is invalid to delete through a DocumentSourceNeedsMongoProcessInterface-typed pointer.
+    virtual ~DocumentSourceNeedsMongoProcessInterface() {}
 
-    // Gives subclasses access to a MongodInterface implementation
-    std::shared_ptr<MongodInterface> _mongod;
+    // Gives subclasses access to a MongoProcessInterface implementation
+    std::shared_ptr<MongoProcessInterface> _mongoProcessInterface;
 };
 
 
