@@ -43,20 +43,27 @@ namespace mongo {
  */
 class DocumentSourceBucketAuto final : public DocumentSource, public SplittableDocumentSource {
 public:
-    Value serialize(bool explain = false) const final;
+    Value serialize(boost::optional<ExplainOptions::Verbosity> explain = boost::none) const final;
     GetDepsReturn getDependencies(DepsTracker* deps) const final;
     GetNextResult getNext() final;
-    void dispose() final;
     const char* getSourceName() const final;
 
+    StageConstraints constraints(Pipeline::SplitState pipeState) const final {
+        return {StreamType::kBlocking,
+                PositionRequirement::kNone,
+                HostTypeRequirement::kNone,
+                DiskUseRequirement::kWritesTmpData,
+                FacetRequirement::kAllowed};
+    }
+
     /**
      * The $bucketAuto stage must be run on the merging shard.
      */
     boost::intrusive_ptr<DocumentSource> getShardSource() final {
         return nullptr;
     }
-    boost::intrusive_ptr<DocumentSource> getMergeSource() final {
-        return this;
+    std::list<boost::intrusive_ptr<DocumentSource>> getMergeSources() final {
+        return {this};
     }
 
     static const uint64_t kDefaultMaxMemoryUsageBytes = 100 * 1024 * 1024;
@@ -70,7 +77,6 @@ public:
     static boost::intrusive_ptr<DocumentSourceBucketAuto> create(
         const boost::intrusive_ptr<ExpressionContext>& expCtx,
         const boost::intrusive_ptr<Expression>& groupByExpression,
-        Variables::Id numVariables,
         int numBuckets,
         std::vector<AccumulationStatement> accumulationStatements = {},
         const boost::intrusive_ptr<GranularityRounder>& granularityRounder = nullptr,
@@ -82,10 +88,12 @@ public:
     static boost::intrusive_ptr<DocumentSource> createFromBson(
         BSONElement elem, const boost::intrusive_ptr<ExpressionContext>& pExpCtx);
 
+protected:
+    void doDispose() final;
+
 private:
     DocumentSourceBucketAuto(const boost::intrusive_ptr<ExpressionContext>& pExpCtx,
                              const boost::intrusive_ptr<Expression>& groupByExpression,
-                             Variables::Id numVariables,
                              int numBuckets,
                              std::vector<AccumulationStatement> accumulationStatements,
                              const boost::intrusive_ptr<GranularityRounder>& granularityRounder,
@@ -96,7 +104,7 @@ private:
         Bucket(const boost::intrusive_ptr<ExpressionContext>& expCtx,
                Value min,
                Value max,
-               std::vector<Accumulator::Factory> accumulatorFactories);
+               const std::vector<AccumulationStatement>& accumulationStatements);
         Value _min;
         Value _max;
         std::vector<boost::intrusive_ptr<Accumulator>> _accums;
@@ -139,20 +147,13 @@ private:
     std::unique_ptr<Sorter<Value, Document>> _sorter;
     std::unique_ptr<Sorter<Value, Document>::Iterator> _sortedInput;
 
-    // _fieldNames contains the field names for the result documents, _accumulatorFactories contains
-    // the accumulator factories for the result documents, and _expressions contains the common
-    // expressions used by each instance of each accumulator in order to find the right-hand side of
-    // what gets added to the accumulator. These three vectors parallel each other.
-    std::vector<std::string> _fieldNames;
-    std::vector<Accumulator::Factory> _accumulatorFactories;
-    std::vector<boost::intrusive_ptr<Expression>> _expressions;
+    std::vector<AccumulationStatement> _accumulatedFields;
 
     int _nBuckets;
     uint64_t _maxMemoryUsageBytes;
     bool _populated = false;
     std::vector<Bucket> _buckets;
     std::vector<Bucket>::iterator _bucketsIterator;
-    std::unique_ptr<Variables> _variables;
     boost::intrusive_ptr<Expression> _groupByExpression;
     boost::intrusive_ptr<GranularityRounder> _granularityRounder;
     long long _nDocuments = 0;
