@@ -33,8 +33,9 @@
 #include "mongo/base/init.h"
 #include "mongo/db/bson/dotted_path_support.h"
 #include "mongo/db/jsobj.h"
-#include "mongo/db/matcher/extensions_callback_disallow_extensions.h"
 #include "mongo/db/pipeline/document.h"
+#include "mongo/db/pipeline/document_comparator.h"
+#include "mongo/db/pipeline/document_path_support.h"
 #include "mongo/db/pipeline/expression.h"
 #include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/pipeline/value.h"
@@ -47,27 +48,34 @@ using boost::intrusive_ptr;
 
 namespace dps = ::mongo::dotted_path_support;
 
-std::unique_ptr<LiteParsedDocumentSourceOneForeignCollection> DocumentSourceGraphLookUp::liteParse(
+std::unique_ptr<LiteParsedDocumentSourceForeignCollections> DocumentSourceGraphLookUp::liteParse(
     const AggregationRequest& request, const BSONElement& spec) {
-    uassert(40327,
+    uassert(ErrorCodes::FailedToParse,
             str::stream() << "the $graphLookup stage specification must be an object, but found "
                           << typeName(spec.type()),
             spec.type() == BSONType::Object);
 
     auto specObj = spec.Obj();
     auto fromElement = specObj["from"];
-    uassert(40328,
+    uassert(ErrorCodes::FailedToParse,
             str::stream() << "missing 'from' option to $graphLookup stage specification: "
                           << specObj,
             fromElement);
-    uassert(40329,
+    uassert(ErrorCodes::FailedToParse,
             str::stream() << "'from' option to $graphLookup must be a string, but was type "
                           << typeName(specObj["from"].type()),
             fromElement.type() == BSONType::String);
 
     NamespaceString nss(request.getNamespaceString().db(), fromElement.valueStringData());
-    uassert(40330, str::stream() << "invalid $graphLookup namespace: " << nss.ns(), nss.isValid());
-    return stdx::make_unique<LiteParsedDocumentSourceOneForeignCollection>(std::move(nss));
+    uassert(ErrorCodes::InvalidNamespace,
+            str::stream() << "invalid $graphLookup namespace: " << nss.ns(),
+            nss.isValid());
+
+    PrivilegeVector privileges{
+        Privilege(ResourcePattern::forExactNamespace(nss), ActionType::find)};
+
+    return stdx::make_unique<LiteParsedDocumentSourceForeignCollections>(std::move(nss),
+                                                                         std::move(privileges));
 }
 
 REGISTER_DOCUMENT_SOURCE(graphLookup,
@@ -162,11 +170,10 @@ DocumentSource::GetNextResult DocumentSourceGraphLookUp::getNextUnwound() {
     }
 }
 
-void DocumentSourceGraphLookUp::dispose() {
+void DocumentSourceGraphLookUp::doDispose() {
     _cache.clear();
-    _frontier->clear();
+    _frontier.clear();
     _visited.clear();
-    pSource->dispose();
 }
 
 void DocumentSourceGraphLookUp::doBreadthFirstSearch() {
@@ -176,19 +183,19 @@ void DocumentSourceGraphLookUp::doBreadthFirstSearch() {
         shouldPerformAnotherQuery = false;
 
         // Check whether each key in the frontier exists in the cache or needs to be queried.
-        BSONObjSet cached = SimpleBSONObjComparator::kInstance.makeBSONObjSet();
+        auto cached = pExpCtx->getDocumentComparator().makeUnorderedDocumentSet();
         auto matchStage = makeMatchStageFromFrontier(&cached);
 
         ValueUnorderedSet queried = pExpCtx->getValueComparator().makeUnorderedValueSet();
-        _frontier->swap(queried);
+        _frontier.swap(queried);
         _frontierUsageBytes = 0;
 
         // Process cached values, populating '_frontier' for the next iteration of search.
         while (!cached.empty()) {
-            auto it = cached.begin();
+            auto doc = *cached.begin();
+            cached.erase(cached.begin());
             shouldPerformAnotherQuery =
-                addToVisitedAndFrontier(*it, depth) || shouldPerformAnotherQuery;
-            cached.erase(it);
+                addToVisitedAndFrontier(std::move(doc), depth) || shouldPerformAnotherQuery;
             checkMemoryUsage();
         }
 
@@ -198,7 +205,8 @@ void DocumentSourceGraphLookUp::doBreadthFirstSearch() {
 
             // We've already allocated space for the trailing $match stage in '_fromPipeline'.
             _fromPipeline.back() = *matchStage;
-            auto pipeline = uassertStatusOK(_mongod->makePipeline(_fromPipeline, _fromExpCtx));
+            auto pipeline =
+                uassertStatusOK(_mongoProcessInterface->makePipeline(_fromPipeline, _fromExpCtx));
             while (auto next = pipeline->getNext()) {
                 uassert(40271,
                         str::stream()
@@ -207,10 +215,9 @@ void DocumentSourceGraphLookUp::doBreadthFirstSearch() {
                             << "' namespace must contain an _id for de-duplication in $graphLookup",
                         !(*next)["_id"].missing());
 
-                BSONObj result = next->toBson();
                 shouldPerformAnotherQuery =
-                    addToVisitedAndFrontier(result.getOwned(), depth) || shouldPerformAnotherQuery;
-                addToCache(result, queried);
+                    addToVisitedAndFrontier(*next, depth) || shouldPerformAnotherQuery;
+                addToCache(std::move(*next), queried);
             }
             checkMemoryUsage();
         }
@@ -219,105 +226,75 @@ void DocumentSourceGraphLookUp::doBreadthFirstSearch() {
     } while (shouldPerformAnotherQuery && depth < std::numeric_limits<long long>::max() &&
              (!_maxDepth || depth <= *_maxDepth));
 
-    _frontier->clear();
+    _frontier.clear();
     _frontierUsageBytes = 0;
 }
 
-namespace {
+bool DocumentSourceGraphLookUp::addToVisitedAndFrontier(Document result, long long depth) {
+    auto id = result.getField("_id");
 
-BSONObj addDepthFieldToObject(const std::string& field, long long depth, BSONObj object) {
-    BSONObjBuilder bob;
-    bob.appendElements(object);
-    bob.append(field, depth);
-    return bob.obj();
-}
-
-}  // namespace
-
-bool DocumentSourceGraphLookUp::addToVisitedAndFrontier(BSONObj result, long long depth) {
-    Value _id = Value(result.getField("_id"));
-
-    if (_visited.find(_id) != _visited.end()) {
+    if (_visited.find(id) != _visited.end()) {
         // We've already seen this object, don't repeat any work.
         return false;
     }
 
     // We have not seen this node before. If '_depthField' was specified, add the field to the
     // object.
-    BSONObj fullObject =
-        _depthField ? addDepthFieldToObject(_depthField->fullPath(), depth, result) : result;
+    if (_depthField) {
+        MutableDocument mutableDoc(std::move(result));
+        mutableDoc.setNestedField(*_depthField, Value(depth));
+        result = mutableDoc.freeze();
+    }
 
-    // Add the object to our '_visited' list.
-    _visited[_id] = fullObject;
-
-    // Update the size of '_visited' appropriately.
-    _visitedUsageBytes += _id.getApproximateSize();
-    _visitedUsageBytes += static_cast<size_t>(fullObject.objsize());
-
-    // Add the 'connectFrom' field of 'result' into '_frontier'. If the 'connectFrom' field is an
+    // Add the 'connectFromField' of 'result' into '_frontier'. If the 'connectFromField' is an
     // array, we treat it as connecting to multiple values, so we must add each element to
     // '_frontier'.
-    BSONElementSet recurseOnValues;
-    dps::extractAllElementsAlongPath(result, _connectFromField.fullPath(), recurseOnValues);
+    document_path_support::visitAllValuesAtPath(
+        result, _connectFromField, [this](const Value& nextFrontierValue) {
+            _frontier.insert(nextFrontierValue);
+            _frontierUsageBytes += nextFrontierValue.getApproximateSize();
+        });
 
-    for (auto&& elem : recurseOnValues) {
-        Value recurseOn = Value(elem);
-        if (recurseOn.isArray()) {
-            for (auto&& subElem : recurseOn.getArray()) {
-                _frontier->insert(subElem);
-                _frontierUsageBytes += subElem.getApproximateSize();
-            }
-        } else if (!recurseOn.missing()) {
-            // Don't recurse on a missing value.
-            _frontier->insert(recurseOn);
-            _frontierUsageBytes += recurseOn.getApproximateSize();
-        }
-    }
+    // Add the object to our '_visited' list and update the size of '_visited' appropriately.
+    _visitedUsageBytes += id.getApproximateSize();
+    _visitedUsageBytes += result.getApproximateSize();
+
+    _visited[id] = std::move(result);
 
     // We inserted into _visited, so return true.
     return true;
 }
 
-void DocumentSourceGraphLookUp::addToCache(const BSONObj& result,
+void DocumentSourceGraphLookUp::addToCache(const Document& result,
                                            const ValueUnorderedSet& queried) {
-    BSONElementSet cacheByValues;
-    dps::extractAllElementsAlongPath(result, _connectToField.fullPath(), cacheByValues);
-
-    for (auto&& elem : cacheByValues) {
-        Value cacheBy(elem);
-        if (cacheBy.isArray()) {
-            for (auto&& val : cacheBy.getArray()) {
-                if (queried.find(val) != queried.end()) {
-                    _cache.insert(val.getOwned(), result.getOwned());
-                }
-            }
-        } else if (!cacheBy.missing() && queried.find(cacheBy) != queried.end()) {
-            // It is possible that 'cacheBy' is a single value, but was not queried for. For
+    document_path_support::visitAllValuesAtPath(
+        result, _connectToField, [this, &queried, &result](const Value& connectToValue) {
+            // It is possible that 'connectToValue' is a single value, but was not queried for. For
             // instance, with a connectToField of "a.b" and a document with the structure:
             // {a: [{b: 1}, {b: 0}]}, this document will be retrieved by querying for "{b: 1}", but
-            // the outer for loop will split this into two separate cacheByValues. {b: 0} was not
+            // the outer for loop will split this into two separate connectToValues. {b: 0} was not
             // queried for, and thus, we cannot cache under it.
-            _cache.insert(cacheBy.getOwned(), result.getOwned());
-        }
-    }
+            if (queried.find(connectToValue) != queried.end()) {
+                _cache.insert(connectToValue, result);
+            }
+        });
 }
 
-boost::optional<BSONObj> DocumentSourceGraphLookUp::makeMatchStageFromFrontier(BSONObjSet* cached) {
+boost::optional<BSONObj> DocumentSourceGraphLookUp::makeMatchStageFromFrontier(
+    DocumentUnorderedSet* cached) {
     // Add any cached values to 'cached' and remove them from '_frontier'.
-    for (auto it = _frontier->begin(); it != _frontier->end();) {
+    for (auto it = _frontier.begin(); it != _frontier.end();) {
         if (auto entry = _cache[*it]) {
-            for (auto&& obj : *entry) {
-                cached->insert(obj);
-            }
+            cached->insert(entry->begin(), entry->end());
             size_t valueSize = it->getApproximateSize();
-            it = _frontier->erase(it);
+            it = _frontier.erase(it);
 
             // If the cached value increased in size while in the cache, we don't want to underflow
             // '_frontierUsageBytes'.
             invariant(valueSize <= _frontierUsageBytes);
             _frontierUsageBytes -= valueSize;
         } else {
-            it = std::next(it);
+            ++it;
         }
     }
 
@@ -340,7 +317,7 @@ boost::optional<BSONObj> DocumentSourceGraphLookUp::makeMatchStageFromFrontier(B
                     BSONObjBuilder subObj(connectToObj.subobjStart(_connectToField.fullPath()));
                     {
                         BSONArrayBuilder in(subObj.subarrayStart("$in"));
-                        for (auto&& value : *_frontier) {
+                        for (auto&& value : _frontier) {
                             in << value;
                         }
                     }
@@ -349,25 +326,23 @@ boost::optional<BSONObj> DocumentSourceGraphLookUp::makeMatchStageFromFrontier(B
         }
     }
 
-    return _frontier->empty() ? boost::none : boost::optional<BSONObj>(match.obj());
+    return _frontier.empty() ? boost::none : boost::optional<BSONObj>(match.obj());
 }
 
 void DocumentSourceGraphLookUp::performSearch() {
     // Make sure _input is set before calling performSearch().
     invariant(_input);
 
-    _variables->setRoot(*_input);
-    Value startingValue = _startWith->evaluateInternal(_variables.get());
-    _variables->clearRoot();
+    Value startingValue = _startWith->evaluate(*_input);
 
     // If _startWith evaluates to an array, treat each value as a separate starting point.
     if (startingValue.isArray()) {
         for (auto value : startingValue.getArray()) {
-            _frontier->insert(value);
+            _frontier.insert(value);
             _frontierUsageBytes += value.getApproximateSize();
         }
     } else {
-        _frontier->insert(startingValue);
+        _frontier.insert(startingValue);
         _frontierUsageBytes += startingValue.getApproximateSize();
     }
 
@@ -382,7 +357,7 @@ DocumentSource::GetModPathsReturn DocumentSourceGraphLookUp::getModifiedPaths()
         modifiedPaths.insert(pathsModifiedByUnwind.paths.begin(),
                              pathsModifiedByUnwind.paths.end());
     }
-    return {GetModPathsReturn::Type::kFiniteSet, std::move(modifiedPaths)};
+    return {GetModPathsReturn::Type::kFiniteSet, std::move(modifiedPaths), {}};
 }
 
 Pipeline::SourceContainer::iterator DocumentSourceGraphLookUp::doOptimizeAt(
@@ -420,7 +395,8 @@ void DocumentSourceGraphLookUp::checkMemoryUsage() {
     _cache.evictDownTo(_maxMemoryUsageBytes - _frontierUsageBytes - _visitedUsageBytes);
 }
 
-void DocumentSourceGraphLookUp::serializeToArray(std::vector<Value>& array, bool explain) const {
+void DocumentSourceGraphLookUp::serializeToArray(
+    std::vector<Value>& array, boost::optional<ExplainOptions::Verbosity> explain) const {
     // Serialize default options.
     MutableDocument spec(DOC("from" << _from.coll() << "as" << _as.fullPath() << "connectToField"
                                     << _connectToField.fullPath()
@@ -460,24 +436,6 @@ void DocumentSourceGraphLookUp::serializeToArray(std::vector<Value>& array, bool
     }
 }
 
-void DocumentSourceGraphLookUp::doInjectExpressionContext() {
-    _startWith->injectExpressionContext(pExpCtx);
-
-    auto it = pExpCtx->resolvedNamespaces.find(_from.coll());
-    invariant(it != pExpCtx->resolvedNamespaces.end());
-    const auto& resolvedNamespace = it->second;
-    _fromExpCtx = pExpCtx->copyWith(resolvedNamespace.ns);
-    _fromPipeline = resolvedNamespace.pipeline;
-
-    // We append an additional BSONObj to '_fromPipeline' as a placeholder for the $match stage
-    // we'll eventually construct from the input document.
-    _fromPipeline.reserve(_fromPipeline.size() + 1);
-    _fromPipeline.push_back(BSONObj());
-
-    _frontier = pExpCtx->getValueComparator().makeUnorderedValueSet();
-    _cache.setValueComparator(pExpCtx->getValueComparator());
-}
-
 void DocumentSourceGraphLookUp::doDetachFromOperationContext() {
     _fromExpCtx->opCtx = nullptr;
 }
@@ -497,7 +455,7 @@ DocumentSourceGraphLookUp::DocumentSourceGraphLookUp(
     boost::optional<FieldPath> depthField,
     boost::optional<long long> maxDepth,
     boost::optional<boost::intrusive_ptr<DocumentSourceUnwind>> unwindSrc)
-    : DocumentSourceNeedsMongod(expCtx),
+    : DocumentSourceNeedsMongoProcessInterface(expCtx),
       _from(std::move(from)),
       _as(std::move(as)),
       _connectFromField(std::move(connectFromField)),
@@ -506,9 +464,19 @@ DocumentSourceGraphLookUp::DocumentSourceGraphLookUp(
       _additionalFilter(additionalFilter),
       _depthField(depthField),
       _maxDepth(maxDepth),
-      _visited(ValueComparator::kInstance.makeUnorderedValueMap<BSONObj>()),
-      _cache(expCtx->getValueComparator()),
-      _unwind(unwindSrc) {}
+      _frontier(pExpCtx->getValueComparator().makeUnorderedValueSet()),
+      _visited(ValueComparator::kInstance.makeUnorderedValueMap<Document>()),
+      _cache(pExpCtx->getValueComparator()),
+      _unwind(unwindSrc) {
+    const auto& resolvedNamespace = pExpCtx->getResolvedNamespace(_from);
+    _fromExpCtx = pExpCtx->copyWith(resolvedNamespace.ns);
+    _fromPipeline = resolvedNamespace.pipeline;
+
+    // We append an additional BSONObj to '_fromPipeline' as a placeholder for the $match stage
+    // we'll eventually construct from the input document.
+    _fromPipeline.reserve(_fromPipeline.size() + 1);
+    _fromPipeline.push_back(BSONObj());
+}
 
 intrusive_ptr<DocumentSourceGraphLookUp> DocumentSourceGraphLookUp::create(
     const intrusive_ptr<ExpressionContext>& expCtx,
@@ -532,9 +500,6 @@ intrusive_ptr<DocumentSourceGraphLookUp> DocumentSourceGraphLookUp::create(
                                       depthField,
                                       maxDepth,
                                       unwindSrc));
-    source->_variables.reset(new Variables());
-
-    source->injectExpressionContext(expCtx);
     return source;
 }
 
@@ -549,14 +514,13 @@ intrusive_ptr<DocumentSource> DocumentSourceGraphLookUp::createFromBson(
     boost::optional<long long> maxDepth;
     boost::optional<BSONObj> additionalFilter;
 
-    VariablesIdGenerator idGenerator;
-    VariablesParseState vps(&idGenerator);
+    VariablesParseState vps = expCtx->variablesParseState;
 
     for (auto&& argument : elem.Obj()) {
         const auto argName = argument.fieldNameStringData();
 
         if (argName == "startWith") {
-            startWith = Expression::parseOperand(argument, vps);
+            startWith = Expression::parseOperand(expCtx, argument, vps);
             continue;
         } else if (argName == "maxDepth") {
             uassert(40100,
@@ -580,9 +544,9 @@ intrusive_ptr<DocumentSource> DocumentSourceGraphLookUp::createFromBson(
                     argument.type() == Object);
 
             // We don't need to keep ahold of the MatchExpression, but we do need to ensure that
-            // the specified object is parseable.
-            auto parsedMatchExpression = MatchExpressionParser::parse(
-                argument.embeddedObject(), ExtensionsCallbackDisallowExtensions(), nullptr);
+            // the specified object is parseable and does not contain extensions.
+            auto parsedMatchExpression =
+                MatchExpressionParser::parse(argument.embeddedObject(), expCtx);
 
             uassert(40186,
                     str::stream()
@@ -590,13 +554,6 @@ intrusive_ptr<DocumentSource> DocumentSourceGraphLookUp::createFromBson(
                         << parsedMatchExpression.getStatus().reason(),
                     parsedMatchExpression.isOK());
 
-            uassert(40187,
-                    str::stream()
-                        << "Failed to parse 'restrictSearchWithMatch' option to $graphLookup: "
-                        << "$near not supported.",
-                    !QueryPlannerCommon::hasNode(parsedMatchExpression.getValue().get(),
-                                                 MatchExpression::GEO_NEAR));
-
             additionalFilter = argument.embeddedObject().getOwned();
             continue;
         }
@@ -647,8 +604,6 @@ intrusive_ptr<DocumentSource> DocumentSourceGraphLookUp::createFromBson(
                                       maxDepth,
                                       boost::none));
 
-    newSource->_variables.reset(new Variables(idGenerator.getIdCount()));
-
     return std::move(newSource);
 }
 }  // namespace mongo
