@@ -36,18 +36,13 @@
 #include "mongo/db/pipeline/document_source_graph_lookup.h"
 #include "mongo/db/pipeline/document_source_mock.h"
 #include "mongo/db/pipeline/document_value_test_util.h"
-#include "mongo/db/pipeline/stub_mongod_interface.h"
+#include "mongo/db/pipeline/stub_mongo_process_interface.h"
 #include "mongo/unittest/unittest.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/mongoutils/str.h"
 
 namespace mongo {
 
-// Crutch.
-bool isMongos() {
-    return false;
-}
-
 namespace {
 
 // This provides access to getExpCtx(), but we'll use a different name for this test suite.
@@ -58,29 +53,40 @@ using DocumentSourceGraphLookUpTest = AggregationContextFixture;
 //
 
 /**
- * A MongodInterface use for testing that supports making pipelines with an initial
+ * A MongoProcessInterface use for testing that supports making pipelines with an initial
  * DocumentSourceMock source.
  */
-class MockMongodImplementation final : public StubMongodInterface {
+class MockMongoProcessInterfaceImplementation final : public StubMongoProcessInterface {
 public:
-    MockMongodImplementation(std::deque<DocumentSource::GetNextResult> results)
+    MockMongoProcessInterfaceImplementation(std::deque<DocumentSource::GetNextResult> results)
         : _results(std::move(results)) {}
 
-    StatusWith<boost::intrusive_ptr<Pipeline>> makePipeline(
+    StatusWith<std::unique_ptr<Pipeline, Pipeline::Deleter>> makePipeline(
         const std::vector<BSONObj>& rawPipeline,
-        const boost::intrusive_ptr<ExpressionContext>& expCtx) final {
+        const boost::intrusive_ptr<ExpressionContext>& expCtx,
+        const MakePipelineOptions opts) final {
         auto pipeline = Pipeline::parse(rawPipeline, expCtx);
         if (!pipeline.isOK()) {
             return pipeline.getStatus();
         }
 
-        pipeline.getValue()->addInitialSource(DocumentSourceMock::create(_results));
-        pipeline.getValue()->injectExpressionContext(expCtx);
-        pipeline.getValue()->optimizePipeline();
+        if (opts.optimize) {
+            pipeline.getValue()->optimizePipeline();
+        }
+
+        if (opts.attachCursorSource) {
+            uassertStatusOK(attachCursorSourceToPipeline(expCtx, pipeline.getValue().get()));
+        }
 
         return pipeline;
     }
 
+    Status attachCursorSourceToPipeline(const boost::intrusive_ptr<ExpressionContext>& expCtx,
+                                        Pipeline* pipeline) override {
+        pipeline->addInitialSource(DocumentSourceMock::create(_results));
+        return Status::OK();
+    }
+
 private:
     std::deque<DocumentSource::GetNextResult> _results;
 };
@@ -95,22 +101,23 @@ TEST_F(DocumentSourceGraphLookUpTest,
     std::deque<DocumentSource::GetNextResult> fromContents{Document{{"to", 0}}};
 
     NamespaceString fromNs("test", "graph_lookup");
-    expCtx->resolvedNamespaces[fromNs.coll()] = {fromNs, std::vector<BSONObj>{}};
-    auto graphLookupStage = DocumentSourceGraphLookUp::create(expCtx,
-                                                              fromNs,
-                                                              "results",
-                                                              "from",
-                                                              "to",
-                                                              ExpressionFieldPath::create("_id"),
-                                                              boost::none,
-                                                              boost::none,
-                                                              boost::none,
-                                                              boost::none);
+    expCtx->setResolvedNamespace(fromNs, {fromNs, std::vector<BSONObj>{}});
+    auto graphLookupStage =
+        DocumentSourceGraphLookUp::create(expCtx,
+                                          fromNs,
+                                          "results",
+                                          "from",
+                                          "to",
+                                          ExpressionFieldPath::create(expCtx, "_id"),
+                                          boost::none,
+                                          boost::none,
+                                          boost::none,
+                                          boost::none);
     graphLookupStage->setSource(inputMock.get());
-    graphLookupStage->injectMongodInterface(
-        std::make_shared<MockMongodImplementation>(std::move(fromContents)));
+    graphLookupStage->injectMongoProcessInterface(
+        std::make_shared<MockMongoProcessInterfaceImplementation>(std::move(fromContents)));
 
-    ASSERT_THROWS_CODE(graphLookupStage->getNext(), UserException, 40271);
+    ASSERT_THROWS_CODE(graphLookupStage->getNext(), AssertionException, 40271);
 }
 
 TEST_F(DocumentSourceGraphLookUpTest,
@@ -121,25 +128,26 @@ TEST_F(DocumentSourceGraphLookUpTest,
     auto inputMock = DocumentSourceMock::create(std::move(inputs));
 
     std::deque<DocumentSource::GetNextResult> fromContents{
-        Document{{"_id", "a"}, {"to", 0}, {"from", 1}}, Document{{"to", 1}}};
+        Document{{"_id", "a"_sd}, {"to", 0}, {"from", 1}}, Document{{"to", 1}}};
 
     NamespaceString fromNs("test", "graph_lookup");
-    expCtx->resolvedNamespaces[fromNs.coll()] = {fromNs, std::vector<BSONObj>{}};
-    auto graphLookupStage = DocumentSourceGraphLookUp::create(expCtx,
-                                                              fromNs,
-                                                              "results",
-                                                              "from",
-                                                              "to",
-                                                              ExpressionFieldPath::create("_id"),
-                                                              boost::none,
-                                                              boost::none,
-                                                              boost::none,
-                                                              boost::none);
+    expCtx->setResolvedNamespace(fromNs, {fromNs, std::vector<BSONObj>{}});
+    auto graphLookupStage =
+        DocumentSourceGraphLookUp::create(expCtx,
+                                          fromNs,
+                                          "results",
+                                          "from",
+                                          "to",
+                                          ExpressionFieldPath::create(expCtx, "_id"),
+                                          boost::none,
+                                          boost::none,
+                                          boost::none,
+                                          boost::none);
     graphLookupStage->setSource(inputMock.get());
-    graphLookupStage->injectMongodInterface(
-        std::make_shared<MockMongodImplementation>(std::move(fromContents)));
+    graphLookupStage->injectMongoProcessInterface(
+        std::make_shared<MockMongoProcessInterfaceImplementation>(std::move(fromContents)));
 
-    ASSERT_THROWS_CODE(graphLookupStage->getNext(), UserException, 40271);
+    ASSERT_THROWS_CODE(graphLookupStage->getNext(), AssertionException, 40271);
 }
 
 TEST_F(DocumentSourceGraphLookUpTest,
@@ -152,23 +160,24 @@ TEST_F(DocumentSourceGraphLookUpTest,
     std::deque<DocumentSource::GetNextResult> fromContents{Document{{"to", 0}}};
 
     NamespaceString fromNs("test", "graph_lookup");
-    expCtx->resolvedNamespaces[fromNs.coll()] = {fromNs, std::vector<BSONObj>{}};
+    expCtx->setResolvedNamespace(fromNs, {fromNs, std::vector<BSONObj>{}});
     auto unwindStage = DocumentSourceUnwind::create(expCtx, "results", false, boost::none);
-    auto graphLookupStage = DocumentSourceGraphLookUp::create(expCtx,
-                                                              fromNs,
-                                                              "results",
-                                                              "from",
-                                                              "to",
-                                                              ExpressionFieldPath::create("_id"),
-                                                              boost::none,
-                                                              boost::none,
-                                                              boost::none,
-                                                              unwindStage);
-    graphLookupStage->injectMongodInterface(
-        std::make_shared<MockMongodImplementation>(std::move(fromContents)));
+    auto graphLookupStage =
+        DocumentSourceGraphLookUp::create(expCtx,
+                                          fromNs,
+                                          "results",
+                                          "from",
+                                          "to",
+                                          ExpressionFieldPath::create(expCtx, "_id"),
+                                          boost::none,
+                                          boost::none,
+                                          boost::none,
+                                          unwindStage);
+    graphLookupStage->injectMongoProcessInterface(
+        std::make_shared<MockMongoProcessInterfaceImplementation>(std::move(fromContents)));
     graphLookupStage->setSource(inputMock.get());
 
-    ASSERT_THROWS_CODE(graphLookupStage->getNext(), UserException, 40271);
+    ASSERT_THROWS_CODE(graphLookupStage->getNext(), AssertionException, 40271);
 }
 
 bool arrayContains(const boost::intrusive_ptr<ExpressionContext>& expCtx,
@@ -187,28 +196,29 @@ TEST_F(DocumentSourceGraphLookUpTest,
     std::deque<DocumentSource::GetNextResult> inputs{Document{{"_id", 0}}};
     auto inputMock = DocumentSourceMock::create(std::move(inputs));
 
-    Document to0from1{{"_id", "a"}, {"to", 0}, {"from", 1}};
-    Document to0from2{{"_id", "a"}, {"to", 0}, {"from", 2}};
-    Document to1{{"_id", "b"}, {"to", 1}};
-    Document to2{{"_id", "c"}, {"to", 2}};
+    Document to0from1{{"_id", "a"_sd}, {"to", 0}, {"from", 1}};
+    Document to0from2{{"_id", "a"_sd}, {"to", 0}, {"from", 2}};
+    Document to1{{"_id", "b"_sd}, {"to", 1}};
+    Document to2{{"_id", "c"_sd}, {"to", 2}};
     std::deque<DocumentSource::GetNextResult> fromContents{
         Document(to1), Document(to2), Document(to0from1), Document(to0from2)};
 
     NamespaceString fromNs("test", "graph_lookup");
-    expCtx->resolvedNamespaces[fromNs.coll()] = {fromNs, std::vector<BSONObj>{}};
-    auto graphLookupStage = DocumentSourceGraphLookUp::create(expCtx,
-                                                              fromNs,
-                                                              "results",
-                                                              "from",
-                                                              "to",
-                                                              ExpressionFieldPath::create("_id"),
-                                                              boost::none,
-                                                              boost::none,
-                                                              boost::none,
-                                                              boost::none);
+    expCtx->setResolvedNamespace(fromNs, {fromNs, std::vector<BSONObj>{}});
+    auto graphLookupStage =
+        DocumentSourceGraphLookUp::create(expCtx,
+                                          fromNs,
+                                          "results",
+                                          "from",
+                                          "to",
+                                          ExpressionFieldPath::create(expCtx, "_id"),
+                                          boost::none,
+                                          boost::none,
+                                          boost::none,
+                                          boost::none);
     graphLookupStage->setSource(inputMock.get());
-    graphLookupStage->injectMongodInterface(
-        std::make_shared<MockMongodImplementation>(std::move(fromContents)));
+    graphLookupStage->injectMongoProcessInterface(
+        std::make_shared<MockMongoProcessInterfaceImplementation>(std::move(fromContents)));
     graphLookupStage->setSource(inputMock.get());
 
     auto next = graphLookupStage->getNext();
@@ -256,17 +266,17 @@ TEST_F(DocumentSourceGraphLookUpTest, ShouldPropagatePauses) {
                                     DocumentSource::GetNextResult::makePauseExecution()});
 
     std::deque<DocumentSource::GetNextResult> fromContents{
-        Document{{"_id", "a"}, {"to", 0}, {"from", 1}}, Document{{"_id", "b"}, {"to", 1}}};
+        Document{{"_id", "a"_sd}, {"to", 0}, {"from", 1}}, Document{{"_id", "b"_sd}, {"to", 1}}};
 
     NamespaceString fromNs("test", "foreign");
-    expCtx->resolvedNamespaces[fromNs.coll()] = {fromNs, std::vector<BSONObj>{}};
+    expCtx->setResolvedNamespace(fromNs, {fromNs, std::vector<BSONObj>{}});
     auto graphLookupStage =
         DocumentSourceGraphLookUp::create(expCtx,
                                           fromNs,
                                           "results",
                                           "from",
                                           "to",
-                                          ExpressionFieldPath::create("startPoint"),
+                                          ExpressionFieldPath::create(expCtx, "startPoint"),
                                           boost::none,
                                           boost::none,
                                           boost::none,
@@ -274,8 +284,8 @@ TEST_F(DocumentSourceGraphLookUpTest, ShouldPropagatePauses) {
 
     graphLookupStage->setSource(inputMock.get());
 
-    graphLookupStage->injectMongodInterface(
-        std::make_shared<MockMongodImplementation>(std::move(fromContents)));
+    graphLookupStage->injectMongoProcessInterface(
+        std::make_shared<MockMongoProcessInterfaceImplementation>(std::move(fromContents)));
 
     auto next = graphLookupStage->getNext();
     ASSERT_TRUE(next.isAdvanced());
@@ -289,9 +299,9 @@ TEST_F(DocumentSourceGraphLookUpTest, ShouldPropagatePauses) {
     ASSERT_EQ(result["results"].getArray().size(), 2UL);
     ASSERT_TRUE(arrayContains(expCtx,
                               result["results"].getArray(),
-                              Value(Document{{"_id", "a"}, {"to", 0}, {"from", 1}})));
+                              Value(Document{{"_id", "a"_sd}, {"to", 0}, {"from", 1}})));
     ASSERT_TRUE(arrayContains(
-        expCtx, result["results"].getArray(), Value(Document{{"_id", "b"}, {"to", 1}})));
+        expCtx, result["results"].getArray(), Value(Document{{"_id", "b"_sd}, {"to", 1}})));
 
     ASSERT_TRUE(graphLookupStage->getNext().isPaused());
 
@@ -303,9 +313,9 @@ TEST_F(DocumentSourceGraphLookUpTest, ShouldPropagatePauses) {
     ASSERT_EQ(result["results"].getArray().size(), 2UL);
     ASSERT_TRUE(arrayContains(expCtx,
                               result["results"].getArray(),
-                              Value(Document{{"_id", "a"}, {"to", 0}, {"from", 1}})));
+                              Value(Document{{"_id", "a"_sd}, {"to", 0}, {"from", 1}})));
     ASSERT_TRUE(arrayContains(
-        expCtx, result["results"].getArray(), Value(Document{{"_id", "b"}, {"to", 1}})));
+        expCtx, result["results"].getArray(), Value(Document{{"_id", "b"_sd}, {"to", 1}})));
 
     ASSERT_TRUE(graphLookupStage->getNext().isPaused());
 
@@ -324,10 +334,10 @@ TEST_F(DocumentSourceGraphLookUpTest, ShouldPropagatePausesWhileUnwinding) {
                                     DocumentSource::GetNextResult::makePauseExecution()});
 
     std::deque<DocumentSource::GetNextResult> fromContents{
-        Document{{"_id", "a"}, {"to", 0}, {"from", 1}}, Document{{"_id", "b"}, {"to", 1}}};
+        Document{{"_id", "a"_sd}, {"to", 0}, {"from", 1}}, Document{{"_id", "b"_sd}, {"to", 1}}};
 
     NamespaceString fromNs("test", "foreign");
-    expCtx->resolvedNamespaces[fromNs.coll()] = {fromNs, std::vector<BSONObj>{}};
+    expCtx->setResolvedNamespace(fromNs, {fromNs, std::vector<BSONObj>{}});
 
     const bool preserveNullAndEmptyArrays = false;
     const boost::optional<std::string> includeArrayIndex = boost::none;
@@ -340,7 +350,7 @@ TEST_F(DocumentSourceGraphLookUpTest, ShouldPropagatePausesWhileUnwinding) {
                                           "results",
                                           "from",
                                           "to",
-                                          ExpressionFieldPath::create("startPoint"),
+                                          ExpressionFieldPath::create(expCtx, "startPoint"),
                                           boost::none,
                                           boost::none,
                                           boost::none,
@@ -348,13 +358,13 @@ TEST_F(DocumentSourceGraphLookUpTest, ShouldPropagatePausesWhileUnwinding) {
 
     graphLookupStage->setSource(inputMock.get());
 
-    graphLookupStage->injectMongodInterface(
-        std::make_shared<MockMongodImplementation>(std::move(fromContents)));
+    graphLookupStage->injectMongoProcessInterface(
+        std::make_shared<MockMongoProcessInterfaceImplementation>(std::move(fromContents)));
 
     // Assert it has the expected results. Note the results can be in either order.
     auto expectedA =
-        Document{{"startPoint", 0}, {"results", Document{{"_id", "a"}, {"to", 0}, {"from", 1}}}};
-    auto expectedB = Document{{"startPoint", 0}, {"results", Document{{"_id", "b"}, {"to", 1}}}};
+        Document{{"startPoint", 0}, {"results", Document{{"_id", "a"_sd}, {"to", 0}, {"from", 1}}}};
+    auto expectedB = Document{{"startPoint", 0}, {"results", Document{{"_id", "b"_sd}, {"to", 1}}}};
     auto next = graphLookupStage->getNext();
     ASSERT_TRUE(next.isAdvanced());
     if (expCtx->getDocumentComparator().evaluate(next.getDocument() == expectedA)) {
@@ -392,14 +402,14 @@ TEST_F(DocumentSourceGraphLookUpTest, ShouldPropagatePausesWhileUnwinding) {
 TEST_F(DocumentSourceGraphLookUpTest, GraphLookupShouldReportAsFieldIsModified) {
     auto expCtx = getExpCtx();
     NamespaceString fromNs("test", "foreign");
-    expCtx->resolvedNamespaces[fromNs.coll()] = {fromNs, std::vector<BSONObj>{}};
+    expCtx->setResolvedNamespace(fromNs, {fromNs, std::vector<BSONObj>{}});
     auto graphLookupStage =
         DocumentSourceGraphLookUp::create(expCtx,
                                           fromNs,
                                           "results",
                                           "from",
                                           "to",
-                                          ExpressionFieldPath::create("startPoint"),
+                                          ExpressionFieldPath::create(expCtx, "startPoint"),
                                           boost::none,
                                           boost::none,
                                           boost::none,
@@ -414,7 +424,7 @@ TEST_F(DocumentSourceGraphLookUpTest, GraphLookupShouldReportAsFieldIsModified)
 TEST_F(DocumentSourceGraphLookUpTest, GraphLookupShouldReportFieldsModifiedByAbsorbedUnwind) {
     auto expCtx = getExpCtx();
     NamespaceString fromNs("test", "foreign");
-    expCtx->resolvedNamespaces[fromNs.coll()] = {fromNs, std::vector<BSONObj>{}};
+    expCtx->setResolvedNamespace(fromNs, {fromNs, std::vector<BSONObj>{}});
     auto unwindStage =
         DocumentSourceUnwind::create(expCtx, "results", false, std::string("arrIndex"));
     auto graphLookupStage =
@@ -423,7 +433,7 @@ TEST_F(DocumentSourceGraphLookUpTest, GraphLookupShouldReportFieldsModifiedByAbs
                                           "results",
                                           "from",
                                           "to",
-                                          ExpressionFieldPath::create("startPoint"),
+                                          ExpressionFieldPath::create(expCtx, "startPoint"),
                                           boost::none,
                                           boost::none,
                                           boost::none,
@@ -442,7 +452,7 @@ TEST_F(DocumentSourceGraphLookUpTest, GraphLookupWithComparisonExpressionForStar
     auto inputMock = DocumentSourceMock::create(Document({{"_id", 0}, {"a", 1}, {"b", 2}}));
 
     NamespaceString fromNs("test", "foreign");
-    expCtx->resolvedNamespaces[fromNs.coll()] = {fromNs, std::vector<BSONObj>{}};
+    expCtx->setResolvedNamespace(fromNs, {fromNs, std::vector<BSONObj>{}});
     std::deque<DocumentSource::GetNextResult> fromContents{Document{{"_id", 0}, {"to", true}},
                                                            Document{{"_id", 1}, {"to", false}}};
 
@@ -452,17 +462,18 @@ TEST_F(DocumentSourceGraphLookUpTest, GraphLookupWithComparisonExpressionForStar
         "results",
         "from",
         "to",
-        ExpressionCompare::create(ExpressionCompare::GT,
-                                  ExpressionFieldPath::create("a"),
-                                  ExpressionFieldPath::create("b")),
+        ExpressionCompare::create(expCtx,
+                                  ExpressionCompare::GT,
+                                  ExpressionFieldPath::create(expCtx, "a"),
+                                  ExpressionFieldPath::create(expCtx, "b")),
         boost::none,
         boost::none,
         boost::none,
         boost::none);
 
     graphLookupStage->setSource(inputMock.get());
-    graphLookupStage->injectMongodInterface(
-        std::make_shared<MockMongodImplementation>(std::move(fromContents)));
+    graphLookupStage->injectMongoProcessInterface(
+        std::make_shared<MockMongoProcessInterfaceImplementation>(std::move(fromContents)));
 
     auto next = graphLookupStage->getNext();
     ASSERT_TRUE(next.isAdvanced());
@@ -478,5 +489,140 @@ TEST_F(DocumentSourceGraphLookUpTest, GraphLookupWithComparisonExpressionForStar
     ASSERT_DOCUMENT_EQ(actualResult, expectedResult);
 }
 
+TEST_F(DocumentSourceGraphLookUpTest, ShouldExpandArraysAtEndOfConnectFromField) {
+    auto expCtx = getExpCtx();
+
+    std::deque<DocumentSource::GetNextResult> inputs{Document{{"_id", 0}, {"startVal", 0}}};
+    auto inputMock = DocumentSourceMock::create(std::move(inputs));
+
+    /* Make the following graph:
+     *   ,> 1 .
+     *  /      \
+     * 0 -> 2 --+-> 4
+     *  \      /
+     *   `> 3 '
+     */
+    Document startDoc{{"_id", 0},
+                      {"to", std::vector<Value>{Value(1), Value(2), Value(3)}}};  // Note the array.
+    Document middle1{{"_id", 1}, {"to", 4}};
+    Document middle2{{"_id", 2}, {"to", 4}};
+    Document middle3{{"_id", 3}, {"to", 4}};
+    Document sinkDoc{{"_id", 4}};
+
+    // GetNextResults are only constructable from an rvalue reference to a Document, so we have to
+    // explicitly copy.
+    std::deque<DocumentSource::GetNextResult> fromContents{Document(startDoc),
+                                                           Document(middle1),
+                                                           Document(middle2),
+                                                           Document(middle3),
+                                                           Document(sinkDoc)};
+
+    NamespaceString fromNs("test", "graph_lookup");
+    expCtx->setResolvedNamespace(fromNs, {fromNs, std::vector<BSONObj>{}});
+    auto graphLookupStage =
+        DocumentSourceGraphLookUp::create(expCtx,
+                                          fromNs,
+                                          "results",
+                                          "to",
+                                          "_id",
+                                          ExpressionFieldPath::create(expCtx, "startVal"),
+                                          boost::none,
+                                          boost::none,
+                                          boost::none,
+                                          boost::none);
+    graphLookupStage->setSource(inputMock.get());
+    graphLookupStage->injectMongoProcessInterface(
+        std::make_shared<MockMongoProcessInterfaceImplementation>(std::move(fromContents)));
+    graphLookupStage->setSource(inputMock.get());
+
+    auto next = graphLookupStage->getNext();
+    ASSERT_TRUE(next.isAdvanced());
+
+    ASSERT_EQ(3U, next.getDocument().size());
+    ASSERT_VALUE_EQ(Value(0), next.getDocument().getField("_id"));
+
+    auto resultsValue = next.getDocument().getField("results");
+    ASSERT(resultsValue.isArray());
+    auto resultsArray = resultsValue.getArray();
+
+    ASSERT(arrayContains(expCtx, resultsArray, Value(middle1)));
+    ASSERT(arrayContains(expCtx, resultsArray, Value(middle2)));
+    ASSERT(arrayContains(expCtx, resultsArray, Value(middle3)));
+    ASSERT(arrayContains(expCtx, resultsArray, Value(sinkDoc)));
+    ASSERT(graphLookupStage->getNext().isEOF());
+}
+
+TEST_F(DocumentSourceGraphLookUpTest, ShouldNotExpandArraysWithinArraysAtEndOfConnectFromField) {
+    auto expCtx = getExpCtx();
+
+    auto makeTupleValue = [](int left, int right) {
+        return Value(std::vector<Value>{Value(left), Value(right)});
+    };
+
+    std::deque<DocumentSource::GetNextResult> inputs{
+        Document{{"_id", 0}, {"startVal", makeTupleValue(0, 0)}}};
+    auto inputMock = DocumentSourceMock::create(std::move(inputs));
+
+    // Make the following graph:
+    //
+    // [0, 0] -> [1, 1]
+    //  |
+    //  v
+    // [2, 2]
+    //
+    // (unconnected)
+    // [1, 2]
+
+    // If the connectFromField were doubly expanded, we would query for connectToValues with an
+    // expression like {$in: [1, 2]} instead of {$in: [[1, 1], [2, 2]]}, the former of which would
+    // also include [1, 2].
+    Document startDoc{
+        {"_id", 0},
+        {"coordinate", makeTupleValue(0, 0)},
+        {"connectedTo",
+         std::vector<Value>{makeTupleValue(1, 1), makeTupleValue(2, 2)}}};  // Note the extra array.
+    Document target1{{"_id", 1}, {"coordinate", makeTupleValue(1, 1)}};
+    Document target2{{"_id", 2}, {"coordinate", makeTupleValue(2, 2)}};
+    Document soloDoc{{"_id", 3}, {"coordinate", makeTupleValue(1, 2)}};
+
+    // GetNextResults are only constructable from an rvalue reference to a Document, so we have to
+    // explicitly copy.
+    std::deque<DocumentSource::GetNextResult> fromContents{
+        Document(startDoc), Document(target1), Document(target2), Document(soloDoc)};
+
+    NamespaceString fromNs("test", "graph_lookup");
+    expCtx->setResolvedNamespace(fromNs, {fromNs, std::vector<BSONObj>{}});
+    auto graphLookupStage =
+        DocumentSourceGraphLookUp::create(expCtx,
+                                          fromNs,
+                                          "results",
+                                          "connectedTo",
+                                          "coordinate",
+                                          ExpressionFieldPath::create(expCtx, "startVal"),
+                                          boost::none,
+                                          boost::none,
+                                          boost::none,
+                                          boost::none);
+    graphLookupStage->setSource(inputMock.get());
+    graphLookupStage->injectMongoProcessInterface(
+        std::make_shared<MockMongoProcessInterfaceImplementation>(std::move(fromContents)));
+    graphLookupStage->setSource(inputMock.get());
+
+    auto next = graphLookupStage->getNext();
+    ASSERT_TRUE(next.isAdvanced());
+
+    ASSERT_EQ(3U, next.getDocument().size());
+    ASSERT_VALUE_EQ(Value(0), next.getDocument().getField("_id"));
+
+    auto resultsValue = next.getDocument().getField("results");
+    ASSERT(resultsValue.isArray());
+    auto resultsArray = resultsValue.getArray();
+
+    ASSERT(arrayContains(expCtx, resultsArray, Value(target1)));
+    ASSERT(arrayContains(expCtx, resultsArray, Value(target2)));
+    ASSERT(!arrayContains(expCtx, resultsArray, Value(soloDoc)));
+    ASSERT(graphLookupStage->getNext().isEOF());
+}
+
 }  // namespace
 }  // namespace mongo
