@@ -45,7 +45,7 @@
 #include "mongo/db/pipeline/document_source_mock.h"
 #include "mongo/db/pipeline/document_value_test_util.h"
 #include "mongo/db/pipeline/expression.h"
-#include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/pipeline/expression_context_for_test.h"
 #include "mongo/db/pipeline/value_comparator.h"
 #include "mongo/db/query/query_test_service_context.h"
 #include "mongo/dbtests/dbtests.h"
@@ -70,13 +70,13 @@ using DocumentSourceGroupTest = AggregationContextFixture;
 
 TEST_F(DocumentSourceGroupTest, ShouldBeAbleToPauseLoading) {
     auto expCtx = getExpCtx();
-    expCtx->inRouter = true;  // Disallow external sort.
+    expCtx->inMongos = true;  // Disallow external sort.
                               // This is the only way to do this in a debug build.
     AccumulationStatement countStatement{"count",
-                                         AccumulationStatement::getFactory("$sum"),
-                                         ExpressionConstant::create(expCtx, Value(1))};
+                                         ExpressionConstant::create(expCtx, Value(1)),
+                                         AccumulationStatement::getFactory("$sum")};
     auto group = DocumentSourceGroup::create(
-        expCtx, ExpressionConstant::create(expCtx, Value(BSONNULL)), {countStatement}, 0);
+        expCtx, ExpressionConstant::create(expCtx, Value(BSONNULL)), {countStatement});
     auto mock = DocumentSourceMock::create({DocumentSource::GetNextResult::makePauseExecution(),
                                             Document(),
                                             DocumentSource::GetNextResult::makePauseExecution(),
@@ -103,17 +103,16 @@ TEST_F(DocumentSourceGroupTest, ShouldBeAbleToPauseLoadingWhileSpilled) {
     // Allow the $group stage to spill to disk.
     TempDir tempDir("DocumentSourceGroupTest");
     expCtx->tempDir = tempDir.path();
-    expCtx->extSortAllowed = true;
+    expCtx->allowDiskUse = true;
     const size_t maxMemoryUsageBytes = 1000;
 
-    VariablesIdGenerator idGen;
-    VariablesParseState vps(&idGen);
+    VariablesParseState vps = expCtx->variablesParseState;
     AccumulationStatement pushStatement{"spaceHog",
-                                        AccumulationStatement::getFactory("$push"),
-                                        ExpressionFieldPath::parse("$largeStr", vps)};
-    auto groupByExpression = ExpressionFieldPath::parse("$_id", vps);
+                                        ExpressionFieldPath::parse(expCtx, "$largeStr", vps),
+                                        AccumulationStatement::getFactory("$push")};
+    auto groupByExpression = ExpressionFieldPath::parse(expCtx, "$_id", vps);
     auto group = DocumentSourceGroup::create(
-        expCtx, groupByExpression, {pushStatement}, idGen.getIdCount(), maxMemoryUsageBytes);
+        expCtx, groupByExpression, {pushStatement}, maxMemoryUsageBytes);
 
     string largeStr(maxMemoryUsageBytes, 'x');
     auto mock = DocumentSourceMock::create({Document{{"_id", 0}, {"largeStr", largeStr}},
@@ -143,40 +142,38 @@ TEST_F(DocumentSourceGroupTest, ShouldBeAbleToPauseLoadingWhileSpilled) {
 TEST_F(DocumentSourceGroupTest, ShouldErrorIfNotAllowedToSpillToDiskAndResultSetIsTooLarge) {
     auto expCtx = getExpCtx();
     const size_t maxMemoryUsageBytes = 1000;
-    expCtx->inRouter = true;  // Disallow external sort.
+    expCtx->inMongos = true;  // Disallow external sort.
                               // This is the only way to do this in a debug build.
 
-    VariablesIdGenerator idGen;
-    VariablesParseState vps(&idGen);
+    VariablesParseState vps = expCtx->variablesParseState;
     AccumulationStatement pushStatement{"spaceHog",
-                                        AccumulationStatement::getFactory("$push"),
-                                        ExpressionFieldPath::parse("$largeStr", vps)};
-    auto groupByExpression = ExpressionFieldPath::parse("$_id", vps);
+                                        ExpressionFieldPath::parse(expCtx, "$largeStr", vps),
+                                        AccumulationStatement::getFactory("$push")};
+    auto groupByExpression = ExpressionFieldPath::parse(expCtx, "$_id", vps);
     auto group = DocumentSourceGroup::create(
-        expCtx, groupByExpression, {pushStatement}, idGen.getIdCount(), maxMemoryUsageBytes);
+        expCtx, groupByExpression, {pushStatement}, maxMemoryUsageBytes);
 
     string largeStr(maxMemoryUsageBytes, 'x');
     auto mock = DocumentSourceMock::create({Document{{"_id", 0}, {"largeStr", largeStr}},
                                             Document{{"_id", 1}, {"largeStr", largeStr}}});
     group->setSource(mock.get());
 
-    ASSERT_THROWS_CODE(group->getNext(), UserException, 16945);
+    ASSERT_THROWS_CODE(group->getNext(), AssertionException, 16945);
 }
 
 TEST_F(DocumentSourceGroupTest, ShouldCorrectlyTrackMemoryUsageBetweenPauses) {
     auto expCtx = getExpCtx();
     const size_t maxMemoryUsageBytes = 1000;
-    expCtx->inRouter = true;  // Disallow external sort.
+    expCtx->inMongos = true;  // Disallow external sort.
                               // This is the only way to do this in a debug build.
 
-    VariablesIdGenerator idGen;
-    VariablesParseState vps(&idGen);
+    VariablesParseState vps = expCtx->variablesParseState;
     AccumulationStatement pushStatement{"spaceHog",
-                                        AccumulationStatement::getFactory("$push"),
-                                        ExpressionFieldPath::parse("$largeStr", vps)};
-    auto groupByExpression = ExpressionFieldPath::parse("$_id", vps);
+                                        ExpressionFieldPath::parse(expCtx, "$largeStr", vps),
+                                        AccumulationStatement::getFactory("$push")};
+    auto groupByExpression = ExpressionFieldPath::parse(expCtx, "$_id", vps);
     auto group = DocumentSourceGroup::create(
-        expCtx, groupByExpression, {pushStatement}, idGen.getIdCount(), maxMemoryUsageBytes);
+        expCtx, groupByExpression, {pushStatement}, maxMemoryUsageBytes);
 
     string largeStr(maxMemoryUsageBytes / 2, 'x');
     auto mock = DocumentSourceMock::create({Document{{"_id", 0}, {"largeStr", largeStr}},
@@ -189,7 +186,7 @@ TEST_F(DocumentSourceGroupTest, ShouldCorrectlyTrackMemoryUsageBetweenPauses) {
     ASSERT_TRUE(group->getNext().isPaused());
 
     // The next should realize it's used too much memory.
-    ASSERT_THROWS_CODE(group->getNext(), UserException, 16945);
+    ASSERT_THROWS_CODE(group->getNext(), AssertionException, 16945);
 }
 
 BSONObj toBson(const intrusive_ptr<DocumentSource>& source) {
@@ -204,23 +201,24 @@ public:
     Base()
         : _queryServiceContext(stdx::make_unique<QueryTestServiceContext>()),
           _opCtx(_queryServiceContext->makeOperationContext()),
-          _ctx(new ExpressionContext(_opCtx.get(), AggregationRequest(NamespaceString(ns), {}))),
+          _ctx(new ExpressionContextForTest(_opCtx.get(),
+                                            AggregationRequest(NamespaceString(ns), {}))),
           _tempDir("DocumentSourceGroupTest") {}
 
 protected:
-    void createGroup(const BSONObj& spec, bool inShard = false, bool inRouter = false) {
+    void createGroup(const BSONObj& spec, bool inShard = false, bool inMongos = false) {
         BSONObj namedSpec = BSON("$group" << spec);
         BSONElement specElement = namedSpec.firstElement();
 
-        intrusive_ptr<ExpressionContext> expressionContext =
-            new ExpressionContext(_opCtx.get(), AggregationRequest(NamespaceString(ns), {}));
-        expressionContext->inShard = inShard;
-        expressionContext->inRouter = inRouter;
+        intrusive_ptr<ExpressionContextForTest> expressionContext =
+            new ExpressionContextForTest(_opCtx.get(), AggregationRequest(NamespaceString(ns), {}));
+        // For $group, 'inShard' implies 'fromMongos' and 'needsMerge'.
+        expressionContext->fromMongos = expressionContext->needsMerge = inShard;
+        expressionContext->inMongos = inMongos;
         // Won't spill to disk properly if it needs to.
         expressionContext->tempDir = _tempDir.path();
 
         _group = DocumentSourceGroup::createFromBson(specElement, expressionContext);
-        _group->injectExpressionContext(expressionContext);
         assertRoundTrips(_group);
     }
     DocumentSourceGroup* group() {
@@ -234,7 +232,7 @@ protected:
         ASSERT(source->getNext().isEOF());
     }
 
-    intrusive_ptr<ExpressionContext> ctx() const {
+    intrusive_ptr<ExpressionContextForTest> ctx() const {
         return _ctx;
     }
 
@@ -251,7 +249,7 @@ private:
     }
     std::unique_ptr<QueryTestServiceContext> _queryServiceContext;
     ServiceContext::UniqueOperationContext _opCtx;
-    intrusive_ptr<ExpressionContext> _ctx;
+    intrusive_ptr<ExpressionContextForTest> _ctx;
     intrusive_ptr<DocumentSource> _group;
     TempDir _tempDir;
 };
@@ -260,7 +258,7 @@ class ParseErrorBase : public Base {
 public:
     virtual ~ParseErrorBase() {}
     void run() {
-        ASSERT_THROWS(createGroup(spec()), UserException);
+        ASSERT_THROWS(createGroup(spec()), AssertionException);
     }
 
 protected:
@@ -304,7 +302,7 @@ public:
         BSONObj spec = BSON("$group"
                             << "foo");
         BSONElement specElement = spec.firstElement();
-        ASSERT_THROWS(DocumentSourceGroup::createFromBson(specElement, ctx()), UserException);
+        ASSERT_THROWS(DocumentSourceGroup::createFromBson(specElement, ctx()), AssertionException);
     }
 };
 
@@ -546,9 +544,10 @@ protected:
         // case only one shard is in use.
         SplittableDocumentSource* splittable = dynamic_cast<SplittableDocumentSource*>(group());
         ASSERT(splittable);
-        intrusive_ptr<DocumentSource> routerSource = splittable->getMergeSource();
-        ASSERT_NOT_EQUALS(group(), routerSource.get());
-        return routerSource;
+        auto routerSources = splittable->getMergeSources();
+        ASSERT_EQ(routerSources.size(), 1UL);
+        ASSERT_NOT_EQUALS(group(), routerSources.front().get());
+        return routerSources.front();
     }
     void checkResultSet(const intrusive_ptr<DocumentSource>& sink) {
         // Load the results from the DocumentSourceGroup and sort them by _id.
@@ -683,21 +682,21 @@ class GroupNullUndefinedIds : public CheckResultsBase {
 class ComplexId : public CheckResultsBase {
     deque<DocumentSource::GetNextResult> inputData() {
         return {DOC("a"
-                    << "de"
+                    << "de"_sd
                     << "b"
-                    << "ad"
+                    << "ad"_sd
                     << "c"
-                    << "beef"
+                    << "beef"_sd
                     << "d"
-                    << ""),
+                    << ""_sd),
                 DOC("a"
-                    << "d"
+                    << "d"_sd
                     << "b"
-                    << "eadbe"
+                    << "eadbe"_sd
                     << "c"
-                    << ""
+                    << ""_sd
                     << "d"
-                    << "ef")};
+                    << "ef"_sd)};
     }
     virtual BSONObj groupSpec() {
         return BSON("_id" << BSON("$concat" << BSON_ARRAY("$a"
@@ -932,7 +931,7 @@ public:
         ASSERT_TRUE(res.isAdvanced());
         ASSERT_VALUE_EQ(res.getDocument().getField("_id")["sub"]["x"], Value(5));
         ASSERT_VALUE_EQ(res.getDocument().getField("_id")["sub"]["y"], Value(1));
-        ASSERT_VALUE_EQ(res.getDocument().getField("_id")["sub"]["z"], Value("c"));
+        ASSERT_VALUE_EQ(res.getDocument().getField("_id")["sub"]["z"], Value("c"_sd));
 
         ASSERT_TRUE(group()->isStreaming());
 
@@ -1014,7 +1013,7 @@ public:
 
         // We pretend to be in the router so that we don't spill to disk, because this produces
         // inconsistent output on debug vs. non-debug builds.
-        const bool inRouter = true;
+        const bool inMongos = true;
         const bool inShard = false;
 
         createGroup(BSON("_id" << BSON("x"
@@ -1022,7 +1021,7 @@ public:
                                        << "y"
                                        << "$b")),
                     inShard,
-                    inRouter);
+                    inMongos);
         group()->setSource(source.get());
 
         group()->getNext();
@@ -1041,7 +1040,7 @@ public:
 
         // We pretend to be in the router so that we don't spill to disk, because this produces
         // inconsistent output on debug vs. non-debug builds.
-        const bool inRouter = true;
+        const bool inMongos = true;
         const bool inShard = false;
 
         createGroup(BSON("_id" << BSON("a"
@@ -1049,7 +1048,7 @@ public:
                                        << "b"
                                        << "$a")),
                     inShard,
-                    inRouter);
+                    inMongos);
         group()->setSource(source.get());
 
         group()->getNext();
@@ -1068,10 +1067,10 @@ public:
 
         // We pretend to be in the router so that we don't spill to disk, because this produces
         // inconsistent output on debug vs. non-debug builds.
-        const bool inRouter = true;
+        const bool inMongos = true;
         const bool inShard = false;
 
-        createGroup(fromjson("{_id: {$sum: ['$a', '$b']}}"), inShard, inRouter);
+        createGroup(fromjson("{_id: {$sum: ['$a', '$b']}}"), inShard, inMongos);
         group()->setSource(source.get());
 
         group()->getNext();
@@ -1103,7 +1102,7 @@ class ArrayConstantAccumulatorExpression : public CheckResultsBase {
 public:
     void run() {
         // A parse exception is thrown when a raw array is provided to an accumulator.
-        ASSERT_THROWS(createGroup(fromjson("{_id:1,a:{$push:[4,5,6]}}")), UserException);
+        ASSERT_THROWS(createGroup(fromjson("{_id:1,a:{$push:[4,5,6]}}")), AssertionException);
         // Run standard base tests.
         CheckResultsBase::run();
     }
