@@ -49,9 +49,7 @@ REGISTER_DOCUMENT_SOURCE(limit,
                          LiteParsedDocumentSourceDefault::parse,
                          DocumentSourceLimit::createFromBson);
 
-const char* DocumentSourceLimit::getSourceName() const {
-    return "$limit";
-}
+constexpr StringData DocumentSourceLimit::kStageName;
 
 Pipeline::SourceContainer::iterator DocumentSourceLimit::doOptimizeAt(
     Pipeline::SourceContainer::iterator itr, Pipeline::SourceContainer* container) {
@@ -85,7 +83,7 @@ DocumentSource::GetNextResult DocumentSourceLimit::getNext() {
     return nextInput;
 }
 
-Value DocumentSourceLimit::serialize(bool explain) const {
+Value DocumentSourceLimit::serialize(boost::optional<ExplainOptions::Verbosity> explain) const {
     return Value(Document{{getSourceName(), _limit}});
 }
 
@@ -93,7 +91,6 @@ intrusive_ptr<DocumentSourceLimit> DocumentSourceLimit::create(
     const intrusive_ptr<ExpressionContext>& pExpCtx, long long limit) {
     uassert(15958, "the limit must be positive", limit > 0);
     intrusive_ptr<DocumentSourceLimit> source(new DocumentSourceLimit(pExpCtx, limit));
-    source->injectExpressionContext(pExpCtx);
     return source;
 }
 
