@@ -38,6 +38,7 @@
 #include "mongo/db/pipeline/dependencies.h"
 #include "mongo/db/pipeline/document.h"
 #include "mongo/db/pipeline/document_value_test_util.h"
+#include "mongo/db/pipeline/expression_context_for_test.h"
 #include "mongo/db/pipeline/value.h"
 #include "mongo/unittest/unittest.h"
 
@@ -47,68 +48,82 @@ namespace {
 using std::vector;
 
 // These ParsedAddFields spec tests are a subset of the ParsedAggregationProjection creation tests.
-// ParsedAddField should behave the same way, but does not use the same creation, so we include
+// ParsedAddFields should behave the same way, but does not use the same creation, so we include
 // an abbreviation of the same tests here.
 
 // Verify that ParsedAddFields rejects specifications with conflicting field paths.
 TEST(ParsedAddFieldsSpec, ThrowsOnCreationWithConflictingFieldPaths) {
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     // These specs contain the same exact path.
-    ASSERT_THROWS(ParsedAddFields::create(BSON("a" << 1 << "a" << 2)), UserException);
-    ASSERT_THROWS(ParsedAddFields::create(BSON("a" << BSON("b" << 1 << "b" << 2))), UserException);
-    ASSERT_THROWS(ParsedAddFields::create(BSON("_id" << 3 << "_id" << true)), UserException);
+    ASSERT_THROWS(ParsedAddFields::create(expCtx, BSON("a" << 1 << "a" << 2)), AssertionException);
+    ASSERT_THROWS(ParsedAddFields::create(expCtx, BSON("a" << BSON("b" << 1 << "b" << 2))),
+                  AssertionException);
+    ASSERT_THROWS(ParsedAddFields::create(expCtx, BSON("_id" << 3 << "_id" << true)),
+                  AssertionException);
 
     // These specs contain overlapping paths.
-    ASSERT_THROWS(ParsedAddFields::create(BSON("a" << 1 << "a.b" << 2)), UserException);
-    ASSERT_THROWS(ParsedAddFields::create(BSON("a.b.c" << 1 << "a" << 2)), UserException);
-    ASSERT_THROWS(ParsedAddFields::create(BSON("_id" << true << "_id.x" << true)), UserException);
+    ASSERT_THROWS(ParsedAddFields::create(expCtx, BSON("a" << 1 << "a.b" << 2)),
+                  AssertionException);
+    ASSERT_THROWS(ParsedAddFields::create(expCtx, BSON("a.b.c" << 1 << "a" << 2)),
+                  AssertionException);
+    ASSERT_THROWS(ParsedAddFields::create(expCtx, BSON("_id" << true << "_id.x" << true)),
+                  AssertionException);
 }
 
 // Verify that ParsedAddFields rejects specifications that contain invalid field paths.
 TEST(ParsedAddFieldsSpec, ThrowsOnCreationWithInvalidFieldPath) {
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     // Dotted subfields are not allowed.
-    ASSERT_THROWS(ParsedAddFields::create(BSON("a" << BSON("b.c" << true))), UserException);
+    ASSERT_THROWS(ParsedAddFields::create(expCtx, BSON("a" << BSON("b.c" << true))),
+                  AssertionException);
 
     // The user cannot start a field with $.
-    ASSERT_THROWS(ParsedAddFields::create(BSON("$dollar" << 0)), UserException);
-    ASSERT_THROWS(ParsedAddFields::create(BSON("c.$d" << true)), UserException);
+    ASSERT_THROWS(ParsedAddFields::create(expCtx, BSON("$dollar" << 0)), AssertionException);
+    ASSERT_THROWS(ParsedAddFields::create(expCtx, BSON("c.$d" << true)), AssertionException);
 
     // Empty field names should throw an error.
-    ASSERT_THROWS(ParsedAddFields::create(BSON("" << 2)), UserException);
-    ASSERT_THROWS(ParsedAddFields::create(BSON("a" << BSON("" << true))), UserException);
-    ASSERT_THROWS(ParsedAddFields::create(BSON("" << BSON("a" << true))), UserException);
-    ASSERT_THROWS(ParsedAddFields::create(BSON("a." << true)), UserException);
-    ASSERT_THROWS(ParsedAddFields::create(BSON(".a" << true)), UserException);
+    ASSERT_THROWS(ParsedAddFields::create(expCtx, BSON("" << 2)), AssertionException);
+    ASSERT_THROWS(ParsedAddFields::create(expCtx, BSON("a" << BSON("" << true))),
+                  AssertionException);
+    ASSERT_THROWS(ParsedAddFields::create(expCtx, BSON("" << BSON("a" << true))),
+                  AssertionException);
+    ASSERT_THROWS(ParsedAddFields::create(expCtx, BSON("a." << true)), AssertionException);
+    ASSERT_THROWS(ParsedAddFields::create(expCtx, BSON(".a" << true)), AssertionException);
 }
 
 // Verify that ParsedAddFields rejects specifications that contain empty objects or invalid
 // expressions.
 TEST(ParsedAddFieldsSpec, ThrowsOnCreationWithInvalidObjectsOrExpressions) {
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     // Invalid expressions should be rejected.
-    ASSERT_THROWS(
-        ParsedAddFields::create(BSON("a" << BSON("$add" << BSON_ARRAY(4 << 2) << "b" << 1))),
-        UserException);
-    ASSERT_THROWS(ParsedAddFields::create(BSON("a" << BSON("$gt" << BSON("bad"
-                                                                         << "arguments")))),
-                  UserException);
     ASSERT_THROWS(ParsedAddFields::create(
-                      BSON("a" << false << "b" << BSON("$unknown" << BSON_ARRAY(4 << 2)))),
-                  UserException);
+                      expCtx, BSON("a" << BSON("$add" << BSON_ARRAY(4 << 2) << "b" << 1))),
+                  AssertionException);
+    ASSERT_THROWS(ParsedAddFields::create(expCtx,
+                                          BSON("a" << BSON("$gt" << BSON("bad"
+                                                                         << "arguments")))),
+                  AssertionException);
+    ASSERT_THROWS(ParsedAddFields::create(
+                      expCtx, BSON("a" << false << "b" << BSON("$unknown" << BSON_ARRAY(4 << 2)))),
+                  AssertionException);
 
     // Empty specifications are not allowed.
-    ASSERT_THROWS(ParsedAddFields::create(BSONObj()), UserException);
+    ASSERT_THROWS(ParsedAddFields::create(expCtx, BSONObj()), AssertionException);
 
     // Empty nested objects are not allowed.
-    ASSERT_THROWS(ParsedAddFields::create(BSON("a" << BSONObj())), UserException);
+    ASSERT_THROWS(ParsedAddFields::create(expCtx, BSON("a" << BSONObj())), AssertionException);
 }
 
 TEST(ParsedAddFields, DoesNotErrorOnTwoNestedFields) {
-    ParsedAddFields::create(BSON("a.b" << true << "a.c" << true));
-    ParsedAddFields::create(BSON("a.b" << true << "a" << BSON("c" << true)));
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedAddFields::create(expCtx, BSON("a.b" << true << "a.c" << true));
+    ParsedAddFields::create(expCtx, BSON("a.b" << true << "a" << BSON("c" << true)));
 }
 
 // Verify that replaced fields are not included as dependencies.
 TEST(ParsedAddFieldsDeps, RemovesReplaceFieldsFromDependencies) {
-    ParsedAddFields addition;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedAddFields addition(expCtx);
     addition.parse(BSON("a" << true));
 
     DepsTracker deps;
@@ -121,7 +136,8 @@ TEST(ParsedAddFieldsDeps, RemovesReplaceFieldsFromDependencies) {
 
 // Verify that adding nested fields keeps the top-level field as a dependency.
 TEST(ParsedAddFieldsDeps, IncludesTopLevelFieldInDependenciesWhenAddingNestedFields) {
-    ParsedAddFields addition;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedAddFields addition(expCtx);
     addition.parse(BSON("x.y" << true));
 
     DepsTracker deps;
@@ -135,7 +151,8 @@ TEST(ParsedAddFieldsDeps, IncludesTopLevelFieldInDependenciesWhenAddingNestedFie
 
 // Verify that fields that an expression depends on are added to the dependencies.
 TEST(ParsedAddFieldsDeps, AddsDependenciesForComputedFields) {
-    ParsedAddFields addition;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedAddFields addition(expCtx);
     addition.parse(BSON("x.y"
                         << "$z"
                         << "a"
@@ -155,28 +172,40 @@ TEST(ParsedAddFieldsDeps, AddsDependenciesForComputedFields) {
 // Verify that the serialization produces the correct output: converting numbers and literals to
 // their corresponding $const form.
 TEST(ParsedAddFieldsSerialize, SerializesToCorrectForm) {
-    ParsedAddFields addition;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedAddFields addition(expCtx);
     addition.parse(fromjson("{a: {$add: ['$a', 2]}, b: {d: 3}, 'x.y': {$literal: 4}}"));
 
     auto expectedSerialization = Document(
         fromjson("{a: {$add: [\"$a\", {$const: 2}]}, b: {d: {$const: 3}}, x: {y: {$const: 4}}}"));
 
     // Should be the same if we're serializing for explain or for internal use.
-    ASSERT_DOCUMENT_EQ(expectedSerialization, addition.serialize(false));
-    ASSERT_DOCUMENT_EQ(expectedSerialization, addition.serialize(true));
+    ASSERT_DOCUMENT_EQ(expectedSerialization, addition.serializeStageOptions(boost::none));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       addition.serializeStageOptions(ExplainOptions::Verbosity::kQueryPlanner));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       addition.serializeStageOptions(ExplainOptions::Verbosity::kExecStats));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       addition.serializeStageOptions(ExplainOptions::Verbosity::kExecAllPlans));
 }
 
 // Verify that serialize treats the _id field as any other field: including when explicity included.
 TEST(ParsedAddFieldsSerialize, AddsIdToSerializeWhenExplicitlyIncluded) {
-    ParsedAddFields addition;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedAddFields addition(expCtx);
     addition.parse(BSON("_id" << false));
 
     // Adds explicit "_id" setting field, serializes expressions.
     auto expectedSerialization = Document(fromjson("{_id: {$const: false}}"));
 
     // Should be the same if we're serializing for explain or for internal use.
-    ASSERT_DOCUMENT_EQ(expectedSerialization, addition.serialize(false));
-    ASSERT_DOCUMENT_EQ(expectedSerialization, addition.serialize(true));
+    ASSERT_DOCUMENT_EQ(expectedSerialization, addition.serializeStageOptions(boost::none));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       addition.serializeStageOptions(ExplainOptions::Verbosity::kQueryPlanner));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       addition.serializeStageOptions(ExplainOptions::Verbosity::kExecStats));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       addition.serializeStageOptions(ExplainOptions::Verbosity::kExecAllPlans));
 }
 
 // Verify that serialize treats the _id field as any other field: excluded when not explicitly
@@ -184,39 +213,57 @@ TEST(ParsedAddFieldsSerialize, AddsIdToSerializeWhenExplicitlyIncluded) {
 // yet they derive from the same parent class. If the parent class were to change, this test would
 // fail.
 TEST(ParsedAddFieldsSerialize, OmitsIdFromSerializeWhenNotIncluded) {
-    ParsedAddFields addition;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedAddFields addition(expCtx);
     addition.parse(BSON("a" << true));
 
     // Does not implicitly include "_id" field.
     auto expectedSerialization = Document(fromjson("{a: {$const: true}}"));
 
     // Should be the same if we're serializing for explain or for internal use.
-    ASSERT_DOCUMENT_EQ(expectedSerialization, addition.serialize(false));
-    ASSERT_DOCUMENT_EQ(expectedSerialization, addition.serialize(true));
+    ASSERT_DOCUMENT_EQ(expectedSerialization, addition.serializeStageOptions(boost::none));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       addition.serializeStageOptions(ExplainOptions::Verbosity::kQueryPlanner));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       addition.serializeStageOptions(ExplainOptions::Verbosity::kExecStats));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       addition.serializeStageOptions(ExplainOptions::Verbosity::kExecAllPlans));
 }
 
 // Verify that the $addFields stage optimizes expressions into simpler forms when possible.
 TEST(ParsedAddFieldsOptimize, OptimizesTopLevelExpressions) {
-    ParsedAddFields addition;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedAddFields addition(expCtx);
     addition.parse(BSON("a" << BSON("$add" << BSON_ARRAY(1 << 2))));
     addition.optimize();
     auto expectedSerialization = Document{{"a", Document{{"$const", 3}}}};
 
     // Should be the same if we're serializing for explain or for internal use.
-    ASSERT_DOCUMENT_EQ(expectedSerialization, addition.serialize(false));
-    ASSERT_DOCUMENT_EQ(expectedSerialization, addition.serialize(true));
+    ASSERT_DOCUMENT_EQ(expectedSerialization, addition.serializeStageOptions(boost::none));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       addition.serializeStageOptions(ExplainOptions::Verbosity::kQueryPlanner));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       addition.serializeStageOptions(ExplainOptions::Verbosity::kExecStats));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       addition.serializeStageOptions(ExplainOptions::Verbosity::kExecAllPlans));
 }
 
 // Verify that the $addFields stage optimizes expressions even when they are nested.
 TEST(ParsedAddFieldsOptimize, ShouldOptimizeNestedExpressions) {
-    ParsedAddFields addition;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedAddFields addition(expCtx);
     addition.parse(BSON("a.b" << BSON("$add" << BSON_ARRAY(1 << 2))));
     addition.optimize();
     auto expectedSerialization = Document{{"a", Document{{"b", Document{{"$const", 3}}}}}};
 
     // Should be the same if we're serializing for explain or for internal use.
-    ASSERT_DOCUMENT_EQ(expectedSerialization, addition.serialize(false));
-    ASSERT_DOCUMENT_EQ(expectedSerialization, addition.serialize(true));
+    ASSERT_DOCUMENT_EQ(expectedSerialization, addition.serializeStageOptions(boost::none));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       addition.serializeStageOptions(ExplainOptions::Verbosity::kQueryPlanner));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       addition.serializeStageOptions(ExplainOptions::Verbosity::kExecStats));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       addition.serializeStageOptions(ExplainOptions::Verbosity::kExecAllPlans));
 }
 
 //
@@ -225,7 +272,8 @@ TEST(ParsedAddFieldsOptimize, ShouldOptimizeNestedExpressions) {
 
 // Verify that a new field is added to the end of the document.
 TEST(ParsedAddFieldsExecutionTest, AddsNewFieldToEndOfDocument) {
-    ParsedAddFields addition;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedAddFields addition(expCtx);
     addition.parse(BSON("c" << 3));
 
     // There are no fields in the document.
@@ -241,7 +289,8 @@ TEST(ParsedAddFieldsExecutionTest, AddsNewFieldToEndOfDocument) {
 
 // Verify that an existing field is replaced and stays in the same order in the document.
 TEST(ParsedAddFieldsExecutionTest, ReplacesFieldThatAlreadyExistsInDocument) {
-    ParsedAddFields addition;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedAddFields addition(expCtx);
     addition.parse(BSON("c" << 3));
 
     // Specified field is the only field in the document, and is replaced.
@@ -257,19 +306,21 @@ TEST(ParsedAddFieldsExecutionTest, ReplacesFieldThatAlreadyExistsInDocument) {
 
 // Verify that replacing multiple fields preserves the original field order in the document.
 TEST(ParsedAddFieldsExecutionTest, ReplacesMultipleFieldsWhilePreservingInputFieldOrder) {
-    ParsedAddFields addition;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedAddFields addition(expCtx);
     addition.parse(BSON("second"
                         << "SECOND"
                         << "first"
                         << "FIRST"));
     auto result = addition.applyProjection(Document{{"first", 0}, {"second", 1}, {"third", 2}});
-    auto expectedResult = Document{{"first", "FIRST"}, {"second", "SECOND"}, {"third", 2}};
+    auto expectedResult = Document{{"first", "FIRST"_sd}, {"second", "SECOND"_sd}, {"third", 2}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
 // Verify that adding multiple fields adds the fields in the order specified.
 TEST(ParsedAddFieldsExecutionTest, AddsNewFieldsAfterExistingFieldsInOrderSpecified) {
-    ParsedAddFields addition;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedAddFields addition(expCtx);
     addition.parse(BSON("firstComputed"
                         << "FIRST"
                         << "secondComputed"
@@ -278,55 +329,58 @@ TEST(ParsedAddFieldsExecutionTest, AddsNewFieldsAfterExistingFieldsInOrderSpecif
     auto expectedResult = Document{{"first", 0},
                                    {"second", 1},
                                    {"third", 2},
-                                   {"firstComputed", "FIRST"},
-                                   {"secondComputed", "SECOND"}};
+                                   {"firstComputed", "FIRST"_sd},
+                                   {"secondComputed", "SECOND"_sd}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
 // Verify that both adding and replacing fields at the same time follows the same rules as doing
 // each independently.
 TEST(ParsedAddFieldsExecutionTest, ReplacesAndAddsNewFieldsWithSameOrderingRulesAsSeparately) {
-    ParsedAddFields addition;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedAddFields addition(expCtx);
     addition.parse(BSON("firstComputed"
                         << "FIRST"
                         << "second"
                         << "SECOND"));
     auto result = addition.applyProjection(Document{{"first", 0}, {"second", 1}, {"third", 2}});
-    auto expectedResult =
-        Document{{"first", 0}, {"second", "SECOND"}, {"third", 2}, {"firstComputed", "FIRST"}};
+    auto expectedResult = Document{
+        {"first", 0}, {"second", "SECOND"_sd}, {"third", 2}, {"firstComputed", "FIRST"_sd}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
 // Verify that _id is included just like a regular field, in whatever order it appears in the
 // input document, when adding new fields.
 TEST(ParsedAddFieldsExecutionTest, IdFieldIsKeptInOrderItAppearsInInputDocument) {
-    ParsedAddFields addition;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedAddFields addition(expCtx);
     addition.parse(BSON("newField"
                         << "computedVal"));
-    auto result = addition.applyProjection(Document{{"_id", "ID"}, {"a", 1}});
-    auto expectedResult = Document{{"_id", "ID"}, {"a", 1}, {"newField", "computedVal"}};
+    auto result = addition.applyProjection(Document{{"_id", "ID"_sd}, {"a", 1}});
+    auto expectedResult = Document{{"_id", "ID"_sd}, {"a", 1}, {"newField", "computedVal"_sd}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 
-    result = addition.applyProjection(Document{{"a", 1}, {"_id", "ID"}});
-    expectedResult = Document{{"a", 1}, {"_id", "ID"}, {"newField", "computedVal"}};
+    result = addition.applyProjection(Document{{"a", 1}, {"_id", "ID"_sd}});
+    expectedResult = Document{{"a", 1}, {"_id", "ID"_sd}, {"newField", "computedVal"_sd}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
 // Verify that replacing or adding _id works just like any other field.
 TEST(ParsedAddFieldsExecutionTest, ShouldReplaceIdWithComputedId) {
-    ParsedAddFields addition;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedAddFields addition(expCtx);
     addition.parse(BSON("_id"
                         << "newId"));
-    auto result = addition.applyProjection(Document{{"_id", "ID"}, {"a", 1}});
-    auto expectedResult = Document{{"_id", "newId"}, {"a", 1}};
+    auto result = addition.applyProjection(Document{{"_id", "ID"_sd}, {"a", 1}});
+    auto expectedResult = Document{{"_id", "newId"_sd}, {"a", 1}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 
-    result = addition.applyProjection(Document{{"a", 1}, {"_id", "ID"}});
-    expectedResult = Document{{"a", 1}, {"_id", "newId"}};
+    result = addition.applyProjection(Document{{"a", 1}, {"_id", "ID"_sd}});
+    expectedResult = Document{{"a", 1}, {"_id", "newId"_sd}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 
     result = addition.applyProjection(Document{{"a", 1}});
-    expectedResult = Document{{"a", 1}, {"_id", "newId"}};
+    expectedResult = Document{{"a", 1}, {"_id", "newId"_sd}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
@@ -336,7 +390,8 @@ TEST(ParsedAddFieldsExecutionTest, ShouldReplaceIdWithComputedId) {
 
 // Verify that adding a dotted field keeps the other fields in the subdocument.
 TEST(ParsedAddFieldsExecutionTest, KeepsExistingSubFieldsWhenAddingSimpleDottedFieldToSubDoc) {
-    ParsedAddFields addition;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedAddFields addition(expCtx);
     addition.parse(BSON("a.b" << true));
 
     // More than one field in sub document.
@@ -362,7 +417,8 @@ TEST(ParsedAddFieldsExecutionTest, KeepsExistingSubFieldsWhenAddingSimpleDottedF
 
 // Verify that creating a dotted field creates the subdocument structure necessary.
 TEST(ParsedAddFieldsExecutionTest, CreatesSubDocIfDottedAddedFieldDoesNotExist) {
-    ParsedAddFields addition;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedAddFields addition(expCtx);
     addition.parse(BSON("sub.target" << true));
 
     // Should add the path if it doesn't exist.
@@ -371,7 +427,7 @@ TEST(ParsedAddFieldsExecutionTest, CreatesSubDocIfDottedAddedFieldDoesNotExist)
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 
     // Should replace the second part of the path if that part already exists.
-    result = addition.applyProjection(Document{{"sub", "notADocument"}});
+    result = addition.applyProjection(Document{{"sub", "notADocument"_sd}});
     expectedResult = Document{{"sub", Document{{"target", true}}}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
@@ -379,7 +435,8 @@ TEST(ParsedAddFieldsExecutionTest, CreatesSubDocIfDottedAddedFieldDoesNotExist)
 // Verify that adding a dotted value to an array field sets the field in every element of the array.
 // SERVER-25200: make this agree with $set.
 TEST(ParsedAddFieldsExecutionTest, AppliesDottedAdditionToEachElementInArray) {
-    ParsedAddFields addition;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedAddFields addition(expCtx);
     addition.parse(BSON("a.b" << true));
 
     vector<Value> nestedValues = {Value(1),
@@ -404,35 +461,38 @@ TEST(ParsedAddFieldsExecutionTest, AppliesDottedAdditionToEachElementInArray) {
 
 // Verify that creation of the subdocument structure works for many layers of nesting.
 TEST(ParsedAddFieldsExecutionTest, CreatesNestedSubDocumentsAllTheWayToAddedField) {
-    ParsedAddFields addition;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedAddFields addition(expCtx);
     addition.parse(BSON("a.b.c.d"
                         << "computedVal"));
 
     // Should add the path if it doesn't exist.
     auto result = addition.applyProjection(Document{});
     auto expectedResult =
-        Document{{"a", Document{{"b", Document{{"c", Document{{"d", "computedVal"}}}}}}}};
+        Document{{"a", Document{{"b", Document{{"c", Document{{"d", "computedVal"_sd}}}}}}}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 
     // Should replace non-documents with documents.
-    result = addition.applyProjection(Document{{"a", Document{{"b", "other"}}}});
+    result = addition.applyProjection(Document{{"a", Document{{"b", "other"_sd}}}});
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
 // Verify that _id is not special: we can add subfields to it as well.
 TEST(ParsedAddFieldsExecutionTest, AddsSubFieldsOfId) {
-    ParsedAddFields addition;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedAddFields addition(expCtx);
     addition.parse(BSON("_id.X" << true << "_id.Z"
                                 << "NEW"));
     auto result = addition.applyProjection(Document{{"_id", Document{{"X", 1}, {"Y", 2}}}});
-    auto expectedResult = Document{{"_id", Document{{"X", true}, {"Y", 2}, {"Z", "NEW"}}}};
+    auto expectedResult = Document{{"_id", Document{{"X", true}, {"Y", 2}, {"Z", "NEW"_sd}}}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
 // Verify that both ways of specifying nested fields -- both dotted notation and nesting --
 // can be used together in the same specification.
 TEST(ParsedAddFieldsExecutionTest, ShouldAllowMixedNestedAndDottedFields) {
-    ParsedAddFields addition;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedAddFields addition(expCtx);
     // Include all of "a.b", "a.c", "a.d", and "a.e".
     // Add new computed fields "a.W", "a.X", "a.Y", and "a.Z".
     addition.parse(BSON("a.b" << true << "a.c" << true << "a.W"
@@ -444,30 +504,32 @@ TEST(ParsedAddFieldsExecutionTest, ShouldAllowMixedNestedAndDottedFields) {
                                           << "Y"
                                           << "Z"
                                           << "Z")));
-    auto result = addition.applyProjection(
-        Document{{"a", Document{{"b", "b"}, {"c", "c"}, {"d", "d"}, {"e", "e"}, {"f", "f"}}}});
+    auto result = addition.applyProjection(Document{
+        {"a",
+         Document{{"b", "b"_sd}, {"c", "c"_sd}, {"d", "d"_sd}, {"e", "e"_sd}, {"f", "f"_sd}}}});
     auto expectedResult = Document{{"a",
                                     Document{{"b", true},
                                              {"c", true},
                                              {"d", true},
                                              {"e", true},
-                                             {"f", "f"},
-                                             {"W", "W"},
-                                             {"X", "X"},
-                                             {"Y", "Y"},
-                                             {"Z", "Z"}}}};
+                                             {"f", "f"_sd},
+                                             {"W", "W"_sd},
+                                             {"X", "X"_sd},
+                                             {"Y", "Y"_sd},
+                                             {"Z", "Z"_sd}}}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
 // Verify that adding nested fields preserves the addition order in the spec.
 TEST(ParsedAddFieldsExecutionTest, AddsNestedAddedFieldsInOrderSpecified) {
-    ParsedAddFields addition;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedAddFields addition(expCtx);
     addition.parse(BSON("b.d"
                         << "FIRST"
                         << "b.c"
                         << "SECOND"));
     auto result = addition.applyProjection(Document{});
-    auto expectedResult = Document{{"b", Document{{"d", "FIRST"}, {"c", "SECOND"}}}};
+    auto expectedResult = Document{{"b", Document{{"d", "FIRST"_sd}, {"c", "SECOND"_sd}}}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
@@ -477,7 +539,8 @@ TEST(ParsedAddFieldsExecutionTest, AddsNestedAddedFieldsInOrderSpecified) {
 
 // Verify that the metadata is kept from the original input document.
 TEST(ParsedAddFieldsExecutionTest, AlwaysKeepsMetadataFromOriginalDoc) {
-    ParsedAddFields addition;
+    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedAddFields addition(expCtx);
     addition.parse(BSON("a" << true));
 
     MutableDocument inputDocBuilder(Document{{"a", 1}});
