@@ -46,17 +46,25 @@
 namespace mongo {
 namespace parsed_aggregation_projection {
 
+using TransformerType =
+    DocumentSourceSingleDocumentTransformation::TransformerInterface::TransformerType;
+
 using expression::isPathPrefixOf;
 
 //
 // ProjectionSpecValidator
 //
 
-Status ProjectionSpecValidator::validate(const BSONObj& spec) {
-    return ProjectionSpecValidator(spec).validate();
+void ProjectionSpecValidator::uassertValid(const BSONObj& spec, StringData stageName) {
+    try {
+        ProjectionSpecValidator(spec).validate();
+    } catch (DBException& ex) {
+        ex.addContext("Invalid " + stageName.toString());
+        throw;
+    }
 }
 
-Status ProjectionSpecValidator::ensurePathDoesNotConflictOrThrow(const std::string& path) {
+void ProjectionSpecValidator::ensurePathDoesNotConflictOrThrow(const std::string& path) {
     auto result = _seenPaths.emplace(path);
     auto pos = result.first;
 
@@ -75,48 +83,41 @@ Status ProjectionSpecValidator::ensurePathDoesNotConflictOrThrow(const std::stri
             boost::make_optional(isPathPrefixOf(path, *std::next(pos)), *std::next(pos));
     }
 
-    if (conflictingPath) {
-        return Status(ErrorCodes::FailedToParse,
-                      str::stream() << "specification contains two conflicting paths. "
-                                       "Cannot specify both '"
-                                    << path
-                                    << "' and '"
-                                    << *conflictingPath
-                                    << "': "
-                                    << _rawObj.toString(),
-                      40176);
-    }
-    return Status::OK();
+    uassert(40176,
+            str::stream() << "specification contains two conflicting paths. "
+                             "Cannot specify both '"
+                          << path
+                          << "' and '"
+                          << *conflictingPath
+                          << "': "
+                          << _rawObj.toString(),
+            !conflictingPath);
 }
 
-Status ProjectionSpecValidator::validate() {
+void ProjectionSpecValidator::validate() {
     if (_rawObj.isEmpty()) {
-        return Status(
-            ErrorCodes::FailedToParse, "specification must have at least one field", 40177);
+        uasserted(40177, "specification must have at least one field");
     }
     for (auto&& elem : _rawObj) {
-        Status status = parseElement(elem, FieldPath(elem.fieldName()));
-        if (!status.isOK())
-            return status;
+        parseElement(elem, FieldPath(elem.fieldName()));
     }
-    return Status::OK();
 }
 
-Status ProjectionSpecValidator::parseElement(const BSONElement& elem, const FieldPath& pathToElem) {
+void ProjectionSpecValidator::parseElement(const BSONElement& elem, const FieldPath& pathToElem) {
     if (elem.type() == BSONType::Object) {
-        return parseNestedObject(elem.Obj(), pathToElem);
+        parseNestedObject(elem.Obj(), pathToElem);
+    } else {
+        ensurePathDoesNotConflictOrThrow(pathToElem.fullPath());
     }
-    return ensurePathDoesNotConflictOrThrow(pathToElem.fullPath());
 }
 
-Status ProjectionSpecValidator::parseNestedObject(const BSONObj& thisLevelSpec,
-                                                  const FieldPath& prefix) {
+void ProjectionSpecValidator::parseNestedObject(const BSONObj& thisLevelSpec,
+                                                const FieldPath& prefix) {
     if (thisLevelSpec.isEmpty()) {
-        return Status(ErrorCodes::FailedToParse,
-                      str::stream()
-                          << "an empty object is not a valid value. Found empty object at path "
-                          << prefix.fullPath(),
-                      40180);
+        uasserted(
+            40180,
+            str::stream() << "an empty object is not a valid value. Found empty object at path "
+                          << prefix.fullPath());
     }
     for (auto&& elem : thisLevelSpec) {
         auto fieldName = elem.fieldNameStringData();
@@ -125,34 +126,26 @@ Status ProjectionSpecValidator::parseNestedObject(const BSONObj& thisLevelSpec,
             // into an Expression later, but for now, just track that the prefix has been
             // specified and skip it.
             if (thisLevelSpec.nFields() != 1) {
-                return Status(ErrorCodes::FailedToParse,
-                              str::stream() << "an expression specification must contain exactly "
-                                               "one field, the name of the expression. Found "
-                                            << thisLevelSpec.nFields()
-                                            << " fields in "
-                                            << thisLevelSpec.toString()
-                                            << ", while parsing object "
-                                            << _rawObj.toString(),
-                              40181);
+                uasserted(40181,
+                          str::stream() << "an expression specification must contain exactly "
+                                           "one field, the name of the expression. Found "
+                                        << thisLevelSpec.nFields()
+                                        << " fields in "
+                                        << thisLevelSpec.toString()
+                                        << ", while parsing object "
+                                        << _rawObj.toString());
             }
-            Status status = ensurePathDoesNotConflictOrThrow(prefix.fullPath());
-            if (!status.isOK())
-                return status;
+            ensurePathDoesNotConflictOrThrow(prefix.fullPath());
             continue;
         }
         if (fieldName.find('.') != std::string::npos) {
-            return Status(ErrorCodes::FailedToParse,
-                          str::stream() << "cannot use dotted field name '" << fieldName
-                                        << "' in a sub object: "
-                                        << _rawObj.toString(),
-                          40183);
+            uasserted(40183,
+                      str::stream() << "cannot use dotted field name '" << fieldName
+                                    << "' in a sub object: "
+                                    << _rawObj.toString());
         }
-        Status status =
-            parseElement(elem, FieldPath::getFullyQualifiedPath(prefix.fullPath(), fieldName));
-        if (!status.isOK())
-            return status;
+        parseElement(elem, FieldPath::getFullyQualifiedPath(prefix.fullPath(), fieldName));
     }
-    return Status::OK();
 }
 
 namespace {
@@ -167,7 +160,7 @@ public:
      * fields (ones which are defined by an expression or a literal) are treated as inclusion
      * projections for in this context of the$project stage.
      */
-    static ProjectionType parse(const BSONObj& spec) {
+    static TransformerType parse(const BSONObj& spec) {
         ProjectTypeParser parser(spec);
         parser.parse();
         invariant(parser._parsedType);
@@ -193,13 +186,14 @@ private:
             BSONElement elem = _rawObj.firstElement();
             if (elem.fieldNameStringData() == "_id" && (elem.isBoolean() || elem.isNumber()) &&
                 !elem.trueValue()) {
-                _parsedType = ProjectionType::kExclusion;
+                _parsedType = TransformerType::kExclusionProjection;
             }
         }
 
         // Default to inclusion if nothing (except maybe '_id') is explicitly included or excluded.
         if (!_parsedType) {
-            _parsedType = ProjectionType::kInclusion;
+            _parsedType = DocumentSourceSingleDocumentTransformation::TransformerInterface::
+                TransformerType::kInclusionProjection;
         }
     }
 
@@ -209,7 +203,7 @@ private:
      * Delegates to parseSubObject() if 'elem' is an object. Otherwise updates '_parsedType' if
      * appropriate.
      *
-     * Throws a UserException if this element represents a mix of projection types.
+     * Throws a AssertionException if this element represents a mix of projection types.
      */
     void parseElement(const BSONElement& elem, const FieldPath& pathToElem) {
         if (elem.type() == BSONType::Object) {
@@ -224,8 +218,12 @@ private:
                         str::stream() << "Bad projection specification, cannot exclude fields "
                                          "other than '_id' in an inclusion projection: "
                                       << _rawObj.toString(),
-                        !_parsedType || (*_parsedType == ProjectionType::kExclusion));
-                _parsedType = ProjectionType::kExclusion;
+                        !_parsedType ||
+                            (*_parsedType ==
+                             DocumentSourceSingleDocumentTransformation::TransformerInterface::
+                                 TransformerType::kExclusionProjection));
+                _parsedType = DocumentSourceSingleDocumentTransformation::TransformerInterface::
+                    TransformerType::kExclusionProjection;
             }
         } else {
             // A boolean true, a truthy numeric value, or any expression can only be used with an
@@ -235,15 +233,19 @@ private:
                     str::stream() << "Bad projection specification, cannot include fields or "
                                      "add computed fields during an exclusion projection: "
                                   << _rawObj.toString(),
-                    !_parsedType || (*_parsedType == ProjectionType::kInclusion));
-            _parsedType = ProjectionType::kInclusion;
+                    !_parsedType ||
+                        (*_parsedType ==
+                         DocumentSourceSingleDocumentTransformation::TransformerInterface::
+                             TransformerType::kInclusionProjection));
+            _parsedType = DocumentSourceSingleDocumentTransformation::TransformerInterface::
+                TransformerType::kInclusionProjection;
         }
     }
 
     /**
      * Traverses 'thisLevelSpec', parsing each element in turn.
      *
-     * Throws a UserException if 'thisLevelSpec' represents an invalid mix of projections.
+     * Throws a AssertionException if 'thisLevelSpec' represents an invalid mix of projections.
      */
     void parseNestedObject(const BSONObj& thisLevelSpec, const FieldPath& prefix) {
 
@@ -257,8 +259,12 @@ private:
                         str::stream() << "Bad projection specification, cannot include fields or "
                                          "add computed fields during an exclusion projection: "
                                       << _rawObj.toString(),
-                        !_parsedType || _parsedType == ProjectionType::kInclusion);
-                _parsedType = ProjectionType::kInclusion;
+                        !_parsedType ||
+                            _parsedType ==
+                                DocumentSourceSingleDocumentTransformation::TransformerInterface::
+                                    TransformerType::kInclusionProjection);
+                _parsedType = DocumentSourceSingleDocumentTransformation::TransformerInterface::
+                    TransformerType::kInclusionProjection;
                 continue;
             }
             parseElement(elem, FieldPath::getFullyQualifiedPath(prefix.fullPath(), fieldName));
@@ -269,7 +275,7 @@ private:
     const BSONObj& _rawObj;
 
     // This will be populated during parse().
-    boost::optional<ProjectionType> _parsedType;
+    boost::optional<TransformerType> _parsedType;
 };
 
 }  // namespace
@@ -279,26 +285,22 @@ std::unique_ptr<ParsedAggregationProjection> ParsedAggregationProjection::create
     // Check that the specification was valid. Status returned is unspecific because validate()
     // is used by the $addFields stage as well as $project.
     // If there was an error, uassert with a $project-specific message.
-    Status status = ProjectionSpecValidator::validate(spec);
-    if (!status.isOK()) {
-        uasserted(status.location(),
-                  str::stream() << "Invalid $project specification: " << status.reason());
-    }
+    ProjectionSpecValidator::uassertValid(spec, "$project");
 
     // Check for any conflicting specifications, and determine the type of the projection.
     auto projectionType = ProjectTypeParser::parse(spec);
     // kComputed is a projection type reserved for $addFields, and should never be detected by the
     // ProjectTypeParser.
-    invariant(projectionType != ProjectionType::kComputed);
+    invariant(projectionType != TransformerType::kComputedProjection);
 
     // We can't use make_unique() here, since the branches have different types.
     std::unique_ptr<ParsedAggregationProjection> parsedProject(
-        projectionType == ProjectionType::kInclusion
-            ? static_cast<ParsedAggregationProjection*>(new ParsedInclusionProjection())
-            : static_cast<ParsedAggregationProjection*>(new ParsedExclusionProjection()));
+        projectionType == TransformerType::kInclusionProjection
+            ? static_cast<ParsedAggregationProjection*>(new ParsedInclusionProjection(expCtx))
+            : static_cast<ParsedAggregationProjection*>(new ParsedExclusionProjection(expCtx)));
 
     // Actually parse the specification.
-    parsedProject->parse(expCtx, spec);
+    parsedProject->parse(spec);
     return parsedProject;
 }
 
