@@ -38,6 +38,7 @@
 #include "mongo/db/pipeline/dependencies.h"
 #include "mongo/db/pipeline/document.h"
 #include "mongo/db/pipeline/document_value_test_util.h"
+#include "mongo/db/pipeline/expression_context_for_test.h"
 #include "mongo/db/pipeline/value.h"
 #include "mongo/unittest/unittest.h"
 
@@ -52,19 +53,22 @@ BSONObj wrapInLiteral(const T& arg) {
 }
 
 TEST(InclusionProjection, ShouldThrowWhenParsingInvalidExpression) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     ASSERT_THROWS(inclusion.parse(BSON("a" << BSON("$gt" << BSON("bad"
                                                                  << "arguments")))),
-                  UserException);
+                  AssertionException);
 }
 
 TEST(InclusionProjection, ShouldRejectProjectionWithNoOutputFields) {
-    ParsedInclusionProjection inclusion;
-    ASSERT_THROWS(inclusion.parse(BSON("_id" << false)), UserException);
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
+    ASSERT_THROWS(inclusion.parse(BSON("_id" << false)), AssertionException);
 }
 
 TEST(InclusionProjection, ShouldAddIncludedFieldsToDependencies) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("_id" << false << "a" << true << "x.y" << true));
 
     DepsTracker deps;
@@ -77,7 +81,8 @@ TEST(InclusionProjection, ShouldAddIncludedFieldsToDependencies) {
 }
 
 TEST(InclusionProjection, ShouldAddIdToDependenciesIfNotSpecified) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("a" << true));
 
     DepsTracker deps;
@@ -89,7 +94,8 @@ TEST(InclusionProjection, ShouldAddIdToDependenciesIfNotSpecified) {
 }
 
 TEST(InclusionProjection, ShouldAddDependenciesOfComputedFields) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("a"
                          << "$a"
                          << "x"
@@ -105,7 +111,8 @@ TEST(InclusionProjection, ShouldAddDependenciesOfComputedFields) {
 }
 
 TEST(InclusionProjection, ShouldAddPathToDependenciesForNestedComputedFields) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("x.y"
                          << "$z"));
 
@@ -122,7 +129,8 @@ TEST(InclusionProjection, ShouldAddPathToDependenciesForNestedComputedFields) {
 }
 
 TEST(InclusionProjection, ShouldSerializeToEquivalentProjection) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(fromjson("{a: {$add: ['$a', 2]}, b: {d: 3}, 'x.y': {$literal: 4}}"));
 
     // Adds implicit "_id" inclusion, converts numbers to bools, serializes expressions.
@@ -130,25 +138,37 @@ TEST(InclusionProjection, ShouldSerializeToEquivalentProjection) {
         "{_id: true, a: {$add: [\"$a\", {$const: 2}]}, b: {d: true}, x: {y: {$const: 4}}}"));
 
     // Should be the same if we're serializing for explain or for internal use.
-    ASSERT_DOCUMENT_EQ(expectedSerialization, inclusion.serialize(false));
-    ASSERT_DOCUMENT_EQ(expectedSerialization, inclusion.serialize(true));
+    ASSERT_DOCUMENT_EQ(expectedSerialization, inclusion.serializeStageOptions(boost::none));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       inclusion.serializeStageOptions(ExplainOptions::Verbosity::kQueryPlanner));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       inclusion.serializeStageOptions(ExplainOptions::Verbosity::kExecStats));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       inclusion.serializeStageOptions(ExplainOptions::Verbosity::kExecAllPlans));
 }
 
 TEST(InclusionProjection, ShouldSerializeExplicitExclusionOfId) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("_id" << false << "a" << true));
 
     // Adds implicit "_id" inclusion, converts numbers to bools, serializes expressions.
     auto expectedSerialization = Document{{"_id", false}, {"a", true}};
 
     // Should be the same if we're serializing for explain or for internal use.
-    ASSERT_DOCUMENT_EQ(expectedSerialization, inclusion.serialize(false));
-    ASSERT_DOCUMENT_EQ(expectedSerialization, inclusion.serialize(true));
+    ASSERT_DOCUMENT_EQ(expectedSerialization, inclusion.serializeStageOptions(boost::none));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       inclusion.serializeStageOptions(ExplainOptions::Verbosity::kQueryPlanner));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       inclusion.serializeStageOptions(ExplainOptions::Verbosity::kExecStats));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       inclusion.serializeStageOptions(ExplainOptions::Verbosity::kExecAllPlans));
 }
 
 
 TEST(InclusionProjection, ShouldOptimizeTopLevelExpressions) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("a" << BSON("$add" << BSON_ARRAY(1 << 2))));
 
     inclusion.optimize();
@@ -156,12 +176,18 @@ TEST(InclusionProjection, ShouldOptimizeTopLevelExpressions) {
     auto expectedSerialization = Document{{"_id", true}, {"a", Document{{"$const", 3}}}};
 
     // Should be the same if we're serializing for explain or for internal use.
-    ASSERT_DOCUMENT_EQ(expectedSerialization, inclusion.serialize(false));
-    ASSERT_DOCUMENT_EQ(expectedSerialization, inclusion.serialize(true));
+    ASSERT_DOCUMENT_EQ(expectedSerialization, inclusion.serializeStageOptions(boost::none));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       inclusion.serializeStageOptions(ExplainOptions::Verbosity::kQueryPlanner));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       inclusion.serializeStageOptions(ExplainOptions::Verbosity::kExecStats));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       inclusion.serializeStageOptions(ExplainOptions::Verbosity::kExecAllPlans));
 }
 
 TEST(InclusionProjection, ShouldOptimizeNestedExpressions) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("a.b" << BSON("$add" << BSON_ARRAY(1 << 2))));
 
     inclusion.optimize();
@@ -170,12 +196,18 @@ TEST(InclusionProjection, ShouldOptimizeNestedExpressions) {
         Document{{"_id", true}, {"a", Document{{"b", Document{{"$const", 3}}}}}};
 
     // Should be the same if we're serializing for explain or for internal use.
-    ASSERT_DOCUMENT_EQ(expectedSerialization, inclusion.serialize(false));
-    ASSERT_DOCUMENT_EQ(expectedSerialization, inclusion.serialize(true));
+    ASSERT_DOCUMENT_EQ(expectedSerialization, inclusion.serializeStageOptions(boost::none));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       inclusion.serializeStageOptions(ExplainOptions::Verbosity::kQueryPlanner));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       inclusion.serializeStageOptions(ExplainOptions::Verbosity::kExecStats));
+    ASSERT_DOCUMENT_EQ(expectedSerialization,
+                       inclusion.serializeStageOptions(ExplainOptions::Verbosity::kExecAllPlans));
 }
 
 TEST(InclusionProjection, ShouldReportThatAllExceptIncludedFieldsAreModified) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON(
         "a" << wrapInLiteral("computedVal") << "b.c" << wrapInLiteral("computedVal") << "d" << true
             << "e.f"
@@ -194,7 +226,8 @@ TEST(InclusionProjection, ShouldReportThatAllExceptIncludedFieldsAreModified) {
 }
 
 TEST(InclusionProjection, ShouldReportThatAllExceptIncludedFieldsAreModifiedWithIdExclusion) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("_id" << false << "a" << wrapInLiteral("computedVal") << "b.c"
                                << wrapInLiteral("computedVal")
                                << "d"
@@ -221,7 +254,8 @@ TEST(InclusionProjection, ShouldReportThatAllExceptIncludedFieldsAreModifiedWith
 //
 
 TEST(InclusionProjectionExecutionTest, ShouldIncludeTopLevelField) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("a" << true));
 
     // More than one field in document.
@@ -246,28 +280,31 @@ TEST(InclusionProjectionExecutionTest, ShouldIncludeTopLevelField) {
 }
 
 TEST(InclusionProjectionExecutionTest, ShouldAddComputedTopLevelField) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("newField" << wrapInLiteral("computedVal")));
     auto result = inclusion.applyProjection(Document{});
-    auto expectedResult = Document{{"newField", "computedVal"}};
+    auto expectedResult = Document{{"newField", "computedVal"_sd}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 
     // Computed field should replace existing field.
-    result = inclusion.applyProjection(Document{{"newField", "preExisting"}});
-    expectedResult = Document{{"newField", "computedVal"}};
+    result = inclusion.applyProjection(Document{{"newField", "preExisting"_sd}});
+    expectedResult = Document{{"newField", "computedVal"_sd}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
 TEST(InclusionProjectionExecutionTest, ShouldApplyBothInclusionsAndComputedFields) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("a" << true << "newField" << wrapInLiteral("computedVal")));
     auto result = inclusion.applyProjection(Document{{"a", 1}});
-    auto expectedResult = Document{{"a", 1}, {"newField", "computedVal"}};
+    auto expectedResult = Document{{"a", 1}, {"newField", "computedVal"_sd}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
 TEST(InclusionProjectionExecutionTest, ShouldIncludeFieldsInOrderOfInputDoc) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("first" << true << "second" << true << "third" << true));
     auto inputDoc = Document{{"second", 1}, {"first", 0}, {"third", 2}};
     auto result = inclusion.applyProjection(inputDoc);
@@ -275,56 +312,63 @@ TEST(InclusionProjectionExecutionTest, ShouldIncludeFieldsInOrderOfInputDoc) {
 }
 
 TEST(InclusionProjectionExecutionTest, ShouldApplyComputedFieldsInOrderSpecified) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("firstComputed" << wrapInLiteral("FIRST") << "secondComputed"
                                          << wrapInLiteral("SECOND")));
     auto result = inclusion.applyProjection(Document{{"first", 0}, {"second", 1}, {"third", 2}});
-    auto expectedResult = Document{{"firstComputed", "FIRST"}, {"secondComputed", "SECOND"}};
+    auto expectedResult = Document{{"firstComputed", "FIRST"_sd}, {"secondComputed", "SECOND"_sd}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
 TEST(InclusionProjectionExecutionTest, ShouldImplicitlyIncludeId) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("a" << true));
-    auto result = inclusion.applyProjection(Document{{"_id", "ID"}, {"a", 1}, {"b", 2}});
-    auto expectedResult = Document{{"_id", "ID"}, {"a", 1}};
+    auto result = inclusion.applyProjection(Document{{"_id", "ID"_sd}, {"a", 1}, {"b", 2}});
+    auto expectedResult = Document{{"_id", "ID"_sd}, {"a", 1}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 
     // Should leave the "_id" in the same place as in the original document.
-    result = inclusion.applyProjection(Document{{"a", 1}, {"b", 2}, {"_id", "ID"}});
-    expectedResult = Document{{"a", 1}, {"_id", "ID"}};
+    result = inclusion.applyProjection(Document{{"a", 1}, {"b", 2}, {"_id", "ID"_sd}});
+    expectedResult = Document{{"a", 1}, {"_id", "ID"_sd}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
 TEST(InclusionProjectionExecutionTest, ShouldImplicitlyIncludeIdWithComputedFields) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("newField" << wrapInLiteral("computedVal")));
-    auto result = inclusion.applyProjection(Document{{"_id", "ID"}, {"a", 1}});
-    auto expectedResult = Document{{"_id", "ID"}, {"newField", "computedVal"}};
+    auto result = inclusion.applyProjection(Document{{"_id", "ID"_sd}, {"a", 1}});
+    auto expectedResult = Document{{"_id", "ID"_sd}, {"newField", "computedVal"_sd}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
 TEST(InclusionProjectionExecutionTest, ShouldIncludeIdIfExplicitlyIncluded) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("a" << true << "_id" << true << "b" << true));
-    auto result = inclusion.applyProjection(Document{{"_id", "ID"}, {"a", 1}, {"b", 2}, {"c", 3}});
-    auto expectedResult = Document{{"_id", "ID"}, {"a", 1}, {"b", 2}};
+    auto result =
+        inclusion.applyProjection(Document{{"_id", "ID"_sd}, {"a", 1}, {"b", 2}, {"c", 3}});
+    auto expectedResult = Document{{"_id", "ID"_sd}, {"a", 1}, {"b", 2}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
 TEST(InclusionProjectionExecutionTest, ShouldExcludeIdIfExplicitlyExcluded) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("a" << true << "_id" << false));
-    auto result = inclusion.applyProjection(Document{{"a", 1}, {"b", 2}, {"_id", "ID"}});
+    auto result = inclusion.applyProjection(Document{{"a", 1}, {"b", 2}, {"_id", "ID"_sd}});
     auto expectedResult = Document{{"a", 1}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
 TEST(InclusionProjectionExecutionTest, ShouldReplaceIdWithComputedId) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("_id" << wrapInLiteral("newId")));
-    auto result = inclusion.applyProjection(Document{{"a", 1}, {"b", 2}, {"_id", "ID"}});
-    auto expectedResult = Document{{"_id", "newId"}};
+    auto result = inclusion.applyProjection(Document{{"a", 1}, {"b", 2}, {"_id", "ID"_sd}});
+    auto expectedResult = Document{{"_id", "newId"_sd}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
@@ -333,7 +377,8 @@ TEST(InclusionProjectionExecutionTest, ShouldReplaceIdWithComputedId) {
 //
 
 TEST(InclusionProjectionExecutionTest, ShouldIncludeSimpleDottedFieldFromSubDoc) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("a.b" << true));
 
     // More than one field in sub document.
@@ -358,7 +403,8 @@ TEST(InclusionProjectionExecutionTest, ShouldIncludeSimpleDottedFieldFromSubDoc)
 }
 
 TEST(InclusionProjectionExecutionTest, ShouldNotCreateSubDocIfDottedIncludedFieldDoesNotExist) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("sub.target" << true));
 
     // Should not add the path if it doesn't exist.
@@ -367,13 +413,14 @@ TEST(InclusionProjectionExecutionTest, ShouldNotCreateSubDocIfDottedIncludedFiel
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 
     // Should not replace the first part of the path if that part exists.
-    result = inclusion.applyProjection(Document{{"sub", "notADocument"}});
+    result = inclusion.applyProjection(Document{{"sub", "notADocument"_sd}});
     expectedResult = Document{};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
 TEST(InclusionProjectionExecutionTest, ShouldApplyDottedInclusionToEachElementInArray) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("a.b" << true));
 
     vector<Value> nestedValues = {Value(1),
@@ -397,56 +444,60 @@ TEST(InclusionProjectionExecutionTest, ShouldApplyDottedInclusionToEachElementIn
 }
 
 TEST(InclusionProjectionExecutionTest, ShouldAddComputedDottedFieldToSubDocument) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("sub.target" << wrapInLiteral("computedVal")));
 
     // Other fields exist in sub document, one of which is the specified field.
     auto result = inclusion.applyProjection(Document{{"sub", Document{{"target", 1}, {"c", 2}}}});
-    auto expectedResult = Document{{"sub", Document{{"target", "computedVal"}}}};
+    auto expectedResult = Document{{"sub", Document{{"target", "computedVal"_sd}}}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 
     // Specified field is not present in the sub document.
     result = inclusion.applyProjection(Document{{"sub", Document{{"c", 1}}}});
-    expectedResult = Document{{"sub", Document{{"target", "computedVal"}}}};
+    expectedResult = Document{{"sub", Document{{"target", "computedVal"_sd}}}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 
     // There are no fields in sub document.
     result = inclusion.applyProjection(Document{{"sub", Document{}}});
-    expectedResult = Document{{"sub", Document{{"target", "computedVal"}}}};
+    expectedResult = Document{{"sub", Document{{"target", "computedVal"_sd}}}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
 TEST(InclusionProjectionExecutionTest, ShouldCreateSubDocIfDottedComputedFieldDoesntExist) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("sub.target" << wrapInLiteral("computedVal")));
 
     // Should add the path if it doesn't exist.
     auto result = inclusion.applyProjection(Document{});
-    auto expectedResult = Document{{"sub", Document{{"target", "computedVal"}}}};
+    auto expectedResult = Document{{"sub", Document{{"target", "computedVal"_sd}}}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 
     // Should replace non-documents with documents.
-    result = inclusion.applyProjection(Document{{"sub", "notADocument"}});
+    result = inclusion.applyProjection(Document{{"sub", "notADocument"_sd}});
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
 TEST(InclusionProjectionExecutionTest, ShouldCreateNestedSubDocumentsAllTheWayToComputedField) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("a.b.c.d" << wrapInLiteral("computedVal")));
 
     // Should add the path if it doesn't exist.
     auto result = inclusion.applyProjection(Document{});
     auto expectedResult =
-        Document{{"a", Document{{"b", Document{{"c", Document{{"d", "computedVal"}}}}}}}};
+        Document{{"a", Document{{"b", Document{{"c", Document{{"d", "computedVal"_sd}}}}}}}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 
     // Should replace non-documents with documents.
-    result = inclusion.applyProjection(Document{{"a", Document{{"b", "other"}}}});
+    result = inclusion.applyProjection(Document{{"a", Document{{"b", "other"_sd}}}});
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
 TEST(InclusionProjectionExecutionTest, ShouldAddComputedDottedFieldToEachElementInArray) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("a.b" << wrapInLiteral("COMPUTED")));
 
     vector<Value> nestedValues = {Value(1),
@@ -455,20 +506,22 @@ TEST(InclusionProjectionExecutionTest, ShouldAddComputedDottedFieldToEachElement
                                   Value(Document{{"b", 1}, {"c", 2}}),
                                   Value(vector<Value>{}),
                                   Value(vector<Value>{Value(1), Value(Document{{"c", 1}})})};
-    vector<Value> expectedNestedValues = {Value(Document{{"b", "COMPUTED"}}),
-                                          Value(Document{{"b", "COMPUTED"}}),
-                                          Value(Document{{"b", "COMPUTED"}}),
-                                          Value(Document{{"b", "COMPUTED"}}),
-                                          Value(vector<Value>{}),
-                                          Value(vector<Value>{Value(Document{{"b", "COMPUTED"}}),
-                                                              Value(Document{{"b", "COMPUTED"}})})};
+    vector<Value> expectedNestedValues = {
+        Value(Document{{"b", "COMPUTED"_sd}}),
+        Value(Document{{"b", "COMPUTED"_sd}}),
+        Value(Document{{"b", "COMPUTED"_sd}}),
+        Value(Document{{"b", "COMPUTED"_sd}}),
+        Value(vector<Value>{}),
+        Value(vector<Value>{Value(Document{{"b", "COMPUTED"_sd}}),
+                            Value(Document{{"b", "COMPUTED"_sd}})})};
     auto result = inclusion.applyProjection(Document{{"a", nestedValues}});
     auto expectedResult = Document{{"a", expectedNestedValues}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
 TEST(InclusionProjectionExecutionTest, ShouldApplyInclusionsAndAdditionsToEachElementInArray) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("a.inc" << true << "a.comp" << wrapInLiteral("COMPUTED")));
 
     vector<Value> nestedValues = {Value(1),
@@ -476,34 +529,36 @@ TEST(InclusionProjectionExecutionTest, ShouldApplyInclusionsAndAdditionsToEachEl
                                   Value(Document{{"inc", 1}}),
                                   Value(Document{{"inc", 1}, {"c", 2}}),
                                   Value(Document{{"c", 2}, {"inc", 1}}),
-                                  Value(Document{{"inc", 1}, {"c", 2}, {"comp", "original"}}),
+                                  Value(Document{{"inc", 1}, {"c", 2}, {"comp", "original"_sd}}),
                                   Value(vector<Value>{}),
                                   Value(vector<Value>{Value(1), Value(Document{{"inc", 1}})})};
     vector<Value> expectedNestedValues = {
-        Value(Document{{"comp", "COMPUTED"}}),
-        Value(Document{{"comp", "COMPUTED"}}),
-        Value(Document{{"inc", 1}, {"comp", "COMPUTED"}}),
-        Value(Document{{"inc", 1}, {"comp", "COMPUTED"}}),
-        Value(Document{{"inc", 1}, {"comp", "COMPUTED"}}),
-        Value(Document{{"inc", 1}, {"comp", "COMPUTED"}}),
+        Value(Document{{"comp", "COMPUTED"_sd}}),
+        Value(Document{{"comp", "COMPUTED"_sd}}),
+        Value(Document{{"inc", 1}, {"comp", "COMPUTED"_sd}}),
+        Value(Document{{"inc", 1}, {"comp", "COMPUTED"_sd}}),
+        Value(Document{{"inc", 1}, {"comp", "COMPUTED"_sd}}),
+        Value(Document{{"inc", 1}, {"comp", "COMPUTED"_sd}}),
         Value(vector<Value>{}),
-        Value(vector<Value>{Value(Document{{"comp", "COMPUTED"}}),
-                            Value(Document{{"inc", 1}, {"comp", "COMPUTED"}})})};
+        Value(vector<Value>{Value(Document{{"comp", "COMPUTED"_sd}}),
+                            Value(Document{{"inc", 1}, {"comp", "COMPUTED"_sd}})})};
     auto result = inclusion.applyProjection(Document{{"a", nestedValues}});
     auto expectedResult = Document{{"a", expectedNestedValues}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
 TEST(InclusionProjectionExecutionTest, ShouldAddOrIncludeSubFieldsOfId) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("_id.X" << true << "_id.Z" << wrapInLiteral("NEW")));
     auto result = inclusion.applyProjection(Document{{"_id", Document{{"X", 1}, {"Y", 2}}}});
-    auto expectedResult = Document{{"_id", Document{{"X", 1}, {"Z", "NEW"}}}};
+    auto expectedResult = Document{{"_id", Document{{"X", 1}, {"Z", "NEW"_sd}}}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
 TEST(InclusionProjectionExecutionTest, ShouldAllowMixedNestedAndDottedFields) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     // Include all of "a.b", "a.c", "a.d", and "a.e".
     // Add new computed fields "a.W", "a.X", "a.Y", and "a.Z".
     inclusion.parse(BSON(
@@ -511,33 +566,36 @@ TEST(InclusionProjectionExecutionTest, ShouldAllowMixedNestedAndDottedFields) {
               << "a"
               << BSON("d" << true << "e" << true << "Y" << wrapInLiteral("Y") << "Z"
                           << wrapInLiteral("Z"))));
-    auto result = inclusion.applyProjection(
-        Document{{"a", Document{{"b", "b"}, {"c", "c"}, {"d", "d"}, {"e", "e"}, {"f", "f"}}}});
+    auto result = inclusion.applyProjection(Document{
+        {"a",
+         Document{{"b", "b"_sd}, {"c", "c"_sd}, {"d", "d"_sd}, {"e", "e"_sd}, {"f", "f"_sd}}}});
     auto expectedResult = Document{{"a",
-                                    Document{{"b", "b"},
-                                             {"c", "c"},
-                                             {"d", "d"},
-                                             {"e", "e"},
-                                             {"W", "W"},
-                                             {"X", "X"},
-                                             {"Y", "Y"},
-                                             {"Z", "Z"}}}};
+                                    Document{{"b", "b"_sd},
+                                             {"c", "c"_sd},
+                                             {"d", "d"_sd},
+                                             {"e", "e"_sd},
+                                             {"W", "W"_sd},
+                                             {"X", "X"_sd},
+                                             {"Y", "Y"_sd},
+                                             {"Z", "Z"_sd}}}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
 TEST(InclusionProjectionExecutionTest, ShouldApplyNestedComputedFieldsInOrderSpecified) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("a" << wrapInLiteral("FIRST") << "b.c" << wrapInLiteral("SECOND")));
     auto result = inclusion.applyProjection(Document{});
-    auto expectedResult = Document{{"a", "FIRST"}, {"b", Document{{"c", "SECOND"}}}};
+    auto expectedResult = Document{{"a", "FIRST"_sd}, {"b", Document{{"c", "SECOND"_sd}}}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
 TEST(InclusionProjectionExecutionTest, ShouldApplyComputedFieldsAfterAllInclusions) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("b.c" << wrapInLiteral("NEW") << "a" << true));
     auto result = inclusion.applyProjection(Document{{"a", 1}});
-    auto expectedResult = Document{{"a", 1}, {"b", Document{{"c", "NEW"}}}};
+    auto expectedResult = Document{{"a", 1}, {"b", Document{{"c", "NEW"_sd}}}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 
     result = inclusion.applyProjection(Document{{"a", 1}, {"b", 4}});
@@ -548,15 +606,16 @@ TEST(InclusionProjectionExecutionTest, ShouldApplyComputedFieldsAfterAllInclusio
     // inconsistent, but the expected behavior, and a consequence of applying the projection
     // recursively to each sub-document.
     result = inclusion.applyProjection(Document{{"b", 4}, {"a", 1}});
-    expectedResult = Document{{"b", Document{{"c", "NEW"}}}, {"a", 1}};
+    expectedResult = Document{{"b", Document{{"c", "NEW"_sd}}}, {"a", 1}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 }
 
 TEST(InclusionProjectionExecutionTest, ComputedFieldReplacingExistingShouldAppearAfterInclusions) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("b" << wrapInLiteral("NEW") << "a" << true));
     auto result = inclusion.applyProjection(Document{{"b", 1}, {"a", 1}});
-    auto expectedResult = Document{{"a", 1}, {"b", "NEW"}};
+    auto expectedResult = Document{{"a", 1}, {"b", "NEW"_sd}};
     ASSERT_DOCUMENT_EQ(result, expectedResult);
 
     result = inclusion.applyProjection(Document{{"a", 1}, {"b", 4}});
@@ -568,7 +627,8 @@ TEST(InclusionProjectionExecutionTest, ComputedFieldReplacingExistingShouldAppea
 //
 
 TEST(InclusionProjectionExecutionTest, ShouldAlwaysKeepMetadataFromOriginalDoc) {
-    ParsedInclusionProjection inclusion;
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
     inclusion.parse(BSON("a" << true));
 
     MutableDocument inputDocBuilder(Document{{"a", 1}});
@@ -583,6 +643,104 @@ TEST(InclusionProjectionExecutionTest, ShouldAlwaysKeepMetadataFromOriginalDoc)
     ASSERT_DOCUMENT_EQ(result, expectedDoc.freeze());
 }
 
+//
+// Detection of subset projection.
+//
+
+TEST(InclusionProjectionSubsetTest, ShouldDetectSubsetForIdenticalProjection) {
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
+    inclusion.parse(BSON("a" << true << "b" << true));
+
+    auto proj = BSON("_id" << false << "a" << true << "b" << true);
+
+    ASSERT_TRUE(inclusion.isSubsetOfProjection(proj));
+}
+
+TEST(InclusionProjectionSubsetTest, ShouldDetectSubsetForSupersetProjection) {
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
+    inclusion.parse(BSON("a" << true << "b" << true));
+
+    auto proj = BSON("_id" << false << "a" << true << "b" << true << "c" << true);
+
+    ASSERT_TRUE(inclusion.isSubsetOfProjection(proj));
+}
+
+TEST(InclusionProjectionSubsetTest, ShouldDetectSubsetForIdenticalNestedProjection) {
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
+    inclusion.parse(BSON("a.b" << true));
+
+    auto proj = BSON("_id" << false << "a.b" << true);
+
+    ASSERT_TRUE(inclusion.isSubsetOfProjection(proj));
+}
+
+TEST(InclusionProjectionSubsetTest, ShouldDetectSubsetForSupersetProjectionWithNestedFields) {
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
+    inclusion.parse(BSON("a" << true << "c" << BSON("d" << true)));
+
+    auto proj = BSON("_id" << false << "a" << true << "b" << true << "c.d" << true);
+
+    ASSERT_TRUE(inclusion.isSubsetOfProjection(proj));
+}
+
+TEST(InclusionProjectionSubsetTest, ShouldDetectNonSubsetForProjectionWithMissingFields) {
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
+    inclusion.parse(BSON("a" << true << "b" << true));
+
+    auto proj = BSON("_id" << false << "a" << true);
+    ASSERT_FALSE(inclusion.isSubsetOfProjection(proj));
+
+    proj = BSON("_id" << false << "a" << true << "c" << true);
+    ASSERT_FALSE(inclusion.isSubsetOfProjection(proj));
+}
+
+TEST(InclusionProjectionSubsetTest,
+     ShouldDetectNonSubsetForSupersetProjectionWithoutComputedFields) {
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
+    inclusion.parse(BSON("a" << true << "b" << true << "c" << BSON("$literal" << 1)));
+
+    auto proj = BSON("_id" << false << "a" << true << "b" << true);
+
+    ASSERT_FALSE(inclusion.isSubsetOfProjection(proj));
+}
+
+TEST(InclusionProjectionSubsetTest, ShouldDetectNonSubsetForProjectionWithMissingNestedFields) {
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
+    inclusion.parse(BSON("a.b" << true << "a.c" << true));
+
+    auto proj = BSON("_id" << false << "a.b" << true);
+
+    ASSERT_FALSE(inclusion.isSubsetOfProjection(proj));
+}
+
+TEST(InclusionProjectionSubsetTest, ShouldDetectNonSubsetForProjectionWithRenamedFields) {
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
+    inclusion.parse(BSON("a"
+                         << "$b"));
+
+    auto proj = BSON("_id" << false << "b" << true);
+
+    ASSERT_FALSE(inclusion.isSubsetOfProjection(proj));
+}
+
+TEST(InclusionProjectionSubsetTest, ShouldDetectNonSubsetForProjectionWithMissingIdField) {
+    const boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
+    ParsedInclusionProjection inclusion(expCtx);
+    inclusion.parse(BSON("a" << true));
+
+    auto proj = BSON("a" << true);
+
+    ASSERT_FALSE(inclusion.isSubsetOfProjection(proj));
+}
+
 }  // namespace
 }  // namespace parsed_aggregation_projection
 }  // namespace mongo
