@@ -33,15 +33,17 @@
 
 #include "mongo/bson/bsonobj.h"
 #include "mongo/db/namespace_string.h"
+#include "mongo/db/pipeline/aggregation_request.h"
+#include "mongo/db/query/plan_executor.h"
 
 namespace mongo {
 class Collection;
 class DocumentSourceCursor;
+class DocumentSourceMatch;
 class DocumentSourceSort;
-struct ExpressionContext;
+class ExpressionContext;
 class OperationContext;
 class Pipeline;
-class PlanExecutor;
 struct PlanSummaryStats;
 class BSONObj;
 struct DepsTracker;
@@ -58,27 +60,34 @@ struct DepsTracker;
 class PipelineD {
 public:
     /**
-     * Create a Cursor wrapped in a DocumentSourceCursor, which is suitable
-     * to be the first source for a pipeline to begin with.  This source
-     * will feed the execution of the pipeline.
+     * If the first stage in the pipeline does not generate its own output documents, attaches a
+     * DocumentSourceCursor to the front of the pipeline which will output documents from the
+     * collection to feed into the pipeline.
      *
-     * This method looks for early pipeline stages that can be folded into
-     * the underlying cursor, and when a cursor can absorb those, they
-     * are removed from the head of the pipeline.  For example, an
-     * early match can be removed and replaced with a Cursor that will
-     * do an index scan.
+     * This method looks for early pipeline stages that can be folded into the underlying
+     * PlanExecutor, and removes those stages from the pipeline when they can be absorbed by the
+     * PlanExecutor. For example, an early $match can be removed and replaced with a
+     * DocumentSourceCursor containing a PlanExecutor that will do an index scan.
      *
-     * The cursor is added to the front of the pipeline's sources.
+     * Callers must take care to ensure that 'nss' is locked in at least IS-mode.
      *
-     * Callers must take care to ensure that 'collection' is locked in at least IS-mode.
+     * When not null, 'aggRequest' provides access to pipeline command options such as hint.
      */
     static void prepareCursorSource(Collection* collection,
-                                    const boost::intrusive_ptr<Pipeline>& pipeline);
+                                    const NamespaceString& nss,
+                                    const AggregationRequest* aggRequest,
+                                    Pipeline* pipeline);
 
-    static std::string getPlanSummaryStr(const boost::intrusive_ptr<Pipeline>& pPipeline);
+    /**
+     * Injects a MongodInterface into stages which require access to mongod-specific functionality.
+     */
+    static void injectMongodInterface(Pipeline* pipeline);
 
-    static void getPlanSummaryStats(const boost::intrusive_ptr<Pipeline>& pPipeline,
-                                    PlanSummaryStats* statsOut);
+    static std::string getPlanSummaryStr(const Pipeline* pipeline);
+
+    static void getPlanSummaryStats(const Pipeline* pipeline, PlanSummaryStats* statsOut);
+
+    static Timestamp getLatestOplogTimestamp(const Pipeline* pipeline);
 
 private:
     PipelineD();  // does not exist:  prevent instantiation
@@ -92,15 +101,17 @@ private:
      * sort, and 'projectionObj' will be set to an empty object if the query system cannot provide a
      * covered projection.
      */
-    static StatusWith<std::unique_ptr<PlanExecutor>> prepareExecutor(
-        OperationContext* txn,
+    static StatusWith<std::unique_ptr<PlanExecutor, PlanExecutor::Deleter>> prepareExecutor(
+        OperationContext* opCtx,
         Collection* collection,
         const NamespaceString& nss,
-        const boost::intrusive_ptr<Pipeline>& pipeline,
+        Pipeline* pipeline,
         const boost::intrusive_ptr<ExpressionContext>& expCtx,
+        bool oplogReplay,
         const boost::intrusive_ptr<DocumentSourceSort>& sortStage,
         const DepsTracker& deps,
         const BSONObj& queryObj,
+        const AggregationRequest* aggRequest,
         BSONObj* sortObj,
         BSONObj* projectionObj);
 
@@ -108,9 +119,10 @@ private:
      * Creates a DocumentSourceCursor from the given PlanExecutor and adds it to the front of the
      * Pipeline.
      */
-    static void addCursorSource(const boost::intrusive_ptr<Pipeline>& pipeline,
+    static void addCursorSource(Collection* collection,
+                                Pipeline* pipeline,
                                 const boost::intrusive_ptr<ExpressionContext>& expCtx,
-                                std::unique_ptr<PlanExecutor> exec,
+                                std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> exec,
                                 DepsTracker deps,
                                 const BSONObj& queryObj = BSONObj(),
                                 const BSONObj& sortObj = BSONObj(),
