@@ -41,6 +41,7 @@
 #include "mongo/bson/simple_bsonobj_comparator.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/pipeline/document.h"
+#include "mongo/db/query/datetime/date_time_support.h"
 #include "mongo/platform/decimal128.h"
 #include "mongo/util/hex.h"
 #include "mongo/util/mongoutils/str.h"
@@ -55,6 +56,10 @@ using std::string;
 using std::stringstream;
 using std::vector;
 
+namespace {
+constexpr StringData kISOFormatString = "%Y-%m-%dT%H:%M:%S.%LZ"_sd;
+}
+
 void ValueStorage::verifyRefCountingIfShould() const {
     switch (type) {
         case MinKey:
@@ -343,7 +348,7 @@ BSONObjBuilder& operator<<(BSONObjBuilderValueStream& builder, const Value& val)
         case Bool:
             return builder << val.getBool();
         case Date:
-            return builder << Date_t::fromMillisSinceEpoch(val.getDate());
+            return builder << val.getDate();
         case bsonTimestamp:
             return builder << val.getTimestamp();
         case Object:
@@ -556,13 +561,16 @@ Decimal128 Value::coerceToDecimal() const {
     }  // switch(getType())
 }
 
-long long Value::coerceToDate() const {
+Date_t Value::coerceToDate() const {
     switch (getType()) {
         case Date:
             return getDate();
 
         case bsonTimestamp:
-            return getTimestamp().getSecs() * 1000LL;
+            return Date_t::fromMillisSinceEpoch(getTimestamp().getSecs() * 1000LL);
+
+        case jstOID:
+            return getOid().asDateT();
 
         default:
             uassert(16006,
@@ -572,57 +580,6 @@ long long Value::coerceToDate() const {
     }  // switch(getType())
 }
 
-time_t Value::coerceToTimeT() const {
-    long long millis = coerceToDate();
-    if (millis < 0) {
-        // We want the division below to truncate toward -inf rather than 0
-        // eg Dec 31, 1969 23:59:58.001 should be -2 seconds rather than -1
-        // This is needed to get the correct values from coerceToTM
-        if (-1999 / 1000 != -2) {  // this is implementation defined
-            millis -= 1000 - 1;
-        }
-    }
-    const long long seconds = millis / 1000;
-
-    uassert(16421,
-            "Can't handle date values outside of time_t range",
-            seconds >= std::numeric_limits<time_t>::min() &&
-                seconds <= std::numeric_limits<time_t>::max());
-
-    return static_cast<time_t>(seconds);
-}
-tm Value::coerceToTm() const {
-    // See implementation in Date_t.
-    // Can't reuse that here because it doesn't support times before 1970
-    time_t dtime = coerceToTimeT();
-    tm out;
-
-#if defined(_WIN32)  // Both the argument order and the return values differ
-    bool itWorked = gmtime_s(&out, &dtime) == 0;
-#else
-    bool itWorked = gmtime_r(&dtime, &out) != NULL;
-#endif
-
-    if (!itWorked) {
-        if (dtime < 0) {
-            // Windows docs say it doesn't support these, but empirically it seems to work
-            uasserted(16422, "gmtime failed - your system doesn't support dates before 1970");
-        } else {
-            uasserted(16423, str::stream() << "gmtime failed to convert time_t of " << dtime);
-        }
-    }
-
-    return out;
-}
-
-static string tmToISODateString(const tm& time) {
-    char buf[128];
-    size_t len = strftime(buf, 128, "%Y-%m-%dT%H:%M:%S", &time);
-    verify(len > 0);
-    verify(len < 128);
-    return buf;
-}
-
 string Value::coerceToString() const {
     switch (getType()) {
         case NumberDouble:
@@ -646,7 +603,7 @@ string Value::coerceToString() const {
             return getTimestamp().toStringPretty();
 
         case Date:
-            return tmToISODateString(coerceToTm());
+            return TimeZoneDatabase::utcZone().formatDate(kISOFormatString, getDate());
 
         case EOO:
         case jstNULL:
@@ -691,7 +648,7 @@ inline static int cmp(const T& left, const T& right) {
 int Value::compare(const Value& rL,
                    const Value& rR,
                    const StringData::ComparatorInterface* stringComparator) {
-    // Note, this function needs to behave identically to BSON's compareElementValues().
+    // Note, this function needs to behave identically to BSONElement::compareElements().
     // Additionally, any changes here must be replicated in hash_combine().
     BSONType lType = rL.getType();
     BSONType rType = rR.getType();
@@ -703,7 +660,8 @@ int Value::compare(const Value& rL,
         return ret;
 
     switch (lType) {
-        // Order of types is the same as in compareElementValues() to make it easier to verify
+        // Order of types is the same as in BSONElement::compareElements() to make it easier to
+        // verify.
 
         // These are valueless types
         case EOO:
@@ -845,7 +803,9 @@ int Value::compare(const Value& rL,
             return rL.getStringData().compare(rR.getStringData());
         }
 
-        case RegEx:  // same as String in this impl but keeping order same as compareElementValues
+        case RegEx:
+            // same as String in this impl but keeping order same as
+            // BSONElement::compareElements().
             return rL.getStringData().compare(rR.getStringData());
 
         case CodeWScope: {
@@ -869,7 +829,7 @@ void Value::hash_combine(size_t& seed,
     boost::hash_combine(seed, canonicalizeBSONType(type));
 
     switch (type) {
-        // Order of types is the same as in Value::compare() and compareElementValues().
+        // Order of types is the same as in Value::compare() and BSONElement::compareElements().
 
         // These are valueless types
         case EOO:
@@ -1163,7 +1123,8 @@ ostream& operator<<(ostream& out, const Value& val) {
         case Undefined:
             return out << "undefined";
         case Date:
-            return out << tmToISODateString(val.coerceToTm());
+            return out << TimeZoneDatabase::utcZone().formatDate(kISOFormatString,
+                                                                 val.coerceToDate());
         case bsonTimestamp:
             return out << val.getTimestamp().toString();
         case Object:
@@ -1365,4 +1326,17 @@ Value Value::deserializeForSorter(BufReader& buf, const SorterDeserializeSetting
     }
     verify(false);
 }
+
+void Value::serializeForIDL(StringData fieldName, BSONObjBuilder* builder) const {
+    addToBsonObj(builder, fieldName);
 }
+
+void Value::serializeForIDL(BSONArrayBuilder* builder) const {
+    addToBsonArray(builder);
+}
+
+Value Value::deserializeForIDL(const BSONElement& element) {
+    return Value(element);
+}
+
+}  // namespace mongo
