@@ -32,6 +32,7 @@
 #include "mongo/base/string_data.h"
 #include "mongo/db/pipeline/value_internal.h"
 #include "mongo/platform/unordered_set.h"
+#include "mongo/util/uuid.h"
 
 namespace mongo {
 class BSONElement;
@@ -99,7 +100,6 @@ public:
     explicit Value(const OID& value) : _storage(jstOID, value) {}
     explicit Value(StringData value) : _storage(String, value) {}
     explicit Value(const std::string& value) : _storage(String, StringData(value)) {}
-    explicit Value(const char* value) : _storage(String, StringData(value)) {}
     explicit Value(const Document& doc) : _storage(Object, doc) {}
     explicit Value(const BSONObj& obj);
     explicit Value(const BSONArray& arr);
@@ -117,6 +117,12 @@ public:
     explicit Value(const MinKeyLabeler&) : _storage(MinKey) {}        // MINKEY
     explicit Value(const MaxKeyLabeler&) : _storage(MaxKey) {}        // MAXKEY
     explicit Value(const Date_t& date) : _storage(Date, date.toMillisSinceEpoch()) {}
+    explicit Value(const UUID& uuid)
+        : _storage(BinData,
+                   BSONBinData(uuid.toCDR().data(), uuid.toCDR().length(), BinDataType::newUUID)) {}
+
+    explicit Value(const char*) = delete;  // Use StringData instead to prevent accidentally
+                                           // terminating the string at the first null byte.
 
     // TODO: add an unsafe version that can share storage with the BSONElement
     /// Deep-convert from BSONElement to Value
@@ -177,7 +183,7 @@ public:
     Document getDocument() const;
     OID getOid() const;
     bool getBool() const;
-    long long getDate() const;  // in milliseconds
+    Date_t getDate() const;
     Timestamp getTimestamp() const;
     const char* getRegex() const;
     const char* getRegexFlags() const;
@@ -185,6 +191,9 @@ public:
     std::string getCode() const;
     int getInt() const;
     long long getLong() const;
+    UUID getUuid() const;
+    // The returned BSONBinData remains owned by this Value.
+    BSONBinData getBinData() const;
     const std::vector<Value>& getArray() const {
         return _storage.getArray();
     }
@@ -196,11 +205,20 @@ public:
     /// Access a field of a subdocument. Returns Value() if missing or getType() != Object
     Value operator[](StringData name) const;
 
-    /// Add this value to the BSON object under construction.
-    void addToBsonObj(BSONObjBuilder* pBuilder, StringData fieldName) const;
+    /**
+     * Recursively serializes this value as a field in the object in 'builder' with the field name
+     * 'fieldName'. This function throws a AssertionException if the recursion exceeds the server's
+     * BSON depth limit.
+     */
+    void addToBsonObj(BSONObjBuilder* builder,
+                      StringData fieldName,
+                      size_t recursionLevel = 1) const;
 
-    /// Add this field to the BSON array under construction.
-    void addToBsonArray(BSONArrayBuilder* pBuilder) const;
+    /**
+     * Recursively serializes this value as an element in the array in 'builder'. This function
+     * throws a AssertionException if the recursion exceeds the server's BSON depth limit.
+     */
+    void addToBsonArray(BSONArrayBuilder* builder, size_t recursionLevel = 1) const;
 
     // Support BSONObjBuilder and BSONArrayBuilder "stream" API
     friend BSONObjBuilder& operator<<(BSONObjBuilderValueStream& builder, const Value& val);
@@ -220,9 +238,7 @@ public:
     double coerceToDouble() const;
     Decimal128 coerceToDecimal() const;
     Timestamp coerceToTimestamp() const;
-    long long coerceToDate() const;
-    time_t coerceToTimeT() const;
-    tm coerceToTm() const;  // broken-out time struct (see man gmtime)
+    Date_t coerceToDate() const;
 
     //
     // Comparison API.
@@ -319,6 +335,11 @@ public:
         return *this;
     }
 
+    /// Members to support parsing/deserialization from IDL generated code.
+    void serializeForIDL(StringData fieldName, BSONObjBuilder* builder) const;
+    void serializeForIDL(BSONArrayBuilder* builder) const;
+    static Value deserializeForIDL(const BSONElement& element);
+
 private:
     /** This is a "honeypot" to prevent unexpected implicit conversions to the accepted argument
      *  types. bool is especially bad since without this it will accept any pointer.
@@ -350,6 +371,16 @@ class ImplicitValue : public Value {
 public:
     template <typename T>
     ImplicitValue(T arg) : Value(std::move(arg)) {}
+
+    /**
+     * Converts a vector of Implicit values to a single Value object.
+     */
+    static Value convertToValue(const std::vector<ImplicitValue>& vec) {
+        std::vector<Value> values;
+        for_each(
+            vec.begin(), vec.end(), ([&](const ImplicitValue& val) { values.push_back(val); }));
+        return Value(values);
+    }
 };
 }
 
@@ -381,9 +412,9 @@ inline bool Value::getBool() const {
     return _storage.boolValue;
 }
 
-inline long long Value::getDate() const {
+inline Date_t Value::getDate() const {
     verify(getType() == Date);
-    return _storage.dateValue;
+    return Date_t::fromMillisSinceEpoch(_storage.dateValue);
 }
 
 inline Timestamp Value::getTimestamp() const {
@@ -425,4 +456,16 @@ inline long long Value::getLong() const {
     verify(type == NumberLong);
     return _storage.longValue;
 }
+
+inline UUID Value::getUuid() const {
+    verify(_storage.binDataType() == BinDataType::newUUID);
+    auto stringData = _storage.getString();
+    return UUID::fromCDR({stringData.rawData(), stringData.size()});
+}
+
+inline BSONBinData Value::getBinData() const {
+    verify(getType() == BinData);
+    auto stringData = _storage.getString();
+    return BSONBinData(stringData.rawData(), stringData.size(), _storage.binDataType());
+}
 };
