@@ -29,7 +29,6 @@
 #pragma once
 
 #include <algorithm>
-#include <boost/config.hpp>
 #include <boost/intrusive_ptr.hpp>
 
 #include "mongo/base/static_assert.h"
@@ -168,7 +167,7 @@ public:
         memcpyed();
     }
 
-    ValueStorage(ValueStorage&& rhs) BOOST_NOEXCEPT {
+    ValueStorage(ValueStorage&& rhs) noexcept {
         memcpy(this, &rhs, sizeof(*this));
         rhs.zero();  // Reset rhs to the missing state. TODO consider only doing this if refCounter.
     }
@@ -196,7 +195,7 @@ public:
         return *this;
     }
 
-    ValueStorage& operator=(ValueStorage&& rhs) BOOST_NOEXCEPT {
+    ValueStorage& operator=(ValueStorage&& rhs) noexcept {
         DEV verifyRefCountingIfShould();
         if (refCounter)
             intrusive_ptr_release(genericRCPtr);
