@@ -34,6 +34,6 @@ class OperationContext;
 namespace repl {
 
 // page in possible index and/or data pages for an op from the oplog
-void prefetchPagesForReplicatedOp(OperationContext* txn, Database* db, const BSONObj& op);
+void prefetchPagesForReplicatedOp(OperationContext* opCtx, Database* db, const BSONObj& op);
 }  // namespace repl
 }  // namespace mongo
