@@ -32,9 +32,12 @@
 
 #include "mongo/bson/simple_bsonobj_comparator.h"
 #include "mongo/db/catalog/collection.h"
+#include "mongo/db/catalog/database.h"
+#include "mongo/db/catalog/database_holder.h"
 #include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/exec/cached_plan.h"
+#include "mongo/db/exec/collection_scan.h"
 #include "mongo/db/exec/multi_plan.h"
 #include "mongo/db/exec/pipeline_proxy.h"
 #include "mongo/db/exec/plan_stage.h"
@@ -42,11 +45,15 @@
 #include "mongo/db/exec/subplan.h"
 #include "mongo/db/exec/working_set.h"
 #include "mongo/db/exec/working_set_common.h"
+#include "mongo/db/query/find_common.h"
+#include "mongo/db/query/mock_yield_policies.h"
 #include "mongo/db/query/plan_yield_policy.h"
+#include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/storage/record_fetcher.h"
 #include "mongo/stdx/memory.h"
 #include "mongo/util/fail_point_service.h"
+#include "mongo/util/scopeguard.h"
 #include "mongo/util/stacktrace.h"
 
 namespace mongo {
@@ -55,13 +62,41 @@ using std::shared_ptr;
 using std::string;
 using std::unique_ptr;
 using std::vector;
-using stdx::make_unique;
+
+const OperationContext::Decoration<repl::OpTime> clientsLastKnownCommittedOpTime =
+    OperationContext::declareDecoration<repl::OpTime>();
+
+struct CappedInsertNotifierData {
+    shared_ptr<CappedInsertNotifier> notifier;
+    uint64_t lastEOFVersion = ~0;
+};
 
 namespace {
 
-namespace {
-MONGO_FP_DECLARE(planExecutorAlwaysDead);
-}  // namespace
+MONGO_FP_DECLARE(planExecutorAlwaysFails);
+
+/**
+ * Constructs a PlanYieldPolicy based on 'policy'.
+ */
+std::unique_ptr<PlanYieldPolicy> makeYieldPolicy(PlanExecutor* exec,
+                                                 PlanExecutor::YieldPolicy policy) {
+    switch (policy) {
+        case PlanExecutor::YieldPolicy::YIELD_AUTO:
+        case PlanExecutor::YieldPolicy::YIELD_MANUAL:
+        case PlanExecutor::YieldPolicy::NO_YIELD:
+        case PlanExecutor::YieldPolicy::WRITE_CONFLICT_RETRY_ONLY: {
+            return stdx::make_unique<PlanYieldPolicy>(exec, policy);
+        }
+        case PlanExecutor::YieldPolicy::ALWAYS_TIME_OUT: {
+            return stdx::make_unique<AlwaysTimeOutYieldPolicy>(exec);
+        }
+        case PlanExecutor::YieldPolicy::ALWAYS_MARK_KILLED: {
+            return stdx::make_unique<AlwaysPlanKilledYieldPolicy>(exec);
+        }
+        default:
+            MONGO_UNREACHABLE;
+    }
+}
 
 /**
  * Retrieves the first stage of a given type from the plan tree, or NULL
@@ -82,71 +117,91 @@ PlanStage* getStageByType(PlanStage* root, StageType type) {
 
     return NULL;
 }
-}
+}  // namespace
 
 // static
-StatusWith<unique_ptr<PlanExecutor>> PlanExecutor::make(OperationContext* opCtx,
-                                                        unique_ptr<WorkingSet> ws,
-                                                        unique_ptr<PlanStage> rt,
-                                                        const Collection* collection,
-                                                        YieldPolicy yieldPolicy) {
+StatusWith<unique_ptr<PlanExecutor, PlanExecutor::Deleter>> PlanExecutor::make(
+    OperationContext* opCtx,
+    unique_ptr<WorkingSet> ws,
+    unique_ptr<PlanStage> rt,
+    const Collection* collection,
+    YieldPolicy yieldPolicy) {
     return PlanExecutor::make(
-        opCtx, std::move(ws), std::move(rt), nullptr, nullptr, collection, "", yieldPolicy);
+        opCtx, std::move(ws), std::move(rt), nullptr, nullptr, collection, {}, yieldPolicy);
 }
 
 // static
-StatusWith<unique_ptr<PlanExecutor>> PlanExecutor::make(OperationContext* opCtx,
-                                                        unique_ptr<WorkingSet> ws,
-                                                        unique_ptr<PlanStage> rt,
-                                                        const string& ns,
-                                                        YieldPolicy yieldPolicy) {
+StatusWith<unique_ptr<PlanExecutor, PlanExecutor::Deleter>> PlanExecutor::make(
+    OperationContext* opCtx,
+    unique_ptr<WorkingSet> ws,
+    unique_ptr<PlanStage> rt,
+    NamespaceString nss,
+    YieldPolicy yieldPolicy) {
+    return PlanExecutor::make(opCtx,
+                              std::move(ws),
+                              std::move(rt),
+                              nullptr,
+                              nullptr,
+                              nullptr,
+                              std::move(nss),
+                              yieldPolicy);
+}
+
+// static
+StatusWith<unique_ptr<PlanExecutor, PlanExecutor::Deleter>> PlanExecutor::make(
+    OperationContext* opCtx,
+    unique_ptr<WorkingSet> ws,
+    unique_ptr<PlanStage> rt,
+    unique_ptr<CanonicalQuery> cq,
+    const Collection* collection,
+    YieldPolicy yieldPolicy) {
     return PlanExecutor::make(
-        opCtx, std::move(ws), std::move(rt), nullptr, nullptr, nullptr, ns, yieldPolicy);
+        opCtx, std::move(ws), std::move(rt), nullptr, std::move(cq), collection, {}, yieldPolicy);
 }
 
 // static
-StatusWith<unique_ptr<PlanExecutor>> PlanExecutor::make(OperationContext* opCtx,
-                                                        unique_ptr<WorkingSet> ws,
-                                                        unique_ptr<PlanStage> rt,
-                                                        unique_ptr<CanonicalQuery> cq,
-                                                        const Collection* collection,
-                                                        YieldPolicy yieldPolicy) {
-    return PlanExecutor::make(
-        opCtx, std::move(ws), std::move(rt), nullptr, std::move(cq), collection, "", yieldPolicy);
-}
-
-// static
-StatusWith<unique_ptr<PlanExecutor>> PlanExecutor::make(OperationContext* opCtx,
-                                                        unique_ptr<WorkingSet> ws,
-                                                        unique_ptr<PlanStage> rt,
-                                                        unique_ptr<QuerySolution> qs,
-                                                        unique_ptr<CanonicalQuery> cq,
-                                                        const Collection* collection,
-                                                        YieldPolicy yieldPolicy) {
+StatusWith<unique_ptr<PlanExecutor, PlanExecutor::Deleter>> PlanExecutor::make(
+    OperationContext* opCtx,
+    unique_ptr<WorkingSet> ws,
+    unique_ptr<PlanStage> rt,
+    unique_ptr<QuerySolution> qs,
+    unique_ptr<CanonicalQuery> cq,
+    const Collection* collection,
+    YieldPolicy yieldPolicy) {
     return PlanExecutor::make(opCtx,
                               std::move(ws),
                               std::move(rt),
                               std::move(qs),
                               std::move(cq),
                               collection,
-                              "",
+                              {},
                               yieldPolicy);
 }
 
 // static
-StatusWith<unique_ptr<PlanExecutor>> PlanExecutor::make(OperationContext* txn,
-                                                        unique_ptr<WorkingSet> ws,
-                                                        unique_ptr<PlanStage> rt,
-                                                        unique_ptr<QuerySolution> qs,
-                                                        unique_ptr<CanonicalQuery> cq,
-                                                        const Collection* collection,
-                                                        const string& ns,
-                                                        YieldPolicy yieldPolicy) {
-    unique_ptr<PlanExecutor> exec(new PlanExecutor(
-        txn, std::move(ws), std::move(rt), std::move(qs), std::move(cq), collection, ns));
+StatusWith<unique_ptr<PlanExecutor, PlanExecutor::Deleter>> PlanExecutor::make(
+    OperationContext* opCtx,
+    unique_ptr<WorkingSet> ws,
+    unique_ptr<PlanStage> rt,
+    unique_ptr<QuerySolution> qs,
+    unique_ptr<CanonicalQuery> cq,
+    const Collection* collection,
+    NamespaceString nss,
+    YieldPolicy yieldPolicy) {
+
+    std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> exec(
+        new PlanExecutor(opCtx,
+                         std::move(ws),
+                         std::move(rt),
+                         std::move(qs),
+                         std::move(cq),
+                         collection,
+                         std::move(nss),
+                         yieldPolicy),
+        PlanExecutor::Deleter(opCtx, collection));
 
     // Perform plan selection, if necessary.
-    Status status = exec->pickBestPlan(yieldPolicy, collection);
+    Status status = exec->pickBestPlan(collection);
     if (!status.isOK()) {
         return status;
     }
@@ -160,33 +215,34 @@ PlanExecutor::PlanExecutor(OperationContext* opCtx,
                            unique_ptr<QuerySolution> qs,
                            unique_ptr<CanonicalQuery> cq,
                            const Collection* collection,
-                           const string& ns)
+                           NamespaceString nss,
+                           YieldPolicy yieldPolicy)
     : _opCtx(opCtx),
       _cq(std::move(cq)),
       _workingSet(std::move(ws)),
       _qs(std::move(qs)),
       _root(std::move(rt)),
-      _ns(ns),
-      _yieldPolicy(new PlanYieldPolicy(this, YIELD_MANUAL)) {
-    // We may still need to initialize _ns from either collection or _cq.
-    if (!_ns.empty()) {
-        // We already have an _ns set, so there's nothing more to do.
-        return;
+      _nss(std::move(nss)),
+      // There's no point in yielding if the collection doesn't exist.
+      _yieldPolicy(makeYieldPolicy(this, collection ? yieldPolicy : NO_YIELD)) {
+    // We may still need to initialize _nss from either collection or _cq.
+    if (!_nss.isEmpty()) {
+        return;  // We already have an _nss set, so there's nothing more to do.
     }
 
     if (collection) {
-        _ns = collection->ns().ns();
+        _nss = collection->ns();
+        if (_yieldPolicy->canReleaseLocksDuringExecution()) {
+            _registrationToken = collection->getCursorManager()->registerExecutor(this);
+        }
     } else {
         invariant(_cq);
-        _ns = _cq->getQueryRequest().ns();
+        _nss = _cq->getQueryRequest().nss();
     }
 }
 
-Status PlanExecutor::pickBestPlan(YieldPolicy policy, const Collection* collection) {
+Status PlanExecutor::pickBestPlan(const Collection* collection) {
     invariant(_currentState == kUsable);
-    // For YIELD_AUTO, this will both set an auto yield policy on the PlanExecutor and
-    // register it to receive notifications.
-    this->setYieldPolicy(policy, collection);
 
     // First check if we need to do subplanning.
     PlanStage* foundStage = getStageByType(_root.get(), STAGE_SUBPLAN);
@@ -216,7 +272,9 @@ Status PlanExecutor::pickBestPlan(YieldPolicy policy, const Collection* collecti
     return Status::OK();
 }
 
-PlanExecutor::~PlanExecutor() {}
+PlanExecutor::~PlanExecutor() {
+    invariant(_currentState == kDisposed);
+}
 
 // static
 string PlanExecutor::statestr(ExecState s) {
@@ -285,17 +343,17 @@ void PlanExecutor::saveState() {
     // boundaries.
     WorkingSetCommon::prepareForSnapshotChange(_workingSet.get());
 
-    if (!killed()) {
+    if (!isMarkedAsKilled()) {
         _root->saveState();
     }
     _currentState = kSaved;
 }
 
-bool PlanExecutor::restoreState() {
+Status PlanExecutor::restoreState() {
     try {
         return restoreStateWithoutRetrying();
-    } catch (const WriteConflictException& wce) {
-        if (!_yieldPolicy->allowedToYield())
+    } catch (const WriteConflictException&) {
+        if (!_yieldPolicy->canAutoYield())
             throw;
 
         // Handles retries by calling restoreStateWithoutRetrying() in a loop.
@@ -303,15 +361,17 @@ bool PlanExecutor::restoreState() {
     }
 }
 
-bool PlanExecutor::restoreStateWithoutRetrying() {
+Status PlanExecutor::restoreStateWithoutRetrying() {
     invariant(_currentState == kSaved);
 
-    if (!killed()) {
+    if (!isMarkedAsKilled()) {
         _root->restoreState();
     }
 
     _currentState = kUsable;
-    return !killed();
+    return isMarkedAsKilled()
+        ? Status{ErrorCodes::QueryPlanKilled, "query killed during yield: " + *_killReason}
+        : Status::OK();
 }
 
 void PlanExecutor::detachFromOperationContext() {
@@ -322,21 +382,21 @@ void PlanExecutor::detachFromOperationContext() {
     _everDetachedFromOperationContext = true;
 }
 
-void PlanExecutor::reattachToOperationContext(OperationContext* txn) {
+void PlanExecutor::reattachToOperationContext(OperationContext* opCtx) {
     invariant(_currentState == kDetached);
 
     // We're reattaching for a getMore now.  Reset the yield timer in order to prevent from
     // yielding again right away.
     _yieldPolicy->resetTimer();
 
-    _opCtx = txn;
-    _root->reattachToOperationContext(txn);
+    _opCtx = opCtx;
+    _root->reattachToOperationContext(opCtx);
     _currentState = kSaved;
 }
 
-void PlanExecutor::invalidate(OperationContext* txn, const RecordId& dl, InvalidationType type) {
-    if (!killed()) {
-        _root->invalidate(txn, dl, type);
+void PlanExecutor::invalidate(OperationContext* opCtx, const RecordId& dl, InvalidationType type) {
+    if (!isMarkedAsKilled()) {
+        _root->invalidate(opCtx, dl, type);
     }
 }
 
@@ -358,18 +418,87 @@ PlanExecutor::ExecState PlanExecutor::getNextSnapshotted(Snapshotted<BSONObj>* o
     return getNextImpl(objOut, dlOut);
 }
 
-PlanExecutor::ExecState PlanExecutor::getNextImpl(Snapshotted<BSONObj>* objOut, RecordId* dlOut) {
-    MONGO_FAIL_POINT_BLOCK(planExecutorAlwaysDead, customKill) {
-        const BSONObj& data = customKill.getData();
-        BSONElement customKillNS = data["namespace"];
-        if (!customKillNS || _ns == customKillNS.str()) {
-            deregisterExec();
-            kill("hit planExecutorAlwaysDead fail point");
+
+bool PlanExecutor::shouldWaitForInserts() {
+    // If this is an awaitData-respecting operation and we have time left and we're not interrupted,
+    // we should wait for inserts.
+    if (_cq && _cq->getQueryRequest().isTailableAndAwaitData() &&
+        awaitDataState(_opCtx).shouldWaitForInserts && _opCtx->checkForInterruptNoAssert().isOK() &&
+        awaitDataState(_opCtx).waitForInsertsDeadline >
+            _opCtx->getServiceContext()->getPreciseClockSource()->now()) {
+        // We expect awaitData cursors to be yielding.
+        invariant(_yieldPolicy->canReleaseLocksDuringExecution());
+
+        // For operations with a last committed opTime, we should not wait if the replication
+        // coordinator's lastCommittedOpTime has changed.
+        if (!clientsLastKnownCommittedOpTime(_opCtx).isNull()) {
+            auto replCoord = repl::ReplicationCoordinator::get(_opCtx);
+            return clientsLastKnownCommittedOpTime(_opCtx) == replCoord->getLastCommittedOpTime();
         }
+        return true;
+    }
+    return false;
+}
+
+std::shared_ptr<CappedInsertNotifier> PlanExecutor::getCappedInsertNotifier() {
+    // We don't expect to need a capped insert notifier for non-yielding plans.
+    invariant(_yieldPolicy->canReleaseLocksDuringExecution());
+
+    // We can only wait if we have a collection; otherwise we should retry immediately when
+    // we hit EOF.
+    dassert(_opCtx->lockState()->isCollectionLockedForMode(_nss.ns(), MODE_IS));
+    auto db = dbHolder().get(_opCtx, _nss.db());
+    invariant(db);
+    auto collection = db->getCollection(_opCtx, _nss);
+    invariant(collection);
+
+    return collection->getCappedInsertNotifier();
+}
+
+PlanExecutor::ExecState PlanExecutor::waitForInserts(CappedInsertNotifierData* notifierData,
+                                                     Snapshotted<BSONObj>* errorObj) {
+    invariant(notifierData->notifier);
+
+    // The notifier wait() method will not wait unless the version passed to it matches the
+    // current version of the notifier.  Since the version passed to it is the current version
+    // of the notifier at the time of the previous EOF, we require two EOFs in a row with no
+    // notifier version change in order to wait.  This is sufficient to ensure we never wait
+    // when data is available.
+    auto curOp = CurOp::get(_opCtx);
+    curOp->pauseTimer();
+    ON_BLOCK_EXIT([curOp] { curOp->resumeTimer(); });
+    auto opCtx = _opCtx;
+    uint64_t currentNotifierVersion = notifierData->notifier->getVersion();
+    auto yieldResult = _yieldPolicy->yield(nullptr, [opCtx, notifierData] {
+        const auto deadline = awaitDataState(opCtx).waitForInsertsDeadline;
+        notifierData->notifier->waitUntil(notifierData->lastEOFVersion, deadline);
+    });
+    notifierData->lastEOFVersion = currentNotifierVersion;
+
+    if (yieldResult.isOK()) {
+        // There may be more results, try to get more data.
+        return ADVANCED;
+    }
+
+    if (errorObj) {
+        *errorObj = Snapshotted<BSONObj>(SnapshotId(),
+                                         WorkingSetCommon::buildMemberStatusObject(yieldResult));
+    }
+    return DEAD;
+}
+
+PlanExecutor::ExecState PlanExecutor::getNextImpl(Snapshotted<BSONObj>* objOut, RecordId* dlOut) {
+    if (MONGO_FAIL_POINT(planExecutorAlwaysFails)) {
+        Status status(ErrorCodes::OperationFailed,
+                      str::stream() << "PlanExecutor hit planExecutorAlwaysFails fail point");
+        *objOut =
+            Snapshotted<BSONObj>(SnapshotId(), WorkingSetCommon::buildMemberStatusObject(status));
+
+        return PlanExecutor::FAILURE;
     }
 
     invariant(_currentState == kUsable);
-    if (killed()) {
+    if (isMarkedAsKilled()) {
         if (NULL != objOut) {
             Status status(ErrorCodes::OperationFailed,
                           str::stream() << "Operation aborted because: " << *_killReason);
@@ -395,6 +524,13 @@ PlanExecutor::ExecState PlanExecutor::getNextImpl(Snapshotted<BSONObj>* objOut,
     // Incremented on every writeConflict, reset to 0 on any successful call to _root->work.
     size_t writeConflictsInARow = 0;
 
+    // Capped insert data; declared outside the loop so we hold a shared pointer to the capped
+    // insert notifier the entire time we are in the loop.  Holding a shared pointer to the capped
+    // insert notifier is necessary for the notifierVersion to advance.
+    CappedInsertNotifierData cappedInsertNotifierData;
+    if (shouldWaitForInserts()) {
+        cappedInsertNotifierData.notifier = getCappedInsertNotifier();
+    }
     for (;;) {
         // These are the conditions which can cause us to yield:
         //   1) The yield policy's timer elapsed, or
@@ -402,16 +538,11 @@ PlanExecutor::ExecState PlanExecutor::getNextImpl(Snapshotted<BSONObj>* objOut,
         //   3) we need to yield and retry due to a WriteConflictException.
         // In all cases, the actual yielding happens here.
         if (_yieldPolicy->shouldYield()) {
-            if (!_yieldPolicy->yield(fetcher.get())) {
-                // A return of false from a yield should only happen if we've been killed during the
-                // yield.
-                invariant(killed());
-
-                if (NULL != objOut) {
-                    Status status(ErrorCodes::OperationFailed,
-                                  str::stream() << "Operation aborted because: " << *_killReason);
+            auto yieldStatus = _yieldPolicy->yield(fetcher.get());
+            if (!yieldStatus.isOK()) {
+                if (objOut) {
                     *objOut = Snapshotted<BSONObj>(
-                        SnapshotId(), WorkingSetCommon::buildMemberStatusObject(status));
+                        SnapshotId(), WorkingSetCommon::buildMemberStatusObject(yieldStatus));
                 }
                 return PlanExecutor::DEAD;
             }
@@ -465,11 +596,12 @@ PlanExecutor::ExecState PlanExecutor::getNextImpl(Snapshotted<BSONObj>* objOut,
             // This result didn't have the data the caller wanted, try again.
         } else if (PlanStage::NEED_YIELD == code) {
             if (id == WorkingSet::INVALID_ID) {
-                if (!_yieldPolicy->allowedToYield())
+                if (!_yieldPolicy->canAutoYield())
                     throw WriteConflictException();
                 CurOp::get(_opCtx)->debug().writeConflicts++;
                 writeConflictsInARow++;
-                WriteConflictException::logAndBackoff(writeConflictsInARow, "plan execution", _ns);
+                WriteConflictException::logAndBackoff(
+                    writeConflictsInARow, "plan execution", _nss.ns());
 
             } else {
                 WorkingSetMember* member = _workingSet->get(id);
@@ -480,12 +612,20 @@ PlanExecutor::ExecState PlanExecutor::getNextImpl(Snapshotted<BSONObj>* objOut,
             }
 
             // If we're allowed to, we will yield next time through the loop.
-            if (_yieldPolicy->allowedToYield())
+            if (_yieldPolicy->canAutoYield())
                 _yieldPolicy->forceYield();
         } else if (PlanStage::NEED_TIME == code) {
             // Fall through to yield check at end of large conditional.
         } else if (PlanStage::IS_EOF == code) {
-            return PlanExecutor::IS_EOF;
+            if (!shouldWaitForInserts()) {
+                return PlanExecutor::IS_EOF;
+            }
+            const ExecState waitResult = waitForInserts(&cappedInsertNotifierData, objOut);
+            if (waitResult == PlanExecutor::ADVANCED) {
+                // There may be more results, keep going.
+                continue;
+            }
+            return waitResult;
         } else {
             invariant(PlanStage::DEAD == code || PlanStage::FAILURE == code);
 
@@ -502,25 +642,31 @@ PlanExecutor::ExecState PlanExecutor::getNextImpl(Snapshotted<BSONObj>* objOut,
 
 bool PlanExecutor::isEOF() {
     invariant(_currentState == kUsable);
-    return killed() || (_stash.empty() && _root->isEOF());
+    return isMarkedAsKilled() || (_stash.empty() && _root->isEOF());
 }
 
-void PlanExecutor::registerExec(const Collection* collection) {
-    // There's no need to register a PlanExecutor for which the underlying collection
-    // doesn't exist.
-    if (collection) {
-        _safety.reset(new ScopedExecutorRegistration(this, collection));
-    }
-}
-
-void PlanExecutor::deregisterExec() {
-    _safety.reset();
-}
-
-void PlanExecutor::kill(string reason) {
+void PlanExecutor::markAsKilled(string reason) {
     _killReason = std::move(reason);
 }
 
+void PlanExecutor::dispose(OperationContext* opCtx, CursorManager* cursorManager) {
+    if (_currentState == kDisposed) {
+        return;
+    }
+
+    // If we are registered with the CursorManager we need to be sure to deregister ourselves.
+    // However, if we have been killed we should not attempt to deregister ourselves, since the
+    // caller of markAsKilled() will have done that already, and the CursorManager may no longer
+    // exist. Note that the caller's collection lock prevents us from being marked as killed during
+    // this method, since any interruption event requires a lock in at least MODE_IX.
+    if (cursorManager && _registrationToken && !isMarkedAsKilled()) {
+        dassert(opCtx->lockState()->isCollectionLockedForMode(_nss.ns(), MODE_IS));
+        cursorManager->deregisterExecutor(this);
+    }
+    _root->dispose(opCtx);
+    _currentState = kDisposed;
+}
+
 Status PlanExecutor::executePlan() {
     invariant(_currentState == kUsable);
     BSONObj obj;
@@ -530,71 +676,52 @@ Status PlanExecutor::executePlan() {
     }
 
     if (PlanExecutor::DEAD == state || PlanExecutor::FAILURE == state) {
-        if (killed()) {
+        if (isMarkedAsKilled()) {
             return Status(ErrorCodes::QueryPlanKilled,
                           str::stream() << "Operation aborted because: " << *_killReason);
         }
 
-        return Status(ErrorCodes::OperationFailed,
-                      str::stream() << "Exec error: " << WorkingSetCommon::toStatusString(obj)
-                                    << ", state: "
-                                    << PlanExecutor::statestr(state));
+        auto errorStatus = WorkingSetCommon::getMemberObjectStatus(obj);
+        invariant(!errorStatus.isOK());
+        return errorStatus.withContext(str::stream() << "Exec error resulting in state "
+                                                     << PlanExecutor::statestr(state));
     }
 
-    invariant(!killed());
+    invariant(!isMarkedAsKilled());
     invariant(PlanExecutor::IS_EOF == state);
     return Status::OK();
 }
 
-const string& PlanExecutor::ns() {
-    return _ns;
-}
-
-void PlanExecutor::setYieldPolicy(YieldPolicy policy,
-                                  const Collection* collection,
-                                  bool registerExecutor) {
-    if (!collection) {
-        // If the collection doesn't exist, then there's no need to yield at all.
-        invariant(!_yieldPolicy->allowedToYield());
-        return;
-    }
-
-    _yieldPolicy->setPolicy(policy);
-    if (PlanExecutor::YIELD_AUTO == policy) {
-        // Runners that yield automatically generally need to be registered so that
-        // after yielding, they receive notifications of events like deletions and
-        // index drops. The only exception is that a few PlanExecutors get registered
-        // by ClientCursor instead of being registered here. This is unneeded if we only do
-        // partial "yields" for WriteConflict retrying.
-        if (registerExecutor) {
-            this->registerExec(collection);
-        }
-    }
-}
 
 void PlanExecutor::enqueue(const BSONObj& obj) {
     _stash.push(obj.getOwned());
 }
 
-//
-// ScopedExecutorRegistration
-//
-
-// PlanExecutor::ScopedExecutorRegistration
-PlanExecutor::ScopedExecutorRegistration::ScopedExecutorRegistration(PlanExecutor* exec,
-                                                                     const Collection* collection)
-    : _exec(exec), _collection(collection) {
-    invariant(_collection);
-    _collection->getCursorManager()->registerExecutor(_exec);
+Timestamp PlanExecutor::getLatestOplogTimestamp() {
+    if (auto pipelineProxy = getStageByType(_root.get(), STAGE_PIPELINE_PROXY))
+        return static_cast<PipelineProxyStage*>(pipelineProxy)->getLatestOplogTimestamp();
+    if (auto collectionScan = getStageByType(_root.get(), STAGE_COLLSCAN))
+        return static_cast<CollectionScan*>(collectionScan)->getLatestOplogTimestamp();
+    return Timestamp();
 }
 
-PlanExecutor::ScopedExecutorRegistration::~ScopedExecutorRegistration() {
-    if (_exec->killed()) {
-        // If the plan executor has been killed, then it's possible that the collection
-        // no longer exists.
-        return;
+//
+// PlanExecutor::Deleter
+//
+
+PlanExecutor::Deleter::Deleter(OperationContext* opCtx, const Collection* collection)
+    : _opCtx(opCtx), _cursorManager(collection ? collection->getCursorManager() : nullptr) {}
+
+void PlanExecutor::Deleter::operator()(PlanExecutor* execPtr) {
+    try {
+        invariant(_opCtx);  // It is illegal to invoke operator() on a default constructed Deleter.
+        if (!_dismissed) {
+            execPtr->dispose(_opCtx, _cursorManager);
+        }
+        delete execPtr;
+    } catch (...) {
+        std::terminate();
     }
-    _collection->getCursorManager()->deregisterExecutor(_exec);
 }
 
 }  // namespace mongo
