@@ -32,10 +32,14 @@
 #include <boost/optional.hpp>
 #include <boost/optional/optional_io.hpp>
 
+#include "mongo/db/catalog/collection_mock.h"
+#include "mongo/db/catalog/uuid_catalog.h"
+#include "mongo/db/dbmessage.h"
 #include "mongo/db/json.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/pipeline/aggregation_request.h"
 #include "mongo/db/query/query_request.h"
+#include "mongo/db/service_context_noop.h"
 #include "mongo/unittest/unittest.h"
 
 namespace mongo {
@@ -443,7 +447,7 @@ TEST(QueryRequestTest, ParseFromCommandAllFlagsTrue) {
     ASSERT(!qr->isSlaveOk());
     ASSERT(qr->isOplogReplay());
     ASSERT(qr->isNoCursorTimeout());
-    ASSERT(qr->isAwaitData());
+    ASSERT(qr->isTailableAndAwaitData());
     ASSERT(qr->isAllowPartialResults());
 }
 
@@ -473,6 +477,7 @@ TEST(QueryRequestTest, ParseFromCommandAllNonOptionFields) {
         "projection: {c: 1},"
         "hint: {d: 1},"
         "readConcern: {e: 1},"
+        "$queryOptions: {$readPreference: 'secondary'},"
         "collation: {f: 1},"
         "limit: 3,"
         "skip: 5,"
@@ -494,6 +499,9 @@ TEST(QueryRequestTest, ParseFromCommandAllNonOptionFields) {
     ASSERT_EQUALS(0, expectedHint.woCompare(qr->getHint()));
     BSONObj expectedReadConcern = BSON("e" << 1);
     ASSERT_EQUALS(0, expectedReadConcern.woCompare(qr->getReadConcern()));
+    BSONObj expectedUnwrappedReadPref = BSON("$readPreference"
+                                             << "secondary");
+    ASSERT_EQUALS(0, expectedUnwrappedReadPref.woCompare(qr->getUnwrappedReadPref()));
     BSONObj expectedCollation = BSON("f" << 1);
     ASSERT_EQUALS(0, expectedCollation.woCompare(qr->getCollation()));
     ASSERT_EQUALS(3, *qr->getLimit());
@@ -589,6 +597,7 @@ TEST(QueryRequestTest, ParseFromCommandSkipWrongType) {
     ASSERT_NOT_OK(result.getStatus());
 }
 
+
 TEST(QueryRequestTest, ParseFromCommandLimitWrongType) {
     BSONObj cmdObj = fromjson(
         "{find: 'testns',"
@@ -624,6 +633,17 @@ TEST(QueryRequestTest, ParseFromCommandCommentWrongType) {
     ASSERT_NOT_OK(result.getStatus());
 }
 
+TEST(QueryRequestTest, ParseFromCommandUnwrappedReadPrefWrongType) {
+    BSONObj cmdObj = fromjson(
+        "{find: 'testns',"
+        "filter:  {a: 1},"
+        "$queryOptions: 1}");
+    const NamespaceString nss("test.testns");
+    bool isExplain = false;
+    auto result = QueryRequest::makeFromFindCommand(nss, cmdObj, isExplain);
+    ASSERT_NOT_OK(result.getStatus());
+}
+
 TEST(QueryRequestTest, ParseFromCommandMaxScanWrongType) {
     BSONObj cmdObj = fromjson(
         "{find: 'testns',"
@@ -1030,7 +1050,7 @@ TEST(QueryRequestTest, DefaultQueryParametersCorrect) {
     ASSERT_EQUALS(false, qr->isSlaveOk());
     ASSERT_EQUALS(false, qr->isOplogReplay());
     ASSERT_EQUALS(false, qr->isNoCursorTimeout());
-    ASSERT_EQUALS(false, qr->isAwaitData());
+    ASSERT_EQUALS(false, qr->isTailableAndAwaitData());
     ASSERT_EQUALS(false, qr->isExhaust());
     ASSERT_EQUALS(false, qr->isAllowPartialResults());
 }
@@ -1102,14 +1122,14 @@ TEST(QueryRequestTest, ConvertToAggregationSucceeds) {
 
     auto ar = AggregationRequest::parseFromBSON(testns, agg.getValue());
     ASSERT_OK(ar.getStatus());
-    ASSERT(!ar.getValue().isExplain());
+    ASSERT(!ar.getValue().getExplain());
     ASSERT(ar.getValue().getPipeline().empty());
-    ASSERT(ar.getValue().isCursorCommand());
+    ASSERT_EQ(ar.getValue().getBatchSize(), AggregationRequest::kDefaultBatchSize);
     ASSERT_EQ(ar.getValue().getNamespaceString(), testns);
     ASSERT_BSONOBJ_EQ(ar.getValue().getCollation(), BSONObj());
 }
 
-TEST(QueryRequestTest, ConvertToAggregationWithExplainSucceeds) {
+TEST(QueryRequestTest, ConvertToAggregationOmitsExplain) {
     QueryRequest qr(testns);
     qr.setExplain(true);
     auto agg = qr.asAggregationCommand();
@@ -1117,13 +1137,23 @@ TEST(QueryRequestTest, ConvertToAggregationWithExplainSucceeds) {
 
     auto ar = AggregationRequest::parseFromBSON(testns, agg.getValue());
     ASSERT_OK(ar.getStatus());
-    ASSERT(ar.getValue().isExplain());
+    ASSERT_FALSE(ar.getValue().getExplain());
     ASSERT(ar.getValue().getPipeline().empty());
-    ASSERT(ar.getValue().isCursorCommand());
     ASSERT_EQ(ar.getValue().getNamespaceString(), testns);
     ASSERT_BSONOBJ_EQ(ar.getValue().getCollation(), BSONObj());
 }
 
+TEST(QueryRequestTest, ConvertToAggregationWithHintSucceeds) {
+    QueryRequest qr(testns);
+    qr.setHint(fromjson("{a_1: -1}"));
+    const auto aggCmd = qr.asAggregationCommand();
+    ASSERT_OK(aggCmd);
+
+    auto ar = AggregationRequest::parseFromBSON(testns, aggCmd.getValue());
+    ASSERT_OK(ar.getStatus());
+    ASSERT_BSONOBJ_EQ(qr.getHint(), ar.getValue().getHint());
+}
+
 TEST(QueryRequestTest, ConvertToAggregationWithMinFails) {
     QueryRequest qr(testns);
     qr.setMin(fromjson("{a: 1}"));
@@ -1168,16 +1198,15 @@ TEST(QueryRequestTest, ConvertToAggregationWithReturnKeyFails) {
     ASSERT_NOT_OK(qr.asAggregationCommand());
 }
 
-TEST(QueryRequestTest, ConvertToAggregationWithHintFails) {
-    QueryRequest qr(testns);
-    qr.setHint(fromjson("{a_1: -1}"));
-    ASSERT_NOT_OK(qr.asAggregationCommand());
-}
-
-TEST(QueryRequestTest, ConvertToAggregationWithCommentFails) {
+TEST(QueryRequestTest, ConvertToAggregationWithCommentSucceeds) {
     QueryRequest qr(testns);
     qr.setComment("test");
-    ASSERT_NOT_OK(qr.asAggregationCommand());
+    const auto aggCmd = qr.asAggregationCommand();
+    ASSERT_OK(aggCmd);
+
+    auto ar = AggregationRequest::parseFromBSON(testns, aggCmd.getValue());
+    ASSERT_OK(ar.getStatus());
+    ASSERT_EQ(qr.getComment(), ar.getValue().getComment());
 }
 
 TEST(QueryRequestTest, ConvertToAggregationWithShowRecordIdFails) {
@@ -1194,7 +1223,7 @@ TEST(QueryRequestTest, ConvertToAggregationWithSnapshotFails) {
 
 TEST(QueryRequestTest, ConvertToAggregationWithTailableFails) {
     QueryRequest qr(testns);
-    qr.setTailable(true);
+    qr.setTailableMode(TailableMode::kTailable);
     ASSERT_NOT_OK(qr.asAggregationCommand());
 }
 
@@ -1212,7 +1241,7 @@ TEST(QueryRequestTest, ConvertToAggregationWithNoCursorTimeoutFails) {
 
 TEST(QueryRequestTest, ConvertToAggregationWithAwaitDataFails) {
     QueryRequest qr(testns);
-    qr.setAwaitData(true);
+    qr.setTailableMode(TailableMode::kTailableAndAwaitData);
     ASSERT_NOT_OK(qr.asAggregationCommand());
 }
 
@@ -1241,8 +1270,8 @@ TEST(QueryRequestTest, ConvertToAggregationWithPipeline) {
 
     auto ar = AggregationRequest::parseFromBSON(testns, agg.getValue());
     ASSERT_OK(ar.getStatus());
-    ASSERT(!ar.getValue().isExplain());
-    ASSERT(ar.getValue().isCursorCommand());
+    ASSERT(!ar.getValue().getExplain());
+    ASSERT_EQ(ar.getValue().getBatchSize(), AggregationRequest::kDefaultBatchSize);
     ASSERT_EQ(ar.getValue().getNamespaceString(), testns);
     ASSERT_BSONOBJ_EQ(ar.getValue().getCollation(), BSONObj());
 
@@ -1266,8 +1295,7 @@ TEST(QueryRequestTest, ConvertToAggregationWithBatchSize) {
 
     auto ar = AggregationRequest::parseFromBSON(testns, agg.getValue());
     ASSERT_OK(ar.getStatus());
-    ASSERT(ar.getValue().isCursorCommand());
-    ASSERT(!ar.getValue().isExplain());
+    ASSERT(!ar.getValue().getExplain());
     ASSERT_EQ(ar.getValue().getNamespaceString(), testns);
     ASSERT_EQ(ar.getValue().getBatchSize(), 4LL);
     ASSERT_BSONOBJ_EQ(ar.getValue().getCollation(), BSONObj());
@@ -1285,8 +1313,8 @@ TEST(QueryRequestTest, ConvertToAggregationWithMaxTimeMS) {
 
     auto ar = AggregationRequest::parseFromBSON(testns, cmdObj);
     ASSERT_OK(ar.getStatus());
-    ASSERT(!ar.getValue().isExplain());
-    ASSERT(ar.getValue().isCursorCommand());
+    ASSERT(!ar.getValue().getExplain());
+    ASSERT_EQ(ar.getValue().getBatchSize(), AggregationRequest::kDefaultBatchSize);
     ASSERT_EQ(ar.getValue().getNamespaceString(), testns);
     ASSERT_BSONOBJ_EQ(ar.getValue().getCollation(), BSONObj());
 }
@@ -1299,12 +1327,116 @@ TEST(QueryRequestTest, ConvertToAggregationWithCollationSucceeds) {
 
     auto ar = AggregationRequest::parseFromBSON(testns, agg.getValue());
     ASSERT_OK(ar.getStatus());
-    ASSERT(!ar.getValue().isExplain());
+    ASSERT(!ar.getValue().getExplain());
     ASSERT(ar.getValue().getPipeline().empty());
-    ASSERT(ar.getValue().isCursorCommand());
+    ASSERT_EQ(ar.getValue().getBatchSize(), AggregationRequest::kDefaultBatchSize);
     ASSERT_EQ(ar.getValue().getNamespaceString(), testns);
     ASSERT_BSONOBJ_EQ(ar.getValue().getCollation(), BSON("f" << 1));
 }
 
+TEST(QueryRequestTest, ParseFromLegacyObjMetaOpComment) {
+    BSONObj queryObj = fromjson(
+        "{$query: {a: 1},"
+        "$comment: {b: 2, c: {d: 'ParseFromLegacyObjMetaOpComment'}}}");
+    const NamespaceString nss("test.testns");
+    unique_ptr<QueryRequest> qr(
+        assertGet(QueryRequest::fromLegacyQuery(nss, queryObj, BSONObj(), 0, 0, 0)));
+
+    // Ensure that legacy comment meta-operator is parsed to a string comment
+    ASSERT_EQ(qr->getComment(), "{ b: 2, c: { d: \"ParseFromLegacyObjMetaOpComment\" } }");
+    ASSERT_BSONOBJ_EQ(qr->getFilter(), fromjson("{a: 1}"));
+}
+
+TEST(QueryRequestTest, ParseFromLegacyStringMetaOpComment) {
+    BSONObj queryObj = fromjson(
+        "{$query: {a: 1},"
+        "$comment: 'ParseFromLegacyStringMetaOpComment'}");
+    const NamespaceString nss("test.testns");
+    unique_ptr<QueryRequest> qr(
+        assertGet(QueryRequest::fromLegacyQuery(nss, queryObj, BSONObj(), 0, 0, 0)));
+
+    ASSERT_EQ(qr->getComment(), "ParseFromLegacyStringMetaOpComment");
+    ASSERT_BSONOBJ_EQ(qr->getFilter(), fromjson("{a: 1}"));
+}
+
+TEST(QueryRequestTest, ParseFromLegacyQuery) {
+    const auto kSkip = 1;
+    const auto kNToReturn = 2;
+
+    BSONObj queryObj = fromjson(R"({
+            query: {query: 1},
+            orderby: {sort: 1},
+            $hint: {hint: 1},
+            $explain: false,
+            $min: {x: 'min'},
+            $max: {x: 'max'},
+            $maxScan: 7
+         })");
+    const NamespaceString nss("test.testns");
+    unique_ptr<QueryRequest> qr(assertGet(QueryRequest::fromLegacyQuery(
+        nss, queryObj, BSON("proj" << 1), kSkip, kNToReturn, QueryOption_Exhaust)));
+
+    ASSERT_EQ(qr->nss(), nss);
+    ASSERT_BSONOBJ_EQ(qr->getFilter(), fromjson("{query: 1}"));
+    ASSERT_BSONOBJ_EQ(qr->getProj(), fromjson("{proj: 1}"));
+    ASSERT_BSONOBJ_EQ(qr->getSort(), fromjson("{sort: 1}"));
+    ASSERT_BSONOBJ_EQ(qr->getHint(), fromjson("{hint: 1}"));
+    ASSERT_BSONOBJ_EQ(qr->getMin(), fromjson("{x: 'min'}"));
+    ASSERT_BSONOBJ_EQ(qr->getMax(), fromjson("{x: 'max'}"));
+    ASSERT_EQ(qr->getSkip(), boost::optional<long long>(kSkip));
+    ASSERT_EQ(qr->getNToReturn(), boost::optional<long long>(kNToReturn));
+    ASSERT_EQ(qr->wantMore(), true);
+    ASSERT_EQ(qr->isExplain(), false);
+    ASSERT_EQ(qr->getMaxScan(), 7);
+    ASSERT_EQ(qr->isSlaveOk(), false);
+    ASSERT_EQ(qr->isOplogReplay(), false);
+    ASSERT_EQ(qr->isNoCursorTimeout(), false);
+    ASSERT_EQ(qr->isTailable(), false);
+    ASSERT_EQ(qr->isExhaust(), true);
+    ASSERT_EQ(qr->isAllowPartialResults(), false);
+    ASSERT_EQ(qr->getOptions(), QueryOption_Exhaust);
+}
+
+TEST(QueryRequestTest, ParseFromLegacyQueryUnwrapped) {
+    BSONObj queryObj = fromjson(R"({
+            foo: 1
+         })");
+    const NamespaceString nss("test.testns");
+    unique_ptr<QueryRequest> qr(assertGet(
+        QueryRequest::fromLegacyQuery(nss, queryObj, BSONObj(), 0, 0, QueryOption_Exhaust)));
+
+    ASSERT_EQ(qr->nss(), nss);
+    ASSERT_BSONOBJ_EQ(qr->getFilter(), fromjson("{foo: 1}"));
+}
+
+TEST(QueryRequestTest, ParseFromLegacyQueryTooNegativeNToReturn) {
+    BSONObj queryObj = fromjson(R"({
+            foo: 1
+         })");
+    const NamespaceString nss("test.testns");
+
+    ASSERT_NOT_OK(
+        QueryRequest::fromLegacyQuery(
+            nss, queryObj, BSONObj(), 0, std::numeric_limits<int>::min(), QueryOption_Exhaust)
+            .getStatus());
+}
+
+TEST(QueryRequestTest, ParseFromUUID) {
+    ServiceContextNoop service;
+    auto client = service.makeClient("test");
+    auto opCtxNoop = client->makeOperationContext();
+    auto opCtx = opCtxNoop.get();
+    // Register a UUID/Collection pair in the UUIDCatalog.
+    const CollectionUUID uuid = UUID::gen();
+    const NamespaceString nss("test.testns");
+    Collection coll(stdx::make_unique<CollectionMock>(nss));
+    UUIDCatalog& catalog = UUIDCatalog::get(opCtx);
+    catalog.onCreateCollection(opCtx, &coll, uuid);
+    QueryRequest qr(uuid);
+    // Ensure a call to refreshNSS succeeds.
+    qr.refreshNSS(opCtx);
+    ASSERT_EQ(nss, qr.nss());
+}
+
 }  // namespace mongo
 }  // namespace
