@@ -36,7 +36,9 @@
 #include "mongo/db/repl/data_replicator_external_state.h"
 #include "mongo/db/repl/oplog_buffer.h"
 #include "mongo/db/repl/oplog_fetcher.h"
+#include "mongo/db/repl/oplog_interface_remote.h"
 #include "mongo/db/repl/optime.h"
+#include "mongo/db/repl/rollback_impl.h"
 #include "mongo/db/repl/sync_source_resolver.h"
 #include "mongo/stdx/condition_variable.h"
 #include "mongo/stdx/functional.h"
@@ -51,8 +53,11 @@ class OperationContext;
 
 namespace repl {
 
+class OplogInterface;
 class ReplicationCoordinator;
 class ReplicationCoordinatorExternalState;
+class ReplicationProcess;
+class StorageInterface;
 
 class BackgroundSync {
     MONGO_DISALLOW_COPYING(BackgroundSync);
@@ -73,6 +78,7 @@ public:
     enum class ProducerState { Starting, Running, Stopped };
 
     BackgroundSync(ReplicationCoordinatorExternalState* replicationCoordinatorExternalState,
+                   ReplicationProcess* replicationProcess,
                    std::unique_ptr<OplogBuffer> oplogBuffer);
 
     // stop syncing (when this node becomes a primary, e.g.)
@@ -84,17 +90,17 @@ public:
     /**
      * Starts oplog buffer, task executor and producer thread, in that order.
      */
-    void startup(OperationContext* txn);
+    void startup(OperationContext* opCtx);
 
     /**
      * Signals producer thread to stop.
      */
-    void shutdown(OperationContext* txn);
+    void shutdown(OperationContext* opCtx);
 
     /**
      * Waits for producer thread to stop before shutting down the task executor and oplog buffer.
      */
-    void join(OperationContext* txn);
+    void join(OperationContext* opCtx);
 
     /**
      * Returns true if shutdown() has been called.
@@ -109,8 +115,8 @@ public:
 
     // Interface implementation
 
-    bool peek(OperationContext* txn, BSONObj* op);
-    void consume(OperationContext* txn);
+    bool peek(OperationContext* opCtx, BSONObj* op);
+    void consume(OperationContext* opCtx);
     void clearSyncTarget();
     void waitForMore();
 
@@ -118,7 +124,7 @@ public:
     BSONObj getCounters();
 
     // Clears any fetched and buffered oplog entries.
-    void clearBuffer(OperationContext* txn);
+    void clearBuffer(OperationContext* opCtx);
 
     /**
      * Returns true if any of the following is true:
@@ -134,7 +140,7 @@ public:
     void startProducerIfStopped();
 
     // Adds a fake oplog entry to buffer. Used for testing only.
-    void pushTestOpToBuffer(OperationContext* txn, const BSONObj& op);
+    void pushTestOpToBuffer(OperationContext* opCtx, const BSONObj& op);
 
 private:
     bool _inShutdown_inlock() const;
@@ -160,10 +166,47 @@ private:
                              Fetcher::Documents::const_iterator end,
                              const OplogFetcher::DocumentsInfo& info);
 
-    // restart syncing
-    void start(OperationContext* txn);
+    /**
+     * Executes a rollback.
+     */
+    void _runRollback(OperationContext* opCtx,
+                      const Status& fetcherReturnStatus,
+                      const HostAndPort& source,
+                      int requiredRBID,
+                      StorageInterface* storageInterface);
 
-    OpTimeWithHash _readLastAppliedOpTimeWithHash(OperationContext* txn);
+    /**
+     * Executes a rollback with the recover to checkpoint algorithm. This is the default rollback
+     * algorithm.
+     */
+    void _runRollbackViaRecoverToCheckpoint(OperationContext* opCtx,
+                                            const HostAndPort& source,
+                                            OplogInterface* localOplog,
+                                            StorageInterface* storageInterface,
+                                            OplogInterfaceRemote::GetConnectionFn getConnection);
+
+    /**
+     * Executes a rollback via refetch in either rs_rollback.cpp or rs_rollback_no_uuid.cpp
+     *
+     * We fall back on the rollback via refetch algorithm when:
+     * 1)  the server parameter "rollbackMethod" is set to "rollbackViaRefetch" or
+     *     "rollbackViaRefetchNoUUID"; or
+     * 2)  the storage engine does not support "rollback to a checkpoint."
+     *
+     * Must be called from _runRollback() which ensures that all the conditions for entering
+     * rollback have been met.
+     */
+    void _fallBackOnRollbackViaRefetch(OperationContext* opCtx,
+                                       const HostAndPort& source,
+                                       int requiredRBID,
+                                       OplogInterface* localOplog,
+                                       bool useUUID,
+                                       OplogInterfaceRemote::GetConnectionFn getConnection);
+
+    // restart syncing
+    void start(OperationContext* opCtx);
+
+    OpTimeWithHash _readLastAppliedOpTimeWithHash(OperationContext* opCtx);
 
     // Production thread
     std::unique_ptr<OplogBuffer> _oplogBuffer;
@@ -174,6 +217,9 @@ private:
     // A pointer to the replication coordinator external state.
     ReplicationCoordinatorExternalState* _replicationCoordinatorExternalState;
 
+    // A pointer to the replication process.
+    ReplicationProcess* _replicationProcess;
+
     /**
       * All member variables are labeled with one of the following codes indicating the
       * synchronization rules for accessing them:
@@ -218,6 +264,11 @@ private:
 
     // Current oplog fetcher tailing the oplog on the sync source.
     std::unique_ptr<OplogFetcher> _oplogFetcher;
+
+    // Current rollback process. If this component is active, we are currently reverting local
+    // operations in the local oplog in order to bring this server to a consistent state relative
+    // to the sync source.
+    std::unique_ptr<RollbackImpl> _rollback;  // (PR)
 };
 
 
