@@ -41,16 +41,19 @@
 #include "mongo/db/commands/list_collections_filter.h"
 #include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/server_parameters.h"
-#include "mongo/rpc/metadata/server_selection_metadata.h"
 #include "mongo/stdx/functional.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/destructor_guard.h"
+#include "mongo/util/fail_point_service.h"
 #include "mongo/util/log.h"
 #include "mongo/util/mongoutils/str.h"
 
 namespace mongo {
 namespace repl {
 
+// Failpoint which causes the initial sync function to hang before running listCollections.
+MONGO_FP_DECLARE(initialSyncHangBeforeListCollections);
+
 namespace {
 
 using LockGuard = stdx::lock_guard<stdx::mutex>;
@@ -58,10 +61,23 @@ using UniqueLock = stdx::unique_lock<stdx::mutex>;
 
 const char* kNameFieldName = "name";
 const char* kOptionsFieldName = "options";
+const char* kInfoFieldName = "info";
+const char* kUUIDFieldName = "uuid";
+
+// The batchSize to use for the find/getMore queries called by the CollectionCloner
+constexpr int kUseARMDefaultBatchSize = -1;
+MONGO_EXPORT_STARTUP_SERVER_PARAMETER(collectionClonerBatchSize, int, kUseARMDefaultBatchSize);
 
 // The number of attempts for the listCollections commands.
 MONGO_EXPORT_SERVER_PARAMETER(numInitialSyncListCollectionsAttempts, int, 3);
 
+// The number of cursors to use in the collection cloning process.
+MONGO_EXPORT_SERVER_PARAMETER(maxNumInitialSyncCollectionClonerCursors, int, 1);
+
+// Failpoint which causes initial sync to hang right after listCollections, but before cloning
+// any colelctions in the 'database' database.
+MONGO_FP_DECLARE(initialSyncHangAfterListCollections);
+
 /**
  * Default listCollections predicate.
  */
@@ -113,11 +129,11 @@ DatabaseCloner::DatabaseCloner(executor::TaskExecutor* executor,
                                          stdx::placeholders::_1,
                                          stdx::placeholders::_2,
                                          stdx::placeholders::_3),
-                              rpc::ServerSelectionMetadata(true, boost::none).toBSON(),
+                              ReadPreferenceSetting::secondaryPreferredMetadata(),
                               RemoteCommandRequest::kNoTimeout /* find network timeout */,
                               RemoteCommandRequest::kNoTimeout /* getMore network timeout */,
                               RemoteCommandRetryScheduler::makeRetryPolicy(
-                                  numInitialSyncListCollectionsAttempts,
+                                  numInitialSyncListCollectionsAttempts.load(),
                                   executor::RemoteCommandRequest::kNoTimeout,
                                   RemoteCommandRetryScheduler::kAllRetriableErrors)),
       _startCollectionCloner([](CollectionCloner& cloner) { return cloner.startup(); }) {
@@ -141,24 +157,6 @@ const std::vector<BSONObj>& DatabaseCloner::getCollectionInfos_forTest() const {
     return _collectionInfos;
 }
 
-std::string DatabaseCloner::getDiagnosticString() const {
-    LockGuard lk(_mutex);
-    return _getDiagnosticString_inlock();
-}
-
-std::string DatabaseCloner::_getDiagnosticString_inlock() const {
-    str::stream output;
-    output << "DatabaseCloner";
-    output << " executor: " << _executor->getDiagnosticString();
-    output << " source: " << _source.toString();
-    output << " database: " << _dbname;
-    output << " listCollections filter" << _listCollectionsFilter;
-    output << " active: " << _isActive_inlock();
-    output << " collection info objects (empty if listCollections is in progress): "
-           << _collectionInfos.size();
-    return output;
-}
-
 bool DatabaseCloner::isActive() const {
     LockGuard lk(_mutex);
     return _isActive_inlock();
@@ -168,8 +166,13 @@ bool DatabaseCloner::_isActive_inlock() const {
     return State::kRunning == _state || State::kShuttingDown == _state;
 }
 
-Status DatabaseCloner::startup() noexcept {
+bool DatabaseCloner::_isShuttingDown() const {
     LockGuard lk(_mutex);
+    return State::kShuttingDown == _state;
+}
+
+Status DatabaseCloner::startup() noexcept {
+    UniqueLock lk(_mutex);
 
     switch (_state) {
         case State::kPreStart:
@@ -183,6 +186,20 @@ Status DatabaseCloner::startup() noexcept {
             return Status(ErrorCodes::ShutdownInProgress, "database cloner completed");
     }
 
+    MONGO_FAIL_POINT_BLOCK(initialSyncHangBeforeListCollections, customArgs) {
+        const auto& data = customArgs.getData();
+        const auto databaseElem = data["database"];
+        if (!databaseElem || databaseElem.checkAndGetStringData() == _dbname) {
+            lk.unlock();
+            log() << "initial sync - initialSyncHangBeforeListCollections fail point "
+                     "enabled. Blocking until fail point is disabled.";
+            while (MONGO_FAIL_POINT(initialSyncHangBeforeListCollections) && !_isShuttingDown()) {
+                mongo::sleepsecs(1);
+            }
+            lk.lock();
+        }
+    }
+
     _stats.start = _executor->now();
     LOG(1) << "Scheduling listCollections call for database: " << _dbname;
     Status scheduleResult = _listCollectionsFetcher.schedule();
@@ -289,6 +306,17 @@ void DatabaseCloner::_listCollectionsCallback(const StatusWith<Fetcher::QueryRes
         return;
     }
 
+    MONGO_FAIL_POINT_BLOCK(initialSyncHangAfterListCollections, options) {
+        const BSONObj& data = options.getData();
+        if (data["database"].String() == _dbname) {
+            log() << "initial sync - initialSyncHangAfterListCollections fail point "
+                     "enabled. Blocking until fail point is disabled.";
+            while (MONGO_FAIL_POINT(initialSyncHangAfterListCollections)) {
+                mongo::sleepsecs(1);
+            }
+        }
+    }
+
     _collectionNamespaces.reserve(_collectionInfos.size());
     std::set<std::string> seen;
     for (auto&& info : _collectionInfos) {
@@ -342,11 +370,26 @@ void DatabaseCloner::_listCollectionsCallback(const StatusWith<Fetcher::QueryRes
         }
         const BSONObj optionsObj = optionsElement.Obj();
         CollectionOptions options;
-        Status parseStatus = options.parse(optionsObj);
+        auto parseStatus = options.parse(optionsObj, CollectionOptions::parseForStorage);
         if (!parseStatus.isOK()) {
             _finishCallback_inlock(lk, parseStatus);
             return;
         }
+
+        BSONElement infoElement = info.getField(kInfoFieldName);
+        if (infoElement.isABSONObj()) {
+            BSONElement uuidElement = infoElement[kUUIDFieldName];
+            if (!uuidElement.eoo()) {
+                auto res = CollectionUUID::parse(uuidElement);
+                if (!res.isOK()) {
+                    _finishCallback_inlock(lk, res.getStatus());
+                    return;
+                }
+                options.uuid = res.getValue();
+            }
+        }
+        // TODO(SERVER-27994): Ensure UUID present when FCV >= "3.6".
+
         seen.insert(collectionName);
 
         _collectionNamespaces.emplace_back(_dbname, collectionName);
@@ -361,8 +404,10 @@ void DatabaseCloner::_listCollectionsCallback(const StatusWith<Fetcher::QueryRes
                 options,
                 stdx::bind(
                     &DatabaseCloner::_collectionClonerCallback, this, stdx::placeholders::_1, nss),
-                _storageInterface);
-        } catch (const UserException& ex) {
+                _storageInterface,
+                collectionClonerBatchSize,
+                maxNumInitialSyncCollectionClonerCursors.load());
+        } catch (const AssertionException& ex) {
             _finishCallback_inlock(lk, ex.toStatus());
             return;
         }
