@@ -41,6 +41,7 @@
 #include "mongo/bson/util/bson_extract.h"
 #include "mongo/client/fetcher.h"
 #include "mongo/client/remote_command_retry_scheduler.h"
+#include "mongo/db/commands/feature_compatibility_version.h"
 #include "mongo/db/commands/server_status_metric.h"
 #include "mongo/db/concurrency/d_concurrency.h"
 #include "mongo/db/jsobj.h"
@@ -51,12 +52,12 @@
 #include "mongo/db/repl/oplog_buffer.h"
 #include "mongo/db/repl/oplog_fetcher.h"
 #include "mongo/db/repl/optime.h"
+#include "mongo/db/repl/replication_process.h"
 #include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/repl/sync_source_selector.h"
 #include "mongo/db/server_parameters.h"
 #include "mongo/executor/task_executor.h"
 #include "mongo/rpc/metadata/repl_set_metadata.h"
-#include "mongo/rpc/metadata/server_selection_metadata.h"
 #include "mongo/stdx/memory.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/destructor_guard.h"
@@ -98,12 +99,18 @@ using QueryResponseStatus = StatusWith<Fetcher::QueryResponse>;
 using UniqueLock = stdx::unique_lock<stdx::mutex>;
 using LockGuard = stdx::lock_guard<stdx::mutex>;
 
+// 16MB max batch size / 12 byte min doc size * 10 (for good measure) = defaultBatchSize to use.
+const auto defaultBatchSize = (16 * 1024 * 1024) / 12 * 10;
+
 // The number of attempts to connect to a sync source.
 MONGO_EXPORT_SERVER_PARAMETER(numInitialSyncConnectAttempts, int, 10);
 
 // The number of attempts to call find on the remote oplog.
 MONGO_EXPORT_SERVER_PARAMETER(numInitialSyncOplogFindAttempts, int, 3);
 
+// The batchSize to use for the find/getMore queries called by the OplogFetcher
+MONGO_EXPORT_STARTUP_SERVER_PARAMETER(initialSyncOplogFetcherBatchSize, int, defaultBatchSize);
+
 // The number of initial sync attempts that have failed since server startup. Each instance of
 // InitialSyncer may run multiple attempts to fulfill an initial sync request that is triggered
 // when InitialSyncer::startup() is called.
@@ -144,64 +151,6 @@ StatusWith<Timestamp> parseTimestampStatus(const QueryResponseStatus& fetchResul
     }
 }
 
-StatusWith<BSONObj> getLatestOplogEntry(executor::TaskExecutor* exec,
-                                        HostAndPort source,
-                                        const NamespaceString& oplogNS) {
-    BSONObj query =
-        BSON("find" << oplogNS.coll() << "sort" << BSON("$natural" << -1) << "limit" << 1);
-
-    BSONObj entry;
-    Status statusToReturn(Status::OK());
-    Fetcher fetcher(
-        exec,
-        source,
-        oplogNS.db().toString(),
-        query,
-        [&entry, &statusToReturn](const QueryResponseStatus& fetchResult,
-                                  Fetcher::NextAction* nextAction,
-                                  BSONObjBuilder*) {
-            if (!fetchResult.isOK()) {
-                statusToReturn = fetchResult.getStatus();
-            } else {
-                const auto docs = fetchResult.getValue().documents;
-                invariant(docs.size() < 2);
-                if (docs.size() == 0) {
-                    statusToReturn = {ErrorCodes::OplogStartMissing, "no oplog entry found."};
-                } else {
-                    entry = docs.back().getOwned();
-                }
-            }
-        });
-    Status scheduleStatus = fetcher.schedule();
-    if (!scheduleStatus.isOK()) {
-        return scheduleStatus;
-    }
-
-    // wait for fetcher to get the oplog position.
-    fetcher.join();
-    if (statusToReturn.isOK()) {
-        LOG(2) << "returning last oplog entry: " << redact(entry) << ", from: " << source
-               << ", ns: " << oplogNS;
-        return entry;
-    }
-    return statusToReturn;
-}
-
-StatusWith<OpTimeWithHash> parseOpTimeWithHash(const BSONObj& oplogEntry) {
-    long long oplogEntryHash = 0LL;
-    auto status = bsonExtractIntegerField(oplogEntry, "h", &oplogEntryHash);
-    if (!status.isOK()) {
-        return status;
-    }
-
-    const auto lastOpTime = OpTime::parseFromOplogEntry(oplogEntry);
-    if (!lastOpTime.isOK()) {
-        return lastOpTime.getStatus();
-    }
-
-    return OpTimeWithHash{oplogEntryHash, lastOpTime.getValue()};
-}
-
 StatusWith<OpTimeWithHash> parseOpTimeWithHash(const QueryResponseStatus& fetchResult) {
     if (!fetchResult.isOK()) {
         return fetchResult.getStatus();
@@ -209,7 +158,7 @@ StatusWith<OpTimeWithHash> parseOpTimeWithHash(const QueryResponseStatus& fetchR
     const auto docs = fetchResult.getValue().documents;
     const auto hasDoc = docs.begin() != docs.end();
     return hasDoc
-        ? parseOpTimeWithHash(docs.front())
+        ? AbstractOplogFetcher::parseOpTimeWithHash(docs.front())
         : StatusWith<OpTimeWithHash>{ErrorCodes::NoMatchingDocument, "no oplog entry found"};
 }
 
@@ -219,15 +168,18 @@ InitialSyncer::InitialSyncer(
     InitialSyncerOptions opts,
     std::unique_ptr<DataReplicatorExternalState> dataReplicatorExternalState,
     StorageInterface* storage,
+    ReplicationProcess* replicationProcess,
     const OnCompletionFn& onCompletion)
     : _fetchCount(0),
       _opts(opts),
       _dataReplicatorExternalState(std::move(dataReplicatorExternalState)),
       _exec(_dataReplicatorExternalState->getTaskExecutor()),
       _storage(storage),
+      _replicationProcess(replicationProcess),
       _onCompletion(onCompletion) {
     uassert(ErrorCodes::BadValue, "task executor cannot be null", _exec);
     uassert(ErrorCodes::BadValue, "invalid storage interface", _storage);
+    uassert(ErrorCodes::BadValue, "invalid replication process", _replicationProcess);
     uassert(ErrorCodes::BadValue, "invalid getMyLastOptime function", _opts.getMyLastOptime);
     uassert(ErrorCodes::BadValue, "invalid setMyLastOptime function", _opts.setMyLastOptime);
     uassert(ErrorCodes::BadValue, "invalid resetOptimes function", _opts.resetOptimes);
@@ -238,7 +190,7 @@ InitialSyncer::InitialSyncer(
 
 InitialSyncer::~InitialSyncer() {
     DESTRUCTOR_GUARD({
-        shutdown();
+        shutdown().transitional_ignore();
         join();
     });
 }
@@ -324,6 +276,7 @@ void InitialSyncer::_cancelRemainingWork_inlock() {
         _shutdownComponent_inlock(_initialSyncState->dbsCloner);
     }
     _shutdownComponent_inlock(_applier);
+    _shutdownComponent_inlock(_fCVFetcher);
     _shutdownComponent_inlock(_lastOplogEntryFetcher);
 }
 
@@ -365,44 +318,30 @@ BSONObj InitialSyncer::getInitialSyncProgress() const {
     return _getInitialSyncProgress_inlock();
 }
 
-void InitialSyncer::_appendInitialSyncProgressMinimal_inlock(BSONObjBuilder* bob) const {
-    _stats.append(bob);
-    if (!_initialSyncState) {
-        return;
-    }
-    bob->appendNumber("fetchedMissingDocs", _initialSyncState->fetchedMissingDocs);
-    bob->appendNumber("appliedOps", _initialSyncState->appliedOps);
-    if (!_initialSyncState->beginTimestamp.isNull()) {
-        bob->append("initialSyncOplogStart", _initialSyncState->beginTimestamp);
-    }
-    if (!_initialSyncState->stopTimestamp.isNull()) {
-        bob->append("initialSyncOplogEnd", _initialSyncState->stopTimestamp);
-    }
-}
-
 BSONObj InitialSyncer::_getInitialSyncProgress_inlock() const {
+    BSONObjBuilder bob;
     try {
-        BSONObjBuilder bob;
-        _appendInitialSyncProgressMinimal_inlock(&bob);
+        _stats.append(&bob);
         if (_initialSyncState) {
+            bob.appendNumber("fetchedMissingDocs", _initialSyncState->fetchedMissingDocs);
+            bob.appendNumber("appliedOps", _initialSyncState->appliedOps);
+            if (!_initialSyncState->beginTimestamp.isNull()) {
+                bob.append("initialSyncOplogStart", _initialSyncState->beginTimestamp);
+            }
+            if (!_initialSyncState->stopTimestamp.isNull()) {
+                bob.append("initialSyncOplogEnd", _initialSyncState->stopTimestamp);
+            }
             if (_initialSyncState->dbsCloner) {
                 BSONObjBuilder dbsBuilder(bob.subobjStart("databases"));
                 _initialSyncState->dbsCloner->getStats().append(&dbsBuilder);
                 dbsBuilder.doneFast();
             }
         }
-        // In 3.4, BSONObjBuilder::obj() does not check the validity of the document it returns so
-        // we check it explicitly here.
-        auto obj = bob.obj();
-        uassert(ErrorCodes::InvalidBSON,
-                str::stream() << "Invalid BSONObj size: " << obj.objsize(),
-                obj.isValid());
-        return obj;
     } catch (const DBException& e) {
+        bob.resetToEmpty();
+        bob.append("error", e.toString());
         log() << "Error creating initial sync progress object: " << e.toString();
     }
-    BSONObjBuilder bob;
-    _appendInitialSyncProgressMinimal_inlock(&bob);
     return bob.obj();
 }
 
@@ -412,9 +351,12 @@ void InitialSyncer::setScheduleDbWorkFn_forTest(const CollectionCloner::Schedule
 }
 
 void InitialSyncer::_setUp_inlock(OperationContext* opCtx, std::uint32_t initialSyncMaxAttempts) {
-    // This will call through to the storageInterfaceImpl to ReplicationCoordinatorImpl.
-    // 'txn' is passed through from startup().
-    _storage->setInitialSyncFlag(opCtx);
+    // 'opCtx' is passed through from startup().
+    _replicationProcess->getConsistencyMarkers()->setInitialSyncFlag(opCtx);
+
+    auto serviceCtx = opCtx->getServiceContext();
+    _storage->setInitialDataTimestamp(serviceCtx, Timestamp::kAllowUnstableCheckpointsSentinel);
+    _storage->setStableTimestamp(serviceCtx, Timestamp::min());
 
     LOG(1) << "Creating oplogBuffer.";
     _oplogBuffer = _dataReplicatorExternalState->makeInitialSyncOplogBuffer(opCtx);
@@ -436,8 +378,15 @@ void InitialSyncer::_tearDown_inlock(OperationContext* opCtx,
     if (!lastApplied.isOK()) {
         return;
     }
-    _storage->clearInitialSyncFlag(opCtx);
-    _opts.setMyLastOptime(lastApplied.getValue().opTime);
+
+    // This is necessary to ensure that the oplog contains at least one visible document prior to
+    // setting an externally visible lastApplied.  That way if any other node attempts to read from
+    // this node's oplog, it won't appear empty.
+    _storage->waitForAllEarlierOplogWritesToBeVisible(opCtx);
+
+    _storage->setInitialDataTimestamp(opCtx->getServiceContext(),
+                                      lastApplied.getValue().opTime.getTimestamp());
+    _replicationProcess->getConsistencyMarkers()->clearInitialSyncFlag(opCtx);
     log() << "initial sync done; took "
           << duration_cast<Seconds>(_stats.initialSyncEnd - _stats.initialSyncStart) << ".";
     initialSyncCompletes.increment();
@@ -480,6 +429,11 @@ void InitialSyncer::_startInitialSyncAttemptCallback(
     _lastApplied = {};
     _lastFetched = {};
 
+    LOG(2) << "Resetting feature compatibility version to 3.4. If the sync source is in feature "
+              "compatibility version 3.6, we will find out when we clone the admin.system.version "
+              "collection.";
+    serverGlobalParams.featureCompatibility.reset();
+
     // Clear the oplog buffer.
     _oplogBuffer->clear(makeOpCtx().get());
 
@@ -561,7 +515,7 @@ void InitialSyncer::_chooseSyncSourceCallback(
     // There is no need to schedule separate task to create oplog collection since we are already in
     // a callback and we are certain there's no existing operation context (required for creating
     // collections and dropping user databases) attached to the current thread.
-    status = _recreateOplogAndDropReplicatedDatabases();
+    status = _truncateOplogAndDropReplicatedDatabases();
     if (!status.isOK()) {
         onCompletionGuard->setResultAndCancelRemainingWork_inlock(lock, status);
         return;
@@ -583,33 +537,31 @@ void InitialSyncer::_chooseSyncSourceCallback(
     _getBaseRollbackIdHandle = scheduleResult.getValue();
 }
 
-Status InitialSyncer::_recreateOplogAndDropReplicatedDatabases() {
-    // drop/create oplog; drop user databases.
-    LOG(1) << "About to drop+create the oplog, if it exists, ns:" << _opts.localOplogNS
+Status InitialSyncer::_truncateOplogAndDropReplicatedDatabases() {
+    // truncate oplog; drop user databases.
+    LOG(1) << "About to truncate the oplog, if it exists, ns:" << _opts.localOplogNS
            << ", and drop all user databases (so that we can clone them).";
 
-    auto txn = makeOpCtx();
+    auto opCtx = makeOpCtx();
 
     // We are not replicating nor validating these writes.
-    UnreplicatedWritesBlock unreplicatedWritesBlock(txn.get());
+    UnreplicatedWritesBlock unreplicatedWritesBlock(opCtx.get());
 
-    // 1.) Drop the oplog.
-    LOG(2) << "Dropping the existing oplog: " << _opts.localOplogNS;
-    auto status = _storage->dropCollection(txn.get(), _opts.localOplogNS);
+    // 1.) Truncate the oplog.
+    LOG(2) << "Truncating the existing oplog: " << _opts.localOplogNS;
+    auto status = _storage->truncateCollection(opCtx.get(), _opts.localOplogNS);
     if (!status.isOK()) {
-        return status;
+        // 1a.) Create the oplog.
+        LOG(2) << "Creating the oplog: " << _opts.localOplogNS;
+        status = _storage->createOplog(opCtx.get(), _opts.localOplogNS);
+        if (!status.isOK()) {
+            return status;
+        }
     }
 
     // 2.) Drop user databases.
-    LOG(2) << "Dropping  user databases";
-    status = _storage->dropReplicatedDatabases(txn.get());
-    if (!status.isOK()) {
-        return status;
-    }
-
-    // 3.) Create the oplog.
-    LOG(2) << "Creating the oplog: " << _opts.localOplogNS;
-    return _storage->createOplog(txn.get(), _opts.localOplogNS);
+    LOG(2) << "Dropping user databases";
+    return _storage->dropReplicatedDatabases(opCtx.get());
 }
 
 void InitialSyncer::_rollbackCheckerResetCallback(
@@ -651,6 +603,89 @@ void InitialSyncer::_lastOplogEntryFetcherCallbackForBeginTimestamp(
         return;
     }
 
+    const auto& lastOpTimeWithHash = opTimeWithHashResult.getValue();
+
+    BSONObjBuilder queryBob;
+    queryBob.append("find", nsToCollectionSubstring(FeatureCompatibilityVersion::kCollection));
+    auto filterBob = BSONObjBuilder(queryBob.subobjStart("filter"));
+    filterBob.append("_id", FeatureCompatibilityVersion::kParameterName);
+    filterBob.done();
+
+    _fCVFetcher = stdx::make_unique<Fetcher>(
+        _exec,
+        _syncSource,
+        nsToDatabaseSubstring(FeatureCompatibilityVersion::kCollection).toString(),
+        queryBob.obj(),
+        stdx::bind(&InitialSyncer::_fcvFetcherCallback,
+                   this,
+                   stdx::placeholders::_1,
+                   onCompletionGuard,
+                   lastOpTimeWithHash),
+        ReadPreferenceSetting::secondaryPreferredMetadata(),
+        RemoteCommandRequest::kNoTimeout /* find network timeout */,
+        RemoteCommandRequest::kNoTimeout /* getMore network timeout */,
+        RemoteCommandRetryScheduler::makeRetryPolicy(
+            numInitialSyncOplogFindAttempts.load(),
+            executor::RemoteCommandRequest::kNoTimeout,
+            RemoteCommandRetryScheduler::kAllRetriableErrors));
+    Status scheduleStatus = _fCVFetcher->schedule();
+    if (!scheduleStatus.isOK()) {
+        _fCVFetcher.reset();
+        onCompletionGuard->setResultAndCancelRemainingWork_inlock(lock, scheduleStatus);
+        return;
+    }
+}
+
+void InitialSyncer::_fcvFetcherCallback(const StatusWith<Fetcher::QueryResponse>& result,
+                                        std::shared_ptr<OnCompletionGuard> onCompletionGuard,
+                                        const OpTimeWithHash& lastOpTimeWithHash) {
+    stdx::unique_lock<stdx::mutex> lock(_mutex);
+    auto status = _checkForShutdownAndConvertStatus_inlock(
+        result.getStatus(), "error while getting the remote feature compatibility version");
+    if (!status.isOK()) {
+        onCompletionGuard->setResultAndCancelRemainingWork_inlock(lock, status);
+        return;
+    }
+
+    const auto docs = result.getValue().documents;
+    if (docs.size() > 1) {
+        onCompletionGuard->setResultAndCancelRemainingWork_inlock(
+            lock,
+            Status(ErrorCodes::TooManyMatchingDocuments,
+                   str::stream() << "Expected to receive one document, but received: "
+                                 << docs.size()
+                                 << ". First: "
+                                 << redact(docs.front())
+                                 << ". Last: "
+                                 << redact(docs.back())));
+        return;
+    }
+    const auto hasDoc = docs.begin() != docs.end();
+    if (!hasDoc) {
+        onCompletionGuard->setResultAndCancelRemainingWork_inlock(
+            lock,
+            Status(ErrorCodes::IncompatibleServerVersion,
+                   "Sync source had no feature compatibility version document"));
+        return;
+    }
+
+    auto fCVParseSW = FeatureCompatibilityVersion::parse(docs.front());
+    if (!fCVParseSW.isOK()) {
+        onCompletionGuard->setResultAndCancelRemainingWork_inlock(lock, fCVParseSW.getStatus());
+        return;
+    }
+
+    auto version = fCVParseSW.getValue();
+    if (version != ServerGlobalParams::FeatureCompatibility::Version::kFullyDowngradedTo34 &&
+        version != ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo36) {
+        onCompletionGuard->setResultAndCancelRemainingWork_inlock(
+            lock,
+            Status(ErrorCodes::IncompatibleServerVersion,
+                   str::stream() << "Sync source had unsafe feature compatibility version: "
+                                 << FeatureCompatibilityVersion::toString(version)));
+        return;
+    }
+
     // This is where the flow of control starts to split into two parallel tracks:
     // - oplog fetcher
     // - data cloning and applier
@@ -675,7 +710,6 @@ void InitialSyncer::_lastOplogEntryFetcherCallbackForBeginTimestamp(
                                                       stdx::placeholders::_1,
                                                       onCompletionGuard)));
 
-    const auto& lastOpTimeWithHash = opTimeWithHashResult.getValue();
     _initialSyncState->beginTimestamp = lastOpTimeWithHash.opTime.getTimestamp();
 
     invariant(!result.getValue().documents.empty());
@@ -693,26 +727,24 @@ void InitialSyncer::_lastOplogEntryFetcherCallbackForBeginTimestamp(
     }
 
     const auto& config = configResult.getValue();
-    _oplogFetcher =
-        stdx::make_unique<OplogFetcher>(_exec,
-                                        lastOpTimeWithHash,
-                                        _syncSource,
-                                        _opts.remoteOplogNS,
-                                        config,
-                                        _opts.oplogFetcherMaxFetcherRestarts,
-                                        _rollbackChecker->getBaseRBID(),
-                                        false /* requireFresherSyncSource */,
-                                        _dataReplicatorExternalState.get(),
-                                        stdx::bind(&InitialSyncer::_enqueueDocuments,
-                                                   this,
-                                                   stdx::placeholders::_1,
-                                                   stdx::placeholders::_2,
-                                                   stdx::placeholders::_3),
-                                        stdx::bind(&InitialSyncer::_oplogFetcherCallback,
-                                                   this,
-                                                   stdx::placeholders::_1,
-                                                   stdx::placeholders::_2,
-                                                   onCompletionGuard));
+    _oplogFetcher = stdx::make_unique<OplogFetcher>(
+        _exec,
+        lastOpTimeWithHash,
+        _syncSource,
+        _opts.remoteOplogNS,
+        config,
+        _opts.oplogFetcherMaxFetcherRestarts,
+        _rollbackChecker->getBaseRBID(),
+        false /* requireFresherSyncSource */,
+        _dataReplicatorExternalState.get(),
+        stdx::bind(&InitialSyncer::_enqueueDocuments,
+                   this,
+                   stdx::placeholders::_1,
+                   stdx::placeholders::_2,
+                   stdx::placeholders::_3),
+        stdx::bind(
+            &InitialSyncer::_oplogFetcherCallback, this, stdx::placeholders::_1, onCompletionGuard),
+        initialSyncOplogFetcherBatchSize);
 
     LOG(2) << "Starting OplogFetcher: " << _oplogFetcher->toString();
 
@@ -757,12 +789,11 @@ void InitialSyncer::_lastOplogEntryFetcherCallbackForBeginTimestamp(
 }
 
 void InitialSyncer::_oplogFetcherCallback(const Status& oplogFetcherFinishStatus,
-                                          const OpTimeWithHash& lastFetched,
                                           std::shared_ptr<OnCompletionGuard> onCompletionGuard) {
-    log() << "Finished fetching oplog during initial sync: " << redact(oplogFetcherFinishStatus)
-          << ". Last fetched optime and hash: " << lastFetched;
-
     stdx::lock_guard<stdx::mutex> lock(_mutex);
+    log() << "Finished fetching oplog during initial sync: " << redact(oplogFetcherFinishStatus)
+          << ". Last fetched optime and hash: " << _lastFetched.toString();
+
     auto status = _checkForShutdownAndConvertStatus_inlock(
         oplogFetcherFinishStatus, "error fetching oplog during initial sync");
 
@@ -777,8 +808,7 @@ void InitialSyncer::_oplogFetcherCallback(const Status& oplogFetcherFinishStatus
     // OplogFetcher to ignore the current sync source response and return early.
     if (status.isOK()) {
         log() << "Finished fetching oplog fetching early. Last fetched optime and hash: "
-              << lastFetched.toString();
-        _lastFetched = lastFetched;
+              << _lastFetched.toString();
         return;
     }
 
@@ -793,6 +823,9 @@ void InitialSyncer::_oplogFetcherCallback(const Status& oplogFetcherFinishStatus
 
 void InitialSyncer::_databasesClonerCallback(const Status& databaseClonerFinishStatus,
                                              std::shared_ptr<OnCompletionGuard> onCompletionGuard) {
+    log() << "Finished cloning data: " << redact(databaseClonerFinishStatus)
+          << ". Beginning oplog replay.";
+
     stdx::lock_guard<stdx::mutex> lock(_mutex);
     auto status = _checkForShutdownAndConvertStatus_inlock(databaseClonerFinishStatus,
                                                            "error cloning databases");
@@ -815,6 +848,7 @@ void InitialSyncer::_databasesClonerCallback(const Status& databaseClonerFinishS
 void InitialSyncer::_lastOplogEntryFetcherCallbackForStopTimestamp(
     const StatusWith<Fetcher::QueryResponse>& result,
     std::shared_ptr<OnCompletionGuard> onCompletionGuard) {
+    OpTimeWithHash optimeWithHash;
     {
         stdx::lock_guard<stdx::mutex> lock(_mutex);
         auto status = _checkForShutdownAndConvertStatus_inlock(
@@ -830,14 +864,10 @@ void InitialSyncer::_lastOplogEntryFetcherCallbackForStopTimestamp(
                 lock, optimeWithHashStatus.getStatus());
             return;
         }
-        auto&& optimeWithHash = optimeWithHashStatus.getValue();
+        optimeWithHash = optimeWithHashStatus.getValue();
         _initialSyncState->stopTimestamp = optimeWithHash.opTime.getTimestamp();
 
-        if (_initialSyncState->beginTimestamp == _initialSyncState->stopTimestamp) {
-            _lastApplied = optimeWithHash;
-            log() << "No need to apply operations. (currently at "
-                  << _initialSyncState->stopTimestamp.toBSON() << ")";
-        } else {
+        if (_initialSyncState->beginTimestamp != _initialSyncState->stopTimestamp) {
             invariant(_lastApplied.opTime.isNull());
             _checkApplierProgressAndScheduleGetNextApplierBatch_inlock(lock, onCompletionGuard);
             return;
@@ -849,7 +879,7 @@ void InitialSyncer::_lastOplogEntryFetcherCallbackForStopTimestamp(
     {
         const auto& documents = result.getValue().documents;
         invariant(!documents.empty());
-        const auto& oplogSeedDoc = documents.front();
+        const BSONObj oplogSeedDoc = documents.front();
         LOG(2) << "Inserting oplog seed document: " << oplogSeedDoc;
 
         auto opCtx = makeOpCtx();
@@ -857,7 +887,11 @@ void InitialSyncer::_lastOplogEntryFetcherCallbackForStopTimestamp(
         // override its behavior in tests. See InitialSyncerReturnsCallbackCanceledAndDoesNot-
         // ScheduleRollbackCheckerIfShutdownAfterInsertingInsertOplogSeedDocument in
         // initial_syncer_test.cpp
-        auto status = _storage->insertDocument(opCtx.get(), _opts.localOplogNS, oplogSeedDoc);
+        auto status = _storage->insertDocument(
+            opCtx.get(),
+            _opts.localOplogNS,
+            TimestampedBSONObj{oplogSeedDoc, optimeWithHash.opTime.getTimestamp()},
+            optimeWithHash.opTime.getTerm());
         if (!status.isOK()) {
             stdx::lock_guard<stdx::mutex> lock(_mutex);
             onCompletionGuard->setResultAndCancelRemainingWork_inlock(lock, status);
@@ -866,6 +900,10 @@ void InitialSyncer::_lastOplogEntryFetcherCallbackForStopTimestamp(
     }
 
     stdx::lock_guard<stdx::mutex> lock(_mutex);
+    _lastApplied = optimeWithHash;
+    log() << "No need to apply operations. (currently at "
+          << _initialSyncState->stopTimestamp.toBSON() << ")";
+
     // This sets the error in 'onCompletionGuard' and shuts down the OplogFetcher on error.
     _scheduleRollbackCheckerCheckForRollback_inlock(lock, onCompletionGuard);
 }
@@ -907,7 +945,7 @@ void InitialSyncer::_getNextApplierBatchCallback(
                                                    stdx::placeholders::_3);
 
         const auto lastEntry = ops.back().raw;
-        const auto opTimeWithHashStatus = parseOpTimeWithHash(lastEntry);
+        const auto opTimeWithHashStatus = AbstractOplogFetcher::parseOpTimeWithHash(lastEntry);
         status = opTimeWithHashStatus.getStatus();
         if (!status.isOK()) {
             onCompletionGuard->setResultAndCancelRemainingWork_inlock(lock, status);
@@ -982,7 +1020,8 @@ void InitialSyncer::_multiApplierCallback(const Status& multiApplierStatus,
 
     _initialSyncState->appliedOps += numApplied;
     _lastApplied = lastApplied;
-    _opts.setMyLastOptime(_lastApplied.opTime);
+    _opts.setMyLastOptime(_lastApplied.opTime,
+                          ReplicationCoordinator::DataConsistency::Inconsistent);
 
     auto fetchCount = _fetchCount.load();
     if (fetchCount > 0) {
@@ -1055,6 +1094,26 @@ void InitialSyncer::_rollbackCheckerCheckForRollbackCallback(
         return;
     }
 
+    // Set UUIDs for all non-replicated collections on secondaries. See comment in
+    // ReplicationCoordinatorExternalStateImpl::initializeReplSetStorage() for the explanation of
+    // why we do this.
+    const NamespaceString nss("admin", "system.version");
+    auto opCtx = makeOpCtx();
+    auto statusWithUUID = _storage->getCollectionUUID(opCtx.get(), nss);
+    if (!statusWithUUID.isOK()) {
+        // If the admin database does not exist, we intentionally fail initial sync. As part of
+        // SERVER-29448, we disallow dropping the admin database, so failing here is fine.
+        onCompletionGuard->setResultAndCancelRemainingWork_inlock(lock, statusWithUUID.getStatus());
+        return;
+    }
+    if (statusWithUUID.getValue()) {
+        auto schemaStatus = _storage->upgradeUUIDSchemaVersionNonReplicated(opCtx.get());
+        if (!schemaStatus.isOK()) {
+            onCompletionGuard->setResultAndCancelRemainingWork_inlock(lock, schemaStatus);
+            return;
+        }
+    }
+
     // Success!
     onCompletionGuard->setResultAndCancelRemainingWork_inlock(lock, _lastApplied);
 }
@@ -1066,7 +1125,7 @@ void InitialSyncer::_finishInitialSyncAttempt(const StatusWith<OpTimeWithHash>&
     // For example, if CollectionCloner fails while inserting documents into the
     // CollectionBulkLoader, we will get here via one of CollectionCloner's TaskRunner callbacks
     // which has an active OperationContext bound to the current Client. This would lead to an
-    // invariant when we attempt to create a new OperationContext for _tearDown(txn).
+    // invariant when we attempt to create a new OperationContext for _tearDown(opCtx).
     // To avoid this, we schedule _finishCallback against the TaskExecutor rather than calling it
     // here synchronously.
 
@@ -1157,8 +1216,8 @@ void InitialSyncer::_finishCallback(StatusWith<OpTimeWithHash> lastApplied) {
     decltype(_onCompletion) onCompletion;
     {
         stdx::lock_guard<stdx::mutex> lock(_mutex);
-        auto txn = makeOpCtx();
-        _tearDown_inlock(txn.get(), lastApplied);
+        auto opCtx = makeOpCtx();
+        _tearDown_inlock(opCtx.get(), lastApplied);
 
         invariant(_onCompletion);
         std::swap(_onCompletion, onCompletion);
@@ -1202,11 +1261,11 @@ Status InitialSyncer::_scheduleLastOplogEntryFetcher_inlock(Fetcher::CallbackFn
                                    _opts.remoteOplogNS.db().toString(),
                                    query,
                                    callback,
-                                   rpc::ServerSelectionMetadata(true, boost::none).toBSON(),
+                                   ReadPreferenceSetting::secondaryPreferredMetadata(),
                                    RemoteCommandRequest::kNoTimeout /* find network timeout */,
                                    RemoteCommandRequest::kNoTimeout /* getMore network timeout */,
                                    RemoteCommandRetryScheduler::makeRetryPolicy(
-                                       numInitialSyncOplogFindAttempts,
+                                       numInitialSyncOplogFindAttempts.load(),
                                        executor::RemoteCommandRequest::kNoTimeout,
                                        RemoteCommandRetryScheduler::kAllRetriableErrors));
     Status scheduleStatus = _lastOplogEntryFetcher->schedule();
@@ -1315,11 +1374,7 @@ Status InitialSyncer::_checkForShutdownAndConvertStatus_inlock(const Status& sta
         return Status(ErrorCodes::CallbackCanceled, message + ": initial syncer is shutting down");
     }
 
-    if (!status.isOK()) {
-        return Status(status.code(), message + ": " + status.reason());
-    }
-
-    return Status::OK();
+    return status.withContext(message);
 }
 
 Status InitialSyncer::_scheduleWorkAndSaveHandle_inlock(
@@ -1415,8 +1470,8 @@ StatusWith<Operations> InitialSyncer::_getNextApplierBatch_inlock() {
     //      * only OplogEntries from before the slaveDelay point
     //      * a single command OplogEntry (including index builds, which appear to be inserts)
     //          * consequently, commands bound the previous batch to be in a batch of their own
-    auto txn = makeOpCtx();
-    while (_oplogBuffer->peek(txn.get(), &op)) {
+    auto opCtx = makeOpCtx();
+    while (_oplogBuffer->peek(opCtx.get(), &op)) {
         auto entry = OplogEntry(std::move(op));
 
         // Check for oplog version change. If it is absent, its value is one.
@@ -1433,11 +1488,11 @@ StatusWith<Operations> InitialSyncer::_getNextApplierBatch_inlock() {
             // Index builds are achieved through the use of an insert op, not a command op.
             // The following line is the same as what the insert code uses to detect an index
             // build.
-            (entry.hasNamespace() && entry.getCollectionName() == "system.indexes")) {
+            (entry.getNamespace().isSystemDotIndexes())) {
             if (ops.empty()) {
                 // Apply commands one-at-a-time.
                 ops.push_back(std::move(entry));
-                invariant(_oplogBuffer->tryPop(txn.get(), &op));
+                invariant(_oplogBuffer->tryPop(opCtx.get(), &op));
                 dassert(SimpleBSONObjComparator::kInstance.evaluate(ops.back().raw == op));
             }
 
@@ -1471,7 +1526,7 @@ StatusWith<Operations> InitialSyncer::_getNextApplierBatch_inlock() {
         // Add op to buffer.
         ops.push_back(std::move(entry));
         totalBytes += ops.back().raw.objsize();
-        invariant(_oplogBuffer->tryPop(txn.get(), &op));
+        invariant(_oplogBuffer->tryPop(opCtx.get(), &op));
         dassert(SimpleBSONObjComparator::kInstance.evaluate(ops.back().raw == op));
     }
     return std::move(ops);
