@@ -35,16 +35,26 @@
 #include "mongo/bson/util/bson_check.h"
 #include "mongo/bson/util/bson_extract.h"
 #include "mongo/db/jsobj.h"
+#include "mongo/db/mongod_options.h"
 #include "mongo/db/server_options.h"
+#include "mongo/db/server_parameters.h"
 #include "mongo/stdx/functional.h"
 #include "mongo/util/stringutils.h"
 
 namespace mongo {
+/**
+ * Dont run any sharding validations. Can not be combined with --configsvr or shardvr. Intended to
+ * allow restarting config server or shard as an independent replica set.
+ */
+MONGO_EXPORT_STARTUP_SERVER_PARAMETER(skipShardingConfigurationChecks, bool, false);
+
 namespace repl {
 
 const size_t ReplSetConfig::kMaxMembers;
 const size_t ReplSetConfig::kMaxVotingMembers;
 const Milliseconds ReplSetConfig::kInfiniteCatchUpTimeout(-1);
+const Milliseconds ReplSetConfig::kCatchUpDisabled(0);
+const Milliseconds ReplSetConfig::kCatchUpTakeoverDisabled(-1);
 
 const std::string ReplSetConfig::kConfigServerFieldName = "configsvr";
 const std::string ReplSetConfig::kVersionFieldName = "version";
@@ -52,8 +62,9 @@ const std::string ReplSetConfig::kMajorityWriteConcernModeName = "$majority";
 const Milliseconds ReplSetConfig::kDefaultHeartbeatInterval(2000);
 const Seconds ReplSetConfig::kDefaultHeartbeatTimeoutPeriod(10);
 const Milliseconds ReplSetConfig::kDefaultElectionTimeoutPeriod(10000);
-const Milliseconds ReplSetConfig::kDefaultCatchUpTimeoutPeriod(60000);
+const Milliseconds ReplSetConfig::kDefaultCatchUpTimeoutPeriod(kInfiniteCatchUpTimeout);
 const bool ReplSetConfig::kDefaultChainingAllowed(true);
+const Milliseconds ReplSetConfig::kDefaultCatchUpTakeoverDelay(30000);
 
 namespace {
 
@@ -81,6 +92,7 @@ const std::string kHeartbeatIntervalFieldName = "heartbeatIntervalMillis";
 const std::string kHeartbeatTimeoutFieldName = "heartbeatTimeoutSecs";
 const std::string kCatchUpTimeoutFieldName = "catchUpTimeoutMillis";
 const std::string kReplicaSetIdFieldName = "replicaSetId";
+const std::string kCatchUpTakeoverDelayFieldName = "catchUpTakeoverDelayMillis";
 
 }  // namespace
 
@@ -271,13 +283,15 @@ Status ReplSetConfig::_parseSettingsSubdocument(const BSONObj& settings) {
     //
     // Parse catchUpTimeoutMillis
     //
-    auto validCatchUpTimeout = [](long long timeout) { return timeout >= 0LL || timeout == -1LL; };
+    auto validCatchUpParameter = [](long long timeout) {
+        return timeout >= 0LL || timeout == -1LL;
+    };
     long long catchUpTimeoutMillis;
     Status catchUpTimeoutStatus = bsonExtractIntegerFieldWithDefaultIf(
         settings,
         kCatchUpTimeoutFieldName,
         durationCount<Milliseconds>(kDefaultCatchUpTimeoutPeriod),
-        validCatchUpTimeout,
+        validCatchUpParameter,
         "catch-up timeout must be positive, 0 (no catch-up) or -1 (infinite catch-up).",
         &catchUpTimeoutMillis);
     if (!catchUpTimeoutStatus.isOK()) {
@@ -285,6 +299,22 @@ Status ReplSetConfig::_parseSettingsSubdocument(const BSONObj& settings) {
     }
     _catchUpTimeoutPeriod = Milliseconds(catchUpTimeoutMillis);
 
+    //
+    // Parse catchUpTakeoverDelayMillis
+    //
+    long long catchUpTakeoverDelayMillis;
+    Status catchUpTakeoverDelayStatus = bsonExtractIntegerFieldWithDefaultIf(
+        settings,
+        kCatchUpTakeoverDelayFieldName,
+        durationCount<Milliseconds>(kDefaultCatchUpTakeoverDelay),
+        validCatchUpParameter,
+        "catch-up takeover delay must be -1 (no catch-up takeover) or greater than or equal to 0.",
+        &catchUpTakeoverDelayMillis);
+    if (!catchUpTakeoverDelayStatus.isOK()) {
+        return catchUpTakeoverDelayStatus;
+    }
+    _catchUpTakeoverDelay = Milliseconds(catchUpTakeoverDelayMillis);
+
     //
     // Parse chainingAllowed
     //
@@ -508,8 +538,6 @@ Status ReplSetConfig::validate() const {
                       "one non-arbiter member with priority > 0");
     }
 
-    // TODO(schwerin): Validate satisfiability of write modes? Omitting for backwards
-    // compatibility.
     if (_defaultWriteConcern.wMode.empty()) {
         if (_defaultWriteConcern.wNumNodes == 0) {
             return Status(ErrorCodes::BadValue,
@@ -552,7 +580,8 @@ Status ReplSetConfig::validate() const {
                               "servers cannot have a non-zero slaveDelay");
             }
         }
-        if (serverGlobalParams.clusterRole != ClusterRole::ConfigServer) {
+        if (serverGlobalParams.clusterRole != ClusterRole::ConfigServer &&
+            !skipShardingConfigurationChecks) {
             return Status(ErrorCodes::BadValue,
                           "Nodes being used for config servers must be started with the "
                           "--configsvr flag");
@@ -631,7 +660,7 @@ const MemberConfig* ReplSetConfig::findMemberByID(int id) const {
     return NULL;
 }
 
-const int ReplSetConfig::findMemberIndexByHostAndPort(const HostAndPort& hap) const {
+int ReplSetConfig::findMemberIndexByHostAndPort(const HostAndPort& hap) const {
     int x = 0;
     for (std::vector<MemberConfig>::const_iterator it = _members.begin(); it != _members.end();
          ++it) {
@@ -643,7 +672,7 @@ const int ReplSetConfig::findMemberIndexByHostAndPort(const HostAndPort& hap) co
     return -1;
 }
 
-const int ReplSetConfig::findMemberIndexByConfigId(long long configId) const {
+int ReplSetConfig::findMemberIndexByConfigId(long long configId) const {
     int x = 0;
     for (const auto& member : _members) {
         if (member.getId() == configId) {
@@ -738,7 +767,7 @@ void ReplSetConfig::_initializeConnectionString() {
     try {
         _connectionString = ConnectionString::forReplicaSet(_replSetName, visibleMembers);
     } catch (const DBException& e) {
-        invariant(e.getCode() == ErrorCodes::FailedToParse);
+        invariant(e.code() == ErrorCodes::FailedToParse);
         // Failure to construct the ConnectionString means either an invalid replica set name
         // or members array, which should be caught in validate()
     }
@@ -785,6 +814,8 @@ BSONObj ReplSetConfig::toBSON() const {
                                   durationCount<Milliseconds>(_electionTimeoutPeriod));
     settingsBuilder.appendIntOrLL(kCatchUpTimeoutFieldName,
                                   durationCount<Milliseconds>(_catchUpTimeoutPeriod));
+    settingsBuilder.appendIntOrLL(kCatchUpTakeoverDelayFieldName,
+                                  durationCount<Milliseconds>(_catchUpTakeoverDelay));
 
 
     BSONObjBuilder gleModes(settingsBuilder.subobjStart(kGetLastErrorModesFieldName));
