@@ -115,6 +115,93 @@ TEST(ValidateConfigForInitiate, SelfMustBeElectable) {
             .getStatus());
 }
 
+TEST(ValidateConfigForInitiate, WriteConcernMustBeSatisfiable) {
+    ReplSetConfig config;
+    ASSERT_OK(
+        config.initializeForInitiate(BSON("_id"
+                                          << "rs0"
+                                          << "version"
+                                          << 1
+                                          << "members"
+                                          << BSON_ARRAY(BSON("_id" << 1 << "host"
+                                                                   << "h1"))
+                                          << "settings"
+                                          << BSON("getLastErrorDefaults" << BSON("w" << 2)))));
+    ReplicationCoordinatorExternalStateMock presentOnceExternalState;
+    presentOnceExternalState.addSelf(HostAndPort("h2"));
+
+    ASSERT_EQUALS(
+        ErrorCodes::CannotSatisfyWriteConcern,
+        validateConfigForInitiate(&presentOnceExternalState, config, getGlobalServiceContext())
+            .getStatus());
+}
+
+TEST(ValidateConfigForInitiate, ArbiterPriorityMustBeZeroOrOne) {
+    ReplSetConfig zeroConfig;
+    ReplSetConfig oneConfig;
+    ReplSetConfig twoConfig;
+    ASSERT_OK(zeroConfig.initialize(BSON("_id"
+                                         << "rs0"
+                                         << "version"
+                                         << 1
+                                         << "members"
+                                         << BSON_ARRAY(BSON("_id" << 1 << "host"
+                                                                  << "h1")
+                                                       << BSON("_id" << 2 << "host"
+                                                                     << "h2"
+                                                                     << "priority"
+                                                                     << 0
+                                                                     << "arbiterOnly"
+                                                                     << true)
+                                                       << BSON("_id" << 3 << "host"
+                                                                     << "h3")))));
+
+    ASSERT_OK(oneConfig.initialize(BSON("_id"
+                                        << "rs0"
+                                        << "version"
+                                        << 1
+                                        << "members"
+                                        << BSON_ARRAY(BSON("_id" << 1 << "host"
+                                                                 << "h1")
+                                                      << BSON("_id" << 2 << "host"
+                                                                    << "h2"
+                                                                    << "priority"
+                                                                    << 1
+                                                                    << "arbiterOnly"
+                                                                    << true)
+                                                      << BSON("_id" << 3 << "host"
+                                                                    << "h3")))));
+
+    ASSERT_OK(twoConfig.initialize(BSON("_id"
+                                        << "rs0"
+                                        << "version"
+                                        << 1
+                                        << "members"
+                                        << BSON_ARRAY(BSON("_id" << 1 << "host"
+                                                                 << "h1")
+                                                      << BSON("_id" << 2 << "host"
+                                                                    << "h2"
+                                                                    << "priority"
+                                                                    << 2
+                                                                    << "arbiterOnly"
+                                                                    << true)
+                                                      << BSON("_id" << 3 << "host"
+                                                                    << "h3")))));
+    ReplicationCoordinatorExternalStateMock presentOnceExternalState;
+    presentOnceExternalState.addSelf(HostAndPort("h1"));
+
+    ASSERT_OK(
+        validateConfigForInitiate(&presentOnceExternalState, zeroConfig, getGlobalServiceContext())
+            .getStatus());
+    ASSERT_OK(
+        validateConfigForInitiate(&presentOnceExternalState, oneConfig, getGlobalServiceContext())
+            .getStatus());
+    ASSERT_EQUALS(
+        ErrorCodes::InvalidReplicaSetConfig,
+        validateConfigForInitiate(&presentOnceExternalState, twoConfig, getGlobalServiceContext())
+            .getStatus());
+}
+
 TEST(ValidateConfigForReconfig, NewConfigVersionNumberMustBeHigherThanOld) {
     ReplicationCoordinatorExternalStateMock externalState;
     externalState.addSelf(HostAndPort("h1"));
@@ -588,6 +675,96 @@ TEST(ValidateConfigForReconfig, MustFindSelf) {
             &presentOnceExternalState, oldConfig, newConfig, getGlobalServiceContext(), true)));
 }
 
+TEST(ValidateConfigForReconfig, ArbiterPriorityValueMustBeZeroOrOne) {
+    ReplicationCoordinatorExternalStateMock externalState;
+    externalState.addSelf(HostAndPort("h1"));
+
+    ReplSetConfig oldConfig;
+    ReplSetConfig zeroConfig;
+    ReplSetConfig oneConfig;
+    ReplSetConfig twoConfig;
+
+    ASSERT_OK(oldConfig.initialize(BSON("_id"
+                                        << "rs0"
+                                        << "version"
+                                        << 1
+                                        << "members"
+                                        << BSON_ARRAY(BSON("_id" << 1 << "host"
+                                                                 << "h1")
+                                                      << BSON("_id" << 2 << "host"
+                                                                    << "h2"
+                                                                    << "arbiterOnly"
+                                                                    << true)
+                                                      << BSON("_id" << 3 << "host"
+                                                                    << "h3")))));
+
+    ASSERT_OK(zeroConfig.initialize(BSON("_id"
+                                         << "rs0"
+                                         << "version"
+                                         << 2
+                                         << "members"
+                                         << BSON_ARRAY(BSON("_id" << 1 << "host"
+                                                                  << "h1")
+                                                       << BSON("_id" << 2 << "host"
+                                                                     << "h2"
+                                                                     << "priority"
+                                                                     << 0
+                                                                     << "arbiterOnly"
+                                                                     << true)
+                                                       << BSON("_id" << 3 << "host"
+                                                                     << "h3")))));
+    ASSERT_OK(oneConfig.initialize(BSON("_id"
+                                        << "rs0"
+                                        << "version"
+                                        << 2
+                                        << "members"
+                                        << BSON_ARRAY(BSON("_id" << 1 << "host"
+                                                                 << "h1")
+                                                      << BSON("_id" << 2 << "host"
+                                                                    << "h2"
+                                                                    << "priority"
+                                                                    << 1
+                                                                    << "arbiterOnly"
+                                                                    << true)
+                                                      << BSON("_id" << 3 << "host"
+                                                                    << "h3")))));
+    ASSERT_OK(twoConfig.initialize(BSON("_id"
+                                        << "rs0"
+                                        << "version"
+                                        << 2
+                                        << "members"
+                                        << BSON_ARRAY(BSON("_id" << 1 << "host"
+                                                                 << "h1")
+                                                      << BSON("_id" << 2 << "host"
+                                                                    << "h2"
+                                                                    << "priority"
+                                                                    << 2
+                                                                    << "arbiterOnly"
+                                                                    << true)
+                                                      << BSON("_id" << 3 << "host"
+                                                                    << "h3")))));
+
+    ASSERT_OK(oldConfig.validate());
+    ASSERT_OK(zeroConfig.validate());
+    ASSERT_OK(oneConfig.validate());
+    ASSERT_OK(twoConfig.validate());
+    ASSERT_OK(validateConfigForReconfig(
+                  &externalState, oldConfig, zeroConfig, getGlobalServiceContext(), false)
+                  .getStatus());
+    ASSERT_OK(validateConfigForReconfig(
+                  &externalState, oldConfig, oneConfig, getGlobalServiceContext(), false)
+                  .getStatus());
+    ASSERT_EQUALS(ErrorCodes::InvalidReplicaSetConfig,
+                  validateConfigForReconfig(
+                      &externalState, oldConfig, twoConfig, getGlobalServiceContext(), false)
+                      .getStatus());
+    // Forced reconfigs also do not allow this.
+    ASSERT_EQUALS(ErrorCodes::InvalidReplicaSetConfig,
+                  validateConfigForReconfig(
+                      &externalState, oldConfig, twoConfig, getGlobalServiceContext(), true)
+                      .getStatus());
+}
+
 TEST(ValidateConfigForReconfig, SelfMustEndElectable) {
     // Old and new config are same except for version change and the electability of one node;
     // this is just testing that we must be electable in the new config.
@@ -694,10 +871,10 @@ TEST(ValidateConfigForReconfig, NewConfigInvalid) {
             .getStatus());
 }
 
-TEST(ValidateConfigForStartUp, NewConfigInvalid) {
-    // The new config is not valid due to a duplicate _id value. This tests that if the new
-    // config is invalid, validateConfigForStartUp will return a status indicating what is wrong
-    // with the new config.
+TEST(ValidateConfigForReconfig, NewConfigWriteConcernNotSatisifiable) {
+    // The new config is not valid due to an unsatisfiable write concern. This tests that if the
+    // new config is invalid, validateConfigForReconfig will return a status indicating what is
+    // wrong with the new config.
     ReplSetConfig oldConfig;
     ASSERT_OK(oldConfig.initialize(BSON("_id"
                                         << "rs0"
@@ -707,6 +884,36 @@ TEST(ValidateConfigForStartUp, NewConfigInvalid) {
                                         << BSON_ARRAY(BSON("_id" << 0 << "host"
                                                                  << "h2")))));
 
+    ReplSetConfig newConfig;
+    ASSERT_OK(newConfig.initialize(BSON("_id"
+                                        << "rs0"
+                                        << "version"
+                                        << 1
+                                        << "members"
+                                        << BSON_ARRAY(BSON("_id" << 0 << "host"
+                                                                 << "h2"))
+                                        << "settings"
+                                        << BSON("getLastErrorDefaults" << BSON("w" << 2)))));
+
+    ReplicationCoordinatorExternalStateMock presentOnceExternalState;
+    presentOnceExternalState.addSelf(HostAndPort("h2"));
+    ASSERT_EQUALS(
+        ErrorCodes::CannotSatisfyWriteConcern,
+        validateConfigForReconfig(
+            &presentOnceExternalState, oldConfig, newConfig, getGlobalServiceContext(), false)
+            .getStatus());
+    // Forced reconfigs also do not allow this.
+    ASSERT_EQUALS(
+        ErrorCodes::CannotSatisfyWriteConcern,
+        validateConfigForReconfig(
+            &presentOnceExternalState, oldConfig, newConfig, getGlobalServiceContext(), true)
+            .getStatus());
+}
+
+TEST(ValidateConfigForStartUp, NewConfigInvalid) {
+    // The new config is not valid due to a duplicate _id value. This tests that if the new
+    // config is invalid, validateConfigForStartUp will return a status indicating what is wrong
+    // with the new config.
     ReplSetConfig newConfig;
     ASSERT_OK(newConfig.initialize(BSON("_id"
                                         << "rs0"
@@ -720,63 +927,15 @@ TEST(ValidateConfigForStartUp, NewConfigInvalid) {
 
     ReplicationCoordinatorExternalStateMock presentOnceExternalState;
     presentOnceExternalState.addSelf(HostAndPort("h2"));
-    ASSERT_EQUALS(ErrorCodes::BadValue,
-                  validateConfigForStartUp(
-                      &presentOnceExternalState, oldConfig, newConfig, getGlobalServiceContext())
-                      .getStatus());
+    ASSERT_EQUALS(
+        ErrorCodes::BadValue,
+        validateConfigForStartUp(&presentOnceExternalState, newConfig, getGlobalServiceContext())
+            .getStatus());
 }
 
-TEST(ValidateConfigForStartUp, OldAndNewConfigIncompatible) {
-    // The new config is not compatible with the old config due to a member changing _ids. This
-    // tests that validateConfigForStartUp will return a status indicating the incompatiblilty
-    // between the old and new config.
-    ReplSetConfig oldConfig;
-    ASSERT_OK(oldConfig.initialize(BSON("_id"
-                                        << "rs0"
-                                        << "version"
-                                        << 1
-                                        << "members"
-                                        << BSON_ARRAY(BSON("_id" << 0 << "host"
-                                                                 << "h2")
-                                                      << BSON("_id" << 1 << "host"
-                                                                    << "h3")))));
-
-
-    ReplSetConfig newConfig;
-    ASSERT_OK(newConfig.initialize(BSON("_id"
-                                        << "rs0"
-                                        << "version"
-                                        << 2
-                                        << "members"
-                                        << BSON_ARRAY(BSON("_id" << 2 << "host"
-                                                                 << "h2")
-                                                      << BSON("_id" << 1 << "host"
-                                                                    << "h3")))));
-
-    ReplicationCoordinatorExternalStateMock presentOnceExternalState;
-    presentOnceExternalState.addSelf(HostAndPort("h2"));
-    ASSERT_EQUALS(ErrorCodes::NewReplicaSetConfigurationIncompatible,
-                  validateConfigForStartUp(
-                      &presentOnceExternalState, oldConfig, newConfig, getGlobalServiceContext())
-                      .getStatus());
-}
-
-TEST(ValidateConfigForStartUp, OldAndNewConfigCompatible) {
-    // The new config is compatible with the old config. This tests that
-    // validateConfigForStartUp will return a Status::OK() indicating the validity of this
-    // config change.
-    ReplSetConfig oldConfig;
-    ASSERT_OK(oldConfig.initialize(BSON("_id"
-                                        << "rs0"
-                                        << "version"
-                                        << 1
-                                        << "members"
-                                        << BSON_ARRAY(BSON("_id" << 0 << "host"
-                                                                 << "h2")
-                                                      << BSON("_id" << 1 << "host"
-                                                                    << "h3")))));
-
-
+TEST(ValidateConfigForStartUp, NewConfigValid) {
+    // The new config is valid. This tests that validateConfigForStartUp will return a
+    // Status::OK() indicating the validity of this configuration.
     ReplSetConfig newConfig;
     ASSERT_OK(newConfig.initialize(BSON("_id"
                                         << "rs0"
@@ -792,9 +951,31 @@ TEST(ValidateConfigForStartUp, OldAndNewConfigCompatible) {
 
     ReplicationCoordinatorExternalStateMock presentOnceExternalState;
     presentOnceExternalState.addSelf(HostAndPort("h2"));
-    ASSERT_OK(validateConfigForStartUp(
-                  &presentOnceExternalState, oldConfig, newConfig, getGlobalServiceContext())
-                  .getStatus());
+    ASSERT_OK(
+        validateConfigForStartUp(&presentOnceExternalState, newConfig, getGlobalServiceContext())
+            .getStatus());
+}
+
+TEST(ValidateConfigForStartUp, NewConfigWriteConcernNotSatisfiable) {
+    // The new config contains an unsatisfiable write concern.  We don't allow these configs to be
+    // created anymore, but we allow any which exist to pass and the database to start up to
+    // maintain backwards compatibility.
+    ReplSetConfig newConfig;
+    ASSERT_OK(newConfig.initialize(BSON("_id"
+                                        << "rs0"
+                                        << "version"
+                                        << 2
+                                        << "members"
+                                        << BSON_ARRAY(BSON("_id" << 0 << "host"
+                                                                 << "h2"))
+                                        << "settings"
+                                        << BSON("getLastErrorDefaults" << BSON("w" << 2)))));
+
+    ReplicationCoordinatorExternalStateMock presentOnceExternalState;
+    presentOnceExternalState.addSelf(HostAndPort("h2"));
+    ASSERT_OK(
+        validateConfigForStartUp(&presentOnceExternalState, newConfig, getGlobalServiceContext())
+            .getStatus());
 }
 
 TEST(ValidateConfigForHeartbeatReconfig, NewConfigInvalid) {
@@ -841,6 +1022,29 @@ TEST(ValidateConfigForHeartbeatReconfig, NewConfigValid) {
                   .getStatus());
 }
 
+TEST(ValidateConfigForHeartbeatReconfig, NewConfigWriteConcernNotSatisfiable) {
+    // The new config contains an unsatisfiable write concern.  We don't allow these configs to be
+    // created anymore, but we allow any which exist to be received in a heartbeat.
+    ReplSetConfig newConfig;
+    ASSERT_OK(newConfig.initialize(BSON("_id"
+                                        << "rs0"
+                                        << "version"
+                                        << 2
+                                        << "members"
+                                        << BSON_ARRAY(BSON("_id" << 0 << "host"
+                                                                 << "h2")
+                                                      << BSON("_id" << 1 << "host"
+                                                                    << "h3"))
+                                        << "settings"
+                                        << BSON("getLastErrorDefaults" << BSON("w" << 2)))));
+
+    ReplicationCoordinatorExternalStateMock presentOnceExternalState;
+    presentOnceExternalState.addSelf(HostAndPort("h2"));
+    ASSERT_OK(validateConfigForHeartbeatReconfig(
+                  &presentOnceExternalState, newConfig, getGlobalServiceContext())
+                  .getStatus());
+}
+
 TEST(ValidateForReconfig, ForceStillNeedsValidConfig) {
     // The new config is invalid due to two nodes with the same _id value. This tests that
     // ValidateForReconfig fails with an invalid config, even if force is true.
